// Chunker.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <Windows.h>
#include "strsafe.h"

TCHAR* m_pFileName(NULL);

HANDLE   m_hFile(INVALID_HANDLE_VALUE);               // Handle to file for dumping
HANDLE   m_hInfoFile(INVALID_HANDLE_VALUE);               // Handle to file for dumping

__int64 currentPosition;
__int64 currentFileLength;

int chunkReserve(1);
int flushBeforeReserve(0);
int currentEndPointer(1);
int fileBuffering(1);
int growBy(104857600);

const TCHAR* strUsage = _T("\
Usage: Chunker [options] -out [filename]\n\n\
The DVB stream must be piped to Chunker.  e.g.:\n\n\
    type 0*.ts | Chunker -out output.ts\n\
or: Chunker < file.ts -out output.ts\n\n\
Options:\n\
  -flushBeforeReserve   Flush file buffer before creating reserve chunk\n\
  -noChunkReserve       No reserve chunk\n\
  -noCurrentEndPointer  No info file\n\
  -noFileBuffering      File write-through\n\
  -growBy [value]       Reserve chunk size; default value is 104857600\n");

//
// OpenFile
//
// Opens the file ready for dumping
//
bool OpenFile()
{
    TCHAR *pFileName = NULL;

    // Is the file already opened
    if (m_hFile != INVALID_HANDLE_VALUE)
	{
        return true;
    }

    // Has a filename been set yet
    if (m_pFileName == NULL)
	{
        return false;
    }

    pFileName = m_pFileName;

	// work out the rile flags to use
	DWORD fileCreateFlags = 0;

	if(fileBuffering == 0)
		fileCreateFlags = fileCreateFlags | FILE_FLAG_WRITE_THROUGH;

    m_hFile = CreateFile((LPCTSTR) pFileName,   // The filename
                         GENERIC_WRITE,         // File access
                         FILE_SHARE_READ,       // Share access
                         NULL,                  // Security
                         CREATE_ALWAYS,         // Open flags
                         fileCreateFlags,			// More flags
                         NULL);                 // Template


    if (m_hFile == INVALID_HANDLE_VALUE) 
    {
        return false;
    }

	if(currentEndPointer == 1 && chunkReserve == 1)
	{
		TCHAR infoFile[MAX_PATH];
		StringCchCopy(infoFile, MAX_PATH, pFileName);
		StringCchCat(infoFile, MAX_PATH, _T(".info"));

		m_hInfoFile = CreateFile((LPCTSTR) infoFile,
			GENERIC_WRITE,
			FILE_SHARE_READ,
			NULL,
			CREATE_ALWAYS,
			fileCreateFlags,
			NULL);
	}

    return true;

} // Open


//
// CloseFile
//
// Closes any dump file we have opened
//
bool CloseFile()
{
    if (m_hFile == INVALID_HANDLE_VALUE)
	{
        return true;
    }

	if(chunkReserve == 1)
	{
		LARGE_INTEGER li;
		li.QuadPart = currentPosition;

		SetFilePointer(
			m_hFile,
			li.LowPart,
			&li.HighPart,
			FILE_BEGIN);

		SetEndOfFile(m_hFile);

		currentPosition = 0;
		currentFileLength = 0;
	}


	CloseHandle(m_hFile);
	m_hFile = INVALID_HANDLE_VALUE; // Invalidate the file 

	if(currentEndPointer == 1 && chunkReserve == 1)
	{
		if (m_hInfoFile != INVALID_HANDLE_VALUE)
		{
			CloseHandle(m_hInfoFile);
			m_hInfoFile = INVALID_HANDLE_VALUE;
		}

		TCHAR *pFileName = NULL;
	    pFileName = m_pFileName;

		TCHAR infoFile[MAX_PATH];
		StringCchCopy(infoFile, MAX_PATH, pFileName);
		StringCchCat(infoFile, MAX_PATH, _T(".info"));

		DeleteFile(infoFile);
	}

    return true;

} // Open

//
// Write
//
// Write raw data to the file
//
bool Write(PBYTE pbData, LONG lDataLength)
{
    // If the file has already been closed, don't continue
    if (m_hFile == INVALID_HANDLE_VALUE)
	{
        return false;
    }

	LARGE_INTEGER li;

	// do the reserve chunk is needed

	if((lDataLength + currentPosition) > currentFileLength)
	{
		// increate the length of the file by growFile
		currentFileLength = currentFileLength + growBy;

		if(flushBeforeReserve == 1)
		{
			FlushFileBuffers(m_hFile);

			if(currentEndPointer == 1)
				FlushFileBuffers(m_hInfoFile);
		}

		if(chunkReserve == 1)
		{
			li.QuadPart = currentFileLength;

			SetFilePointer(
				m_hFile,
				li.LowPart,
				&li.HighPart,
				FILE_BEGIN);

			SetEndOfFile(m_hFile);

			li.QuadPart = currentPosition;

			SetFilePointer(
				m_hFile,
				li.LowPart,
				&li.HighPart,
				FILE_BEGIN);
		}
	}

	DWORD written = 0;
	WriteFile(m_hFile, pbData, lDataLength, &written, NULL);

	currentPosition = currentPosition + lDataLength;

	// only write to end pointer file if it is needed
	if(currentEndPointer == 1 && chunkReserve == 1)
	{
		li.QuadPart = 0;
		SetFilePointer(
				m_hInfoFile,
				li.LowPart,
				&li.HighPart,
				FILE_BEGIN);

		WriteFile(m_hInfoFile, &currentPosition, sizeof(__int64), &written, NULL);
	}

    return true;
}

int _tmain(int argc, _TCHAR* argv[])
{
	_tprintf(_T("\nChunker version 1.00, conversion from FileWriter to Chunker.exe by RS (pirayen at gmail)\n\n"));

	if (argc <= 1)
	{
		_tprintf(strUsage);
		return 0;
	}

	for (int op=1; op<argc; op++)
	{
		if ( !_tcscmp(argv[op], _T("-help")) )
		{
			_tprintf(strUsage);
			return 0;
		}
		else if ( !_tcscmp(argv[op], _T("-noChunkReserve")) )
		{
			chunkReserve = 0;
		}
		else if ( !_tcscmp(argv[op], _T("-flushBeforeReserve")) )
		{
			flushBeforeReserve = 1;
		}
		else if ( !_tcscmp(argv[op], _T("-noCurrentEndPointer")) )
		{
			currentEndPointer = 0;
		}
		else if ( !_tcscmp(argv[op], _T("-noFileBuffering")) )
		{
			fileBuffering = 0;
		}
		else if ( !_tcscmp(argv[op], _T("-growBy")) && (argc > op+1) )
		{
			op++;
			int value = _ttoi(argv[op]);
			if (value > 0)
				growBy = value;		
		}
		else if ( !_tcscmp(argv[op], _T("-out")) && (argc > op+1) )
		{
			op++;
			m_pFileName = argv[op];
		}
		else
		{
			_tprintf(_T("Unknown option:%s\n"), argv[op]);
			return 1;
		}
	}

	if ( !OpenFile() )
	{
		_tprintf(_T("Error opening file:%s\n"), m_pFileName);
		return 1;
	}

	HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE);
	DWORD dwRead;
	BYTE chBuf[8192];
	while ( ReadFile(hStdin, chBuf, 8192, &dwRead, NULL) && (dwRead > 0) )
	{
		if ( !Write(chBuf, dwRead) )
		{
			_tprintf(_T("Error writing file\n"));
			return 1;
		}
	}

	if ( !CloseFile() )
	{
		_tprintf(_T("Error closing file:%s\n"), m_pFileName);
		return 1;
	}

	return 0;
}
