<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:template match="/">
    <html>
    <head>
    <title>DV Scheduler Surf List</title>
    <link rel="stylesheet" href="/css/main.css" type="text/css"/>
    <link rel="stylesheet" href="/css/colour.css" type="text/css" />
<style>
div.listTitle
{
  background: #789AFF;
  cursor:hand;
  cursor:pointer;
}

div.list
{
  overflow:hidden;
}
</style>
<script src="/javascript/extra.js"></script>
<script src="/javascript/-epgPage.js" />
<xsl:comment>[if lt IE 7.0]&gt;
&lt;script src="/javascript/iepng.js"&gt;&lt;/script&gt;
&lt;![endif]</xsl:comment>
<script>
  function loadSetting()
  {
    setList("nowNextList");
    setList("channelList");
    setList("fileList");
    setList("devicesList");
    
    loadScrollPosition();
  }

  function setup()
  {
    loadSetting();
    
    var items = document.getElementsByTagName("body");
    parent.setIFrameFullHeight(items[0].scrollHeight);
    
    if (document.attachEvent)
    {
      document.attachEvent("onkeydown", keyHandler);
    }
    
    if (document.addEventListener)
    {
      document.addEventListener("keydown", keyHandler, false);
    }
  }
  
  function setList(divName)
  {
    var list = readCookie("SurfList-" + divName);
    if (list.length > 0)
    {
      setCookie("SurfList-" + divName, list, 320);
      if (list == '0')
      {
        var divList = document.getElementById(divName);
        divList.style.height = 1;
      }
    }
  }
  
  function saveScrollPosition()
  {
    var items = document.getElementsByTagName("body");
    var scroll = items[0].scrollTop;
  
    var cookieID = document.location.href;
    setCookie("SurfListScrollTo", scroll, 32);
  }
  
  function loadScrollPosition()
  {
    var scrollTo = parseInt(readCookie("SurfListScrollTo"));
    if (!isNaN(scrollTo))
    {
      var items = document.getElementsByTagName("body");
      items[0].scrollTop = scrollTo;
    }
  }

  function keyHandler(e)
  {
    parent.keyHandler(e);
  }

  function toggleShow(divName)
  {
    var divList = document.getElementById(divName);
    if (divList.style.height == '')
    {
      divList.style.height = '1px';
      setCookie("SurfList-" + divName, 0, 320);
    }
    else
    {
      divList.style.height = '';
      setCookie("SurfList-" + divName, 1, 320);
    }
    
    var items = document.getElementsByTagName("body");
    parent.setIFrameFullHeight(items[0].scrollHeight);
  }

  function setCurrentDirectory(path)
  {
    setCookie("SurfCurrentDirectory", path, 320);
    document.location.href = "/servlet/ScheduleDataRes?action=28";
  }

  function delFile(obj)
  {
    var path = obj.parentNode.getAttribute('path');
    if ( confirm("Are you sure you want to delete the following file?\n" + path) )
    {
      document.location.href = '/servlet/FileManagementDataRes?action=01&amp;file=' + encodeURIComponent(path) + '&amp;dari=' + encodeURIComponent(document.location.href);
    }
  }
  
  function addAutoDel(obj)
  {
    var path = obj.parentNode.getAttribute('path');
    document.location.href = '/servlet/FileManagementDataRes?action=04&amp;file=' + encodeURIComponent(path) + '&amp;dari=' + encodeURIComponent(document.location.href);
  }
  
  function removeAutoDel(obj)
  {
    var path = obj.parentNode.getAttribute('path');
    var id = obj.parentNode.getAttribute('autoDeleteId');
    document.location.href = '/servlet/FileManagementDataRes?action=05&amp;id=' + id + '&amp;file=' + encodeURIComponent(path) + '&amp;dari=' + encodeURIComponent(document.location.href);
  }

  function changeChannel(obj)
  {
    var channel = obj.getAttribute('channel');
    parent.changeChannel(1,channel);
  }

  function record(obj)
  {
    var channel = obj.getAttribute('channel');
    parent.record(channel,null);
  }

  function recordFromGuide(obj)
  {
    var channel = obj.getAttribute('channel');
    var guideID = obj.getAttribute('id');
    parent.recordFromGuide(channel,guideID);
  }
</script>
  </head>
  <body onload="setup();" onUnload="saveScrollPosition();">
  <div class="listTitle" title="Hide/Show" onClick="toggleShow('devicesList')"><b>Devices</b></div>
  <div id="devicesList" class="list">
  <xsl:if test="//lists/devicesList/@height > 0">
  <xsl:attribute name="style">height:<xsl:value-of select="//lists/devicesList/@height" />px</xsl:attribute>
  </xsl:if>
  <table width="100%">
  <xsl:choose>
    <xsl:when test="//devices/@total = 0">
    <tr><td>No capture device selected</td></tr>
    </xsl:when>
    <xsl:when test="//devices/@total_running = 0">
    <tr><td>No device is running</td></tr>
    </xsl:when>
    <xsl:otherwise>
    <xsl:apply-templates select="//devices/device" />
    </xsl:otherwise>
  </xsl:choose>
  </table>
  <br/>
  </div>
  <div class="listTitle" title="Hide/Show" onClick="toggleShow('nowNextList')"><b>EPG</b></div>
  <div id="nowNextList" class="list">
  <xsl:if test="//lists/nowNextList/@height > 0">
  <xsl:attribute name="style">height:<xsl:value-of select="//lists/nowNextList/@height" />px</xsl:attribute>
  </xsl:if>
  <table width="100%">
  <xsl:if test="count(//now_and_next/channel) > 0">
    <xsl:apply-templates select="//now_and_next/channel" />
  </xsl:if>
  <xsl:if test="count(//now_and_next/channel) = 0">
    <tr><td colspan="5">No EPG items</td></tr>	
  </xsl:if>
  </table>
  <br/>
  </div>
  <div class="listTitle" title="Hide/Show" onClick="toggleShow('channelList')"><b>Channels</b></div>
  <div id="channelList" class="list">
  <xsl:if test="//lists/channelList/@height > 0">
  <xsl:attribute name="style">height:<xsl:value-of select="//lists/channelList/@height" />px</xsl:attribute>
  </xsl:if>
  <table width="100%">
  <xsl:choose>
    <xsl:when test="count(//orders/channelorder) = 0">
    <tr><td>No channel available</td></tr>
    </xsl:when>
    <xsl:otherwise>
    <xsl:apply-templates select="//orders/channelorder" />
    </xsl:otherwise>
  </xsl:choose>
  </table>
  <br/>
  </div>
  <div class="listTitle" title="Hide/Show" onClick="toggleShow('fileList')"><table width="100%" border="0" cellspacing="0" cellpadding="0"><tr><td><b>Files</b></td><xsl:if test="//files/@currentDirectory"><td><xsl:value-of select="//files/@currentDirectory"/></td><td align='right'>(<span><xsl:if test="number(//files/@minFreeSpace) > number(//files/@freeSpace)"><xsl:attribute name="class">warning</xsl:attribute></xsl:if><xsl:value-of select="format-number(floor(number(//files/@freeSpace) div (1024*1024)),'###,###')" /> MB</span> free)</td></xsl:if></tr></table></div>
  <div id="fileList" class="list">
  <xsl:if test="//lists/fileList/@height > 0">
  <xsl:attribute name="style">height:<xsl:value-of select="//lists/fileList/@height" />px</xsl:attribute>
  </xsl:if>
  <table width="100%">
  <xsl:if test="count(//files/file) > 0">
  <xsl:choose>
    <xsl:when test="//files/@currentDirectory or (//@structuredFileList='0')">
    <xsl:apply-templates select="//files/file" />
    </xsl:when>
    <xsl:otherwise>
    <xsl:apply-templates select="//directories/directory" />
    </xsl:otherwise>
  </xsl:choose>
  </xsl:if>
  </table>
  </div>
<div id="balloon_tooltip" align="left" style="font-family: Tahoma, Verdana, Arial, Helvetica; font-size: 90%; font-weight:normal; filter:alpha(opacity=95);KHTMLOpacity:0.95;MozOpacity:0.95;opacity:0.95; position:absolute; overflow:visible; BACKGROUND-COLOR: #306800; padding:5px; display:none; top:1px; left:1px;" />
<div id="balloon_tooltip2" align="left" style="font-family: Tahoma, Verdana, Arial, Helvetica; font-size: 90%; font-weight:normal; filter:alpha(opacity=95);KHTMLOpacity:0.95;MozOpacity:0.95;opacity:0.95; position:absolute; overflow:visible; BACKGROUND-COLOR: #306800; padding:5px; visibility:hidden; top:1px; left:1px;" />
  </body>
  </html>

</xsl:template>

<xsl:template match="orders/channelorder">
<td class="cellLow" rowspan="1" style='cursor:hand;cursor:pointer;' channel="{@name}">
<xsl:if test="//orders/@channelImage = 1"><xsl:attribute name="align">center</xsl:attribute></xsl:if>
<xsl:if test="(@show = 1) or (@show = 2)">
<xsl:attribute name="onClick">changeChannel(this)</xsl:attribute>
<xsl:attribute name="onContextMenu">record(this); return false</xsl:attribute>
<img style="border:0px;cursor:hand;cursor:pointer;" channel="{@name}"><xsl:attribute name="title"><xsl:value-of select="@number" />. <xsl:value-of select="@name" /></xsl:attribute><xsl:attribute name="src"><xsl:value-of select="@image-file" /></xsl:attribute><xsl:attribute name="width"><xsl:value-of select="@image-width" /></xsl:attribute></img>
</xsl:if>
<xsl:if test="(@show = 2)"><br/></xsl:if>
<xsl:if test="(@show = 0) or (@show = 2)">
<xsl:attribute name="onClick">changeChannel(this)</xsl:attribute>
<xsl:attribute name="onContextMenu">record(this); return false</xsl:attribute>
<xsl:value-of select="@number" />. <xsl:value-of select="@name" />
</xsl:if>
</td>
<xsl:if test="(position() mod 5 = 0)"><tr/></xsl:if>
</xsl:template>	

<xsl:template match="files/file">
<xsl:choose>
  <xsl:when test="//@structuredFileList='1'">
  <xsl:choose>
    <xsl:when test="@type='0'">
    <tr><td class="cellLow" style='cursor:hand;cursor:pointer;' onClick='parent.playFile("{@name}","{@path}","{//files/@directory}","{//files/@relativeDirectory}/{@name}")'>
    <xsl:attribute name="onClick">parent.playFile("<xsl:value-of select="@name" />","<xsl:value-of select="@path" />","<xsl:value-of select="//files/@directory" />","<xsl:if test="//files/@relativeDirectory != ''"><xsl:value-of select="//files/@relativeDirectory" />/</xsl:if><xsl:value-of select="@name" />")</xsl:attribute>
    <xsl:attribute name="title">Server path: <xsl:value-of select="//files/@currentDirectory" /></xsl:attribute>
    <xsl:value-of select="@name" /></td>
    <td class="cellLow" nowrap='1' align="right" width="1%"><xsl:value-of select="format-number(ceiling(number(@size) div 1024),'###,###')" /> KB</td>
    <td class="cellLow" path="{@path}" autoDeleteId="{@autoDeleteId}" nowrap='1' width="1%">
    <img onClick="delFile(this)" class='hand' align='absmiddle' src='/images/delete.png' border='0' alt='Delete' title='Delete' width='24' height='24'/><xsl:text> </xsl:text>
    <xsl:choose>
      <xsl:when test="@autoDeleteId">
      <img onClick="removeAutoDel(this)" class='hand' align='absmiddle' src='/images/removeAutoDel.png' border='0' alt='Remove from Auto-Delete' title='Remove from Auto-Delete' width='24' height='24'/>
      </xsl:when>
      <xsl:otherwise>
      <img onClick="addAutoDel(this)" class='hand' align='absmiddle' src='/images/addAutoDel.png' border='0' alt='Add to Auto-Delete' title='Add to Auto-Delete' width='24' height='24'/>
      </xsl:otherwise>
    </xsl:choose></td></tr>
    </xsl:when>
    <xsl:otherwise>
    <tr><td class="cellLow" colspan="3" style='cursor:hand;cursor:pointer;' onClick='setCurrentDirectory("{@path}")'>
    &lt;<xsl:value-of select="@name" />&gt;</td></tr>
    </xsl:otherwise>
  </xsl:choose>
  </xsl:when>
  <xsl:otherwise>
  <tr><td class="cellLow" rowspan="1" style='cursor:hand;cursor:pointer;' onClick='parent.playFile("{@relativePath}","{@path}","{@directory}","{@relativePath}")'>
  <xsl:attribute name="title">Server path: <xsl:value-of select="@directory" /></xsl:attribute>
  <xsl:value-of select="@relativePath" /></td>
  <td class="cellLow" nowrap='1' align="right" width="1%"><xsl:value-of select="format-number(ceiling(number(@size) div 1024),'###,###')" /> KB</td>
  <td class="cellLow" path="{@path}" autoDeleteId="{@autoDeleteId}" nowrap='1' width="1%">
  <img onClick="delFile(this)" class='hand' align='absmiddle' src='/images/delete.png' border='0' alt='Delete' title='Delete' width='24' height='24'/></td></tr>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="directories/directory">
<tr><td class="cellLow" colspan="3" style='cursor:hand;cursor:pointer;' onClick='setCurrentDirectory("{@name}")'>&lt;<xsl:value-of select="@name" />&gt;</td>
<td class="cellLow" nowrap='1' align="right" width="1%"><span><xsl:if test="number(@minFreeSpace) > number(@freeSpace)"><xsl:attribute name="class">warning</xsl:attribute></xsl:if><xsl:value-of select="format-number(floor(number(@freeSpace) div (1024*1024)),'###,###')" /> MB</span> free</td></tr>
</xsl:template>

<xsl:template match="now_and_next/channel" >
<tr>
<td nowrap="1" class="cellHi">
<xsl:if test="//now_and_next/@channelImage = 1"><xsl:attribute name="align">center</xsl:attribute></xsl:if>
  <xsl:if test="(@show = 1) or (@show = 2)">
  <img style="border:0px;cursor:hand;cursor:pointer;" channel="{@ws_channel}" onClick="changeChannel(this)" onContextMenu="record(this); return false">
  <xsl:attribute name="width"><xsl:value-of select="@image-width" /></xsl:attribute>
  <xsl:attribute name="title"><xsl:value-of select="@ws_channel" /></xsl:attribute>
  <xsl:attribute name="src"><xsl:value-of select="@image-file" /></xsl:attribute></img>
  </xsl:if>
  <xsl:if test="(@show = 2)"><br/></xsl:if>
  <xsl:if test="(@show = 0) or (@show = 2)">
  <span style="border:0px;cursor:hand;cursor:pointer;" channel="{@ws_channel}" onClick="changeChannel(this)" onContextMenu="record(this); return false">
  <xsl:value-of select="@ws_channel" /></span>
  </xsl:if>
</td>	   
      <td nowrap="1" class="cellLow">
      <xsl:if test="string-length(now/start/@hour_12) &gt; 0">    
	      <xsl:value-of select="now/start/@time2" /> (<xsl:value-of select="now/duration" />)
      </xsl:if>
      </td>
<td class="cellLow">
<b><span onMouseOut="hideTip();" onClick="changeChannel(this)" onContextMenu="recordFromGuide(this); return false">
<xsl:attribute name="onMouseMove">showTip(event,this,<xsl:value-of select="//@balloonDelay" />);</xsl:attribute>
<xsl:attribute name="desc2"><xsl:value-of select="now/desc" /></xsl:attribute>
<xsl:attribute name="id"><xsl:value-of select="now/id" /></xsl:attribute>
<xsl:attribute name="channel"><xsl:value-of select="@ws_channel" /></xsl:attribute>
<xsl:attribute name="class">programTitle 
<xsl:if test="now/name/@matchedAA = '1'">programTitleMatched </xsl:if>
<xsl:if test="now/name/@excludedAA = '1'">programTitleExcluded </xsl:if>
<xsl:if test="contains(now/desc, '~Err~ ')">programTitleError </xsl:if>
</xsl:attribute>
      <xsl:value-of select="now/name" />
      </span></b>
      </td>
      <td></td>    
      <td nowrap="1" class="cellLow">  
      <xsl:if test="string-length(next/start/@hour_12) &gt; 0">    
         <xsl:value-of select="next/start/@time2" /> (<xsl:value-of select="next/duration" />)
      </xsl:if>
      </td>
      <td class="cellLow">
      <b><span onMouseOut="hideTip();" onClick="changeChannel(this)" onContextMenu="recordFromGuide(this); return false">
<xsl:attribute name="onMouseMove">showTip(event,this,<xsl:value-of select="//@balloonDelay" />);</xsl:attribute>
<xsl:attribute name="desc2"><xsl:value-of select="next/desc" /></xsl:attribute>
<xsl:attribute name="id"><xsl:value-of select="next/id" /></xsl:attribute>
<xsl:attribute name="channel"><xsl:value-of select="@ws_channel" /></xsl:attribute>
<xsl:attribute name="class">programTitle 
<xsl:if test="next/name/@matchedAA = '1'">programTitleMatched </xsl:if>
<xsl:if test="next/name/@excludedAA = '1'">programTitleExcluded </xsl:if>
<xsl:if test="contains(next/desc, '~Err~ ')">programTitleError </xsl:if>
</xsl:attribute>
      <xsl:value-of select="next/name" />
      </span></b>
      </td>
      </tr>	
	</xsl:template>

	<xsl:template match="devices/device" >
    <xsl:choose>
      <xsl:when test="@status > 0">
	   <tr>
      <td class="cellHi">
      <xsl:value-of select="@name" />
      </td>
      <td class="cellHi">
      Running <xsl:value-of select="@setting" />
      </td>
      <xsl:choose>
        <xsl:when test="@total_capture = 0"><td nowrap="1" class="cellHi">idle</td><td align="right"><a href="/servlet/ScheduleDataRes?action=ScheduleStopIdleDevice&amp;index={@index}"><img title="Stop" src="/images/stop.png" border="0" align="center" width="24" height="24"/></a></td></xsl:when>
        <xsl:when test="@total_capture = 1"><td nowrap="1" colspan="2" class="cellHi">1 capture</td></xsl:when>
        <xsl:otherwise><td nowrap="1" colspan="2" class="cellHi"><xsl:value-of select="@total_capture" /> captures</td></xsl:otherwise>
      </xsl:choose>
      </tr>	
      </xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>
	</xsl:template>
</xsl:stylesheet>
