/*
 Copyright (C) 2006 Shaun Faulds & John Calvi

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

public class CaptureDetails
{
	private DataStore store = null;

	public CaptureDetails()
	{

	}

	public void writeCaptureDetails(CaptureTask task, CaptureDevice deviceID)
	{
		try
		{
			store = DataStore.getInstance();
			String detail = store.getProperty("Capture.details", "none");
			String pathString = store.getProperty("Capture.path.details", "");

			if(detail.trim().equalsIgnoreCase("none"))
				return;

			File outputPath = null;

			CaptureFile cap = task.getCaptureFile();

			if (detail.trim().startsWith("path"))
			{
				File capFile = new File(cap.getFullPath());
				outputPath = new File(pathString + File.separator + capFile.getName() + ".xml");               
			}
			else
			{
				outputPath = new File(cap.getFullPath() + ".xml");
			}

			if(outputPath.getParentFile().exists() == false)
			{
				DVScheduler.log("ERROR details path does not exist: " + outputPath.toString());
				return;
			}

			DVScheduler.log("Writing Capture Details info to: " + outputPath.toString());

			String xmlData = "";
			ScheduleItem schItem = task.getScheduleItem();

			//File file = new File(cap.getFileName());
			//if( file.exists() && (file.length() > 0) )
			{
				xmlData = getXMLData(cap.getFullPath(), deviceID, schItem);

				int count = 1;
				while (outputPath.exists())
				{
					String newPath = outputPath.getCanonicalPath();
					newPath = newPath.substring(0, newPath.length() - 4) + "-" + (count++) + ".xml";
					DVScheduler.log("Details file already exists, creating new name : " + newPath);
					outputPath = new File(newPath);
				}

				FileWriter writer = new FileWriter(outputPath);
				writer.write(xmlData);
				writer.close();
			}
		}
		catch (Exception e)
		{
			DVScheduler.log("Error writing Capture Details XML file.");
			e.printStackTrace();
		}
	}

	private String getXMLData(String fileName, CaptureDevice cardID, ScheduleItem schItem)
	{
		GuideStore guide = GuideStore.getInstance();

		ByteArrayOutputStream buff = new ByteArrayOutputStream();

		try
		{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			DOMImplementation di = db.getDOMImplementation();
			Document doc = di.createDocument("", "capture", null);

			Element root = doc.getDocumentElement();
			Element elm = null;
			Text text = null;

			//Task Items
			elm = doc.createElement("ws_cardID");
			text = doc.createTextNode(new Integer(cardID.getNumber()).toString());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_cardname");
			text = doc.createTextNode(cardID.getName());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_filename");
			text = doc.createTextNode(new File(fileName).getName());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_fullfilename");
			text = doc.createTextNode(new File(fileName).getCanonicalPath());
			elm.appendChild(text);
			root.appendChild(elm);

			ScheduleItem item = schItem;

			// Schedule Items       
			elm = doc.createElement("ws_name");
			text = doc.createTextNode(item.getName());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_start");
			text = doc.createTextNode(item.getStart().toString());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_stop");
			text = doc.createTextNode(item.getStop().toString());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_duration");
			text = doc.createTextNode(new Integer(item.getDuration()).toString());
			elm.appendChild(text);
			root.appendChild(elm);

			elm = doc.createElement("ws_channel");
			text = doc.createTextNode(item.getChannel());
			elm.appendChild(text);
			root.appendChild(elm);

			String epgChan = guide.getEpgChannelIdFromMap(item.getChannel());
			GuideItem[] programs = guide.getProgramsInc(item.getStart(), item.getStop(), epgChan);

			for(int index = 0; index < programs.length; index++)
			{
				GuideItem guideItem = programs[index];

				if(guideItem != null)
				{
					Element epgItem = doc.createElement("epg_item");
					root.appendChild(epgItem);

					// Guide Items
					elm = doc.createElement("epg_title");
					text = doc.createTextNode(guideItem.getName());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_subtitle");
					text = doc.createTextNode(guideItem.getSubName());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_description");
					text = doc.createTextNode(guideItem.getDescription());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_channel");
					text = doc.createTextNode(epgChan);
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_start");
					text = doc.createTextNode(guideItem.getStart().toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_stop");
					text = doc.createTextNode(guideItem.getStop().toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_duration");
					text = doc.createTextNode(new Integer(guideItem.getDuration())
					.toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					Vector<String> categories = guideItem.getCategories();
					if (categories != null)
					{
						for (int i=0; i<categories.size(); i++)
						{
							elm = doc.createElement("epg_category");
							text = doc.createTextNode(categories.get(i));
							elm.appendChild(text);
							epgItem.appendChild(elm);
						}
					}

					elm = doc.createElement("epg_language");
					text = doc.createTextNode(guideItem.getLanguage());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_ratings");
					text = doc.createTextNode(guideItem.getRatings());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_widescreen");
					text = doc.createTextNode(new Boolean(guideItem.getWidescreen()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_highdef");
					text = doc.createTextNode(new Boolean(guideItem.getHighDef()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_repeat");
					text = doc.createTextNode(new Boolean(guideItem.getRepeat()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_premiere");
					text = doc.createTextNode(new Boolean(guideItem.getPremiere()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_live");
					text = doc.createTextNode(new Boolean(guideItem.getLive()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_interactive");
					text = doc.createTextNode(new Boolean(guideItem.getInteractive()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_blackwhite");
					text = doc.createTextNode(new Boolean(guideItem.getBlackWhite()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_surround");
					text = doc.createTextNode(new Boolean(guideItem.getSurround()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_ac3");
					text = doc.createTextNode(new Boolean(guideItem.getAC3()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_lastchance");
					text = doc.createTextNode(new Boolean(guideItem.getLastChance()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_captions");
					text = doc.createTextNode(new Boolean(guideItem.getCaptions()).toString());
					elm.appendChild(text);
					epgItem.appendChild(elm);

					elm = doc.createElement("epg_url");
					text = doc.createTextNode(guideItem.getURL());
					elm.appendChild(text);
					epgItem.appendChild(elm);
				}
			}

			// Transform Document
			TransformerFactory factory = TransformerFactory.newInstance();
			Transformer xformer = factory.newTransformer();

			Source source = new DOMSource(doc);
			Result result = new StreamResult(buff);
			xformer.transform(source, result);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return buff.toString();
	}

}
