/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.Vector;

public class CaptureEngine
{
	private String name = "";
	private boolean m_bStdin = true;
	private boolean m_bStdout = true;
	private boolean m_bExitTerminate = false;
	private boolean bCanRecordMultiple = false;
	private String initialCommand = "";
	private String captureEngineReadyMessage = "";
	private String recordingStartedMessage = "";
	private String recordingErrorMessage = "";
	private String stopRecordingCommand = "";
	private String recordingStoppedMessage = "";
	private String exitCommand = "";
	private String captureEngineExitingMessage = "";
	private Vector<String> charSubstitutionFroms = new Vector<String>(); 
	private Vector<String> charSubstitutionTos = new Vector<String>();
	private String m_initialLocalPreviewCommand = "";
	private String m_initialRemotePreviewCommand = "";
	private String m_initialLocalSurfCommand = "";
	private String m_initialRemoteSurfCommand = "";
	private String m_changeSettingCommand = "";
	private String m_signalStatisticCommand = "";
	private String m_pidChangeMessage = "";

	private String m_initialEpgScanCommand = "";
	private String m_epgScanCommand = "";
	private String m_stopEpgScanCommand = "";
	
	private String m_initialScanCommand = "";

	private String m_deviceListCommand = "";
	private int m_deviceIndexBase = -1;
	
	private Vector<String> m_keywordFroms = new Vector<String>();
	private Vector<String> m_keywordTos = new Vector<String>();

	protected String getValue(String text)
	{
		return text.substring(text.indexOf("->") + 2);
	}

	protected int getValueInt(String text, int defaultValue)
	{
		try
		{
			return Integer.parseInt( getValue(text) );
		}
		catch(Exception e)
		{
			return defaultValue;
		}
	}

	public boolean SetEngine(String name2, String fullPath)
	{
		charSubstitutionFroms.removeAllElements();
		charSubstitutionTos.removeAllElements();
		m_keywordFroms.clear();
		m_keywordTos.clear();
		
		m_changeSettingCommand = "";
		m_signalStatisticCommand = "";
		stopRecordingCommand = "";
		exitCommand = "";
		m_epgScanCommand = "";
		m_stopEpgScanCommand = "";
		m_deviceListCommand = "";

		try
		{
			File dir = new File(fullPath);

			if (!dir.exists() || !dir.isFile())
			{
				DVScheduler.log("Capture engine file (" + fullPath + ") doesn't exist");
				return false;
			}
			else if (dir.length() > 8192)
			{
				DVScheduler.log("Capture engine file (" + fullPath + ") > 8192 bytes - too big, check for error");
				return false;
			}

			BufferedReader r = new BufferedReader(new FileReader(fullPath));
			String line = null;
			while ((line = r.readLine()) != null)
			{
				line = line.trim();

				if (line.startsWith("Keyword->"))
				{
					String[] subs = getValue(line).split("<-To->");
					if ( (subs.length != 2) || (subs[0].length() <= 0) )
					{
						DVScheduler.log("Error with " + line);
						return false;
					}

					m_keywordFroms.add(subs[0]);
					m_keywordTos.add(subs[1]);
					//DVScheduler.log("Keyword->" + subs[0] + "<-To->" + subs[1]);
				}
				else if (line.startsWith("Can Record Multiple->"))
				{
					bCanRecordMultiple = line.substring(21).trim().equals("1") ? true : false;
					//DVScheduler.log("Can Record Multiple->" + bCanRecordMultiple);
				}
				else if (line.startsWith("Stdin->"))
				{
					m_bStdin = line.substring(7).trim().equals("1") ? true : false;
				}
				else if (line.startsWith("Stdout->"))
				{
					m_bStdout = line.substring(8).trim().equals("1") ? true : false;
				}
				else if (line.startsWith("ExitTerminate->"))
				{
					m_bExitTerminate = line.substring(15).trim().equals("1") ? true : false;
				}
				else if (line.startsWith("Initial Command->"))
				{
					initialCommand = line.substring(17);
					//DVScheduler.log("Initial Command->" + initialCommand);
				}
				else if (line.startsWith("Initial Local Preview Command->"))
				{
					m_initialLocalPreviewCommand = getValue(line);
				}
				else if (line.startsWith("Initial Remote Preview Command->"))
				{
					m_initialRemotePreviewCommand = getValue(line);
				}
				else if (line.startsWith("Initial Local Surf Command->"))
				{
					m_initialLocalSurfCommand = getValue(line);
				}
				else if (line.startsWith("Initial Remote Surf Command->"))
				{
					m_initialRemoteSurfCommand = getValue(line);
				}
				else if (line.startsWith("Change Setting Command->"))
				{
					m_changeSettingCommand += getValue(line) + "\n";
				}
				else if (line.startsWith("Signal Statistic Command->"))
				{
					m_signalStatisticCommand += getValue(line) + "\n";
				}
				else if (line.startsWith("Capture Engine Ready Message->"))
				{
					captureEngineReadyMessage = line.substring(30);
					//DVScheduler.log("Capture Engine Ready Message->" + captureEngineReadyMessage);
				}
				else if (line.startsWith("Recording Started Message->"))
				{
					recordingStartedMessage = line.substring(27);
					//DVScheduler.log("Recording Started Message->" + recordingStartedMessage);
				}
				else if (line.startsWith("Recording Error Message->"))
				{
					recordingErrorMessage = line.substring(25);
					//DVScheduler.log("Recording Error Message->" + recordingErrorMessage);
				}
				else if (line.startsWith("PID Change Message->"))
				{
					m_pidChangeMessage = getValue(line);
				}
				else if (line.startsWith("Stop Recording Command->"))
				{
					stopRecordingCommand += getValue(line) + "\n";
					//DVScheduler.log("Stop Recording Command->" + stopRecordingCommand);
				}
				else if (line.startsWith("Recording Stopped Message->"))
				{
					recordingStoppedMessage = line.substring(27);
					//DVScheduler.log("Recording Stopped Message->" + recordingStoppedMessage);
				}
				else if (line.startsWith("Exit Command->"))
				{
					exitCommand += getValue(line) + "\n";
					//DVScheduler.log("Exit Command->" + exitCommand);
				}
				else if (line.startsWith("Capture Engine Exiting Message->"))
				{
					captureEngineExitingMessage = line.substring(32);
					//DVScheduler.log("Capture Engine Exiting Message->" + captureEngineExitingMessage);
				}
				else if (line.startsWith("Filename Character Substitution->"))
				{
					String[] subs = line.substring(33).split("<-To->");
					if ( (subs.length != 2) || (subs[0].length() <= 0) )
					{
						DVScheduler.log("Error with " + line);
						return false;
					}

					charSubstitutionFroms.add(subs[0]);
					charSubstitutionTos.add(subs[1]);
					//DVScheduler.log("Filename Character Substitution->" + subs[0] + "<-To->" + subs[1]);
				}
				else if (line.startsWith("Initial Scan Command->"))
				{
					m_initialScanCommand = getValue(line);
				}
				else if (line.startsWith("Initial EPG Scan Command->"))
				{
					m_initialEpgScanCommand = getValue(line);
				}
				else if (line.startsWith("EPG Scan Command->"))
				{
					m_epgScanCommand += getValue(line) + "\n";
				}
				else if (line.startsWith("Stop EPG Scan Command->"))
				{
					m_stopEpgScanCommand += getValue(line) + "\n";
				}
				else if (line.startsWith("Device List Command->"))
				{
					m_deviceListCommand += getValue(line) + "\n";
				}
				else if (line.startsWith("Device Index Base->"))
				{
					m_deviceIndexBase = getValueInt(line, -1);
				}
			}

			r.close();
		}
		catch (Exception e)
		{
			DVScheduler.log("Error reading capture engine file (" + fullPath + ")");
			e.printStackTrace();

			return false;
		}

		name = name2;
		initialCommand = replaceKeywordsUser(initialCommand);
		captureEngineReadyMessage = replaceKeywordsUser(captureEngineReadyMessage);
		recordingStartedMessage = replaceKeywordsUser(recordingStartedMessage);
		recordingErrorMessage = replaceKeywordsUser(recordingErrorMessage);
		stopRecordingCommand = replaceKeywordsUser(stopRecordingCommand);
		recordingStoppedMessage = replaceKeywordsUser(recordingStoppedMessage);
		exitCommand = replaceKeywordsUser(exitCommand);
		captureEngineExitingMessage = replaceKeywordsUser(captureEngineExitingMessage);
		m_initialLocalPreviewCommand = replaceKeywordsUser(m_initialLocalPreviewCommand);
		m_initialRemotePreviewCommand = replaceKeywordsUser(m_initialRemotePreviewCommand);
		m_initialLocalSurfCommand = replaceKeywordsUser(m_initialLocalSurfCommand);
		m_initialRemoteSurfCommand = replaceKeywordsUser(m_initialRemoteSurfCommand);
		m_changeSettingCommand = replaceKeywordsUser(m_changeSettingCommand);
		m_signalStatisticCommand = replaceKeywordsUser(m_signalStatisticCommand);
		m_pidChangeMessage = replaceKeywordsUser(m_pidChangeMessage);
		m_initialEpgScanCommand = replaceKeywordsUser(m_initialEpgScanCommand);
		m_epgScanCommand = replaceKeywordsUser(m_epgScanCommand);
		m_stopEpgScanCommand = replaceKeywordsUser(m_stopEpgScanCommand);
		m_initialScanCommand = replaceKeywordsUser(m_initialScanCommand);
		m_deviceListCommand = replaceKeywordsUser(m_deviceListCommand);
		return true;
	}

	public String getName()
	{
		return name;
	}

	public boolean isCanRecordMultiple()
	{
		return bCanRecordMultiple;
	}

	public boolean isStdin()
	{
		return m_bStdin;
	}

	public boolean isStdout()
	{
		return m_bStdout;
	}

	public boolean isExitTerminate()
	{
		return m_bExitTerminate;
	}

	public String getInitialCommand()
	{
		return initialCommand;
	}

	public String getInitialLocalPreviewCommand()
	{
		return m_initialLocalPreviewCommand;
	}

	public String getInitialRemotePreviewCommand()
	{
		return m_initialRemotePreviewCommand;
	}

	public String getInitialLocalSurfCommand()
	{
		return m_initialLocalSurfCommand;
	}

	public String getInitialRemoteSurfCommand()
	{
		return m_initialRemoteSurfCommand;
	}

	public String getChangeSettingCommand()
	{
		return m_changeSettingCommand;
	}

	public String getSignalStatisticCommand()
	{
		return m_signalStatisticCommand;
	}

	public String getStopRecordingCommand()
	{
		return stopRecordingCommand;
	}

	public String getExitCommand()
	{
		return exitCommand;
	}

	protected boolean isMessage(String text, String message, long rid)
	{
		if (message.isEmpty() || text.isEmpty())
			return false;

		String key = "$rid";
		int keyPos = message.indexOf(key);

		if (keyPos >= 0)
		{
			String begin = message.substring(0, keyPos);
			String end = message.substring(keyPos + key.length());

			if (text.startsWith(begin) && text.endsWith(end))
			{
				int keyValueLength = text.length() - begin.length() - end.length();
				if (keyValueLength > 0)
				{
					try
					{
						if (rid == Long.parseLong(text.substring(keyPos, keyPos + keyValueLength)))
							return true;
					}
					catch (Exception e)
					{
					}
				}
			}
		}
		else
			return text.equals(message);

		return false;
	}

	protected boolean isMessage(String text, String message)
	{
		if (message.isEmpty() || text.isEmpty())
			return false;

		String key = "$rid";
		int keyPos = message.indexOf(key);

		if (keyPos >= 0)
		{
			String begin = message.substring(0, keyPos);
			String end = message.substring(keyPos + key.length());

			if (text.startsWith(begin) && text.endsWith(end))
			{
				int keyValueLength = text.length() - begin.length() - end.length();
				if (keyValueLength > 0)
				{
					return true;
				}
			}
		}
		else
			return text.equals(message);

		return false;
	}

	protected Long getRID(String text, String message)
	{
		String key = "$rid";
		int keyPos = message.indexOf(key);

		if (keyPos >= 0)
		{
			String begin = message.substring(0, keyPos);
			String end = message.substring(keyPos + key.length());

			if (text.startsWith(begin) && text.endsWith(end))
			{
				int keyValueLength = text.length() - begin.length() - end.length();
				if (keyValueLength > 0)
				{
					try
					{
						return Long.parseLong(text.substring(keyPos, keyPos + keyValueLength));
					}
					catch (Exception e)
					{
					}
				}
			}
		}

		return null;
	}

	public boolean isRecordingStarted(String text, long rid)
	{
		return isMessage(text, recordingStartedMessage, rid);
	}

	public boolean isRecordingStarted(String text)
	{
		return isMessage(text, recordingStartedMessage);
	}

	public boolean isRecordingError(String text, long rid)
	{
		return isMessage(text, recordingErrorMessage, rid);
	}

	public boolean isRecordingError(String text)
	{
		return isMessage(text, recordingErrorMessage);
	}

	public Long getRecordingError(String text)
	{
		return getRID(text, recordingErrorMessage);
	}

	public boolean isPidChange(String text)
	{
		return isMessage(text, m_pidChangeMessage);
	}

	public Long getPidChange(String text)
	{
		return getRID(text, m_pidChangeMessage);
	}

	public boolean isRecordingStopped(String text, long rid)
	{
		return isMessage(text, recordingStoppedMessage, rid);
	}

	public boolean isRecordingStopped(String text)
	{
		return isMessage(text, recordingStoppedMessage);
	}

	public boolean isCaptureEngineReady(String text)
	{
		return text.equals(captureEngineReadyMessage);
	}

	public boolean isCaptureEngineExiting(String text)
	{
		return text.equals(captureEngineExitingMessage);
	}

	public String getInitialScanCommand()
	{
		return m_initialScanCommand;
	}

	public String getInitialEpgScanCommand()
	{
		return m_initialEpgScanCommand;
	}

	public String getEpgScanCommand()
	{
		return m_epgScanCommand;
	}

	public String getStopEpgScanCommand()
	{
		return m_stopEpgScanCommand;
	}

	public String getDeviceListCommand()
	{
		return m_deviceListCommand;
	}

	public int getdeviceIndexBase()
	{
		return m_deviceIndexBase;
	}

	public String formatFileName(String fileName)
	{
		String f2 = fileName;

		for(int x = 0; x < charSubstitutionFroms.size(); x++)
		{
			f2 = f2.replace(charSubstitutionFroms.get(x), charSubstitutionTos.get(x));
		}

		return f2;
	}

	private String replaceKeywordsUser(String text)
	{
		String f2 = text;

		for(int x = 0; x < m_keywordFroms.size(); x++)
		{
			f2 = f2.replace(m_keywordFroms.get(x), m_keywordTos.get(x));
		}

		return f2;
	}
	
	public String replaceKeywords(String text)
	{
		text = replaceKeywordsUser(text);
		return text.replace("$captureenginename", getName());
	}
}
