/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import java.io.File;
import java.util.*;

public class Channel
{
	// PID type.
	public static final int TYPE_VIDEO_MPG = 0;
	public static final int TYPE_AUDIO_MPG = 1;
	public static final int TYPE_AUDIO_AC3 = 2;
	public static final int TYPE_TELETEXT = 3;
	public static final int TYPE_PCR = 4;
	public static final int TYPE_PMT = 5;
	public static final int TYPE_PRIVATE = 6;
	public static final int TYPE_SUBTITLE = 7;
	public static final int TYPE_UNKNOWN = 8;
	public static final int TYPE_VIDEO_H264 = 9;
	public static final int TYPE_AUDIO_AAC = 10;

	public static final int ACCESS_FTA = 0;
	public static final int ACCESS_CA = 1;

	private static String m_defaultDemodulatorData = "";
	private static String m_defaultUserData = "";

	private String m_name = "";
	private int m_number = -1;
	private int programId = -1;
	private int m_videoPid = -1;
	private int m_videoType = -1;
	private int m_audioPid = -1;
	private int m_audioType = -1;
	private int m_teletextPid = -1;
	private int m_teletextSubtitlePage = -1;
	private boolean m_bTeletextRealTime = true;
	private int m_pmtPid = -1;
	private int m_pcrPid = -1;
	private Vector<Integer> m_otherPIDs = new Vector<Integer>();
	private Vector<Integer> m_disabledPIDs = new Vector<Integer>();
	private boolean m_bBuildPat = true;
	private boolean m_bIncludeVideoAudioSubtitleOnly = false;
	private boolean m_bPatchPmtVideo = false;
	private int m_videoPmtType = -1;
	private boolean m_bPatchPmtAudio = false;
	private int m_audioPmtType = -1;
	private int m_selection = 0;	// 0 is from program ID, 1 is full TS, 2 is from given PIDs.
	private String m_captureType = "Capture.deftype";
	private String m_previewType = "Preview.deftype";
	private String m_previewRemoteType = "Preview.remote.deftype";
	private String m_surfType = "Surf.deftype";
	private String m_surfRemoteType = "Surf.remote.deftype";
	private String m_capturePath = "*";
	private String m_userData = "";

	private int m_access = ACCESS_FTA;

	private String m_demodulatorName = "";
	private String m_networkType = "DVB-T";
	private int m_frequency = -2;
	private int m_bandWidth = -2;
	private String m_modulation = "";
	private int m_symbolRate = -2;
	private int m_physicalChannel = -2;
	private String m_polarization = "";
	private int m_highOsc = -2;
	private int m_lowOsc = -2;
	private int m_LNBswitch = -2;
	private double m_longitude = -2;
	private int m_westLongitude = -2;
	private double m_azimuth = -2;
	private double m_elevation = -2;
	private String m_inputRange = "";
	private String m_pilot = "";
	private String m_rollOff = "";
	private String m_diseqc = "";
	private String m_demodulatorData = "";
	private int m_originalNetworkId = -2;
	private int m_transportStreamId = -2;
	private int m_sourceId = -2;
	private String m_characterSet = "";
	private boolean m_bEpgScan = false;

	private Vector<int[]> streams = new Vector<int[]>();

	private Vector<String> m_channels = new Vector<String>();

	public Channel()
	{

	}

	public Channel(String id)
	{
		if (id != null)
			m_name = id.trim();
	}

	public Channel(String id, int freq, int band, int progID)
	{
		if (id != null)
			m_name = id.trim();
		m_frequency = freq > 0 ? freq : -2;
		m_bandWidth = band > 0 ? band : -2;
		programId = progID;
	}

	public Channel(String demodulatorName, String networkType, int freq, String modulation, int symbolRate, int physicalChannel, int band, String polarization, 
			int highOsc, int lowOsc, int lnbSwitch, double longitude, int westLongitude, double azimuth, double elevation, String inputRange, String pilot, String rollOff, 
			String diseqc, String demodulatorData, int originalNetworkId, int transportStreamId, int sourceId, String characterSet)
	{
		setDemodulator(demodulatorName, networkType, freq, modulation, symbolRate, physicalChannel, band, polarization, highOsc, lowOsc, 
				lnbSwitch, longitude, westLongitude, azimuth, elevation, inputRange, pilot, rollOff, diseqc, demodulatorData, 
				originalNetworkId, transportStreamId, sourceId, characterSet);
	}

	public Channel(String demodulatorName, String networkType, int freq, String modulation, int symbolRate, int physicalChannel, int band, String polarization, 
			int highOsc, int lowOsc, int lnbSwitch, double longitude, int westLongitude, double azimuth, double elevation, String inputRange, String pilot, String rollOff, 
			String diseqc, String demodulatorData, int originalNetworkId, int transportStreamId, int sourceId, String characterSet, 
			String name, int access, int progID, int vpid, int videoType, int apid, int audioType, 
			int teletextPID, int teletextSubtitlePage, int pmtPID, int pcrPID, Vector<Integer> otherPids)
	{
		setDemodulator(demodulatorName, networkType, freq, modulation, symbolRate, physicalChannel, band, polarization, highOsc, lowOsc, 
				lnbSwitch, longitude, westLongitude, azimuth, elevation, inputRange, pilot, rollOff, diseqc, demodulatorData, 
				originalNetworkId, transportStreamId, sourceId, characterSet);

		setChannel(name, access, progID, vpid, videoType, apid, audioType, teletextPID, teletextSubtitlePage, pmtPID, pcrPID, otherPids);
	}

	public void setDemodulator(String demodulatorName, String networkType, int freq, String modulation, int symbolRate, int physicalChannel, int band, String polarization, 
			int highOsc, int lowOsc, int lnbSwitch, double longitude, int westLongitude, double azimuth, double elevation, String inputRange, String pilot, String rollOff, 
			String diseqc, String demodulatorData, int originalNetworkId, int transportStreamId, int sourceId, String characterSet)
	{
		if (demodulatorName != null)
			m_demodulatorName = demodulatorName;
		m_networkType = networkType;
		m_frequency = freq;
		if (modulation != null)
			m_modulation = modulation;
		m_symbolRate = symbolRate;
		m_physicalChannel = physicalChannel;
		m_bandWidth = band;
		if (polarization != null)
			m_polarization = polarization;
		m_highOsc = highOsc;
		m_lowOsc = lowOsc;
		m_LNBswitch = lnbSwitch;
		m_longitude = longitude;
		m_westLongitude = westLongitude;
		m_azimuth = azimuth;
		m_elevation = elevation;
		if (inputRange != null)
			m_inputRange = inputRange;
		if (pilot != null)
			m_pilot = pilot;
		if (rollOff != null)
			m_rollOff = rollOff;
		if (diseqc != null)
			m_diseqc = diseqc;
		if (demodulatorData != null)
			m_demodulatorData = DVScheduler.checkBreaks(demodulatorData);
		m_originalNetworkId = originalNetworkId;
		m_transportStreamId = transportStreamId;
		m_sourceId = sourceId;
		m_characterSet = characterSet;
	}

	public void setChannel(String name, int access, int progID, int vpid, int videoType, int apid, int audioType, 
			int teletextPID, int teletextSubtitlePage, int pmtPID, int pcrPID, Vector<Integer> otherPids)
	{
		if (name != null)
			m_name = name.trim();
		m_access = access;
		programId = progID;
		m_videoPid = vpid;
		m_videoType = videoType;
		m_audioPid = apid;
		m_audioType = audioType;
		m_teletextPid = teletextPID;
		m_teletextSubtitlePage = teletextSubtitlePage;
		m_pmtPid = pmtPID;
		m_pcrPid = pcrPID;

		setOtherPid(otherPids);
	}

	public static String getPacketType(int type)
	{
		switch(type)
		{
		case TYPE_VIDEO_MPG:
			return "Video MPG";
		case TYPE_AUDIO_MPG:
			return "Audio MPG";
		case TYPE_AUDIO_AC3:
			return "Audio AC3";
		case TYPE_TELETEXT:
			return "Teletext";
		case TYPE_PCR:
			return "PCR";
		case TYPE_PMT:
			return "PMT";
		case TYPE_PRIVATE:
			return "Private";
		case TYPE_SUBTITLE:
			return "Subtitle";
		case TYPE_VIDEO_H264:
			return "Video H264";
		case TYPE_AUDIO_AAC:
			return "Audio AAC";
		default:
			return "Unknown";
		}
	}

	public void addStream(int[] streamData)
	{
		streams.add(streamData);
	}

	public Vector<int[]> getStreams()
	{
		return streams;
	}

	public Channel(Node item)
	{
		NamedNodeMap itemAttribs = item.getAttributes();
		m_name = itemAttribs.getNamedItem("name").getTextContent().trim();

		try
		{
			m_number = Integer.parseInt(itemAttribs.getNamedItem("number").getTextContent().trim());
		}
		catch (Exception e) {}

		NodeList itemNodes = item.getChildNodes();
		Node itemNode = null;  
		for(int y = 0; y < itemNodes.getLength(); y++)
		{
			itemNode = itemNodes.item(y);

			if(itemNode.getNodeName().equals("access"))
			{
				try
				{
					m_access = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("demodulatorname"))
			{
				m_demodulatorName = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("networktype"))
			{
				m_networkType = itemNode.getTextContent();

				// For backward compatibility.
				if (m_networkType.equals("0"))
					m_networkType = "DVB-T";
				else if (m_networkType.equals("1"))
					m_networkType = "DVB-C";
				else if (m_networkType.equals("2"))
					m_networkType = "DVB-S";
				else if (m_networkType.equals("3"))
					m_networkType = "ATSC";
			}
			else if(itemNode.getNodeName().equals("frequency"))
			{
				try
				{
					m_frequency = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("modulation"))
			{
				m_modulation = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("symbolrate"))
			{
				try
				{
					m_symbolRate = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("physicalchannel"))
			{
				try
				{
					m_physicalChannel = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("bandwidth"))
			{
				try
				{
					m_bandWidth = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}    
			else if(itemNode.getNodeName().equals("polarization"))
			{
				m_polarization = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("highosc"))
			{
				try
				{
					m_highOsc = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("lowosc"))
			{
				try
				{
					m_lowOsc = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("lnbswitch"))
			{
				try
				{
					m_LNBswitch = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("longitude"))
			{
				try
				{
					m_longitude = Double.parseDouble(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("westlongitude"))
			{
				try
				{
					m_westLongitude = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("azimuth"))
			{
				try
				{
					m_azimuth = Double.parseDouble(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("elevation"))
			{
				try
				{
					m_elevation = Double.parseDouble(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("inputrange"))
			{
				m_inputRange = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("pilot"))
			{
				m_pilot = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("rollOff"))
			{
				m_rollOff = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("diseqc"))
			{
				m_diseqc = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("originalNetworkId"))
			{
				try
				{
					m_originalNetworkId = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("transportStreamId"))
			{
				try
				{
					m_transportStreamId = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("sourceId"))
			{
				try
				{
					m_sourceId = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("characterSet"))
			{
				m_characterSet = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("epgScan"))
			{
				try
				{
					m_bEpgScan = Boolean.parseBoolean(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("program_id"))
			{
				try
				{
					programId = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
				NamedNodeMap attribs = itemNode.getAttributes();
				try
				{
					m_bIncludeVideoAudioSubtitleOnly = Boolean.parseBoolean(attribs.getNamedItem("video_audio_subtitle_only").getTextContent());
				}
				catch (Exception e) {}
			} 
			else if(itemNode.getNodeName().equals("video_id"))
			{
				try
				{
					m_videoPid = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
				NamedNodeMap attribs = itemNode.getAttributes();
				try
				{
					m_videoType = Integer.parseInt(attribs.getNamedItem("type").getTextContent());
				}
				catch (Exception e) {}
				try
				{
					m_bPatchPmtVideo = Boolean.parseBoolean(attribs.getNamedItem("patch_pmt").getTextContent());
				}
				catch (Exception e) {}
				try
				{
					m_videoPmtType = Integer.parseInt(attribs.getNamedItem("pmt_type").getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("audio_id"))
			{
				try
				{
					m_audioPid = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
				NamedNodeMap attribs = itemNode.getAttributes();
				try
				{
					m_audioType = Integer.parseInt(attribs.getNamedItem("type").getTextContent());
				}
				catch (Exception e) {}
				try
				{
					m_bPatchPmtAudio = Boolean.parseBoolean(attribs.getNamedItem("patch_pmt").getTextContent());
				}
				catch (Exception e) {}
				try
				{
					m_audioPmtType = Integer.parseInt(attribs.getNamedItem("pmt_type").getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("teletext_id"))
			{
				try
				{
					m_teletextPid = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
				NamedNodeMap attribs = itemNode.getAttributes();
				try
				{
					m_teletextSubtitlePage = Integer.parseInt(attribs.getNamedItem("subtitle_page").getTextContent());
				}
				catch (Exception e) {}
				try
				{
					m_bTeletextRealTime = Boolean.parseBoolean(attribs.getNamedItem("realtime").getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("pmt_id"))
			{
				try
				{
					m_pmtPid = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("pcr_id"))
			{
				try
				{
					m_pcrPid = Integer.parseInt(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("selection"))
			{
				m_selection = Integer.parseInt(itemNode.getTextContent());
			}
			else if(itemNode.getNodeName().equals("build_pat"))
			{
				try
				{
					m_bBuildPat = Boolean.parseBoolean(itemNode.getTextContent());
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("other_id"))
			{
				m_otherPIDs.clear();
				try
				{
					String[] others = itemNode.getTextContent().split(",");
					for(int x = 0; x < others.length; x++)
					{
						String other = others[x].trim();
						if (other.isEmpty())
							continue;
						m_otherPIDs.add( Integer.parseInt(other.trim()) );
					}
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("disabled_id"))
			{
				m_disabledPIDs.clear();
				try
				{
					String[] others = itemNode.getTextContent().split(",");
					for(int x = 0; x < others.length; x++)
					{
						String other = others[x].trim();
						if (other.isEmpty())
							continue;
						m_disabledPIDs.add( Integer.parseInt(other.trim()) );
					}
				}
				catch (Exception e) {}
			}
			else if(itemNode.getNodeName().equals("capturetype"))
			{
				m_captureType = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("previewtype"))
			{
				m_previewType = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("previewremotetype"))
			{
				m_previewRemoteType = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("surftype"))
			{
				m_surfType = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("surfremotetype"))
			{
				m_surfRemoteType = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("capturepath"))
			{
				m_capturePath = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("userdata"))
			{
				m_userData = DVScheduler.checkBreaks(itemNode.getTextContent());
			}
			else if(itemNode.getNodeName().equals("demodulatordata"))
			{
				m_demodulatorData = DVScheduler.checkBreaks(itemNode.getTextContent());
			}
			else if(itemNode.getNodeName().equals("childchannel"))
			{
				m_channels.add(itemNode.getTextContent());
			}
		}
		m_bBuildPat = (m_bBuildPat && ((m_selection == 0) || (m_selection == 1) || (m_selection == 2 && programId > -1 && m_pmtPid > -1))) ? true : false;
	}

	public int getVideoPid()
	{
		return m_videoPid;
	}

	public int getVideoType()
	{
		return m_videoType;
	}

	public String getVideoTypeStr()
	{
		return getVideoTypeStr(m_videoType);
	}

	public static String getVideoTypeStr(int type)
	{
		switch (type)
		{
		case TYPE_VIDEO_MPG:
			return "MPG";
		case TYPE_VIDEO_H264:
			return "H264";
		default:
			return "Unknown";
		}
	}

	public String getVideoPmtTypeStr()
	{
		return getVideoTypeStr(m_videoPmtType);
	}

	public int getAudioPid()
	{
		return m_audioPid;
	}

	public int getSelection()
	{
		return m_selection;
	}

	public String getSelectionStr()
	{
		switch ( getSelection() )
		{
		case 0:
			return "Program ID";

		case 1:
			return "All";

		case 2:
		{
			return "PIDs";
		}

		default:
			return "Unknown";
		}
	}

	public String getSelectionDescription()
	{
		switch ( getSelection() )
		{
		case 0:
			return "Program ID: " + getProgramID();

		case 1:
			return "All";

		case 2:
		{
			return "PIDs: " + getCustomPidString(true);
		}

		default:
			return "Unknown";
		}
	}

	public String getCustomPidString(boolean bSpace)
	{
		Set<Integer> pids = new HashSet<Integer>();
		pids.add(getVideoPid());
		pids.add(getAudioPid());
		pids.add(getTeletextPid());
		pids.add(getPmtPid());
		pids.add(getPcrPid());
		pids.addAll(m_otherPIDs);
		pids.removeAll(m_disabledPIDs);
		pids.remove(-1);
		pids.remove(-2);

		String cps = "";
		Iterator<Integer> iter = pids.iterator();
		while (iter.hasNext())
		{
			if (!cps.isEmpty())
			{
				cps += bSpace ? ", " : ",";
			}

			cps += iter.next();
		}

		return cps;
	}

	public int getOtherPidSize()
	{
		return m_otherPIDs.size();
	}

	public int getOtherPid(int index)
	{
		return m_otherPIDs.get(index);
	}

	public void deleteOtherPid(int index)
	{
		m_otherPIDs.remove(index);
	}

	public int getDisabledPidSize()
	{
		return m_disabledPIDs.size();
	}

	public int getDisabledPid(int index)
	{
		return m_disabledPIDs.get(index);
	}

	public void deleteDisabledPid(int index)
	{
		m_disabledPIDs.remove(index);
	}

	public String getOtherPidString(boolean bSpace)
	{
		String others = "";
		Integer[] pids = m_otherPIDs.toArray(new Integer[0]);
		//Arrays.sort(pids);

		for (int x = 0; x < pids.length; x++)
		{
			if (x > 0)
			{
				if (bSpace)
					others += ", ";
				else
					others += ",";
			}

			others += pids[x];
		}

		return others;
	}

	public String getDisabledPidString(boolean bSpace)
	{
		String others = "";
		Integer[] pids = m_disabledPIDs.toArray(new Integer[0]);
		//Arrays.sort(pids);

		for (int x = 0; x < pids.length; x++)
		{
			if (x > 0)
			{
				if (bSpace)
					others += ", ";
				else
					others += ",";
			}

			others += pids[x];
		}

		return others;
	}

	public void setVideoPid(int id)
	{
		m_videoPid = id;
	}

	public void setAudioPid(int id)
	{
		m_audioPid = id;
	}

	public void setSelection(int select)
	{
		m_selection = select;
	}

	public void setDisabledPid(Vector<Integer> id)
	{
		m_disabledPIDs.clear();

		if (id == null)
			return;

		for (int i=0; i<id.size(); i++)
		{
			int pid = id.get(i);
			//if ( !m_disabledPIDs.contains(pid) && (pid != getVideoPid()) && (pid != getAudioPid()) )
			if ( !m_disabledPIDs.contains(pid) )
				m_disabledPIDs.add(pid);
		}
	}

	public void setOtherPid(Vector<Integer> id)
	{
		m_otherPIDs.clear();

		if (id == null)
			return;

		for (int i=0; i<id.size(); i++)
		{
			int pid = id.get(i);
			if ( !m_otherPIDs.contains(pid) && (pid != getVideoPid()) && (pid != getAudioPid()) )
				m_otherPIDs.add(pid);
		}
	}

	public void setAudioType(int type)
	{
		m_audioType = type;
	}

	public int getAudioType()
	{
		return m_audioType;
	}

	public String getAudioTypeStr()
	{
		return getAudioTypeStr(m_audioType);
	}

	public static String getAudioTypeStr(int type)
	{
		switch (type)
		{
		case TYPE_AUDIO_MPG:
			return "MP2";

		case TYPE_AUDIO_AC3:
			return "AC3";

		case TYPE_AUDIO_AAC:
			return "AAC";

		default:
			return "Unknown";
		}
	}

	public String getAudioPmtTypeStr()
	{
		return getAudioTypeStr(m_audioPmtType);
	}

	public void getAudioTypesXml(Document doc, Element node)
	{
		Element elm2 = doc.createElement("item");
		elm2.setAttribute("name", "");
		elm2.setAttribute("value", "-1");
		node.appendChild(elm2);

		elm2 = doc.createElement("item");
		elm2.setAttribute("name", "MPG");
		elm2.setAttribute("value", "1");
		node.appendChild(elm2);

		elm2 = doc.createElement("item");
		elm2.setAttribute("name", "AC3");
		elm2.setAttribute("value", "2");
		node.appendChild(elm2);

		elm2 = doc.createElement("item");
		elm2.setAttribute("name", "AAC");
		elm2.setAttribute("value", "10");
		node.appendChild(elm2);
	}

	public void getVideoTypesXml(Document doc, Element node)
	{
		Element elm2 = doc.createElement("item");
		elm2.setAttribute("name", "");
		elm2.setAttribute("value", "-1");
		node.appendChild(elm2);

		elm2 = doc.createElement("item");
		elm2.setAttribute("name", "MPG");
		elm2.setAttribute("value", "0");
		node.appendChild(elm2);

		elm2 = doc.createElement("item");
		elm2.setAttribute("name", "H264");
		elm2.setAttribute("value", "9");
		node.appendChild(elm2);
	}

	public int getBandWidth()
	{
		return m_bandWidth;
	}

	public void setBandWidth(int band)
	{
		m_bandWidth = band;
	}

	public String getName()
	{
		return m_name;
	}

	public void setDemodulatorName(String demodulatorName)
	{
		if (demodulatorName != null)
			m_demodulatorName = demodulatorName;
	}

	public String getDemodulatorName()
	{
		return m_demodulatorName;
	}

	public void setName(String id)
	{
		m_name = id.trim();
	}

	public int getNumber()
	{
		return m_number;
	}

	public void setNumber(int number)
	{
		m_number = number;
	}

	public int getFrequency()
	{
		return m_frequency;
	}

	public void setFrequency(int freq)
	{
		m_frequency = freq;
	}

	public void setProgramID(int id)
	{
		programId = id;
	}

	public int getProgramID()
	{
		return programId;
	}

	public String getCapType()
	{
		return m_captureType;
	}

	public void setCaptureType(String type) throws Exception
	{
		m_captureType = ( (type == null) || (type.isEmpty()) || (type.equals("*UNDEFINED*")) ) ? "Capture.deftype" : type;
	}

	public String getPreviewType()
	{
		return m_previewType;
	}

	public void setPreviewType(String type) throws Exception
	{
		m_previewType = ( (type == null) || (type.isEmpty()) || (type.equals("*UNDEFINED*")) ) ? "Preview.deftype" : type;
	}

	public String getPreviewRemoteType()
	{
		return m_previewRemoteType;
	}

	public void setPreviewRemoteType(String type) throws Exception
	{
		m_previewRemoteType = ( (type == null) || (type.isEmpty()) || (type.equals("*UNDEFINED*")) ) ? "Preview.remote.deftype" : type;
	}

	public String getSurfType()
	{
		return m_surfType;
	}

	public void setSurfType(String type) throws Exception
	{
		m_surfType = ( (type == null) || (type.isEmpty()) || (type.equals("*UNDEFINED*")) ) ? "Surf.deftype" : type;
	}

	public String getSurfRemoteType()
	{
		return m_surfRemoteType;
	}

	public void setSurfRemoteType(String type) throws Exception
	{
		m_surfRemoteType = ( (type == null) || (type.isEmpty()) || (type.equals("*UNDEFINED*")) ) ? "Surf.remote.deftype" : type;
	}

	public String getCapturePath()
	{
		return m_capturePath;
	}

	public void setCapturePath(String capturePath) throws Exception
	{
		m_capturePath = ( (capturePath == null) || capturePath.equals("*UNDEFINED*") ) ? "*" : capturePath;		
	}

	public String getUserData()
	{
		return m_userData;
	}

	public void setUserData(String userData) throws Exception
	{
		if (userData != null)
			m_userData = DVScheduler.checkBreaks(userData);
	}

	public String getDemodulatorData()
	{
		return m_demodulatorData;
	}

	public void setDemodulatorData(String demodulatorData) throws Exception
	{
		if (demodulatorData != null)
			m_demodulatorData = DVScheduler.checkBreaks(demodulatorData);
	}

	public void getDemodulatorXML(Document doc, Element node)
	{
		Element elm = null;
		Text text = null;

		elm = doc.createElement("demodulatorname");
		text = doc.createTextNode(m_demodulatorName);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("networktype");
		text = doc.createTextNode(m_networkType);
		elm.appendChild(text);
		node.appendChild(elm);

		if (m_frequency > -2)
		{
			elm = doc.createElement("frequency");
			text = doc.createTextNode(Integer.toString(m_frequency));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("modulation");
		text = doc.createTextNode(m_modulation);
		elm.appendChild(text);
		node.appendChild(elm);  

		if (m_symbolRate > -2)
		{
			elm = doc.createElement("symbolrate");
			text = doc.createTextNode(Integer.toString(m_symbolRate));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_physicalChannel > -2)
		{
			elm = doc.createElement("physicalchannel");
			text = doc.createTextNode(Integer.toString(m_physicalChannel));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_bandWidth > -2)
		{
			elm = doc.createElement("bandwidth");
			text = doc.createTextNode(Integer.toString(m_bandWidth));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("polarization");
		text = doc.createTextNode(m_polarization);
		elm.appendChild(text);
		node.appendChild(elm);

		if (m_highOsc > -2)
		{
			elm = doc.createElement("highosc");
			text = doc.createTextNode(Integer.toString(m_highOsc));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_lowOsc > -2)
		{
			elm = doc.createElement("lowosc");
			text = doc.createTextNode(Integer.toString(m_lowOsc));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_LNBswitch > -2)
		{
			elm = doc.createElement("lnbswitch");
			text = doc.createTextNode(Integer.toString(m_LNBswitch));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_longitude > -2)
		{
			elm = doc.createElement("longitude");
			text = doc.createTextNode(Double.toString(m_longitude));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("westlongitude");
		text = doc.createTextNode(Integer.toString(m_westLongitude));
		elm.appendChild(text);
		node.appendChild(elm);

		if (m_azimuth > -2)
		{
			elm = doc.createElement("azimuth");
			text = doc.createTextNode(Double.toString(m_azimuth));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_elevation > -2)
		{
			elm = doc.createElement("elevation");
			text = doc.createTextNode(Double.toString(m_elevation));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("inputrange");
		text = doc.createTextNode(m_inputRange);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("pilot");
		text = doc.createTextNode(m_pilot);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("rollOff");
		text = doc.createTextNode(m_rollOff);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("diseqc");
		text = doc.createTextNode(m_diseqc);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("demodulatordata");
		text = doc.createTextNode(m_demodulatorData);
		elm.appendChild(text);
		node.appendChild(elm);

		if (m_originalNetworkId > -2)
		{
			elm = doc.createElement("originalNetworkId");
			text = doc.createTextNode(Integer.toString(m_originalNetworkId));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_transportStreamId > -2)
		{
			elm = doc.createElement("transportStreamId");
			text = doc.createTextNode(Integer.toString(m_transportStreamId));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_sourceId > -2)
		{
			elm = doc.createElement("sourceId");
			text = doc.createTextNode(Integer.toString(m_sourceId));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("characterSet");
		text = doc.createTextNode(m_characterSet);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("epgScan");
		text = doc.createTextNode(Boolean.toString(m_bEpgScan));
		elm.appendChild(text);
		node.appendChild(elm);
	}

	public void getXML(Document doc, Element node)
	{
		getDemodulatorXML(doc, node);

		Element elm = null;
		Text text = null;

		node.setAttribute("name", m_name);

		if (m_number > -1)
			node.setAttribute("number", Integer.toString(m_number));

		elm = doc.createElement("access");
		text = doc.createTextNode(Integer.toString(m_access));
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("program_id");
		elm.setAttribute("video_audio_subtitle_only", Boolean.toString(m_bIncludeVideoAudioSubtitleOnly));
		if (programId > -1)
		{
			text = doc.createTextNode(Integer.toString(programId));
			elm.appendChild(text);
		}
		node.appendChild(elm);  

		elm = doc.createElement("video_id");
		elm.setAttribute("type", Integer.toString(m_videoType));
		elm.setAttribute("patch_pmt", Boolean.toString(m_bPatchPmtVideo));
		elm.setAttribute("pmt_type", Integer.toString(m_videoPmtType));
		if (m_videoPid > -1)
		{
			text = doc.createTextNode(Integer.toString(m_videoPid));
			elm.appendChild(text);
		}
		node.appendChild(elm);     

		elm = doc.createElement("audio_id");
		elm.setAttribute("type", Integer.toString(m_audioType));
		elm.setAttribute("patch_pmt", Boolean.toString(m_bPatchPmtAudio));
		elm.setAttribute("pmt_type", Integer.toString(m_audioPmtType));
		if (m_audioPid > -1)
		{
			text = doc.createTextNode(Integer.toString(m_audioPid));
			elm.appendChild(text);
		}
		node.appendChild(elm);

		elm = doc.createElement("teletext_id");
		if (m_teletextSubtitlePage > -1)
		{
			elm.setAttribute("subtitle_page", Integer.toString(m_teletextSubtitlePage));
		}
		elm.setAttribute("realtime", Boolean.toString(m_bTeletextRealTime));
		if (m_teletextPid > -1)
		{
			text = doc.createTextNode(Integer.toString(m_teletextPid));
			elm.appendChild(text);
		}
		node.appendChild(elm);

		if (m_pmtPid > -1)
		{
			elm = doc.createElement("pmt_id");
			text = doc.createTextNode(Integer.toString(m_pmtPid));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		if (m_pcrPid > -1)
		{
			elm = doc.createElement("pcr_id");
			text = doc.createTextNode(Integer.toString(m_pcrPid));
			elm.appendChild(text);
			node.appendChild(elm);
		}

		elm = doc.createElement("build_pat");
		text = doc.createTextNode(Boolean.toString(m_bBuildPat));
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("selection");
		text = doc.createTextNode(Integer.toString(m_selection));
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("other_id");
		String others = getOtherPidString(false);
		text = doc.createTextNode(others);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("disabled_id");
		String disabled = getDisabledPidString(false);
		text = doc.createTextNode(disabled);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("capturetype");
		text = doc.createTextNode(m_captureType);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("previewtype");
		text = doc.createTextNode(m_previewType);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("previewremotetype");
		text = doc.createTextNode(m_previewRemoteType);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("surftype");
		text = doc.createTextNode(m_surfType);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("surfremotetype");
		text = doc.createTextNode(m_surfRemoteType);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("capturepath");
		text = doc.createTextNode(m_capturePath);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("userdata");
		text = doc.createTextNode(m_userData);
		elm.appendChild(text);
		node.appendChild(elm);

		Iterator<String> iter = m_channels.iterator();
		while (iter.hasNext())
		{
			elm = doc.createElement("childchannel");
			text = doc.createTextNode(iter.next());
			elm.appendChild(text);
			node.appendChild(elm);
		}
	}

	public boolean isDataValid()
	{
		if (isGroup())
			return false;
		else if ( (m_selection < 0) || (m_selection > 2) )
			return false;
		else if ( (m_selection == 0) && (programId < 0) )
			return false;
		else if ( m_selection == 2 )
		{
			if ( (m_videoPid < 0) && (m_audioPid < -1)  && (m_teletextPid < -1) && (m_pmtPid < -1) && (m_pcrPid < -1) && (m_otherPIDs == null || m_otherPIDs.isEmpty()) )
				return false;
		}

		return true;
	}

	public String getHardSetting()
	{
		//return m_networkType + "," + m_lowOsc + "," + m_highOsc + "," + m_surfType;
		return m_networkType;
	}

	public boolean isHardSettingEqual(Channel item)
	{
		return getHardSetting().equals(item.getHardSetting());
	}

	public String getSetting()
	{
		return m_networkType + "," + m_longitude + "," + m_westLongitude + "," + m_diseqc + "," + m_inputRange + "," + 
				m_polarization + "," + m_frequency + "," + m_physicalChannel + "," + m_demodulatorData.replace("\n", "|");
	}

	public String getSettingForFilename()
	{
		return Formatter.formatFilename(getSetting());
	}

	public boolean isSettingEqual(Channel ch)
	{
		return m_networkType.equals(ch.m_networkType) && 
				(m_frequency == ch.m_frequency) &&  
				(m_physicalChannel == ch.m_physicalChannel) && m_polarization.equals(ch.m_polarization) &&  
				(m_longitude == ch.m_longitude) && (m_westLongitude == ch.m_westLongitude) &&  
				m_inputRange.equals(ch.m_inputRange) &&  
				m_diseqc.equals(ch.m_diseqc) && m_demodulatorData.equals(ch.m_demodulatorData);
	}

	/*
	public String getSettingInput()
	{
		return m_networkType + "," + m_frequency + "," + m_modulation + "," + m_symbolRate + "," + 
				m_physicalChannel + "," + m_bandWidth + "," + m_polarization + "," + 
				m_lowOsc + "," + m_highOsc + "," + m_LNBswitch + "," + 
				m_longitude + "," + m_westLongitude + "," + m_azimuth + "," + 
				m_elevation + "," + m_inputRange + "," + m_pilot + "," + 
				m_rollOff + "," + m_diseqc + "," + m_demodulatorData;
	}
	 */

	public boolean isSettingInputEqual(Channel ch)
	{
		return /*m_demodulatorName.equals(ch.m_demodulatorName) &&*/ m_networkType.equals(ch.m_networkType) && 
				(m_frequency == ch.m_frequency) && m_modulation.equals(ch.m_modulation) && (m_symbolRate == ch.m_symbolRate) && 
				(m_physicalChannel == ch.m_physicalChannel) && (m_bandWidth == ch.m_bandWidth) && m_polarization.equals(ch.m_polarization) && 
				(m_lowOsc == ch.m_lowOsc) && (m_highOsc == ch.m_highOsc) && (m_LNBswitch == ch.m_LNBswitch) && 
				(m_longitude == ch.m_longitude) && (m_westLongitude == ch.m_westLongitude) && (m_azimuth == ch.m_azimuth) && 
				(m_elevation == ch.m_elevation) && m_inputRange.equals(ch.m_inputRange) && m_pilot.equals(ch.m_pilot) && 
				m_rollOff.equals(ch.m_rollOff) && m_diseqc.equals(ch.m_diseqc) && m_demodulatorData.equals(ch.m_demodulatorData) && 
				(m_originalNetworkId == ch.m_originalNetworkId) && (m_transportStreamId == ch.m_transportStreamId) && 
				(m_sourceId == ch.m_sourceId) && (m_characterSet.equals(ch.m_characterSet));
	}

	public void setAccess(int access)
	{
		m_access = access;
	}

	public int getAccess()
	{
		return m_access;
	}

	public static String getAccessStr(int access)
	{
		switch (access)
		{
		case ACCESS_FTA:
			return "FTA";
		case ACCESS_CA:
			return "CA";
		default:
			return "Unknown";
		}
	}

	public String getAccessStr()
	{
		return getAccessStr(m_access);
	}

	public String getNetworkType()
	{
		return m_networkType;
	}

	public void setNetworkType(String networkType)
	{
		m_networkType = networkType;
	}

	public String getModulation()
	{
		return m_modulation;
	}

	public int getSymbolRate()
	{
		return m_symbolRate;
	}

	public int getPhysicalChannel()
	{
		return m_physicalChannel;
	}

	public String getPolarization()
	{
		return m_polarization;
	}

	public int getHighOsc()
	{
		return m_highOsc;
	}

	public int getLowOsc()
	{
		return m_lowOsc;
	}

	public int getLNBSwitch()
	{
		return m_LNBswitch;
	}

	public double getLongitude()
	{
		return m_longitude;
	}

	public int getWestLongitude()
	{
		return m_westLongitude;
	}

	public String getDirLongitude()
	{
		switch (m_westLongitude)
		{
		case 0:
			return "EAST";
		case 1:
			return "WEST";
		default:
			return "Unknown";
		}
	}

	public double getAzimuth()
	{
		return m_azimuth;
	}

	public double getElevation()
	{
		return m_elevation;
	}

	public String getInputRange()
	{
		return m_inputRange;
	}

	public String getPilot()
	{
		return m_pilot;
	}

	public String getRollOff()
	{
		return m_rollOff;
	}

	public String getDiseqc()
	{
		return m_diseqc;
	}

	public int getOriginalNetworkId()
	{
		return m_originalNetworkId;
	}

	public void setOriginalNetworkId(int value)
	{
		m_originalNetworkId = value;
	}

	public int getTransportStreamId()
	{
		return m_transportStreamId;
	}

	public void setTransportStreamId(int value)
	{
		m_transportStreamId = value;
	}

	public int getSourceId()
	{
		return m_sourceId;
	}

	public void setSourceId(int value)
	{
		m_sourceId = value;
	}

	public String getCharacterSet()
	{
		return m_characterSet;
	}

	public void setEpgScan(boolean bEpgScan)
	{
		m_bEpgScan = bEpgScan;
	}

	public boolean isEpgScan()
	{
		return m_bEpgScan;
	}

	public String getChannelsString()
	{
		String text = "";
		Iterator<String> iter = m_channels.iterator();
		while (iter.hasNext())
		{
			text += iter.next();

			if (iter.hasNext())
				text += ", ";
		}

		return text;
	}

	public Vector<String> getChannels()
	{
		return m_channels;
	}

	public Set<String> getChannelSet()
	{
		HashSet<String> channelSet = new HashSet<String>();
		channelSet.addAll(m_channels);
		return channelSet;
	}

	public void setChannels(Vector<String> channels)
	{
		m_channels = channels;
	}

	public boolean isGroup()
	{
		return !m_channels.isEmpty();
	}

	public void updateChannelGroup(String oldChannelName, String newChannelName)
	{
		if (!isGroup())
			return;

		for (int i=0; i<m_channels.size(); i++)
		{
			if (m_channels.get(i).equals(oldChannelName))
			{
				m_channels.remove(i);
				m_channels.add(i, newChannelName);
			}
		}
	}

	public int getTeletextPid()
	{
		return m_teletextPid;
	}

	public void setTeletextPid(int teletextPID)
	{
		m_teletextPid = teletextPID;
	}

	public int getTeletextSubtitlePage()
	{
		return m_teletextSubtitlePage;
	}

	public void setTeletextSubtitlePage(int teletextSubtitlePage)
	{
		m_teletextSubtitlePage = teletextSubtitlePage;
	}

	public int getPmtPid()
	{
		return m_pmtPid;
	}

	public int getPcrPid()
	{
		return m_pcrPid;
	}

	public void setBuildPat(boolean bBuildPat)
	{
		m_bBuildPat = (bBuildPat && ((m_selection == 0) || (m_selection == 1) || (m_selection == 2 && programId > -1 && m_pmtPid > -1))) ? true : false;
	}

	public boolean isBuildPat()
	{
		return m_bBuildPat;
	}

	public void setIncludeVideoAudioSubtitleOnly(boolean bIncludeVideoAudioSubtitleOnly)
	{
		m_bIncludeVideoAudioSubtitleOnly = bIncludeVideoAudioSubtitleOnly;
	}

	public boolean isIncludeVideoAudioSubtitleOnly()
	{
		return m_bIncludeVideoAudioSubtitleOnly;
	}

	public void setTeletextRealTime(boolean bTeletextRealTime)
	{
		m_bTeletextRealTime = bTeletextRealTime;
	}

	public boolean isTeletextRealTime()
	{
		return m_bTeletextRealTime;
	}

	public void setPatchPmtVideo(boolean bPatchPmtVideo)
	{
		m_bPatchPmtVideo = bPatchPmtVideo;
	}

	public boolean isPatchPmtVideo()
	{
		return m_bPatchPmtVideo;
	}

	public void setVideoPmtType(int videoPmtType)
	{
		m_videoPmtType = videoPmtType;
	}

	public int getVideoPmtType()
	{
		if (m_videoPmtType > -1)
			return m_videoPmtType;
		else
		{
			switch (m_videoType)
			{
			case TYPE_VIDEO_MPG:
				return 2;
			case TYPE_VIDEO_H264:
				return 0x1B;
			default:
				return 0;
			}
		}
	}

	public void setPatchPmtAudio(boolean bPatchPmtAudio)
	{
		m_bPatchPmtAudio = bPatchPmtAudio;
	}

	public boolean isPatchPmtAudio()
	{
		return m_bPatchPmtAudio;
	}

	public void setAudioPmtType(int audioPmtType)
	{
		m_audioPmtType = audioPmtType;
	}

	public int getAudioPmtType()
	{
		if (m_audioPmtType > -1)
			return m_audioPmtType;
		else
		{
			switch (m_audioType)
			{
			case TYPE_AUDIO_MPG:
				return 4;
			case TYPE_AUDIO_AC3:
				return 0x81;
			case TYPE_AUDIO_AAC:
				return 0x11;
			default:
				return 0;
			}
		}
	}

	public void setDemodulator(Channel channel)
	{
		m_demodulatorName = channel.m_demodulatorName;
		m_networkType = channel.m_networkType;
		m_frequency = channel.m_frequency;
		m_modulation = channel.m_modulation;
		m_symbolRate = channel.m_symbolRate;
		m_physicalChannel = channel.m_physicalChannel;
		m_bandWidth = channel.m_bandWidth;
		m_polarization = channel.m_polarization;
		m_lowOsc = channel.m_lowOsc;
		m_highOsc = channel.m_highOsc;
		m_LNBswitch = channel.m_LNBswitch;
		m_longitude = channel.m_longitude;
		m_westLongitude = channel.m_westLongitude;
		m_azimuth = channel.m_azimuth;
		m_elevation = channel.m_elevation;
		m_inputRange = channel.m_inputRange;
		m_pilot = channel.m_pilot;
		m_rollOff = channel.m_rollOff;
		m_diseqc = channel.m_diseqc;
		m_demodulatorData = channel.m_demodulatorData;
		m_originalNetworkId = channel.m_originalNetworkId;
		m_transportStreamId = channel.m_transportStreamId;
		m_sourceId = channel.m_sourceId;
		m_characterSet = channel.m_characterSet;
	}

	public static boolean getChannelLogoXml(Element channelElement, String channelName, String sChannelImageDirectory, String sChannelImageWidth, String sChannelShow, boolean bShowChannelImage, boolean bChannelImage)
	{
		if (bShowChannelImage)
		{
			File image = new File(sChannelImageDirectory + channelName + ".gif");
			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File(sChannelImageDirectory + channelName + ".jpg");
			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File(sChannelImageDirectory + channelName + ".png");

			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File("http/images/channels/" + channelName + ".gif");
			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File("http/images/channels/" + channelName + ".jpg");
			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File("http/images/channels/" + channelName + ".png");
			if ( !image.exists() || image.isDirectory() || (image.length() <= 0) )
				image = new File("http/images/channels/" + channelName + ".pn2");

			if ( image.exists() && !image.isDirectory() && (image.length() > 0) )
			{
				bChannelImage = true;
				channelElement.setAttribute("image-file", "/images/channels/" + image.getName());
				channelElement.setAttribute("image-width", sChannelImageWidth);
				channelElement.setAttribute("show", sChannelShow);
			}
			else
				channelElement.setAttribute("show", "0");
		}
		else
			channelElement.setAttribute("show", "0");

		return bChannelImage;
	}

	public String replaceKeywords(String text)
	{
		text = text.replace("$accessS1", getAccessStr());
		text = text.replace("$networktypeS1", getNetworkType());
		text = text.replace("$frequency", Integer.toString(getFrequency()));
		text = text.replace("$modulation", getModulation());
		text = text.replace("$symbolrate", Integer.toString(getSymbolRate()));
		text = text.replace("$physicalchannel", Integer.toString(getPhysicalChannel()));
		text = text.replace("$bandwidth", Integer.toString(getBandWidth()));
		text = text.replace("$polarization", getPolarization());
		text = text.replace("$highosc", Integer.toString(getHighOsc()));
		text = text.replace("$lowosc", Integer.toString(getLowOsc()));
		text = text.replace("$lnbswitch", Integer.toString(getLNBSwitch()));
		text = text.replace("$longitude", Double.toString(getLongitude()));
		text = text.replace("$westlongitude", Integer.toString(getWestLongitude()));
		text = text.replace("$dirlongitude", getDirLongitude());
		text = text.replace("$azimuth", Double.toString(getAzimuth()));
		text = text.replace("$elevation", Double.toString(getElevation()));
		text = text.replace("$inputrange", getInputRange());
		text = text.replace("$pilot", getPilot());
		text = text.replace("$rolloff", getRollOff());
		text = text.replace("$diseqc", getDiseqc());
		text = text.replace("$charset", getCharacterSet());
		text = text.replace("$progid", Integer.toString(getProgramID()));
		text = text.replace("$scheduleport", Integer.toString(getProgramID()));	// Should have been replaced by ScheduleItem::replaceKeywords(...)
		text = text.replace("$channelname", getName());
		text = text.replace("$encUtf8UrlChannelname", Formatter.encodeURL_UTF8(getName()));
		text = text.replace("$channelnumber", Integer.toString(getNumber()));

		String[] userDatas = DVScheduler.getArray(getUserData());
		for(int x = 0; x < userDatas.length; x++)
		{
			text = text.replace("$userdata" + (x < 9 ? "0" : "") + (x+1), userDatas[x]);
		}

		userDatas = DVScheduler.getArray( m_defaultUserData );
		for(int x = 0; x < userDatas.length; x++)
		{
			text = text.replace("$userdata" + (x < 9 ? "0" : "") + (x+1), userDatas[x]);
		}

		String[] demodulatorDatas = DVScheduler.getArray(getDemodulatorData());
		for(int x = 0; x < demodulatorDatas.length; x++)
		{
			text = text.replace("$demodulatordata" + (x < 9 ? "0" : "") + (x+1), demodulatorDatas[x]);
		}

		demodulatorDatas = DVScheduler.getArray( m_defaultDemodulatorData );
		for(int x = 0; x < demodulatorDatas.length; x++)
		{
			text = text.replace("$demodulatordata" + (x < 9 ? "0" : "") + (x+1), demodulatorDatas[x]);
		}

		text = text.replace("$videopid", Integer.toString(getVideoPid()));
		text = text.replace("$audiopid", Integer.toString(getAudioPid()));
		text = text.replace("$audiotypen01", Integer.toString(getAudioType()));
		text = text.replace("$audiotypex01", getAudioTypeStr());
		text = text.replace("$teletextsub", Integer.toString(getTeletextPid()));
		text = text.replace("$teletextpagesub", Integer.toString(getTeletextSubtitlePage()));
		text = text.replace("$teletextrealtime", isTeletextRealTime() ? "-realtime" : "");
		text = text.replace("$pmtpid", Integer.toString(getPmtPid()));
		text = text.replace("$pcrpid", Integer.toString(getPcrPid()));
		text = text.replace("$buildpat", isBuildPat() ? "buildpat(" + getProgramID() + "," + getPmtPid() + ")" : "");

		for(int x = 0; x < getOtherPidSize(); x++)
		{
			text = text.replace("$otherpid" + (x < 9 ? "0" : "") + (x+1), Integer.toString(getOtherPid(x)));
		}

		int sel = getSelection();
		String pid = "";
		if (sel == 0)
		{
			pid = "progid(" + getProgramID();

			if (isIncludeVideoAudioSubtitleOnly())
				pid += ",vas";

			/*
			if ( getOtherPidSize() > 0 )
				pid += ",include(" + getOtherPidString(false) + ")";
			 */
			pid += ",include(pcr)";

			if ( getDisabledPidSize() > 0 )
			{
				pid += ",exclude(" + getDisabledPidString(false) + ")";

				String patch = "patchpmt(" + getPmtPid();
				for(int x = 0; x < m_disabledPIDs.size(); x++)
				{
					patch += ",pt(" + m_disabledPIDs.get(x) + ",0)";
				}
				patch += ")";
				text = text.replace("$patchpmt", patch);
			}
			else
				text = text.replace("$patchpmt", "");

			pid += ")";
		}
		else if (sel == 2)
		{
			pid = "pid(" + getCustomPidString(false) + ",pcr(" + getPcrPid() + "))";
			text = text.replace("$ttxsub", Integer.toString(getTeletextPid()));
			text = text.replace("$ttxpagesub", Integer.toString(getTeletextSubtitlePage()));

			if ( isPatchPmtVideo() || isPatchPmtAudio() || (getDisabledPidSize() > 0) )
			{
				String patch = "patchpmt(" + getPmtPid();
				if (isPatchPmtVideo())
					patch += ",pt(" + getVideoPid() + "," + getVideoPmtType() + ")"; 
				if (isPatchPmtAudio())
					patch += ",pt(" + getAudioPid() + "," + getAudioPmtType() + ")";
				for(int x = 0; x < m_disabledPIDs.size(); x++)
				{
					patch += ",pt(" + m_disabledPIDs.get(x) + ",0)";
				}
				patch += ")";
				text = text.replace("$patchpmt", patch);
			}
			else
				text = text.replace("$patchpmt", "");
		}
		else
			text = text.replace("$patchpmt", "");

		return text.replace("$pid", pid);
	}

	public static String getDefaultDemodulatorData()
	{
		return m_defaultDemodulatorData;
	}

	public static void setDefaultDemodulatorData(String demodulatorData) throws Exception
	{
		if (demodulatorData != null)
			m_defaultDemodulatorData = DVScheduler.checkBreaks(demodulatorData);
	}

	public static String getDefaultUserData()
	{
		return m_defaultUserData;
	}

	public static void setDefaultUserData(String userData) throws Exception
	{
		if (userData != null)
			m_defaultUserData = DVScheduler.checkBreaks(userData);
	}
}
