import java.util.*;

import org.w3c.dom.*;

class ChannelOrder
{
	private Vector<String> m_defaultOrder = new Vector<String>();
	private Vector<String> m_customOrder = new Vector<String>();
	private Vector<String> m_order = m_defaultOrder;
	private boolean m_bUseDefault = true;
	private Map<Integer, String> m_numberToChannel = new HashMap<Integer, String>();

	public static final int USE_SETTING = 0;
	public static final int USE_DEFAULT = 1;
	public static final int USE_CUSTOM = 2;

	public ChannelOrder()
	{

	}

	public ChannelOrder(ChannelOrder right)
	{
		m_defaultOrder.addAll(right.m_defaultOrder);
		m_customOrder.addAll(right.m_customOrder);
		m_bUseDefault = right.m_bUseDefault;
		m_order = m_bUseDefault ? m_defaultOrder : m_customOrder;
		m_numberToChannel.putAll(right.m_numberToChannel);
	}

	public void copyTo(ChannelOrder to)
	{
		to.m_defaultOrder.clear();
		to.m_defaultOrder.addAll(m_defaultOrder);
		to.m_customOrder.clear();
		to.m_customOrder.addAll(m_customOrder);
		to.m_bUseDefault = m_bUseDefault;
		to.m_order = to.m_bUseDefault ? to.m_defaultOrder : to.m_customOrder;
		to.m_numberToChannel.clear();
		to.m_numberToChannel.putAll(m_numberToChannel);
	}

	public void loadXML(Node item, HashMap<String, Channel> channels)
	{
		m_customOrder.clear();

		NodeList itemNodes = item.getChildNodes();
		Node itemNode = null;
		for(int y = 0; y < itemNodes.getLength(); y++)
		{
			itemNode = itemNodes.item(y);

			if (itemNode.getNodeName().equals("channelorder"))
				m_customOrder.add(itemNode.getAttributes().getNamedItem("name").getTextContent().trim());
		}

		checkCustomOrderingInvalid(channels);

		NamedNodeMap itemAttribs = item.getAttributes();
		setUse(!itemAttribs.getNamedItem("use").getTextContent().equals("custom"));
	}

	public Element getXML(Document doc, HashMap<String, Channel> channels, int nGet, String userAgent)
	{
		Vector<String> order = null;
		switch (nGet)
		{
		case USE_DEFAULT:
			order = m_defaultOrder;
			break;

		case USE_CUSTOM:
			order = m_customOrder;
			break;

		case USE_SETTING: default:
			order = m_order;
		}

		Element elm = null;

		Element item = doc.createElement("orders");
		item.setAttribute("use", m_bUseDefault ? "default" : "custom");
		DataStore store = DataStore.getInstance();
		String sChannelImageDirectory = store.getProperty("path.httproot", "http") + "/themes/" + store.getThemeDirectory(userAgent) + "/images/channels/";
		String sChannelImageWidth = Integer.toString(store.getPropertyInt("channel.logoWidth", 64));
		String sChannelShow = store.getProperty("channel.logo", "1");
		boolean bShowChannelImage = !sChannelShow.equals("0") && store.getProperty("channel.useLogo", "2").equals("2");
		boolean bChannelImage = false;
		for (int x = 0; x < order.size(); x++)
		{
			String sChannel = order.get(x);
			Channel ch = channels.get(sChannel);
			//if ( (ch == null) || (ch.isGroup()) )
			if ( ch == null )
				continue;

			elm = doc.createElement("channelorder");
			elm.setAttribute("name", sChannel);
			elm.setAttribute("number", Integer.toString(ch.getNumber()));

			if (userAgent != null)
				bChannelImage = Channel.getChannelLogoXml(elm, sChannel, sChannelImageDirectory, sChannelImageWidth, sChannelShow, bShowChannelImage, bChannelImage);

			item.appendChild(elm);
		}
		if (bChannelImage)
			item.setAttribute("channelImage", "1");

		return item;
	}

	public void getXML(Document doc, Element root, HashMap<String, Channel> channels, Vector<String> selected, boolean bShowGroup)
	{
		if (selected == null)
		{
			selected = m_customOrder;
		}

		Element elm = null;

		Element item = doc.createElement("selected");

		Iterator<String> iter = selected.iterator();
		while (iter.hasNext())
		{
			String sChannel = iter.next();
			Channel ch = channels.get(sChannel);
			if ( (ch == null) || (!bShowGroup && ch.isGroup()) )
				continue;

			elm = doc.createElement("channelorder");
			elm.setAttribute("name", sChannel);
			elm.setAttribute("number", Integer.toString(ch.getNumber()));

			item.appendChild(elm);
		}
		root.appendChild(item);

		item = doc.createElement("orders");
		item.setAttribute("use", m_bUseDefault ? "default" : "custom");

		for (int x = 0; x < m_defaultOrder.size(); x++)
		{
			String sChannel = m_defaultOrder.get(x);
			Channel ch = channels.get(sChannel);
			if ( (ch == null) || (!bShowGroup && ch.isGroup()) )
				continue;

			elm = doc.createElement("channelorder");
			elm.setAttribute("name", sChannel);
			elm.setAttribute("number", Integer.toString(ch.getNumber()));

			if ( (selected != null) && (selected.contains(sChannel)) )
				elm.setAttribute("selected", "1");

			item.appendChild(elm);
		}
		root.appendChild(item);
	}

	public void setDefaultOrdering(HashMap<String, Channel> channels)
	{
		int highestChannelNumber = -1;
		m_defaultOrder.clear();
		m_numberToChannel.clear();

		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		List<String> chList = new ArrayList<String>();

		for (int x = 0; x < keys.length; x++)
		{
			Channel ch = channels.get(keys[x]);
			chList.add((ch.isGroup() ? "2" : "1") + ch.getSetting() + "\n" + keys[x]);
			if (ch.getNumber() > highestChannelNumber)
				highestChannelNumber = ch.getNumber();
		}
		String[] chArray = chList.toArray(new String[0]);
		Arrays.sort(chArray);

		for (int x = 0; x < chArray.length; x++)
		{
			String[] chs = chArray[x].split("\n");
			Channel ch = channels.get(chs[1]);
			//if ( (ch == null) || (ch.isGroup()) )
			if ( ch == null )
				continue;

			m_defaultOrder.add(chs[1]);

			if (ch.getNumber() < 0)
				ch.setNumber(++highestChannelNumber);

			putChannel(ch.getNumber(), ch.getName());
		}

		checkCustomOrderingInvalid(channels);
	}

	public int getCustomOrderingSize()
	{
		return m_customOrder.size();
	}

	public void setCustomOrdering(String text, HashMap<String, Channel> channels)
	{
		m_customOrder.clear();

		String[] channels2 = text.trim().split("\n");
		for (int x = 0; x < channels2.length; x++)
		{
			String sChannel = channels2[x].trim();
			if (!sChannel.isEmpty())
				m_customOrder.add(sChannel);
		}

		checkCustomOrderingInvalid(channels);
	}

	public void resetCustomOrdering(HashMap<String, Channel> channels)
	{
		m_customOrder.clear();

		checkCustomOrderingMissing(channels);
	}

	public void sortCustomOrdering(HashMap<String, Channel> channels, int sortType)
	{
		if (sortType == 1)
		{
			Collections.sort(m_customOrder);
		}
		else if (sortType == 2)
		{
			Collections.sort(m_customOrder, Collections.reverseOrder());
		}
		else if (sortType == 3)
		{
			Set<String> customOrderSet = new HashSet<String>(m_customOrder);
			m_customOrder.clear();

			Integer[] numbers = m_numberToChannel.keySet().toArray(new Integer[0]);
			Arrays.sort(numbers);
			for (int x = 0; x < numbers.length; x++)
			{
				String[] sChannels = getChannels(numbers[x]);
				for (int y = 0; y < sChannels.length; y++)
				{
					if (customOrderSet.contains(sChannels[y]))
						m_customOrder.add( sChannels[y] );
				}
			}
		}
		else if (sortType == 4)
		{
			Set<String> customOrderSet = new HashSet<String>(m_customOrder);
			m_customOrder.clear();

			Integer[] numbers = m_numberToChannel.keySet().toArray(new Integer[0]);
			Arrays.sort(numbers, Collections.reverseOrder());
			for (int x = 0; x < numbers.length; x++)
			{
				String[] sChannels = getChannels(numbers[x]);
				for (int y = 0; y < sChannels.length; y++)
				{
					if (customOrderSet.contains(sChannels[y]))
						m_customOrder.add( sChannels[y] );
				}
			}
		}
	}

	protected void checkCustomOrderingInvalid(HashMap<String, Channel> channels)
	{
		for (int x = 0; x < m_customOrder.size(); x++)
		{
			String sChannel = m_customOrder.get(x);
			Channel channel = channels.get(sChannel);
			//if ( (channel == null) || (channel.isGroup()) )
			if ( channel == null )
				m_customOrder.remove(x--);
		}

		//		if (m_customOrder.isEmpty())
		//		checkCustomOrderingMissing(channels);
	}

	public void checkCustomOrderingMissing(HashMap<String, Channel> channels)
	{
		// Add missing channels.
		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		List<String> chList = new ArrayList<String>();

		for (int x = 0; x < keys.length; x++)
		{
			chList.add(channels.get(keys[x]).getSetting() + "\n" + keys[x]);
		}
		String[] chArray = chList.toArray(new String[0]);
		Arrays.sort(chArray);

		for (int x = 0; x < chArray.length; x++)
		{
			String[] chs = chArray[x].split("\n");
			Channel ch = channels.get(chs[1]);
			//if ( (ch == null) || (ch.isGroup()) )
			if ( ch == null )
				continue;

			if (!m_customOrder.contains(chs[1]))
				m_customOrder.add(chs[1]);
		}
	}

	public void updateChannel(String oldName, String newName, HashMap<String, Channel> channels)
	{
		for (int x = 0; x < m_customOrder.size(); x++)
		{
			String sChannel = m_customOrder.get(x);
			if (sChannel.equals(oldName))
			{
				m_customOrder.set(x, newName);
				break;
			}
		}

		setDefaultOrdering(channels);
	}

	public void setUse(boolean bUseDefault)
	{
		m_bUseDefault = bUseDefault;

		if (m_bUseDefault)
			m_order = m_defaultOrder;
		else
			m_order = m_customOrder;
	}

	public String getFirstChannel()
	{
		return m_order.isEmpty() ? null : m_order.firstElement();
	}

	protected int nextIndex(int index)
	{
		return (index < 0) || (index >= (m_order.size() - 1)) ? 0 : index + 1;
	}

	protected int previousIndex(int index)
	{
		return (index <= 0) || (index >= m_order.size()) ? m_order.size() - 1 : index - 1;
	}

	public String getNextChannel(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);
		return m_order.get(nextIndex(index));
	}

	public String getPreviousChannel(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);
		return m_order.get(previousIndex(index));
	}

	public String getNextChannelSameHardSetting(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);

		DataStore store = DataStore.getInstance();
		Channel oldChannel = store.getChannel(sChannel);
		for (int x = 0; x < m_order.size(); x++)
		{
			index = nextIndex(index);
			Channel newChannel = store.getChannel(m_order.get(index));
			if (!newChannel.isGroup() && newChannel.isHardSettingEqual(oldChannel))
				return m_order.get(index);
		}

		// ???
		return null;
	}

	public String getPreviousChannelSameHardSetting(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);

		DataStore store = DataStore.getInstance();
		Channel oldChannel = store.getChannel(sChannel);
		for (int x = 0; x < m_order.size(); x++)
		{
			index = previousIndex(index);
			Channel newChannel = store.getChannel(m_order.get(index));
			if (!newChannel.isGroup() && newChannel.isHardSettingEqual(oldChannel))
				return m_order.get(index);
		}

		// ???
		return null;
	}

	public String getNextChannelSameSetting(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);

		DataStore store = DataStore.getInstance();
		Channel oldChannel = store.getChannel(sChannel);
		for (int x = 0; x < m_order.size(); x++)
		{
			index = nextIndex(index);
			Channel newChannel = store.getChannel(m_order.get(index));
			if (!newChannel.isGroup() && newChannel.isSettingEqual(oldChannel))
				return m_order.get(index);
		}

		// ???
		return null;
	}

	public String getPreviousChannelSameSetting(String sChannel)
	{
		if ( m_order.isEmpty() || (sChannel == null) )
			return null;

		int index = m_order.indexOf(sChannel);

		DataStore store = DataStore.getInstance();
		Channel oldChannel = store.getChannel(sChannel);
		for (int x = 0; x < m_order.size(); x++)
		{
			index = previousIndex(index);
			Channel newChannel = store.getChannel(m_order.get(index));
			if (!newChannel.isGroup() && newChannel.isSettingEqual(oldChannel))
				return m_order.get(index);
		}

		// ???
		return null;
	}

	public String getChannel(int channelNumber)
	{
		return m_numberToChannel.get(channelNumber).split("\n")[0];
	}

	public int getChannelNumber(String channelName)
	{
		for (int i=0; i<m_order.size(); i++)
		{
			if (m_order.get(i).equals(channelName))
				return i;
		}
		return -1;
	}

	public String[] getChannels(int channelNumber)
	{
		return m_numberToChannel.get(channelNumber).split("\n");
	}

	public void putChannel(int number, String name)
	{
		String name2 = m_numberToChannel.get(number);
		if (name2 != null)
			m_numberToChannel.put(number, name2 + "\n" + name);
		else
			m_numberToChannel.put(number, name);
	}

	public Vector<String> getChannels()
	{
		return m_order;
	}
}