/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.Map;
import java.awt.*;
import java.awt.image.*;

class ContinuityImageDataRes extends HTTPResponse
{
	public ContinuityImageDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		m_outStream.write(getImage());
	}

	private byte[] getImage() throws Exception
	{
		ByteArrayOutputStream out = new ByteArrayOutputStream(2048);

		out.write("HTTP/1.0 200 OK\n".getBytes());
		out.write("Content-Type: image/png\n".getBytes());
		out.write("Pragma: no-cache\n".getBytes());
		out.write("Cache-Control: no-cache\n\n".getBytes());

		byte[] bytes = getImageData();

		out.write(bytes);       

		return out.toByteArray();
	}

	private byte[] getImageData() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		int width = 600;
		int height = 20;

		Image img = (Image)new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);

		// variables used for drawing hands of clock
		Graphics g;
		g = img.getGraphics();

		g.setColor( new Color( 188, 190, 252) );
		g.fillRect(0, 0, width, height);      

		guide.calculateDateRange();

		long startData = guide.getMinEntry().getTime();
		long endData = guide.getMaxEntry().getTime();

		GuideItem[] items = guide.getProgramsForChannel(m_urlData.getParameter("channel")); 

		long totalLength = endData - startData;//items[items.length-1].getStart().getTime() - items[0].getStart().getTime();
		long startTime = startData;//items[0].getStart().getTime();

		//DVScheduler.log(m_urlData.getParameter("channel"));

		for (int y = 0; y < items.length - 1; y++)
		{
			GuideItem item1 = items[y];
			GuideItem item2 = items[y + 1];

			long diff = item2.getStart().getTime() - item1.getStop().getTime();

			if(diff > 0)
			{
				long x1 = item1.getStop().getTime() - startTime;
				long x2 = diff;

				x1 = (long)(((double)x1 / (double)totalLength) * (double)width);
				x2 = (long)(((double)x2 / (double)totalLength) * (double)width);

				if(x2 < 2)
					x2 = 2;            

				//g.setColor( new Color( 255, 128, 192) );
				g.setColor( new Color( 255, 255, 0) );
				g.fillRect((int)x1, 0, (int)(x2), height); 
				//DVScheduler.log("DIFF : " + diff + " x1=" + x1 + " x2=" + x2);
			}

			if(diff < 0)
			{
				long x1 = item1.getStop().getTime() - startTime;
				long x2 = diff;

				x1 = (long)(((double)x1 / (double)totalLength) * (double)width);
				x2 = (long)(((double)x2 / (double)totalLength) * (double)width);            

				x2 = x2 * -1;
				if(x2 < 2)
					x2 = 2;  

				x1 = x1 - x2;

				g.setColor( new Color( 255, 0, 0) );
				g.fillRect((int)x1, 0, (int)(x2), height); 

				//DVScheduler.log("DIFF : " + diff + " x1=" + x1 + " x2=" + x2);
			}         

		}

		long startX = items[0].getStart().getTime() - startTime;
		startX = (long)(((double)startX / (double)totalLength) * (double)width);
		g.setColor( new Color( 255, 255, 255) );
		if(startX < 0)
			startX = 0;
		g.fillRect((int)startX, 0, 2, height); 

		long stopX = items[items.length-1].getStop().getTime() - startTime;
		stopX = (long)(((double)stopX / (double)totalLength) * (double)width);
		g.setColor( new Color( 255, 255, 255) );
		if(stopX > width-2)
			stopX = width-2;
		g.fillRect((int)stopX, 0, 2, height);       

		byte[] pngbytes;
		PngEncoder png =  new PngEncoder( img, PngEncoder.NO_ALPHA );
		pngbytes = png.pngEncode();
		return pngbytes;
	}


}
