/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Map.Entry;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.OutputKeys;

import org.w3c.dom.*;

public class DataStore
{
	public static final int WEBGUI_HTTP = 0;
	public static final int WEBGUI_HTTPS = 1;

	private Date lastCheched = new Date();
	private HashMap<String, Date> sessionIDs = new HashMap<String, Date>();

	private String version = "N/A";

	private static DataStore instance = null;

	private HashMap<String, ScheduleItem> m_schedules = null;	// Key is schedule ID.
	private HashMap<String, Channel> m_channels = null;	// Key is channel name.
	private Channel m_currentChannel = new Channel();
	public ChannelOrder m_channelOrder = new ChannelOrder();

	private Map<String, String> m_users = new HashMap<String, String>();	// Key is username.
	private Map<String, String> m_trusted = new HashMap<String, String>();	// Key is IP address.
	private Vector<String> m_trustedRange1 = new Vector<String>();
	private Vector<String> m_trustedRange2 = new Vector<String>();
	private Vector<String> m_trustedRangeGroupName = new Vector<String>();
	private Map<String, GroupAuth> m_groups = new HashMap<String, GroupAuth>();	// Key is group name.

	private String m_keyStore = "";
	private String m_keyStorePassword = "";
	private String m_keyPassword = "";
	private int m_webGuiProtocol = WEBGUI_HTTP;
	private int m_httpsPort = 443;
	private boolean m_bAllowUnsecureDownload = true;

	private Vector<String> m_serveFilePaths = new Vector<String>();

	private Properties serverProp = new Properties();
	public Random m_rand = new Random(new Date().getTime());

	public HashMap<Integer, String> monthNameFull = new HashMap<Integer, String>();
	public HashMap<Integer, String> monthNameShort = new HashMap<Integer, String>();
	public HashMap<Integer, String> dayName = new HashMap<Integer, String>();
	public HashMap<Integer, String> dayNameFull = new HashMap<Integer, String>();
	public HashMap<Integer, String> ampm = new HashMap<Integer, String>();

	private HashMap<String, KeepForDetails> m_autoDelList = null;
	private StringBuffer m_autoDelLog = new StringBuffer(2048);
	private int m_autoDelLogCount = 0;

	private Vector<EpgMatch> m_epgAutoAdds = new Vector<EpgMatch>();
	private HashMap<String, EpgMatchList> m_epgMatches = new HashMap<String, EpgMatchList>();

	private HashMap<String, TaskCommand> m_tasks = new HashMap<String, TaskCommand>();
	public HashMap<String, TaskItemThread> m_runningTaskList = new HashMap<String, TaskItemThread>();
	public Vector<TaskItem> m_waitingTaskList = new Vector<TaskItem>();
	public Vector<TaskItemThread> m_finishedTaskList = new Vector<TaskItemThread>();

	public int timerStatus = 0;
	public String timerThreadErrorStack = "";

	public TimerThread m_timerThread = null;
	private boolean m_bToDo = false;
	private boolean m_bSecurity = true;

	public boolean m_bSoonCapturing = false;
	public long m_nextScheduleTime = Long.MAX_VALUE;
	public long m_nextEpgLoadTime = Long.MAX_VALUE;
	public long m_nextEpgLoadWakeUpTime = Long.MAX_VALUE;
	public long m_wakeUpTime = Long.MAX_VALUE;	// Wake-up time (includes System wake-up allowance).
	public long m_wakeUpEventTime = Long.MAX_VALUE;	// Time of the actual event needing system wake-up.
	private int m_nWakeUpTimerStatus = 0;

	public int m_prevEpgLoadHour = -1;
	public int m_prevEpgLoadMinute = -1;
	public boolean m_bPrevEpgLoadEnabled = false;

	public Thread m_epgLoaderThread = null;
	public boolean m_bEpgManualLoadRunning = false;

	public int m_nRecordingErrorCount = 0;
	public int m_nLastRecordingErrorCount = 0;

	private Vector<String> m_networkTypes = new Vector<String>();

	public HashMap<String, CaptureTask> m_captureTasks = new HashMap<String, CaptureTask>();

	private List<Integer> m_availableStreamPorts = new LinkedList<Integer>();
	private int m_streamPort = 8006;

	public Vector<String[]> m_agentThemes = new Vector<String[]>();

	public String m_EpgTitleFilter = "";

	// Time in milliseconds.
	private long m_totalCapture = 0;
	private long m_totalCaptureTime = 0;
	private long m_sessionCapture = 0;
	private long m_sessionCaptureTime = 0;

	Vector<CaptureDevice> m_tuners;

	public static String m_saveFile = "server.prop";	// Location of server.prop
	public static String m_saveFolder = "";	// Either "C:\Program Files\DV Scheduler\" or "C:\ProgramData\DV Scheduler\"
	public static String m_saveFolderData = "data" + File.separator;	// Either "C:\Program Files\DV Scheduler\data\" or "C:\ProgramData\DV Scheduler\"

	private void initMaps()
	{
		monthNameFull.put(new Integer(Calendar.JANUARY), "January");
		monthNameFull.put(new Integer(Calendar.FEBRUARY), "February");
		monthNameFull.put(new Integer(Calendar.MARCH), "March");
		monthNameFull.put(new Integer(Calendar.APRIL), "April");
		monthNameFull.put(new Integer(Calendar.MAY), "May");
		monthNameFull.put(new Integer(Calendar.JUNE), "June");
		monthNameFull.put(new Integer(Calendar.JULY), "July");
		monthNameFull.put(new Integer(Calendar.AUGUST), "August");
		monthNameFull.put(new Integer(Calendar.SEPTEMBER), "September");
		monthNameFull.put(new Integer(Calendar.OCTOBER), "October");
		monthNameFull.put(new Integer(Calendar.NOVEMBER), "November");
		monthNameFull.put(new Integer(Calendar.DECEMBER), "December");

		monthNameShort.put(new Integer(Calendar.JANUARY), "Jan");
		monthNameShort.put(new Integer(Calendar.FEBRUARY), "Feb");
		monthNameShort.put(new Integer(Calendar.MARCH), "Mar");
		monthNameShort.put(new Integer(Calendar.APRIL), "Apr");
		monthNameShort.put(new Integer(Calendar.MAY), "May");
		monthNameShort.put(new Integer(Calendar.JUNE), "Jun");
		monthNameShort.put(new Integer(Calendar.JULY), "Jul");
		monthNameShort.put(new Integer(Calendar.AUGUST), "Aug");
		monthNameShort.put(new Integer(Calendar.SEPTEMBER), "Sep");
		monthNameShort.put(new Integer(Calendar.OCTOBER), "Oct");
		monthNameShort.put(new Integer(Calendar.NOVEMBER), "Nov");
		monthNameShort.put(new Integer(Calendar.DECEMBER), "Dec");

		dayName.put(new Integer(Calendar.MONDAY), "Mon");
		dayName.put(new Integer(Calendar.TUESDAY), "Tue");
		dayName.put(new Integer(Calendar.WEDNESDAY), "Wed");
		dayName.put(new Integer(Calendar.THURSDAY), "Thu");
		dayName.put(new Integer(Calendar.FRIDAY), "Fri");
		dayName.put(new Integer(Calendar.SATURDAY), "Sat");
		dayName.put(new Integer(Calendar.SUNDAY), "Sun");

		dayNameFull.put(new Integer(Calendar.MONDAY), "Monday");
		dayNameFull.put(new Integer(Calendar.TUESDAY), "Tuesday");
		dayNameFull.put(new Integer(Calendar.WEDNESDAY), "Wednesday");
		dayNameFull.put(new Integer(Calendar.THURSDAY), "Thursday");
		dayNameFull.put(new Integer(Calendar.FRIDAY), "Friday");
		dayNameFull.put(new Integer(Calendar.SATURDAY), "Saturday");
		dayNameFull.put(new Integer(Calendar.SUNDAY), "Sunday");      

		ampm.put(new Integer(Calendar.AM), "AM");
		ampm.put(new Integer(Calendar.PM), "PM");
	}

	public static DataStore getInstance()
	{
		synchronized (DataStore.class)
		{
			if (instance == null)
			{
				instance = new DataStore();
				return instance;
			}
			else
			{
				return instance;
			}
		}
	}

	private void checkBreaks(String pKey, String def) throws Exception
	{
		String text = getProperty(pKey, def);
		String text2 = DVScheduler.checkBreaks(text);

		if ( !text.equals(text2) )
		{
			setProperty(pKey, text2);
			DVScheduler.log("Fixed property: " + pKey);
		}
	}

	private DataStore()
	{
		getVersionDetails();
		DVScheduler.log("New DataStore Object created (version " + version + ")");

		try
		{
			serverProp.load(new FileInputStream(m_saveFile));
		}
		catch(Exception e)
		{
			DVScheduler.log(m_saveFile + " not found!");
			System.exit(-10);
		}

		try
		{
			checkBreaks("Capture.path", "capture");
			checkBreaks("user.keywords", "$$vlcexe->C:\\Program Files\\VideoLAN\\VLC\\vlc.exe");
			checkBreaks("path.agentThemes", "");
			checkBreaks("guide.source.file", "data/xmltv");
			checkBreaks("guide.source.http", "");
			checkBreaks("guide.source.http.usr", "");
			checkBreaks("guide.source.http.pwd", "");

			if (getPropertyInt("version", 1) < 2)
			{
				setProperty( "Capture.filename.patterns", getProperty("Capture.filename.patterns", "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d").replace(",", "\n") );
				setProperty("version", "2");
			}
		}
		catch(Exception e)
		{
			DVScheduler.log("Cannot write to " + m_saveFile);
			System.exit(-10);
		}

		m_totalCapture = getPropertyLong("capture.total", 0);
		m_totalCaptureTime = getPropertyLong("capture.totalTime", 0);

		m_channels = new HashMap<String, Channel>();

		loadNetworkTypes();
		loadAutoDelList();
		loadChannels();
		loadUsers();
		loadSchedules(true);
		initMaps();
		loadEpgMatches();
		loadEpgAutoList();
		loadTaskList();
		loadAgentThemes();

		setStreamPorts(false);

		StringBuffer buff = new StringBuffer();
		buff.append("DV Scheduler was started (" + DVScheduler.m_start.toString() + ")\n");
		buff.append("Following is a summary of the startup details:\n\n");

		buff.append("Channel Count           : " + numberOfChannels() + "\n");
		buff.append("Server Version          : " + getVersion() + "\n");
		buff.append("Number of Schedules     : " + getScheduleList().size() + "\n");

		sendEmail("DV Scheduler Started", buff.toString());
	}

	public String toString()
	{
		return "DataStore Properties Loaded:" + serverProp.size();
	}

	// Number of running captures.
	public int getNumCaptureTask()
	{
		return m_captureTasks.size();
	}

	public void setToDo(boolean bToDo)
	{
		m_bToDo = bToDo;
	}

	public void setToDoWake()
	{
		m_bToDo = true;
		m_timerThread.notifyResponse();
	}

	public boolean isToDo()
	{
		return m_bToDo;
	}

	// Get version from version.prop file
	private void getVersionDetails()
	{
		try
		{
			Properties verProp = new Properties();
			verProp.load(new FileInputStream("version.prop"));
			version = verProp.getProperty("version", "N/A");
		}
		catch (Exception e)
		{
			DVScheduler.log("ERROR!: could not load version info");
		}
	}

	public String createSessionId()
	{
		// Keep only Latest 5
		List<Map.Entry> entrylist = new ArrayList<Map.Entry>(sessionIDs.entrySet());
		Collections.sort(entrylist, new SecurityPinSorter());   
		sessionIDs.clear();
		for(int x = 0; x < entrylist.size() && x < 4; x++)
		{
			Map.Entry entry = (Map.Entry)entrylist.get(x);
			sessionIDs.put((String)entry.getKey(), (Date)entry.getValue());
		}

		String[] ids = (String[])sessionIDs.keySet().toArray(new String[0]);
		// remove old id's
		Date now = new Date();
		for(int x = 0; x < ids.length; x++)
		{  
			Date createDate = (Date)sessionIDs.get(ids[x]);
			if((createDate.getTime() + (1000 * 60 * 3)) < now.getTime())
			{
				sessionIDs.remove(ids[x]);
				DVScheduler.log("Removed Old Session ID : " + ids[x] + " : " + ((now.getTime() - createDate.getTime()) / (1000)));
			}
		}

		Random ran = new Random();
		String ranNum = new Integer(ran.nextInt(999999)).toString();
		if(ranNum.length() == 1)
			ranNum = "00000" + ranNum;
		if(ranNum.length() == 2)
			ranNum = "0000" + ranNum;     
		if(ranNum.length() == 3)
			ranNum = "000" + ranNum;  
		if(ranNum.length() == 4)
			ranNum = "00" + ranNum;  
		if(ranNum.length() == 5)
			ranNum = "0" + ranNum;        

		sessionIDs.put(ranNum, new Date());

		entrylist = new ArrayList<Map.Entry>(sessionIDs.entrySet());
		Collections.sort(entrylist, new SecurityPinSorter()); 
		for(int x = 0; x < entrylist.size() && x < 5; x++)
		{
			Map.Entry entry = (Map.Entry)entrylist.get(x);
			DVScheduler.log("Session ID : " + x + " " + (String)entry.getKey() + " " + ((Date)entry.getValue()).toString());
		}      

		return ranNum;
	}

	public boolean checkSessionId(String id)
	{
		if (!isSecurity())
			return true;

		synchronized(this)
		{      
			long sleepFor = (new Date()).getTime() - lastCheched.getTime();
			if(sleepFor < 10000)
			{
				try
				{
					Thread.sleep(10000 - sleepFor);
				}
				catch(Exception e)
				{}
			}

			lastCheched = new Date();

			// remove old id's
			String[] ids = (String[])sessionIDs.keySet().toArray(new String[0]);
			Date now = new Date();
			for(int x = 0; x < ids.length; x++)
			{  
				Date createDate = (Date)sessionIDs.get(ids[x]);
				if((createDate.getTime() + (1000 * 60 * 3)) < now.getTime())
				{
					DVScheduler.log("Removed Old Session ID : " + ids[x] + " - " + createDate);
					sessionIDs.remove(ids[x]);
				}
			}

			Date testDate = (Date)sessionIDs.get(id);
			if(testDate != null)
			{
				sessionIDs.remove(id);
				DVScheduler.log("Removed Used Session ID : " + id + " - " + testDate);
				return true;
			}
			else
				return false;
		}
	}

	private void sendEmail(String subject, String body)
	{
		if ( !getProperty("email.serverStarted", "No").equals("Yes") )
			return;

		try
		{
			EmailSender sender = new EmailSender();
			sender.setSubject(subject);
			sender.setBody(body);

			Thread mailThread = new Thread(Thread.currentThread().getThreadGroup(), sender, sender.getClass().getName());
			mailThread.start();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  tasks methids
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	private void loadTaskList()
	{
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(DataStore.m_saveFolderData + "Tasks.xml");         

			int version = 0;
			try
			{
				version = Integer.parseInt(doc.getDocumentElement().getAttributes().getNamedItem("version").getTextContent());
			}
			catch(Exception e){}
			boolean bKeepDefaultTasks = version >= 2;

			NodeList tasksNodes = doc.getElementsByTagName("task");

			m_tasks = new HashMap<String, TaskCommand>();

			for(int x = 0; x < tasksNodes.getLength(); x++)
			{
				Node item = tasksNodes.item(x);
				TaskCommand taskCommand = new TaskCommand(item);

				if ( !bKeepDefaultTasks && (taskCommand.getName().equals("Suspend Computer") || 
						taskCommand.getName().equals("Hibernate Computer") || 
						taskCommand.getName().equals("Shutdown Computer")) )
				{
					continue;
				}
				if ( bKeepDefaultTasks && (version < 4) && (taskCommand.getName().equals("Suspend Computer") || 
						taskCommand.getName().equals("Hibernate Computer") || 
						taskCommand.getName().equals("Shutdown Computer")) )
				{
					taskCommand.setAllowSuspend(true);
					taskCommand.setWhenNotCapturing(true);
					taskCommand.setMinIdleTime(2);
				}

				m_tasks.put(taskCommand.getName(), taskCommand);
			}         

			DVScheduler.log("Tasks.xml found and loaded (" + tasksNodes.getLength() + ")");
		}
		catch (Exception e)
		{
			m_tasks = new HashMap<String, TaskCommand>();
			DVScheduler.log("Error loading Tasks.xml, starting with no tasks.");
			e.printStackTrace();
		}
	}

	public void saveTaskList(StringBuffer output) throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "tasks", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("version", "4");

		String[] keys = (String[]) m_tasks.keySet().toArray(new String[0]);

		for (int x = 0; x < keys.length; x++)
		{
			TaskCommand taskData = (TaskCommand) m_tasks.get(keys[x]);
			taskData.getXML(doc, root);
		}

		ByteArrayOutputStream buff = new ByteArrayOutputStream();
		TransformerFactory factory = TransformerFactory.newInstance();

		Transformer transformer = factory.newTransformer();
		transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");      

		Source source = new DOMSource(doc);
		Result result = new StreamResult(buff);
		transformer.transform(source, result);

		if(output != null)
		{
			output.append(buff.toString("UTF-8"));
		}
		else
		{
			FileOutputStream fos = new FileOutputStream(DataStore.m_saveFolderData + "Tasks.xml");
			Writer out = new BufferedWriter(new OutputStreamWriter(fos, "UTF-8"));
			out.write(buff.toString("UTF-8"));
			out.close();
		}
	}

	public HashMap<String, TaskCommand> getTaskList()
	{
		TaskCommand task = m_tasks.get("Suspend Computer");
		if (task == null)
		{
			task = new TaskCommand("Suspend Computer");
			task.setCommand("Suspend.exe -s -t 15");
			task.setAllowSuspend(true);
			task.setWhenNotCapturing(true);
			task.setMinIdleTime(2);
			m_tasks.put(task.getName(), task);
		}

		TaskCommand task2 = m_tasks.get("Hibernate Computer");
		if (task2 == null)
		{
			task2 = new TaskCommand("Hibernate Computer");
			task2.setCommand("Suspend.exe -h -t 15");
			task2.setAllowSuspend(true);
			task2.setWhenNotCapturing(true);
			task2.setMinIdleTime(2);
			m_tasks.put(task2.getName(), task2);
		}

		TaskCommand task3 = m_tasks.get("Shutdown Computer");
		if (task3 == null)
		{
			task3 = new TaskCommand("Shutdown Computer");
			task3.setCommand("shutdown.exe -s -t 15");
			task3.setAllowSuspend(true);
			task3.setWhenNotCapturing(true);
			task3.setMinIdleTime(2);
			m_tasks.put(task3.getName(), task3);
		}

		return m_tasks;
	}

	public void getTaskListXml(Document doc, Element node)
	{
		getTaskList();

		String[] keys = (String[]) m_tasks.keySet().toArray(new String[0]);

		Arrays.sort(keys);

		Element elm2 = null;
		for (int i=0; i<keys.length; i++)
		{
			elm2 = doc.createElement("item");
			elm2.setAttribute("name", keys[i]);
			elm2.setAttribute("value", keys[i]);
			node.appendChild(elm2);
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  EPG methods
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	private void loadEpgMatches()
	{
		m_epgMatches = new HashMap<String, EpgMatchList>();
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(DataStore.m_saveFolderData + "MatchList.xml");

			Node item = doc.getDocumentElement().getAttributes().getNamedItem("epgTitleFilter");
			if (item != null)
				m_EpgTitleFilter = DVScheduler.checkBreaks(item.getTextContent());
			if (m_EpgTitleFilter == null)
				m_EpgTitleFilter = "";

			NodeList items = doc.getElementsByTagName("MatchList");

			for(int x = 0; x < items.getLength(); x++)
			{
				item = items.item(x);
				NamedNodeMap itemAttribs = item.getAttributes();
				String name = itemAttribs.getNamedItem("name").getTextContent();

				EpgMatchList matcher = new EpgMatchList(item);

				m_epgMatches.put(name, matcher);
			}
			DVScheduler.log("MatchList.xml found and loaded (" + m_epgMatches.size() + ")");
		}
		catch (Exception e)
		{
			m_epgMatches = new HashMap<String, EpgMatchList>();
			DVScheduler.log("Error loading MatchList.xml, starting with no Match Lists");
			e.printStackTrace();
		}      
	}

	public void saveEpgMatches(StringBuffer output) throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "match-list", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("epgTitleFilter", m_EpgTitleFilter);

		String[] keys = (String[])m_epgMatches.keySet().toArray(new String[0]);
		for (int x = 0; x < keys.length; x++)
		{
			EpgMatchList item = (EpgMatchList) m_epgMatches.get(keys[x]);
			item.getXML(doc, keys[x]);
		}

		ByteArrayOutputStream buff = new ByteArrayOutputStream();
		TransformerFactory factory = TransformerFactory.newInstance();

		Transformer transformer = factory.newTransformer();
		transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");       

		Source source = new DOMSource(doc);
		Result result = new StreamResult(buff);
		transformer.transform(source, result);

		if(output != null)
		{
			output.append(buff.toString("UTF-8"));
		}
		else
		{
			FileOutputStream fos = new FileOutputStream(DataStore.m_saveFolderData + "MatchList.xml");
			Writer out = new BufferedWriter(new OutputStreamWriter(fos, "UTF-8"));
			out.write(buff.toString("UTF-8"));
			out.close();
			refreshAutoAdd();
		}
	}

	public void deleteEpgMatches(Vector<String> names)
	{
		if ( (names == null) || names.isEmpty() )
			return;

		for (int i=0; i<names.size(); i++)
		{
			String name = names.get(i);
			if ( m_epgMatches.containsKey(name) )
			{
				m_epgMatches.remove(name);
			}
		}

		try
		{
			saveEpgMatches(null);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public HashMap<String, EpgMatchList> getEpgMatches()
	{
		return m_epgMatches;
	}

	public EpgMatchList getEpgMatchList(String name)
	{
		return m_epgMatches.get(name);
	}

	public void getEpgMatchesXml(Document doc, Element node)
	{
		String[] keys = (String[])m_epgMatches.keySet().toArray(new String[0]);
		Arrays.sort(keys, String.CASE_INSENSITIVE_ORDER);
		Element elm2 = null;
		for(int x = 0; x < keys.length; x++)
		{
			elm2 = doc.createElement("item");
			elm2.setAttribute("name", keys[x]);
			node.appendChild(elm2);
		}
	}

	private void loadEpgAutoList()
	{
		Vector<String> fileNames = getFileType(null, ".ini");
		Set<Integer> numbers = new HashSet<Integer>();
		for (int i=0; i<fileNames.size(); i++)
		{
			int value = -1;
			try
			{
				value = Integer.parseInt(fileNames.get(i));
			}
			catch (Exception e) {}
			if ( (value >=0) && (value <= 9) )
				numbers.add(value);
		}

		String[] namePatterns = getNamePatterns();
		Set<String> namePatterns2 = new HashSet<String>();
		for (int i=0; i<namePatterns.length; i++)
		{
			namePatterns2.add(namePatterns[i]);
		}

		m_epgAutoAdds = new Vector<EpgMatch>();
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(DataStore.m_saveFolderData + "EpgAutoAdd.xml");

			NodeList items = doc.getElementsByTagName("item");

			int countNewNamePatterns = 0;
			String newNamePatterns = "";
			for(int x = 0; x < items.getLength(); x++)
			{
				Node item = items.item(x);
				EpgMatch matcher = new EpgMatch(item);

				// Old version uses number, so change to default capture type.
				int value = -1;
				try
				{
					value = Integer.parseInt(matcher.getCaptureType());
				}
				catch (Exception e) {}
				if ( (value >= 0) && (value <= 9) && !numbers.contains(value) )
				{
					DVScheduler.log("Unknown single digit capture type: " + matcher.getCaptureType() + ". Changing to Capture.deftype");
					matcher.setCaptureType("Capture.deftype");
				}

				// Old version has different default name pattern.
				if ( !namePatterns2.contains(matcher.GetFileNamePattern()) && (++countNewNamePatterns <= 10) )
				{
					DVScheduler.log("Unknown filename pattern: " + matcher.GetFileNamePattern() + ". Adding to existing filename pattern list");
					newNamePatterns += "\n" + matcher.GetFileNamePattern();
					namePatterns2.add(matcher.GetFileNamePattern());
				}

				m_epgAutoAdds.add(matcher);
			}
			if (countNewNamePatterns > 10)
				DVScheduler.log("Too many unknown filename patterns, only adding the first 10");

			if (!newNamePatterns.isEmpty())
				setProperty("Capture.filename.patterns", getProperty("Capture.filename.patterns", "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d") + newNamePatterns);

			DVScheduler.log("EpgAutoAdd.xml found and loaded (" + items.getLength() + ")");
		}
		catch (Exception e)
		{
			m_epgAutoAdds = new Vector<EpgMatch>();
			DVScheduler.log("Error loading EpgAutoAdd.xml, starting with no AutoAdds");
			e.printStackTrace();
		}
	}

	public void saveEpgAutoList(StringBuffer output) throws Exception
	{
		saveEpgAutoList(output, 0);
	}

	public void saveEpgAutoList(StringBuffer output, int nRescan) throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "auto-add", null);
		Element root = doc.getDocumentElement();

		for (int x = 0; x < m_epgAutoAdds.size(); x++)
		{
			EpgMatch item = (EpgMatch) m_epgAutoAdds.get(x);
			Element elm = doc.createElement("item");
			item.getXML(doc, elm);
			root.appendChild(elm);
		}

		ByteArrayOutputStream buff = new ByteArrayOutputStream();
		TransformerFactory factory = TransformerFactory.newInstance();

		Transformer transformer = factory.newTransformer();
		transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");       

		Source source = new DOMSource(doc);
		Result result = new StreamResult(buff);
		transformer.transform(source, result);

		if(output != null)
		{
			output.append(buff.toString("UTF-8"));
		}
		else
		{
			FileOutputStream fos = new FileOutputStream(DataStore.m_saveFolderData + "EpgAutoAdd.xml");
			Writer out = new BufferedWriter(new OutputStreamWriter(fos, "UTF-8"));
			out.write(buff.toString("UTF-8"));
			out.close();

			switch (nRescan)
			{
			case 1:
				rescanXMLTVdata(null);
				break;
			case 2:
				break;
			case 0: default:
				refreshAutoAdd();
			}
		}
	}

	public Vector<EpgMatch> getEpgAutoAdds()
	{
		return m_epgAutoAdds;
	}

	public boolean isEpgAutoAddUniqueName(String name)
	{
		for (int i=0; i<m_epgAutoAdds.size(); i++)
		{
			if (m_epgAutoAdds.get(i).getName().equals(name))
				return false;
		}
		return true;
	}

	public String getEpgAutoAddUniqueName(String name)
	{
		String name2 = name;
		int i = 2;
		while ( !isEpgAutoAddUniqueName(name2) )
		{
			name2 = name + "-" + i++;
		}
		return name2;
	}

	public void getEpgAutoAddsXml(Document doc, Element node)
	{
		for (int i=0; i<m_epgAutoAdds.size(); i++)
		{
			Element elm = doc.createElement("item");
			m_epgAutoAdds.get(i).getXML(doc, elm, true);
			elm.setAttribute("index", Integer.toString(i));
			node.appendChild(elm);
		}
	}

	public void addEpgMatch(EpgMatch match) throws Exception
	{
		m_epgAutoAdds.add(match);
		saveEpgAutoList(null);
	}

	public void addEpgMatch(EpgMatch match, int index) throws Exception
	{
		addEpgMatch(match, index, 0);
	}

	public void addEpgMatch(EpgMatch match, int index, int nRescan) throws Exception
	{
		m_epgAutoAdds.add(index, match);
		saveEpgAutoList(null, nRescan);
	}

	public void removeEpgMatch(int id) throws Exception
	{
		m_epgAutoAdds.remove(id);
		saveEpgAutoList(null, 0);
	}

	public void moveEpgItem(int id, boolean direction) throws Exception
	{
		int dest = 0;

		if(direction)
			dest = id + 1;
		else
			dest = id - 1;

		if(dest >= 0 && dest < m_epgAutoAdds.size())
		{        
			EpgMatch obj = (EpgMatch)m_epgAutoAdds.remove(id);

			if(direction)
				m_epgAutoAdds.add(id+1, obj);
			else
				m_epgAutoAdds.add(id-1, obj);

			saveEpgAutoList(null, 0);
		}
	}

	public void deleteEpgAutoAdds(Vector<Integer> indexes)
	{
		if ( (indexes == null) || indexes.isEmpty() )
			return;

		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i) - i;
			if ( (index >= 0) && (index < m_epgAutoAdds.size()) )
			{
				m_epgAutoAdds.remove(index);
			}
		}

		try
		{
			saveEpgAutoList(null, 0);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void deleteAllEpgAutoAdds() throws Exception
	{
		m_epgAutoAdds.clear();
		m_epgMatches.clear();
		saveEpgAutoList(null, 0);
		saveEpgMatches(null);
	}

	public void moveEpgAutoAddsBeforeEpgAutoAdd(int beforeIndex, Vector<Integer> indexes)
	{
		if (beforeIndex < 0)
			beforeIndex = m_epgAutoAdds.size();

		Vector<EpgMatch> maps = new Vector<EpgMatch>();
		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i);
			if ( (index >= 0) && (index < m_epgAutoAdds.size()) )
				maps.add(m_epgAutoAdds.get(index));
		}
		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i) - i;
			if ( (index >= 0) && (index < m_epgAutoAdds.size()) )
			{
				m_epgAutoAdds.remove(index);
				if (index < beforeIndex)
					beforeIndex--;
			}
		}

		if ( (beforeIndex >= 0) && (beforeIndex <= m_epgAutoAdds.size()) )
		{
			m_epgAutoAdds.addAll(beforeIndex, maps);

			try
			{
				saveEpgAutoList(null, 0);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public void rescanXMLTVdata(StringBuffer buff) throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		removeSchedulesOfTypeEpg(buff, 1);
		guide.addEpgMatches(buff, 1, false);
		guide.saveEpg();
		saveSchedules(null);
	}

	public void reMarkXMLTVdata(StringBuffer buff) throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		guide.addEpgMatches(buff, 1, true);
		guide.saveEpg();
	}

	private void refreshAutoAdd() throws Exception
	{
		String autoAddRefresh = getProperty("Schedule.autoAddRefresh", "Yes");
		if (autoAddRefresh.equals("Favorites Only"))
		{
			reMarkXMLTVdata(null);
		}
		else if (autoAddRefresh.equals("Yes"))
		{
			rescanXMLTVdata(null);
		}
	}

	public int removeSchedulesOfTypeEpg(StringBuffer buff, int format)
	{
		int num = 0;

		ScheduleItem item = null;

		ThreadLock.getInstance().getLock();
		try {
			String[] ids = (String[]) (m_schedules.keySet()).toArray(new String[0]);
			String itemDetails = null;
			SimpleDateFormat df = new SimpleDateFormat("EE d HH:mm");

			for (int x = 0; x < ids.length; x++)
			{
				item = (ScheduleItem) m_schedules.get(ids[x]);

				if (item.getType() == ScheduleItem.EPG && item.getState() == ScheduleItem.WAITING)
				{
					itemDetails = item.getName()
							+ " ("
							+ df.format(item.getStart(),
									new StringBuffer(),
									new FieldPosition(0)) + " "
									+ item.getDuration() + " "
									+ item.getChannel() + ") ";

					if (item.isUserModified())
					{
						if (buff != null)
						{
							if (format == 1)
								buff.append("Keeping " + HTMLEncoder.encode(itemDetails) + " (user modified)<br>\n");
							else
								buff.append("Keeping " + itemDetails + " (user modified)\n");
						}
					}
					else
					{
						if (buff != null)
						{
							if (format == 1)
								buff.append("Removing " + HTMLEncoder.encode(itemDetails) + "<br>\n");
							else
								buff.append("Removing " + itemDetails + "\n");
						}

						m_schedules.remove(ids[x]);
						num++;
					}
				}
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		if (buff != null)
		{
			if (format == 1)
				buff.append("Removed " + num + " item of type EPG from the schedule list.<br><br>\n\n");
			else
				buff.append("Removed " + num + " item of type EPG from the schedule list.\n\n");
		}

		return num;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  Schedule methods
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public ScheduleItem[] getSchedulesWhen(Date start, Date end, String channel)
	{
		Vector<ScheduleItem> items = new Vector<ScheduleItem>();
		Iterator<ScheduleItem> it = m_schedules.values().iterator();
		ScheduleItem item = null;

		while (it.hasNext())
		{
			item = (ScheduleItem) it.next();

			if (start.getTime() < item.getStart().getTime() && end.getTime() > item.getStart().getTime())
			{
				if (channel.equals(item.getChannel()))
				{
					//DVScheduler.log(item.getStart() + " " + item.getName());
					items.add(item);
				}
			}
		}

		ScheduleItem[] itemList = (ScheduleItem[]) items.toArray(new ScheduleItem[0]);
		Arrays.sort(itemList);
		//DVScheduler.log("Getting Schedules for " + start + " " + end + " " +
		// channel + " Found:" + itemList.length);
		return itemList;
	}

	public void getSchedulesWhenInc(Date start, Date end, String channel, Vector<ScheduleItem> items)
	{
		ScheduleItem[] schedules = (ScheduleItem[])m_schedules.values().toArray(new ScheduleItem[0]);
		Arrays.sort(schedules);

		for(int y = 0; y < schedules.length; y++)
		{
			ScheduleItem item = schedules[y];

			if (item.getStop().getTime() > (start.getTime() + 5000) && item.getStart().getTime() < end.getTime())
			{
				if (channel.equals(item.getChannel()))
				{
					//DVScheduler.log(item.getStart() + " " + item.getName());
					items.add(item);
				}
			} 
		}
	}

	public ScheduleItem getNextSchedule()
	{
		Date now = new Date();
		ScheduleItem[] itemsArray = (ScheduleItem[])m_schedules.values().toArray(new ScheduleItem[0]);
		Arrays.sort(itemsArray);

		for(int x = 0; x < itemsArray.length; x++)
		{
			if(itemsArray[x].getStart().getTime() > now.getTime())
			{
				if(itemsArray[x].getState() == ScheduleItem.WAITING)
					return itemsArray[x];
			}
		}
		return null;
	}

	public HashMap<String, ScheduleItem> getScheduleList()
	{
		return m_schedules;
	}

	public ScheduleItem getScheduleItem(String id)
	{
		return (ScheduleItem) m_schedules.get(id);
	}

	public int addScheduleItemAndSave(ScheduleItem item)
	{
		m_schedules.put(item.toString(), item);
		saveSchedules(item);
		setToDoWake();
		return 1;
	}

	public int addScheduleItem(ScheduleItem item)
	{
		m_schedules.put(item.toString(), item);
		return 1;
	}

	public int removeScheduleItem(String id)
	{
		return removeScheduleItem(id, false);
	}

	private boolean excludeGuideItem(ScheduleItem item, GuideStore guide)
	{
		if ( (item == null) || (guide == null) )
			return false;

		GuideItem guideItem = item.getCreatedFrom();
		String channel = item.getChannel();
		if (guideItem != null && channel != null)
		{
			String epgChannel = guide.getEpgChannelIdFromMap(channel);
			if (epgChannel != null)
			{
				GuideItem guideItem2 = guide.getProgram(epgChannel, guideItem.toString());
				if (guideItem2 != null && !guide.isInEpgExclusion(guideItem2, channel))
				{
					guide.addEpgExclusion(guideItem2, channel);
					return true;
				}
			}
		}
		return false;
	}

	public int removeScheduleItem(String id, boolean bExclude)
	{
		if (bExclude)
		{
			ScheduleItem item = getScheduleItem(id);
			if ( (item != null) && (item.getType() == ScheduleItem.EPG) )
			{
				GuideStore guide = GuideStore.getInstance();
				if (excludeGuideItem(item, guide))
				{
					Vector<ScheduleItem> items = item.getChildren();
					for (int i=0; i<items.size(); i++)
					{
						excludeGuideItem(items.get(i), guide);
					}
					guide.saveEpgExclusion();
					guide.saveEpg();
				}
			}
		}
		m_schedules.remove(id);
		return 1;
	}

	public Vector<String> getFileType(String captureEnginePath, String type)
	{
		File dir = new File((captureEnginePath == null ? serverProp.getProperty("Capture.engine.path", "util") : captureEnginePath) + File.separator);
		File[] files = dir.listFiles(new FileTypeFilter(type, 0));

		Vector<String> fileNames = new Vector<String>();

		if (files == null || files.length <= 0)
		{
			return fileNames;
		}

		for(int x = 0; x  < files.length; x++)
		{
			String name = files[x].getName();

			if (!Formatter.isOk(name))
				continue;

			fileNames.add(name.substring(0, name.length()-4));
		}

		Collections.sort(fileNames);

		return fileNames;
	}

	public Vector<String> getCaptureTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".ini");
		fileNames.add(0, "Capture.deftype");
		return fileNames;
	}

	public Vector<String> getCaptureTypes()
	{
		return getCaptureTypes(null);
	}

	public void getCaptureTypesXml(String captureEnginePath, Document doc, Element node)
	{
		Vector<String> types = getCaptureTypes(captureEnginePath);

		Element elm2 = null;
		for (int i=0; i<types.size(); i++)
		{
			String type = types.get(i);

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", type);
			elm2.setAttribute("value", type);
			node.appendChild(elm2);
		}
	}

	public void getCaptureTypesXml(Document doc, Element item)
	{
		getCaptureTypesXml(null, doc, item);
	}

	public boolean isCaptureTypeValid(String captureType, String captureEnginePath)
	{
		Vector<String> captureTypes = getCaptureTypes(captureEnginePath);

		for(int x = 0; x  < captureTypes.size(); x++)
		{
			if (captureType.equals(captureTypes.get(x)))
				return true;
		}

		return false;
	}

	public Vector<String> getPreviewTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".pre");
		fileNames.add(0, "Preview.deftype");
		return fileNames;
	}

	public Vector<String> getPreviewTypes()
	{
		return getPreviewTypes(null);
	}

	public void getPreviewTypesXml(String captureEnginePath, Document doc, Element node)
	{
		Vector<String> types = getPreviewTypes(captureEnginePath);

		Element elm2 = null;
		for (int i=0; i<types.size(); i++)
		{
			String type = types.get(i);

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", type);
			elm2.setAttribute("value", type);
			node.appendChild(elm2);
		}
	}

	public void getPreviewTypesXml(Document doc, Element item)
	{
		getPreviewTypesXml(null, doc, item);
	}

	public boolean isPreviewTypeValid(String previewType, String captureEnginePath)
	{
		Vector<String> previewTypes = getPreviewTypes(captureEnginePath);

		for(int x = 0; x  < previewTypes.size(); x++)
		{
			if (previewType.equals(previewTypes.get(x)))
				return true;
		}

		return false;
	}

	public boolean isPreviewAvailable()
	{
		String previewType = getProperty("Preview.deftype", "Preview");
		if ( previewType.isEmpty() || previewType.equals("*UNDEFINED*") )
			return false;

		return !getFileType(null, ".pre").isEmpty();
	}

	public Vector<String> getPreviewRemoteTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".prr");
		fileNames.add(0, "Preview.remote.deftype");
		return fileNames;
	}

	public Vector<String> getPreviewRemoteTypes()
	{
		return getPreviewRemoteTypes(null);
	}

	public void getPreviewRemoteTypesXml(String captureEnginePath, Document doc, Element node)
	{
		Vector<String> types = getPreviewRemoteTypes(captureEnginePath);

		Element elm2 = null;
		for (int i=0; i<types.size(); i++)
		{
			String type = types.get(i);

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", type);
			elm2.setAttribute("value", type);
			node.appendChild(elm2);
		}
	}

	public void getPreviewRemoteTypesXml(Document doc, Element item)
	{
		getPreviewRemoteTypesXml(null, doc, item);
	}

	public boolean isPreviewRemoteTypeValid(String previewRemoteType, String captureEnginePath)
	{
		Vector<String> previewRemoteTypes = getPreviewRemoteTypes(captureEnginePath);

		for(int x = 0; x  < previewRemoteTypes.size(); x++)
		{
			if (previewRemoteType.equals(previewRemoteTypes.get(x)))
				return true;
		}

		return false;
	}

	public boolean isPreviewRemoteAvailable()
	{
		String previewType = getProperty("Preview.remote.deftype", "Preview");
		if ( previewType.isEmpty() || previewType.equals("*UNDEFINED*") )
			return false;

		return !getFileType(null, ".prr").isEmpty();
	}

	public Vector<String> getSurfTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".suf");
		fileNames.add(0, "Surf.deftype");
		return fileNames;
	}

	public Vector<String> getSurfTypes()
	{
		return getSurfTypes(null);
	}

	public void getSurfTypesXml(String captureEnginePath, Document doc, Element node)
	{
		Vector<String> types = getSurfTypes(captureEnginePath);

		Element elm2 = null;
		for (int i=0; i<types.size(); i++)
		{
			String type = types.get(i);

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", type);
			elm2.setAttribute("value", type);
			node.appendChild(elm2);
		}
	}

	public void getSurfTypesXml(Document doc, Element item)
	{
		getSurfTypesXml(null, doc, item);
	}

	public boolean isSurfTypeValid(String surfType, String captureEnginePath)
	{
		Vector<String> surfTypes = getSurfTypes(captureEnginePath);

		for(int x = 0; x  < surfTypes.size(); x++)
		{
			if (surfType.equals(surfTypes.get(x)))
				return true;
		}

		return false;
	}

	public Vector<String> getSurfRemoteTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".sur");
		fileNames.add(0, "Surf.remote.deftype");
		return fileNames;
	}

	public Vector<String> getSurfRemoteTypes()
	{
		return getSurfRemoteTypes(null);
	}

	public Vector<String> getPlayFileRemoteTypes(String captureEnginePath)
	{
		Vector<String> fileNames = getFileType(captureEnginePath, ".pfr");
		return fileNames;
	}

	public Vector<String> getPlayFileRemoteTypes()
	{
		return getPlayFileRemoteTypes(null);
	}

	public void getSurfRemoteTypesXml(String captureEnginePath, Document doc, Element node)
	{
		Vector<String> types = getSurfRemoteTypes(captureEnginePath);

		Element elm2 = null;
		for (int i=0; i<types.size(); i++)
		{
			String type = types.get(i);

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", type);
			elm2.setAttribute("value", type);
			node.appendChild(elm2);
		}
	}

	public void getSurfRemoteTypesXml(Document doc, Element item)
	{
		getSurfRemoteTypesXml(null, doc, item);
	}

	public boolean isSurfRemoteTypeValid(String surfType, String captureEnginePath)
	{
		Vector<String> surfRemoteTypes = getSurfRemoteTypes(captureEnginePath);

		for(int x = 0; x  < surfRemoteTypes.size(); x++)
		{
			if (surfType.equals(surfRemoteTypes.get(x)))
				return true;
		}

		return false;
	}

	public void getCapturePathsXml(Document doc, Element node)
	{
		String[] paths = DVScheduler.getArray(getProperty("Capture.path", "capture"));
		Element elm2 = doc.createElement("item");
		elm2.setAttribute("name", "Default");
		elm2.setAttribute("value", "*");
		node.appendChild(elm2);
		for (int i=0; i<paths.length; i++)
		{
			elm2 = doc.createElement("item");
			elm2.setAttribute("name", paths[i]);
			elm2.setAttribute("value", paths[i]);
			node.appendChild(elm2);
		}
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  Channel methods
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	protected void checkChannel(Channel chan)
	{
		if ( (chan.getName() == null) || chan.getName().isEmpty() )
		{
			chan.setName("1");
			if ( m_channels.containsKey(chan.getName()) )
			{
				Integer count = 1;
				while ( m_channels.containsKey((++count).toString()) );

				chan.setName(count.toString());
			}
		}
		else if ( m_channels.containsKey(chan.getName()) )
		{
			int count = 1;
			while (m_channels.containsKey(chan.getName() + "-" + ++count));

			chan.setName(chan.getName() + "-" + count);
		}
	}

	public void addChannel(Channel chan)
	{
		checkChannel(chan);
		m_channels.put(chan.getName(), chan);
		m_channelOrder.setDefaultOrdering(getChannels());
	}

	public Channel getChannel(String chID)
	{
		return (Channel) m_channels.get(chID);
	}

	public boolean isChannelExist(String channel)
	{
		return m_channels.containsKey(channel);
	}

	public int removeChannel(String name)
	{
		m_channels.remove(name);
		m_channelOrder.setDefaultOrdering(getChannels());
		return 0;
	}

	public void removeAllChannels() throws Exception
	{
		m_channels.clear();
		m_channelOrder.setDefaultOrdering(getChannels());
		saveChannels(null);
	}

	public void updateDemodulator(String oldName, Channel chan)
	{
		Channel ch = getChannel(oldName);
		if (ch != null)
		{
			ch.setDemodulator(chan);
		}
	}

	public void updateDemodulatorAll(String oldName, Channel chan)
	{
		Channel ch = getChannel(oldName);
		if (ch == null)
			return;

		Iterator<Entry<String, Channel>> iter = getChannels().entrySet().iterator();
		while (iter.hasNext())
		{
			Channel ch2 = iter.next().getValue();
			if (ch != ch2 && ch2.isSettingInputEqual(ch))
			{
				ch2.setDemodulator(chan);
			}
		}
		ch.setDemodulator(chan);
	}

	public void updateChannel(String oldName, Channel chan)
	{
		m_channels.remove(oldName);
		checkChannel(chan);
		m_channels.put(chan.getName(), chan);
		m_channelOrder.updateChannel(oldName, chan.getName(), getChannels());
	}

	public HashMap<String, Channel> getChannels()
	{
		return m_channels;
	}

	private void loadChannels()
	{
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			Document doc = docBuilder.parse(DataStore.m_saveFolderData + "Channels.xml");

			boolean bKeepAllOtherPids = false;
			boolean bModifyIncludeVideoAudioSubtitleOnly = true;
			boolean bModifyRealtimeTeletext = true;
			try
			{
				String version = doc.getDocumentElement().getAttributes().getNamedItem("version").getTextContent();
				if ( version.equals("2") || version.equals("3") )
				{
					bKeepAllOtherPids = true;
					bModifyIncludeVideoAudioSubtitleOnly = false;
				}
				if ( version.equals("3") )
				{
					bModifyRealtimeTeletext = false;
				}
			}
			catch(Exception e){}

			NodeList items = doc.getElementsByTagName("channel");

			m_channels = new HashMap<String, Channel>();
			for(int x = 0; x < items.getLength(); x++)
			{
				Node item = items.item(x);
				Channel chan = new Channel(item);
				if (!bKeepAllOtherPids)
				{
					if (chan.getNetworkType().equalsIgnoreCase("ATSC"))
					{
						for(int y = 0; y < chan.getOtherPidSize(); )
						{
							if ( chan.getOtherPid(y) == 0x1ffb )
								chan.deleteOtherPid(y);
							else
								y++;
						}
					}
					else if (chan.getNetworkType().startsWith("DVB"))
					{
						for(int y = 0; y < chan.getOtherPidSize(); )
						{
							if ( (chan.getOtherPid(y) == 16) || (chan.getOtherPid(y) == 17) || (chan.getOtherPid(y) == 18) || (chan.getOtherPid(y) == 20) )
								chan.deleteOtherPid(y);
							else
								y++;
						}
					}
				}
				if (bModifyIncludeVideoAudioSubtitleOnly)
				{
					chan.setIncludeVideoAudioSubtitleOnly(true);
				}
				if (bModifyRealtimeTeletext)
				{
					chan.setTeletextRealTime(true);
				}
				m_channels.put(chan.getName(), chan);
			}

			items = doc.getElementsByTagName("defaultDemodulatorData");
			if (items.getLength() == 1)
				Channel.setDefaultDemodulatorData(items.item(0).getTextContent());

			items = doc.getElementsByTagName("defaultUserData");
			if (items.getLength() == 1)
				Channel.setDefaultUserData(items.item(0).getTextContent());

			items = doc.getElementsByTagName("orders");
			if (items.getLength() == 1)
				m_channelOrder.loadXML(items.item(0), getChannels());
			else
				m_channelOrder.setUse(true);

			m_channelOrder.setDefaultOrdering(getChannels());

			DVScheduler.log("Channels.xml found and loaded (" + items.getLength() + ")");
		}
		catch (Exception e)
		{
			m_channels = new HashMap<String, Channel>();
			DVScheduler.log("Error loading Channels.xml, starting with blank channel list.");
			e.printStackTrace();
		}
	}

	public void saveChannels(StringBuffer output) throws Exception
	{
		GuideStore guide = GuideStore.getInstance();
		guide.setInternalEpgChannelMapping();
		guide.setInternalEpgScheduleChannelChange();

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "channels", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("version", "3");

		String[] keys = (String[]) m_channels.keySet().toArray(new String[0]);

		for (int x = 0; x < keys.length; x++)
		{
			Channel chan = (Channel) m_channels.get(keys[x]);
			Element elm = doc.createElement("channel");
			chan.getXML(doc, elm);
			root.appendChild(elm);
		}

		Element item = doc.createElement("defaultDemodulatorData");
		Text text = doc.createTextNode(Channel.getDefaultDemodulatorData());
		item.appendChild(text);
		root.appendChild(item);
		item = doc.createElement("defaultUserData");
		text = doc.createTextNode(Channel.getDefaultUserData());
		item.appendChild(text);
		root.appendChild(item);

		root.appendChild(m_channelOrder.getXML(doc, getChannels(), ChannelOrder.USE_CUSTOM, null));

		ByteArrayOutputStream buff = new ByteArrayOutputStream();
		TransformerFactory factory = TransformerFactory.newInstance();

		Transformer transformer = factory.newTransformer();
		transformer.setOutputProperty(OutputKeys.METHOD, "xml");
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");

		Source source = new DOMSource(doc);
		Result result = new StreamResult(buff);
		transformer.transform(source, result);

		if(output != null)
		{
			output.append(buff.toString("UTF-8"));
		}
		else
		{
			FileOutputStream fos = new FileOutputStream(DataStore.m_saveFolderData + "channels.xml");
			Writer out = new BufferedWriter(new OutputStreamWriter(fos, "UTF-8"));
			out.write(buff.toString("UTF-8"));
			out.close();

			// For DVSDVBGuideBatch. Since DV Scheduler version 5.63, installer has option to save data to App Data folder.
			try {
				Files.copy(Paths.get(DataStore.m_saveFolderData + "channels.xml"), Paths.get("data" + File.separator + "channels.xml"), StandardCopyOption.REPLACE_EXISTING);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	protected class Channel2 implements Comparable<Channel2>
	{
		public int hashCode()
		{
			return m_setting.hashCode();
		}

		public boolean equals(Object anObject)
		{
			return m_setting.equals(((Channel2)anObject).m_setting);
		}

		@Override
		public int compareTo(Channel2 o)
		{
			return m_setting.compareTo(o.m_setting);
		}

		public Channel2(boolean bEpgScan, String setting, String demodName)
		{
			m_bEpgScan = bEpgScan;
			m_setting = setting;
			m_demodName = demodName;
		}

		boolean m_bEpgScan;
		String m_setting, m_demodName;
	}

	public void getDemodulatorsXml(Document doc, Element root)
	{
		Set<Channel2> demods = new TreeSet<Channel2>();
		Iterator<Channel> iter = m_channels.values().iterator();
		while (iter.hasNext())
		{
			Channel channel = iter.next();
			demods.add(new Channel2(channel.isEpgScan(), channel.getSetting(), channel.getDemodulatorName()));
		}

		Element node = doc.createElement("allDemodulators");
		Element node2 = doc.createElement("selectedDemodulators");
		Element elm2 = null;
		Element elm3 = null;
		Iterator<Channel2> iter2 = demods.iterator();
		while (iter2.hasNext())
		{
			Channel2 channel2 = iter2.next();
			/*
			String setting = channel2.m_demodName;
			if (setting.isEmpty())
				setting = channel2.m_setting;
			 */
			elm2 = doc.createElement("item");
			elm2.setAttribute("name", channel2.m_setting);
			elm2.setAttribute("value", channel2.m_setting);
			if ( channel2.m_bEpgScan )
			{
				elm2.setAttribute("selected", "1");

				elm3 = doc.createElement("item");
				elm3.setAttribute("name", channel2.m_setting);
				elm3.setAttribute("value", channel2.m_setting);
				node2.appendChild(elm3);
			}
			node.appendChild(elm2);
		}
		root.appendChild(node);
		root.appendChild(node2);
	}

	public void setEpgScanDemodulators(String demods)
	{
		String[] demods2 = demods.trim().split("\n");
		Set<String> demods3 = new HashSet<String>();
		//demods3.addAll(Arrays.asList(demods2));
		for (int x = 0; x < demods2.length; x++)
		{
			String str = demods2[x].trim();
			if (!str.isEmpty())
				demods3.add(str);
		}

		Iterator<Channel> iter = m_channels.values().iterator();
		while (iter.hasNext())
		{
			Channel channel = iter.next();
			if (demods3.contains(channel.getSetting()))
				channel.setEpgScan(true);
			else
				channel.setEpgScan(false);
		}
	}

	public int numberOfChannels()
	{
		return m_channels.size();
	}

	public Channel getCurrentChannel()
	{
		return m_currentChannel;
	}

	public void setCurrentChannel(Channel channel)
	{
		m_currentChannel = channel;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  User methods
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public boolean checkUser(String user, HTTPurl urlData) throws Exception
	{
		String groupName = m_users.get(user);
		if (groupName == null)
		{
			// Simple delay.
			try
			{
				Thread.sleep(2000);
			}
			catch (Exception e) {}
			return false;
		}

		GroupAuth group = m_groups.get(groupName);
		if (group == null)
			return false;

		return group.checkPermission(urlData);
	}

	public int getUserCount()
	{
		return m_users.size();
	}

	public String getKeyStore()
	{
		return m_keyStore.isEmpty() ? serverProp.getProperty("path.data", "data") + File.separator + "dvs.keystore" : m_keyStore;
	}

	public String getKeyStorePassword()
	{
		return m_keyStorePassword;
	}

	public String getKeyPassword()
	{
		return m_keyPassword;
	}

	public int getWebGuiProtocol()
	{
		return m_webGuiProtocol;
	}

	public int getHttpsPort()
	{
		return m_httpsPort;
	}

	public boolean isTrustedIp(String ip, HTTPurl urlData) throws Exception
	{
		String groupName = m_trusted.get(ip);
		if ( (groupName == null) && !m_trustedRange1.isEmpty() )
		{
			ip = formatIP(ip);
			// TODO: divide and conquer.
			for (int i=0; i<m_trustedRange1.size(); i++)
			{
				if ( (ip.compareTo(m_trustedRange1.get(i)) >= 0) && (ip.compareTo(m_trustedRange2.get(i)) <= 0) )
				{
					groupName = m_trustedRangeGroupName.get(i);
					break;
				}
			}
		}

		if (groupName == null)
			return false;

		GroupAuth group = m_groups.get(groupName);
		if (group == null)
			return false;

		return group.checkPermission(urlData);
	}

	public boolean isSecurity()
	{
		return ((getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) && !getProperty("https.custom.rule", "0").equals("1")) ? false : m_bSecurity;
	}

	protected String formatIP(String ip)
	{
		String formattedIP = "";
		String temp = "";
		if (ip.contains("."))
		{
			for (int i=0; i<ip.length(); i++)
			{
				if (ip.charAt(i) == '.')
				{
					for (int j=3; j>temp.length(); j--)
						formattedIP += "0";

					formattedIP += temp + ".";
					temp = "";
				}
				else
					temp += ip.charAt(i);
			}

			for (int j=3; j>temp.length(); j--)
				formattedIP += "0";

			formattedIP += temp;
			temp = "";

			return formattedIP;
		}
		else if (ip.contains(":"))
		{
			for (int i=0; i<ip.length(); i++)
			{
				if (ip.charAt(i) == ':')
				{
					for (int j=4; j>temp.length(); j--)
						formattedIP += "0";

					formattedIP += temp + ":";
					temp = "";
				}
				else
					temp += ip.charAt(i);
			}

			for (int j=4; j>temp.length(); j--)
				formattedIP += "0";

			formattedIP += temp;
			temp = "";

			return formattedIP;
		}
		else return ip;
	}

	private void loadUsers()
	{
		// Create a default group called main.
		m_groups.put("main", new GroupAuth());

		try
		{
			FileReader fr = new FileReader(serverProp.getProperty("path.data", "data") + File.separator + "users.list");
			BufferedReader br = new BufferedReader(fr);

			while (true)
			{
				String line = br.readLine();
				if (line == null)
					break;
				else
					line = line.trim();

				if (line.startsWith("group="))
				{
					String[] inputs = line.split(":");
					if (inputs.length != 2)
					{
						DVScheduler.log("Error with users.list data: " + line);
						continue;
					}

					String groupName = inputs[0].substring("group=".length(), inputs[0].length()).trim();
					GroupAuth group = m_groups.get(groupName);
					if (group == null)
						group = new GroupAuth();
					if ( inputs[1].startsWith("allowed") )
						group.m_bAllowed = true;
					else if ( inputs[1].startsWith("notallowed") )
						group.m_bAllowed = false;
					else
					{
						DVScheduler.log("Error with users.list allowed/notallowed keyword: " + line);
						continue;
					}
					inputs[1] = inputs[1].replace("<settings>", "EpgDataRes,16,17;SystemDataRes,02,03,08,09,10,12,17,18,22,23");
					String[] servlets = inputs[1].substring(inputs[1].indexOf('=')+1, inputs[1].length()).split(";");
					for (int i=0; i<servlets.length; i++)
					{
						if (servlets[i].trim().isEmpty())
							continue;
						String[] servletActions = servlets[i].split(",");
						String servletName = Formatter.decodeURL_UTF8E(servletActions[0].trim());
						if (servletName.equals("*"))
						{
							if (servletActions.length == 1)
								group.m_bAllFiles = true;
							else
								group.m_bAllServlets = true;
						}
						else
						{
							Set<String> actions = group.m_servlets.get(servletName);
							if (actions == null)
								actions = new HashSet<String>();
							for (int j=1; j<servletActions.length; j++)
							{
								String action = servletActions[j].trim();
								actions.add(action);
							}
							group.m_servlets.put(servletName, actions);
						}
					}

					m_groups.put(groupName, group);
				}

				else if (line.startsWith("user="))
					m_users.put( line.substring("user=".length(), line.length()), "main" );

				else if (line.startsWith("user:"))
				{
					String[] inputs = line.split("=");
					if (inputs.length != 2)
					{
						DVScheduler.log("Error with users.list data: " + line);
						continue;
					}

					m_users.put( inputs[1], inputs[0].substring("user:".length(), inputs[0].length()).trim() );
				}

				else if (line.startsWith("trusted="))
				{
					String ip = line.substring("trusted=".length(), line.length()).trim();
					if (ip.equalsIgnoreCase("localhost") || ip.equals("127.0.0.1") || ip.equals("0:0:0:0:0:0:0:1") )
					{
						m_trusted.put("127.0.0.1", "main");
						m_trusted.put("0:0:0:0:0:0:0:1", "main");
					}
					else if (ip.contains("-"))
					{
						String[] ips = ip.split("-");
						if (ips.length == 2)
						{
							m_trustedRange1.add(formatIP(ips[0].trim()));
							m_trustedRange2.add(formatIP(ips[1].trim()));
							m_trustedRangeGroupName.add("main");
						}
					}
					else if (!ip.isEmpty())
						m_trusted.put(ip, "main");
				}

				else if (line.startsWith("trusted:"))
				{
					String[] inputs = line.split("=");
					if (inputs.length != 2)
					{
						DVScheduler.log("Error with users.list data: " + line);
						continue;
					}

					String ip = inputs[1].trim();
					String groupName = inputs[0].substring("trusted:".length(), inputs[0].length()).trim();
					if (ip.equalsIgnoreCase("localhost") || ip.equals("127.0.0.1") || ip.equals("0:0:0:0:0:0:0:1") )
					{
						m_trusted.put("127.0.0.1", groupName);
						m_trusted.put("0:0:0:0:0:0:0:1", groupName);
					}
					else if (ip.contains("-"))
					{
						String[] ips = ip.split("-");
						if (ips.length == 2)
						{
							m_trustedRange1.add(formatIP(ips[0].trim()));
							m_trustedRange2.add(formatIP(ips[1].trim()));
							m_trustedRangeGroupName.add(groupName);
						}
					}
					else if (!ip.isEmpty())
						m_trusted.put(ip, groupName);
				}

				else if (line.equals("No Security"))
					m_bSecurity = false;

				// Web GUI protocol.
				else if (line.startsWith("protocol="))
				{
					String protocol = line.substring("protocol=".length(), line.length()).trim();
					if (protocol.equalsIgnoreCase("HTTPS"))
						m_webGuiProtocol = WEBGUI_HTTPS;
				}

				else if (line.startsWith("httpsport="))
				{
					try
					{
						m_httpsPort = Integer.parseInt( line.substring("httpsport=".length(), line.length()).trim() );
					}
					catch (Exception e)
					{
						DVScheduler.log("Invalid HTTPS port");
					}

					if ( (m_httpsPort <= 0) || (m_httpsPort > 65535) )
						m_httpsPort = 443;
				}

				else if (line.startsWith("jks="))
				{
					m_keyStore = line.substring("jks=".length(), line.length());
				}

				else if (line.startsWith("keystorepassword="))
				{
					m_keyStorePassword = line.substring("keystorepassword=".length(), line.length());
				}

				else if (line.startsWith("keypassword="))
				{
					m_keyPassword = line.substring("keypassword=".length(), line.length());
				}

				else if (line.startsWith("servefile="))
				{
					String path = line.substring("servefile=".length(), line.length()).trim();
					if ( path.isEmpty() )
						path = "temp";
					m_serveFilePaths.add(path);
				}
				else if (line.equals("NoFileDownload"))
					FileUploader.setDisabled(true);
				else if (line.equals("NoUnsecureDownload"))
					m_bAllowUnsecureDownload = false;
			}

			br.close();
			fr.close();

			DVScheduler.log("Loaded " + m_users.size() + " Users, " + m_trusted.size() + " trusted IPs, " + m_trustedRange1.size() + " trusted IP ranges");

			// Print out groups
			Iterator<Entry<String, GroupAuth>> iter = m_groups.entrySet().iterator();
			String text = "Group definitions (" + m_groups.size() + " total) :";
			while (iter.hasNext())
			{
				Entry<String, GroupAuth> entry = iter.next();
				GroupAuth group = entry.getValue();
				text += "\nGroup: " + entry.getKey() + (group.m_bAllowed ? " (using allowed list)" : " (using not-allowed list)") + ", AllFiles: " + group.m_bAllFiles + ", AllServlets: " + group.m_bAllServlets;

				Iterator<Entry<String, Set<String>>> iter2 = group.m_servlets.entrySet().iterator();
				while (iter2.hasNext())
				{
					Entry<String, Set<String>> entry2 = iter2.next();
					Set<String> actions = entry2.getValue();
					text += "\n  " + entry2.getKey();
					Iterator<String> iter3 = actions.iterator();
					while (iter3.hasNext())
					{
						text += "," + iter3.next();
					}
				}
			}
			DVScheduler.log(text);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public int getGroupSize()
	{
		return m_groups.size();
	}

	public int getTrustedIpSize()
	{
		return m_trusted.size();
	}

	public int getTrustedIpRangeSize()
	{
		return m_trustedRange1.size();
	}

	public int numberUsers()
	{
		return m_users.size();
	}

	public String[] getServeFilePaths()
	{
		if (m_serveFilePaths.isEmpty())
		{
			return DVScheduler.getArray(getProperty("Capture.path", "capture"));
		}
		else
			return m_serveFilePaths.toArray(new String[0]);
	}

	private void loadNetworkTypes()
	{
		// Add built-in types.
		m_networkTypes.add("ATSC");
		m_networkTypes.add("DVB-C");
		m_networkTypes.add("DVB-S");
		m_networkTypes.add("DVB-T");
		m_networkTypes.add("QAM");

		HashSet<String> types = new HashSet<String>();
		types.addAll(m_networkTypes);

		// Add user types.
		try
		{
			FileReader fr = new FileReader(serverProp.getProperty("path.data", "data") + File.separator + "types.list");
			BufferedReader br = new BufferedReader(fr);

			String line = br.readLine();
			while (line != null)
			{
				line = line.trim();

				if ( !line.startsWith("//") && !line.startsWith("#") && !line.isEmpty() )
				{
					/*
					// Check for invalid file names.
					boolean bOk = true; 
					for (int y = 0; y < line.length(); y++)
					{
						char charAt = line.charAt(y);

						if (!((charAt >= 'a' && charAt <= 'z') || (charAt >= 'A' && charAt <= 'Z') 
								|| (charAt >= '0' && charAt <= '9') 
								|| charAt == '-' || charAt == '_' || charAt == ' '))
						{
							bOk = false;
							break;
						}
					}
					if (bOk)
						m_networkTypes.add(line);
					 */

					if (Formatter.isOk(line) && types.add(line))
						m_networkTypes.add(line);
				}

				line = br.readLine();
			}

			br.close();
			fr.close();

			DVScheduler.log("Loaded " + m_networkTypes.size() + " network types");
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public String getNetworkType(int index)
	{
		return m_networkTypes.get(index);
	}

	public int getNetworkTypeSize()
	{
		return m_networkTypes.size();
	}

	public void getNetworkTypeXml(Document doc, Element node, boolean bAllowMultiple)
	{
		Element elm2 = null;
		for (int i=0; i<m_networkTypes.size(); i++)
		{
			String networkType = m_networkTypes.get(i);
			if (!bAllowMultiple && (networkType.indexOf(',') >= 0))
				continue;

			elm2 = doc.createElement("item");
			elm2.setAttribute("name", networkType);
			node.appendChild(elm2);
		}
	}

	public boolean isNetworkTypeValid(String networkType)
	{
		if (networkType == null || networkType.isEmpty())
			return false;

		for (int x = 0; x < m_networkTypes.size(); x++)
		{
			if (m_networkTypes.get(x).equals(networkType))
				return true;
		}

		return false;
	}

	public void setProperty(String pKey, String value) throws Exception
	{
		//serverProp.setProperty(pKey, value.trim());
		serverProp.setProperty(pKey, value);
		serverProp.store(new FileOutputStream(m_saveFile), "DV Scheduler Server Properties");
	}

	public void setPropertyInt(String pKey, int value) throws Exception
	{
		serverProp.setProperty(pKey, Integer.toString(value));
		serverProp.store(new FileOutputStream(m_saveFile), "DV Scheduler Server Properties");
	}

	public void setPropertyLong(String pKey, long value) throws Exception
	{
		serverProp.setProperty(pKey, Long.toString(value));
		serverProp.store(new FileOutputStream(m_saveFile), "DV Scheduler Server Properties");
	}

	public String getProperty(String pKey, String def)
	{
		String prop = serverProp.getProperty(pKey, null);
		if ( (prop != null) && (prop != "*UNDEFINED*") )
			return prop;

		try
		{
			setProperty(pKey, def);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return def;
	}

	public int getPropertyInt(String pKey, int def)
	{
		try
		{
			def = Integer.parseInt(getProperty(pKey, Integer.toString(def)));
		}
		catch(Exception e){}

		return def;
	}

	public long getPropertyLong(String pKey, long def)
	{
		try
		{
			def = Long.parseLong(getProperty(pKey, Long.toString(def)));
		}
		catch(Exception e){}

		return def;
	}

	public Properties getProperties()
	{
		return serverProp;
	}

	public int saveSchedules(ScheduleItem item)
	{
		if ( (item != null) && item.isLive() )
			return 0;

		String fileNameOut = DataStore.m_saveFolderData + "times" + m_rand.nextLong() + ".sof";
		try
		{
			FileOutputStream fos = new FileOutputStream(fileNameOut);
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(m_schedules);
			oos.close();
			refreshWakeupTime();
			//SetToDo(true);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			return -1;
		}

		String fileNameIn = DataStore.m_saveFolderData + "times.sof";
		File fileIn = new File(fileNameIn);
		if (fileIn.exists() && fileIn.isFile())
		{
			if ( !fileIn.delete() )
				DVScheduler.log("Cannot delete " + fileNameIn);
		}
		File fileOut = new File(fileNameOut);
		if ( !fileOut.renameTo(new File(fileNameIn)) )
		{
			DVScheduler.log("Cannot rename " + fileNameOut + " to " + fileNameIn);
			return -1;
		}
		return 0;
	}

	private void convertSavedSchedules()
	{
		String fileNameIn = DataStore.m_saveFolderData + "times.sof";
		File fileIn = new File(fileNameIn);
		if ( !fileIn.exists() || !fileIn.isFile() || fileIn.length() < 400)
		{
			DVScheduler.log("No conversion of times.sof can be done.");
			return;
		}

		int read = 0;
		byte[] block = new byte[4096];
		byte[] className = "ScheduleItem".getBytes();
		FileInputStream fis = null;
		boolean bFound = false;
		int position = -1;
		try
		{
			fis = new FileInputStream(fileNameIn);
			read = fis.read(block);
			if (read < 400)
			{
				DVScheduler.log("times.sof read too short, cannot be converted.");
				fis.close();
				return;
			}
			for (int i=0; i<200; i++)
			{
				bFound = true;
				for (int j=0; j<className.length; j++)
				{
					if (block[i+j] != className[j])
						bFound = false;
				}

				if (bFound)
				{
					position = i + className.length;
					break;
				}
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			try
			{
				fis.close();
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
			return;
		}

		if (position < 0)
		{
			DVScheduler.log("Position not found in times.sof, cannot be converted.");
			try
			{
				fis.close();
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
			return;
		}

		long uid = ScheduleItem.getSerialVersionUid();
		for (int i=0; i<8; i++)
		{
			block[position + 7 - i] = (byte)(uid >> 8*i);
		}
		String fileNameOut = DataStore.m_saveFolderData + "times" + m_rand.nextLong() + ".sof";
		FileOutputStream fos = null;
		try
		{
			fos = new FileOutputStream(fileNameOut);
			while (read > -1)
			{
				fos.write(block, 0, read);
				read = fis.read(block);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			try
			{
				fis.close();
				fos.close();
				File fileOut = new File(fileNameOut);
				if (fileOut.exists())
					fileOut.delete();
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
			return;
		}

		try
		{
			fis.close();
		}
		catch (Exception e2)
		{
			e2.printStackTrace();
		}
		try
		{
			fos.close();
		}
		catch (Exception e2)
		{
			e2.printStackTrace();
		}

		SimpleDateFormat fm = new SimpleDateFormat("yyyy-MM-dd_(HH.mm.ss.SSS)");
		String fileNameBak = DataStore.m_saveFolderData + "times" + fm.format(Calendar.getInstance().getTime()) + ".sof";
		if ( !fileIn.renameTo(new File(fileNameBak)) )
			DVScheduler.log("Cannot rename times.sof");
		File fileOut = new File(fileNameOut);
		if ( !fileOut.renameTo(new File(fileNameIn)) )
			DVScheduler.log("Cannot rename " + fileNameOut + " to times.sof");
		DVScheduler.log("times.sof converted. Old times.sof was renamed to " + fileNameBak);
	}

	@SuppressWarnings("unchecked")
	private int loadSchedules(boolean bRetry)
	{
		ObjectInputStream ois = null;
		String timesFile = DataStore.m_saveFolderData + "times.sof";
		try
		{
			FileInputStream fis = new FileInputStream(timesFile);
			ois = new ObjectInputStream(fis);
			m_schedules = (HashMap<String, ScheduleItem>) ois.readObject();
			ois.close();

			String[] keys = m_schedules.keySet().toArray(new String[0]);
			for (int x = 0; x < keys.length; x++)
			{
				String id = keys[x];
				if ( m_schedules.get(id).isLive() )
					m_schedules.remove(id);
			}

			DVScheduler.log(timesFile + " found and loaded (" + m_schedules.size() + ")");
			refreshWakeupTime();
		}
		catch (Exception e)
		{
			try
			{
				if (ois != null)
					ois.close();
			}
			catch (Exception e2)
			{
				DVScheduler.log("Error with " + timesFile);
				e2.printStackTrace();
			}
			if ( bRetry && (e.getClass() == InvalidClassException.class) )
			{
				DVScheduler.log("Different version of times.sof detected, trying to convert");
				convertSavedSchedules();
				DVScheduler.log("End conversion");
				return loadSchedules(false);
			}
			else
			{
				m_schedules = new HashMap<String, ScheduleItem>();
				DVScheduler.log("Error loading times.sof, starting with blank schedule");
				e.printStackTrace();
				return -1;
			}
		}

		return 0;
	}

	public int refreshWakeupTime()
	{
		m_nextScheduleTime = getClosestStart();

		int allow = 0;
		try
		{
			allow = Integer.parseInt(getProperty("Schedule.wake.system", "45").trim());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		long wakeUpTime = allow == 0 ? Long.MAX_VALUE : m_nextScheduleTime;

		//
		// Now get next Sch Data Load event
		//
		String scheduleOptions = getProperty("guide.source.schedule", "0:0:0:1:10:1:24");
		String[] schOptsArray = scheduleOptions.split(":");
		boolean bWakeSystem = ((allow == 0) || ((schOptsArray.length >= 6) && schOptsArray[5].equals("0"))) ? false : true;
		boolean bEnabled = (schOptsArray.length >= 3) && "1".equals(schOptsArray[0]);

		if (bEnabled)
		{
			int hour = 0;
			try
			{
				hour = Integer.parseInt(schOptsArray[1]);
			}
			catch (Exception e)
			{
			}
			int minute = 0;
			try
			{
				minute = Integer.parseInt(schOptsArray[2]);
			}
			catch (Exception e)
			{
			}
			int epgLoadPeriod = 24;	// default is every 24 hours.
			if (schOptsArray.length >= 7)
				epgLoadPeriod = Integer.parseInt(schOptsArray[6]);
			if (epgLoadPeriod <= 0)
				epgLoadPeriod = 24;

			Calendar guideSch = Calendar.getInstance();
			if ( (m_prevEpgLoadHour < 0) && (getPropertyInt("guide.source.scheduleMinuteAfterLaunch", -1) > 0) )
			{
				guideSch.add(Calendar.MINUTE, getPropertyInt("guide.source.scheduleMinuteAfterLaunch", -1));
				m_nextEpgLoadTime = guideSch.getTimeInMillis();
				m_prevEpgLoadHour = hour;
				m_prevEpgLoadMinute = minute;
				m_bPrevEpgLoadEnabled = true;
			}
			else if (hour != m_prevEpgLoadHour || minute != m_prevEpgLoadMinute || !m_bPrevEpgLoadEnabled)
			{
				m_prevEpgLoadHour = hour;
				m_prevEpgLoadMinute = minute;
				m_bPrevEpgLoadEnabled = true;

				guideSch.set(Calendar.HOUR_OF_DAY, hour);
				guideSch.set(Calendar.MINUTE, minute);
				guideSch.set(Calendar.MILLISECOND, 0);
				guideSch.set(Calendar.SECOND, 0);

				while (guideSch.before(Calendar.getInstance()))
				{
					guideSch.add(Calendar.HOUR, epgLoadPeriod);
				}
				m_nextEpgLoadTime = guideSch.getTimeInMillis();
			}
			else if (bWakeSystem)
				guideSch.setTimeInMillis(m_nextEpgLoadTime);

			if (bWakeSystem)
			{
				if (guideSch.before(Calendar.getInstance()))
					guideSch.add(Calendar.DATE, 1);

				m_nextEpgLoadWakeUpTime = guideSch.getTimeInMillis();

				if (m_nextEpgLoadWakeUpTime < wakeUpTime)
					wakeUpTime = m_nextEpgLoadWakeUpTime;
			}
			else
				m_nextEpgLoadWakeUpTime = Long.MAX_VALUE;
		}
		else
		{
			m_bPrevEpgLoadEnabled = false;
			m_nextEpgLoadTime = Long.MAX_VALUE;
			m_nextEpgLoadWakeUpTime = Long.MAX_VALUE;
		}

		DllWrapper capEng = new DllWrapper();

		if ( (wakeUpTime == Long.MAX_VALUE) && (m_nWakeUpTimerStatus != 0) )
		{
			m_wakeUpEventTime = wakeUpTime;
			m_wakeUpTime = wakeUpTime;
			DVScheduler.log("About to cancel wakeup timer:");

			int result = capEng.setWakeUpTime(0, 0, 0, 0, 0, 0);

			if(result != 0)
				DVScheduler.log("CancelWaitableTimer FAILED! (" + result + ")");
			else
			{
				m_nWakeUpTimerStatus = 0;
				DVScheduler.log("CancelWaitableTimer Succeeded (" + result + ")");
			}

			return result;
		}
		else if ( wakeUpTime != Long.MAX_VALUE )
		{
			m_wakeUpEventTime = wakeUpTime;
			wakeUpTime -= allow * 1000;
			if ( (m_wakeUpTime != wakeUpTime) || (m_nWakeUpTimerStatus != 1) )
			{
				m_wakeUpTime = wakeUpTime;
				Calendar wakeAt = Calendar.getInstance();
				wakeAt.setTimeInMillis(m_wakeUpTime);

				int sec = wakeAt.get(Calendar.SECOND);
				int min = wakeAt.get(Calendar.MINUTE);
				int hour = wakeAt.get(Calendar.HOUR_OF_DAY);
				int day = wakeAt.get(Calendar.DATE);
				int month = wakeAt.get(Calendar.MONTH) + 1;
				int year = wakeAt.get(Calendar.YEAR);

				DVScheduler.log("About to set wakeup time " + day + "/" + month + "/" + year + " " + hour + ":" + min + ":" + sec + " -" + allow + ":");

				int result = capEng.setWakeUpTime(year, month, day, hour, min, sec);

				if(result != 0)
					DVScheduler.log("SetWaitableTimer FAILED! (" + result + ")");
				else
				{
					m_nWakeUpTimerStatus = 1;
					DVScheduler.log("SetWaitableTimer Succeeded (" + result + ")");
				}

				return result;
			}
		}

		return 0;
	}

	private long getClosestStart()
	{
		HashMap<String, ScheduleItem> times = getScheduleList();
		Iterator<ScheduleItem> iter = times.values().iterator();
		long nearest = Long.MAX_VALUE;
		long now = new Date().getTime();
		while (iter.hasNext())
		{
			ScheduleItem item = iter.next();
			if (!item.isWakeSystem())
				continue;

			long start = item.getStart().getTime();
			if ( (start > now) && (start < nearest) )
				nearest = start;
		}
		return nearest;
	}

	public void checkUserKeywords()
	{
		String userKeywords = getProperty("user.keywords", "$$vlcexe->C:\\Program Files\\VideoLAN\\VLC\\vlc.exe");
		String[] userKeywords2 = DVScheduler.getArray(userKeywords);

		// Semi built-in keywords.
		String[] builtinKeyword = {"$$vlcexe"};
		String[] builtinValue = {"C:\\Program Files\\VideoLAN\\VLC\\vlc.exe"};
		boolean bSave = false;
		for(int x = 0; x < builtinKeyword.length; x++)
		{
			boolean bFound = false;
			for(int y = 0; y < userKeywords2.length; y++)
			{
				if (userKeywords2[y].startsWith(builtinKeyword[x] + "->"))
				{
					bFound = true;
					break;
				}
			}

			if (!bFound)
			{
				bSave = true;
				if (!userKeywords.isEmpty())
					userKeywords += "\n";
				userKeywords += builtinKeyword[x] + "->" + builtinValue[x];
			}
		}

		if (bSave)
		{
			try
			{
				setProperty("user.keywords", userKeywords);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public String replaceKeywords(String text)
	{
		// Built-in keywords.
		text = text.replace("$captureenginepath", getProperty("Capture.engine.path", "util"));
		text = text.replace("$logpath", m_saveFolder + "log");

		Calendar cal = Calendar.getInstance();
		text = text.replace("$nowyear", Integer.toString(cal.get(Calendar.YEAR)));
		String month = Integer.toString(cal.get(Calendar.MONTH) + 1);
		text = text.replace("$nowmonth2", (month.length() < 2 ? "0" + month : month));
		text = text.replace("$nowmonth", month);
		String day = Integer.toString(cal.get(Calendar.DAY_OF_MONTH));
		text = text.replace("$nowday2", (day.length() < 2 ? "0" + day : day));
		text = text.replace("$nowday", day);
		String calValue = Integer.toString(cal.get(Calendar.HOUR_OF_DAY));
		text = text.replace("$nowhour2", (calValue.length() < 2 ? "0" + calValue : calValue));
		text = text.replace("$nowhour", calValue);
		calValue = Integer.toString(cal.get(Calendar.MINUTE));
		text = text.replace("$nowminute2", (calValue.length() < 2 ? "0" + calValue : calValue));
		text = text.replace("$nowminute", calValue);
		calValue = Integer.toString(cal.get(Calendar.SECOND));
		text = text.replace("$nowsecond2", (calValue.length() < 2 ? "0" + calValue : calValue));
		text = text.replace("$nowsecond", calValue);
		calValue = Integer.toString(cal.get(Calendar.MILLISECOND));
		text = text.replace("$nowmillisecond3", (calValue.length() < 2 ? "00" + calValue : (calValue.length() < 3 ? "0" + calValue : calValue)));
		text = text.replace("$nowmillisecond", calValue);

		text = text.replace("$serverprotocol", (getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) ? "https" : "https");
		text = text.replace("$serverport", Integer.toString((getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) ? getHttpsPort() : DVScheduler.getHttpPort()));
		text = text.replace("$serverhttpsport", Integer.toString(getHttpsPort()));
		text = text.replace("$serverhttpport", Integer.toString(DVScheduler.getHttpPort()));

		text = text.replace("$epgmaxlang", getProperty("guide.language.max", ""));
		text = text.replace("$epglang", getProperty("guide.language.priority", ""));

		checkUserKeywords();
		String userKeywords = getProperty("user.keywords", "$$vlcexe->C:\\Program Files\\VideoLAN\\VLC\\vlc.exe");
		String[] userKeywords2 = DVScheduler.getArray(userKeywords);

		for(int x = 0; x < userKeywords2.length; x++)
		{
			int pos = userKeywords2[x].indexOf("->");
			if (pos >= 0)
			{
				String key = userKeywords2[x].substring(0, pos);
				String value = userKeywords2[x].substring(pos + 2);
				text = text.replace(key, value);
			}
		}

		return text;
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  Auto Del methods
	//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	public HashMap<String, KeepForDetails> getAutoDelList()
	{
		return m_autoDelList;
	}

	public void saveAutoDelList()
	{
		try
		{
			FileOutputStream fos = new FileOutputStream(DataStore.m_saveFolderData + "AutoDel.sof");
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(m_autoDelList);
			oos.close();
			DVScheduler.log("AutoDel.sof saved.");
		}
		catch (Exception e)
		{
			DVScheduler.log("Error loading AutoDel.sof, will start with no Auto Delete list.");
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	private void loadAutoDelList()
	{
		try
		{
			FileInputStream fis = new FileInputStream(DataStore.m_saveFolderData + "AutoDel.sof");
			ObjectInputStream ois = new ObjectInputStream(fis);
			m_autoDelList = (HashMap<String, KeepForDetails>) ois.readObject();
			ois.close();
			DVScheduler.log("AutoDel.sof loaded");
		}
		catch (Exception e)
		{
			m_autoDelList = new HashMap<String, KeepForDetails>();
			DVScheduler.log("Error loading AutoDel.sof, starting with empty list.");
			e.printStackTrace();
		}
	}

	public void addAutoDeleteItem(String fileName, int keepForMinutes)
	{
		if ( (fileName == null) || (fileName.isEmpty()) )
			return;

		try
		{
			fileName = new File(fileName).getCanonicalPath();
		}
		catch (Exception e) {}
		String[] key = (String[]) m_autoDelList.keySet().toArray(new String[0]);
		for (int x = 0; x < key.length; x++)
		{
			KeepForDetails item = (KeepForDetails)m_autoDelList.get(key[x]);
			if (fileName.equals(item.getFileName()))
				m_autoDelList.remove(key[x]);
		}

		KeepForDetails kpd = new KeepForDetails(fileName, keepForMinutes);
		String key02 = new Long(new Date().getTime()).toString() + "-" + new Long(m_rand.nextLong()).toString();
		m_autoDelList.put(key02, kpd);
		//saveAutoDelList();
	}

	public KeepForDetails removeAutoDeleteItem(String id)
	{
		return m_autoDelList.remove(id);
		//saveAutoDelList();
	}

	public void resetAutoDelLog()
	{
		m_autoDelLog = new StringBuffer(2048);
	}

	public void autoDelLogAdd(String log)
	{
		int maxLine = getPropertyInt("log.autodelete.maxline", 100);
		if (maxLine < 1)
			maxLine = 1;
		try
		{
			if (m_autoDelLogCount >= maxLine)
				m_autoDelLog.delete(0, m_autoDelLog.indexOf("<br>\n") + 5);
			else
				m_autoDelLogCount++;

			m_autoDelLog.append(log + "<br>\n");
		}
		catch(Exception e){}
	}

	public String getAutoDelLog()
	{
		return m_autoDelLog.toString();
	}

	public boolean isAutoDelByDefault()
	{
		return getProperty("AutoDel.SetByDefault", "No").equals("Yes");
	}

	public boolean isAutoDelOverride()
	{
		return getProperty("AutoDel.Override", "0").equals("1");
	}

	public String getVersion()
	{
		return version;
	}

	public static String intToStr(int num)
	{
		if (num > -1 && num < 10)
			return ("0" + (new Integer(num).toString()));
		else
			return (new Integer(num).toString());
	}

	public String[] getNamePatterns()
	{
		String patterns = getProperty("Capture.filename.patterns", "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d");
		String[] line = patterns.split("\n");
		for (int x = 0; x < line.length; x++)
			line[x] = line[x].trim();
		return line;
	}

	public void getNamePatternsXml(Document doc, Element node)
	{
		String patterns = getProperty("Capture.filename.patterns", "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d");
		String[] line = patterns.split("\n");

		Element elm2 = null;
		for (int i=0; i<line.length; i++)
		{
			line[i] = line[i].trim();
			elm2 = doc.createElement("item");
			elm2.setAttribute("name", line[i]);
			elm2.setAttribute("value", line[i]);
			node.appendChild(elm2);
		}
	}

	public String getTemplateFile(String userAgent, String fileName)
	{
		if ( (userAgent == null) || userAgent.isEmpty() )
			return getProperty("path.template", "data/templates") + File.separator + fileName;

		String path = getProperty("path.httproot", "http") + File.separator + "themes" + File.separator + getThemeDirectory(userAgent) + File.separator + "templates" + File.separator + fileName;
		File fileIn = new File(path);
		if (fileIn.exists() == false)
		{
			return getProperty("path.template", "data/templates") + File.separator + fileName;
		}

		return path;
	}

	// relativePath is assumed to starts with path separator "/".
	public String getHttpFile(String userAgent, String relativePath)
	{
		if ( (userAgent == null) || userAgent.isEmpty() )
			return getProperty("path.httproot", "http") + relativePath;

		String path = getProperty("path.httproot", "http") + "themes" + File.separator + getThemeDirectory(userAgent) + relativePath;
		File fileIn = new File(path);
		if (fileIn.exists() == false)
		{
			return getProperty("path.httproot", "http") + relativePath;
		}

		return path;
	}

	public String getThemeDirectory(String userAgent)
	{
		if ( (userAgent == null) || userAgent.isEmpty() )
			return getProperty("path.theme", "..");

		for (int i=0; i<m_agentThemes.size(); i++)
		{
			String[] array = m_agentThemes.get(i);
			if (userAgent.contains(array[0]))
				return array[1];
		}

		return getProperty("path.theme", "..");
	}

	public boolean isThemeExist(String userAgent)
	{
		for (int i=0; i<m_agentThemes.size(); i++)
		{
			String[] array = m_agentThemes.get(i);
			if (userAgent.contains(array[0]))
				return true;
		}

		return false;
	}

	public void addTheme(String agent, String theme)
	{
		if ( (agent == null) || agent.isEmpty() || (theme == null) || theme.isEmpty() )
			return;

		String[] array = {agent, theme};
		m_agentThemes.add(array);
		saveAgentThemes();
	}

	public void addTheme(int index, String agent, String theme)
	{
		if ( (agent == null) || agent.isEmpty() || (theme == null) || theme.isEmpty() )
			return;

		String[] array = {agent, theme};
		m_agentThemes.add(index, array);
		saveAgentThemes();
	}

	public void deleteTheme(int index)
	{
		if ( (index < 0) || (index >= m_agentThemes.size()) )
			return;

		m_agentThemes.remove(index);
		saveAgentThemes();
	}

	public void deleteThemes(Vector<Integer> indexes)
	{
		if ( (indexes == null) || indexes.isEmpty() )
			return;

		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i) - i;
			if ( (index >= 0) && (index < m_agentThemes.size()) )
			{
				m_agentThemes.remove(index);
			}
		}

		saveAgentThemes();
	}

	public void moveThemesBeforeTheme(int beforeIndex, Vector<Integer> indexes)
	{
		if (beforeIndex < 0)
			beforeIndex = m_agentThemes.size();

		Vector<String[]> maps = new Vector<String[]>();
		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i);
			if ( (index >= 0) && (index < m_agentThemes.size()) )
				maps.add(m_agentThemes.get(index));
		}
		for (int i=0; i<indexes.size(); i++)
		{
			int index = indexes.get(i) - i;
			if ( (index >= 0) && (index < m_agentThemes.size()) )
			{
				m_agentThemes.remove(index);
				if (index < beforeIndex)
					beforeIndex--;
			}
		}

		if ( (beforeIndex >= 0) && (beforeIndex <= m_agentThemes.size()) )
		{
			m_agentThemes.addAll(beforeIndex, maps);
			saveAgentThemes();
		}
	}

	public void saveAgentThemes()
	{
		String text = "";
		for (int i=0; i<m_agentThemes.size(); i++)
		{
			String[] array = m_agentThemes.get(i);

			if (i > 0)
				text += "\n";

			text += array[0] + "\n";
			text += array[1];
		}

		try
		{
			setProperty("path.agentThemes", text);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void loadAgentThemes()
	{
		String[] themes = DVScheduler.getArray(getProperty("path.agentThemes", ""));
		if ( (themes.length % 2) != 0 )
			return;

		for (int i=0; i<themes.length; i+=2)
		{
			if (themes[i+1].equalsIgnoreCase("default"))
				themes[i+1] = "..";
			String[] array = {themes[i], themes[i+1]};
			m_agentThemes.add(array);
		}
	}

	public String getTargetUrl(String id, String def, String userAgent)
	{
		String data = def;
		try
		{
			String httpDir = getProperty("path.httproot", "http");
			String themeDir = getThemeDirectory(userAgent); 

			String path;
			path = httpDir + File.separator + "themes" + File.separator + themeDir + File.separator + "xsl" + File.separator + "target.urls";
			File fileIN = new File(path);
			if(fileIN.exists() == false)
			{
				path = httpDir + File.separator + "xsl" + File.separator + "target.urls";
			}

			Properties prop = new Properties();
			prop.load(new FileInputStream(path));
			data = prop.getProperty(id, def);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return data;
	}

	public boolean isEpgLoading()
	{
		if ( (m_epgLoaderThread != null) && !m_epgLoaderThread.isAlive() )
			m_epgLoaderThread = null;

		return (m_epgLoaderThread != null) || m_bEpgManualLoadRunning;
	}

	public boolean setStreamPorts(boolean bCheckOnly)
	{
		boolean bOk = true;
		String[] streamPorts = getProperty("Stream.port", "50000-55000").split("-");
		int nStreamPortFrom = 50000;
		int nStreamPortTo = 55000;
		if ( streamPorts.length != 2 )
		{
			bOk = false;
			try
			{
				setProperty("Stream.port", "50000-55000");
			}
			catch (Exception e) {}
		}
		else
		{
			try
			{
				nStreamPortFrom = Integer.parseInt(streamPorts[0].trim());
				nStreamPortTo = Integer.parseInt(streamPorts[1].trim());
			}
			catch (Exception e)
			{
				bOk = false;
				nStreamPortFrom = 50000;
				nStreamPortTo = 55000;
				try
				{
					setProperty("Stream.port", "50000-55000");
				}
				catch (Exception e2) {}
			}

			if ( (nStreamPortFrom <= 0) || (nStreamPortTo <= 0) || 
					(nStreamPortFrom > 65535) || (nStreamPortTo > 65535) || 
					(nStreamPortTo == nStreamPortFrom) )
			{
				bOk = false;
				nStreamPortFrom = 50000;
				nStreamPortTo = 55000;
				try
				{
					setProperty("Stream.port", "50000-55000");
				}
				catch (Exception e) {}
			}
			else if ( nStreamPortTo < nStreamPortFrom )
			{
				int nTemp = nStreamPortTo;
				nStreamPortTo = nStreamPortFrom;
				nStreamPortFrom = nTemp;
				try
				{
					setProperty("Stream.port", Integer.toString(nStreamPortFrom) + "-" + Integer.toString(nStreamPortTo));
				}
				catch (Exception e) {}
			}
		}

		if (bCheckOnly)
			return bOk;

		m_streamPort = nStreamPortFrom;
		m_availableStreamPorts.clear();

		for (int i=nStreamPortFrom; i<=nStreamPortTo; i++)
		{
			m_availableStreamPorts.add(i);
		}

		return bOk;
	}

	public int getStreamPort()
	{
		if ( m_availableStreamPorts.isEmpty() )
			return m_streamPort;
		else
		{
			int streamPort = m_availableStreamPorts.get(0);
			m_availableStreamPorts.remove(0);
			return streamPort;
		}
	}

	public void returnStreamPort(int streamPort)
	{
		if (streamPort > 0 && streamPort <= 65535)
			m_availableStreamPorts.add(streamPort);
	}

	public void addCaptureTime(long time) throws Exception
	{
		m_sessionCapture++;
		m_sessionCaptureTime += time;

		setPropertyLong("capture.total", ++m_totalCapture);
		m_totalCaptureTime += time;
		setPropertyLong("capture.totalTime", m_totalCaptureTime);
	}

	public long getSessionCapture()
	{
		return m_sessionCapture;
	}

	public long getSessionCaptureTime()
	{
		return m_sessionCaptureTime;
	}

	public long getTotalCapture()
	{
		return m_totalCapture;
	}

	public long getTotalCaptureTime()
	{
		return m_totalCaptureTime;
	}

	public boolean isAllowUnsecureDownload()
	{
		return m_bAllowUnsecureDownload && getProperty("download.unsecure", "No").equals("Yes");
	}

	protected int GetDeviceCommand(StringBuffer command2, CaptureEngine captureEngine)
	{
		String fullPath = getProperty("Capture.engine.path", "util") + File.separator + "scan.ce";
		if ( !captureEngine.SetEngine("Generic", fullPath) )
		{
			DVScheduler.log("Error reading scan engine file (" + fullPath + ")");

			return -1;
		}

		String command = captureEngine.getDeviceListCommand();
		command = captureEngine.replaceKeywords(command);	// Has to be first.
		command = replaceKeywords(command);
		command2.append(command);

		return captureEngine.getdeviceIndexBase();
	}

	public Vector<CaptureDevice> getTunerList(boolean bRefresh) throws Exception
	{
		if ( (m_tuners == null) || bRefresh )
		{
			StringBuffer scanCommandBuff = new StringBuffer();
			CaptureEngine captureEngine = new CaptureEngine();
			int indexBase = GetDeviceCommand(scanCommandBuff, captureEngine);
			if (indexBase < 0)
			{
				//throw new Exception("Scan command not found");
				DVScheduler.log("Scan command not found in capture engine path");
				return null;
			}
			else
			{
				DVScheduler.log("Running device scan command: " + scanCommandBuff);
				Runtime runner = Runtime.getRuntime();
				String scanCommand = Formatter.formatCommand(scanCommandBuff.toString());
				Process scan = runner.exec(Formatter.parseCommand(scanCommand));

				TunerScanResult tuners = new TunerScanResult();
				tuners.readInput(scan.getInputStream());
				tuners.parseXML(indexBase);

				m_tuners = tuners.getResult();
				scan.waitFor();
			}
		}
		return m_tuners;
	}
}
