/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

public class EmailSender implements Runnable
{
	private String m_subject = "";
	private String m_body = "";
	private boolean m_bFinished = false;

	public EmailSender()
	{
	}

	public EmailSender(String subject, String body)
	{
		setSubject(subject);
		setBody(body);
	}

	public void setSubject(String subject)
	{
		m_subject = subject;
	}

	public void setBody(String body)
	{
		StringBuffer buff = new StringBuffer(body);

		int index = 0;
		index = buff.indexOf("\n", index);

		while(index > -1)
		{
			if(index == 0 || buff.charAt(index-1) != '\r')
			{
				buff.replace(index, index + 1, "\r\n");
				//DVScheduler.log("Replacing text at : " + index);
				index++;
			}

			index++;
			index = buff.indexOf("\n", index);
		}

		this.m_body = buff.toString();
	}

	public boolean isFinished()
	{
		return m_bFinished;
	}

	public void run()
	{
		try
		{
			DVScheduler.log("Email Thread Started (" + m_subject + ")");

			Properties props = new Properties();
			DataStore store = DataStore.getInstance();

			props.put("mail.smtp.host", store.getProperty("email.server", ""));
			props.put("mail.smtp.port", store.getProperty("email.server.port", "25"));
			//props.put("mail.user", store.getProperty("email.server.user", ""));
			//props.put("mail.password", store.getProperty("email.server.password", ""));
			String user = store.getProperty("email.server.user", "");
			String password = store.getProperty("email.server.password", "");
			String security = store.getProperty("email.connection.security", "None");
			if ( security.equals("SSL") )
				props.put("mail.smtp.ssl.enable", store.getProperty("email.server.auth", "true"));
			else if ( security.equals("STARTTLS") )
				props.put("mail.smtp.starttls.enable", "true");
			else if ( !user.isEmpty() || !password.isEmpty() )
				props.put("mail.smtp.auth", store.getProperty("email.server.auth", "true"));

			Session mailSession = Session.getInstance(props);
			MimeMessage message = new MimeMessage(mailSession);

			Address fromAddress = null;
			Vector<InternetAddress> vTos = null;
			fromAddress = new InternetAddress(store.getProperty("email.from", ""), store.getProperty("email.from.name", "DVScheduler"));

			String[] tos = store.getProperty("email.to", "").split(",");
			vTos = new Vector<InternetAddress>();
			for (int i=0; i<tos.length; i++)
			{
				vTos.add( new InternetAddress(tos[i].trim()) );
			}

			message.setFrom(fromAddress);
			message.setRecipients(Message.RecipientType.TO, vTos.toArray(new Address[0]));
			message.setSubject(m_subject, "UTF-8");
			message.setText(m_body, "UTF-8");

			//Transport.send(message);
			Transport t = mailSession.getTransport("smtp");
			try {
				if ( !user.isEmpty() || !password.isEmpty() )
					t.connect( user, password );
				else
					t.connect();
				t.sendMessage(message, message.getAllRecipients());
			} finally {
				t.close();
			}
		}
		catch(Exception e)
		{
			DVScheduler.log("Error email Thread");
			e.printStackTrace();
		}
		finally
		{
			m_bFinished = true;
			DVScheduler.log("Email Thread Exiting (" + m_subject + ")");
		}
	}

	public void finalize()
	{
		DVScheduler.log("Email Sending Thread Cleaned Up");
	}
}