/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;

import org.w3c.dom.*;

import javax.xml.parsers.*;

class EpgDataRes extends HTTPResponse
{

	public EpgDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if ("02".equals(action))
		{
			m_outStream.write(checkGuideData());
			return;
		}
		else if ("03".equals(action))
		{
			reloadXMLTVdata();
			return;
		}
		else if ("04".equals(action))
		{
			m_outStream.write(showChannelMapping());
			return;
		}
		else if ("05".equals(action))
		{
			m_outStream.write(addUpdateChannelMapping());
			return;
		}
		else if ("06".equals(action))
		{
			m_outStream.write(showProgramDetails());
			return;
		}
		else if ("07".equals(action))
		{
			m_outStream.write(deleteChannelMap());
			return;
		}      
		else if ("12".equals(action))
		{
			m_outStream.write(xmlEpg(0));
			return;
		}
		else if ("14".equals(action))
		{
			m_outStream.write(epgAutoAction());
			return;
		}
		else if ("16".equals(action))
		{
			m_outStream.write(xmlSourceSetup());
			return;
		}
		else if ("17".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try {
				m_outStream.write(updateXMLTVsourceData());
			} finally {
				ThreadLock.getInstance().releaseLock();
			}

			return;
		}
		else if ("18".equals(action))
		{
			m_outStream.write(getRegionsFromXMLservice());
			return;
		}
		else if ("19".equals(action))
		{
			m_outStream.write(getChannelsFromXMLservice());
			return;
		}
		else if ("20".equals(action))
		{
			m_outStream.write(setChannelsForXMLservice());
			return;
		}
		else if ("21".equals(action))
		{
			m_outStream.write(nownextEPG());
			return;
		}
		else if ("22".equals(action))
		{
			m_outStream.write(quicksearchEPG("epg-qsearch.html", "SearchEpgListSort", null));
			return;
		}
		else if ("24".equals(action))
		{
			m_outStream.write(moveChannel());
			return;
		}     
		else if ("25".equals(action))
		{
			m_outStream.write(xmlEpg(1));
			return;
		}     
		else if ("26".equals(action))
		{
			m_outStream.write(xmlEpg(2));
			return;
		}
		else if ("27".equals(action))
		{
			m_outStream.write(toggleEpgExclusion(true));
			return;
		}
		else if ("28".equals(action))
		{
			m_outStream.write(toggleEpgExclusion(false));
			return;
		}
		else if ("29".equals(action))
		{
			m_outStream.write(quicksearchEPG("epg-favorites.html", "FavoriteListSort", "autoAdd"));
			return;
		}
		else if ("30".equals(action))
		{
			m_outStream.write(quicksearchEPG("epg-exclusions.html", "AutoAddExclusionListSort", null));
			return;
		}
		else if ("31".equals(action))
		{
			m_outStream.write(moveSelectedChannels());
			return;
		}		
		else if ("32".equals(action))
		{
			m_outStream.write(deleteAllChannelMaps());
			return;
		}      
		else if ("EpgSetMapping".equals(action))
		{
			m_outStream.write(setEpgMapping());
			return;
		}
		else if ("EpgAddDefaultMapping".equals(action))
		{
			m_outStream.write(addDefaultEpgMapping());
			return;
		}      
		else if ("EpgSortMapping".equals(action))
		{
			m_outStream.write(sortEpgMapping());
			return;
		}      
		else
		{
			m_outStream.write("Action Not Supported".getBytes("UTF-8"));
			return;
		}
	}

	private byte[] checkGuideData() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		StringBuffer buff = new StringBuffer();
		String[] epgChannels = guide.getChannelList();

		DecimalFormat oneDec = new DecimalFormat("0.0");
		buff.append("<table>");
		buff.append("<tr>");
		buff.append("<td class='itemheading'><strong>Channel</strong></td>");
		buff.append("<td class='itemheading' align='center'><strong>Count</strong></td>");
		buff.append("<td class='itemheading' align='center'><strong>Start</strong></td>");
		buff.append("<td class='itemheading' align='center'><strong>Span</strong></td>");
		buff.append("</tr>\n");
		for (int x = 0; x < epgChannels.length; x++)
		{
			GuideItem[] items = guide.getProgramsForChannel(epgChannels[x]);
			if ((items == null) || (items.length <= 0))
				continue;
			buff.append("<tr><td align='left'>" + HTMLEncoder.encode(epgChannels[x]) + "</td><td align='center'>" + items.length + "</td>");

			long span = items[items.length-1].getStart().getTime() - items[0].getStart().getTime(); 
			double spanDays = (double)span / (double)(1000 * 60 * 60 * 24);

			long start = items[0].getStart().getTime() - new Date().getTime(); 
			double startDays = (double)start / (double)(1000 * 60 * 60 * 24);

			buff.append("<td align='center'>" + oneDec.format(startDays) + "</td><td align='center'>" + oneDec.format(spanDays) + "</td>");         

			buff.append("</tr>\n");
		}
		buff.append("</table>\n");  

		buff.append("<br>\n");  

		buff.append("<table border='0' cellspacing='0' cellpadding='0'>");
		buff.append("<tr><td class='itemheading'><strong>Continuity Report</strong></td></tr><tr><td>");
		for (int x = 0; x < epgChannels.length; x++)
		{
			if (guide.getProgramSizeForEpgChannel(epgChannels[x]) <= 0)
				continue;
			buff.append("<table>");
			buff.append("<tr><td>" + HTMLEncoder.encode(epgChannels[x]) + "</td></tr>");
			buff.append("<tr><td><img src='/servlet/ContinuityImageDataRes?action=1&channel=" + Formatter.encodeURL(epgChannels[x], "UTF-8") + "'></td></tr>\n");
			buff.append("</table>");
		}
		buff.append("</td></tr></table>\n");

		StringBuffer overaps = new StringBuffer();
		DateFormatter2 dateFormatter2 = new DateFormatter2();

		for (int x = 0; x < epgChannels.length; x++)
		{
			GuideItem[] items = guide.getProgramsForChannel(epgChannels[x]);

			for (int y = 0; y < items.length - 1; y++)
			{
				GuideItem item1 = items[y];
				GuideItem item2 = items[y + 1];

				if(item2.getStart().getTime() < item1.getStop().getTime())
				{
					overaps.append("<tr>\n");
					overaps.append("<td valign='top'>" + HTMLEncoder.encode(epgChannels[x]) + "</td>\n");
					overaps.append("<td>");

					overaps.append("<table>");
					overaps.append("<tr><td>" + item1.getName() + "</td><td>" + dateFormatter2.formatDateTime(item1.getStart()) + "</td><td>(" + item1.getDuration() + ")</td></tr>\n");
					overaps.append("<tr><td>" + item2.getName() + "</td><td>" + dateFormatter2.formatDateTime(item2.getStart()) + "</td><td>(" + item2.getDuration() + ")</td></tr>\n");
					overaps.append("</table>");

					overaps.append("</td>");

					long diff = item1.getStop().getTime() - item2.getStart().getTime();
					diff = diff / (1000 * 60);
					overaps.append("<td valign='top'>" + diff + "</td>\n");

					overaps.append("</tr>\n");
				}
			}
		}      

		if(overaps.length() > 0)
		{
			buff.append("<br><table border='0' cellspacing='5' cellpadding='5'>");
			buff.append("<tr><td nowrap class='itemheading' colspan='3'><strong>Overlapping Item Report</strong></td></tr>\n");
			buff.append("<tr>\n");
			buff.append("<td nowrap><strong>Channel</strong></td>\n");
			buff.append("<td nowrap><strong>Items</strong></td>\n");
			buff.append("<td nowrap><strong>Duration</strong></td>\n");
			buff.append(overaps);
			buff.append("</tr></table>\n");
		}

		StringBuffer gaps = new StringBuffer();

		for (int x = 0; x < epgChannels.length; x++)
		{
			GuideItem[] items = guide.getProgramsForChannel(epgChannels[x]);

			for (int y = 0; y < items.length - 1; y++)
			{
				GuideItem item1 = items[y];
				GuideItem item2 = items[y + 1];

				if(item2.getStart().getTime() > item1.getStop().getTime())
				{
					gaps.append("<tr>\n");
					gaps.append("<td valign='top'>" + HTMLEncoder.encode(epgChannels[x]) + "</td>\n");
					gaps.append("<td>" + dateFormatter2.formatDateTime(item1.getStop()) + "</td>\n");
					gaps.append("<td>" + dateFormatter2.formatDateTime(item2.getStart()) + "</td>\n");

					long diff = item2.getStart().getTime() - item1.getStop().getTime();
					diff = diff / (1000 * 60);
					gaps.append("<td valign='top'>" + diff + "</td>\n");

					gaps.append("</tr>\n");
				}
			}
		}        

		if(gaps.length() > 0)
		{
			buff.append("<br><table border='0' cellspacing='5' cellpadding='5'>");
			buff.append("<tr><td nowrap class='itemheading' colspan='4'><strong>Gap Report</strong></td></tr>\n");
			buff.append("<tr>\n");
			buff.append("<td nowrap><strong>Channel</strong></td>\n");
			buff.append("<td nowrap><strong>From</strong></td>\n");
			buff.append("<td nowrap><strong>To</strong></td>\n");
			buff.append("<td nowrap><strong>Duration</strong></td></tr>\n");
			buff.append(gaps);
			buff.append("</table>\n");
		}      

		buff.append("<br><table border='0' cellspacing='5' cellpadding='5'>");
		buff.append("<tr><td nowrap class='itemheading' colspan='2' id='epgAge'><strong>EPG Age</strong></td></tr>\n");
		buff.append("<tr>\n");
		buff.append("<td nowrap><strong>Channel</strong></td>\n");
		buff.append("<td nowrap><strong>Last load time</strong></td></tr>\n");
		guide.getChannelLastLoadTimeHtml(buff);
		buff.append("</table>\n");

		buff.append("<br><table border='0' cellspacing='5' cellpadding='5'>");
		buff.append("<tr><td nowrap class='itemheading' colspan='2'><strong>Built-in EPG Scanner Schedule</strong></td></tr>\n");
		buff.append("<tr>\n");
		buff.append("<td nowrap><strong>Demodulator</strong></td>\n");
		buff.append("<td nowrap><strong>Next load time</strong></td></tr>\n");
		guide.getInternalEpgScheduleHtml(buff);
		buff.append("</table>\n");

		// write out template
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-conflicts.html", true);
		template.replace("$title" , "EPG Data Report");      
		template.replace("$list", buff.toString());

		return template.getPageBytes();   
	}

	private byte[] deleteChannelMap() throws Exception
	{  
		String redir = getServletRedir("?action=04");

		GuideStore guide = GuideStore.getInstance();

		int id = -1;
		try
		{
			id = Integer.parseInt(m_urlData.getParameter("id"));
		}
		catch(Exception e){}

		int subIndex = m_urlData.getParameterInt("subIndex", -1);

		if(id > -1 && subIndex > -1)
		{
			guide.deleteChannelMapCustom(id, subIndex);
		}
		else if (id > -1)
		{
			guide.deleteChannelMapCustom(id);
		}

		return redir.getBytes("UTF-8");        
	}

	private byte[] deleteAllChannelMaps() throws Exception
	{  
		GuideStore.getInstance().deleteAllChannels();

		return getServletRedir("?action=04").getBytes("UTF-8");
	}

	private byte[] moveChannel() throws Exception
	{    
		GuideStore guide = GuideStore.getInstance();

		int id = -1;
		try
		{
			id = Integer.parseInt(m_urlData.getParameter("id"));
		}
		catch(Exception e){}

		int dir = 1;
		try
		{
			dir = Integer.parseInt(m_urlData.getParameter("dir"));
		}
		catch(Exception e){}

		boolean direction = false;
		if(dir == 1)
			direction = true;

		if(id != -1)
			guide.moveChannel(id, direction);

		StringBuffer buff = new StringBuffer();
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/EpgDataRes?action=04\n\n");

		return buff.toString().getBytes("UTF-8");
	}

	private byte[] moveSelectedChannels() throws Exception
	{
		String redir = getServletRedir("?action=04");

		String selected = m_urlData.getParameter("sel");
		if ( (selected == null) || (selected.isEmpty()) )
			return redir.getBytes("UTF-8");
		String[] selectedArray = selected.split("\n");
		Vector<Integer> selectedVector = new Vector<Integer>();
		for (int i=0; i<selectedArray.length; i++)
		{
			int sel = -1;
			try
			{
				sel = Integer.parseInt(selectedArray[i]);
			}
			catch(Exception e){}

			if (sel > -1)
				selectedVector.add(sel);
		}
		if (selectedVector.isEmpty())
			return redir.getBytes("UTF-8");

		GuideStore guide = GuideStore.getInstance();
		if (m_urlData.getParameterStartsWith("delSel") != null)
		{
			guide.deleteChannels(selectedVector);
		}
		else if (m_urlData.getParameterStartsWith("moveEnd") != null)
		{
			guide.moveChannelsBeforeChannel(-1, selectedVector);
		}
		else
		{
			int beforeIndex = -1;
			try
			{
				beforeIndex = Integer.parseInt(m_urlData.getParameterStartsWith("move.").split("\\.")[1]);
			}
			catch(Exception e){}

			if (beforeIndex > -1)
				guide.moveChannelsBeforeChannel(beforeIndex, selectedVector);
		}

		return redir.getBytes("UTF-8");
	}

	private byte[] setChannelsForXMLservice() throws Exception
	{
		String channelID = "";
		String fullChannelList = "";

		for (int x = 0; x < 50; x++)
		{
			channelID = m_urlData.getParameter("channel" + x);
			if (channelID != null && channelID.length() > 0)
			{
				fullChannelList += channelID + ";";
			}
		}

		m_store.setProperty("guide.source.xml.channelList", fullChannelList);

		StringBuffer buff = new StringBuffer();
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/EpgDataRes?action=16\n\n");

		return buff.toString().getBytes("UTF-8");
	}

	private byte[] getChannelsFromXMLservice() throws Exception
	{
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-xmlServiceChannels.html");

		String channelsURL = m_store.getProperty("guide.source.xml.channelsUrl", "");
		String regionID = m_urlData.getParameter("region");
		channelsURL = channelsURL.replaceFirst("<region>", regionID);

		byte[] pageData = getContentUsingThread(new URL(channelsURL));

		if (pageData != null)
		{
			HashMap<String, String> channelHash = getFreeChannels(pageData);

			String[] keys = (String[]) channelHash.keySet().toArray(new String[0]);
			Arrays.sort(keys);

			StringBuffer buff = new StringBuffer();

			int index = 0;

			for (int x = 0; x < keys.length; x++)
			{
				String value = (String) channelHash.get(keys[x]);
				value = Formatter.encodeURL(value, "UTF-8");

				buff.append("<input type='checkbox' name='channel" + (index++) + "' value='" + value + "'>" + keys[x] + "<br>\n");
			}

			template.replace("$channelList", buff.toString());
		}
		else
		{
			template.replace("$channelList", "");
		}

		return template.getPageBytes();
	}

	public HashMap<String, String> getFreeChannels(byte[] data) throws Exception
	{
		HashMap<String, String> channels = new HashMap<String, String>();

		ByteArrayInputStream reader = new ByteArrayInputStream(data);
		Document doc = null;
		NodeList region_nodes = null;

		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			doc = docBuilder.parse(reader);
			region_nodes = doc.getElementsByTagName("NewDataSet").item(0).getChildNodes();
		}
		catch (Exception e)
		{
			DVScheduler.log("ERROR: no valid XML data returned from query");
		}

		if (region_nodes == null)
			return channels;

		Node thisNode = null;
		for (int x = 0; x < region_nodes.getLength(); x++)
		{
			thisNode = region_nodes.item(x);
			if (thisNode == null)
				break;

			if (thisNode.getNodeName().equals("Channels"))
			{
				String channelID = getNodeValue(getChildByType(thisNode, "ChannelID"));
				String channelName = getNodeValue(getChildByType(thisNode, "Name"));
				channels.put(channelName, channelID);
				//DVScheduler.log(channelID + " : " + channelName);
			}
		}

		return channels;
	}

	private byte[] getRegionsFromXMLservice() throws Exception
	{
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-xmlServiceRegions.html");

		String regionsURL = m_store.getProperty("guide.source.xml.regionsUrl", "");

		byte[] pageData = getContentUsingThread(new URL(regionsURL));

		if (pageData != null)
		{
			HashMap<String, String> regionHash = getRegions(pageData);

			String[] keys = (String[]) regionHash.keySet().toArray(new String[0]);
			Arrays.sort(keys);

			StringBuffer buff = new StringBuffer();
			buff.append("<ul>\n");

			for (int x = 0; x < keys.length; x++)
			{
				String value = (String) regionHash.get(keys[x]);
				value = Formatter.encodeURL(value, "UTF-8");
				buff.append("<li><a href='/servlet/EpgDataRes?action=19&region=" + value + "' class='noUnder'>" + keys[x] + "</a></li>\n");
			}
			buff.append("</ul>\n");

			template.replace("$regions", buff.toString());
		}
		else
		{
			template.replace("$regions", "");
		}

		return template.getPageBytes();
	}

	private byte[] getContentUsingThread(URL location)
	{
		Thread loaderThread = null;
		LoaderTread loader = null;
		try
		{
			loader = new LoaderTread(location, "", "");
			loaderThread = new Thread(Thread.currentThread().getThreadGroup(), loader, loader.getClass().getName());
			loaderThread.start();

			while (!loader.isFinished())
			{
				if (loader.isTimedOut(30))
				{
					DVScheduler.log("\nTimeout reached : Killing loader Thread");
					loaderThread.interrupt();
					break;
				}
				Thread.sleep(500);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		loaderThread.interrupt();

		if(loader.getResponceCode() == 200)
		{
			return loader.getDataBytes();
		}

		return null;
	}

	private HashMap<String, String> getRegions(byte[] data) throws Exception
	{
		HashMap<String, String> regions = new HashMap<String, String>();

		ByteArrayInputStream reader = new ByteArrayInputStream(data);
		Document doc = null;
		NodeList region_nodes = null;

		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
			doc = docBuilder.parse(reader);
			region_nodes = doc.getElementsByTagName("NewDataSet").item(0).getChildNodes();
		}
		catch (Exception e)
		{
			DVScheduler.log("ERROR: no valid XML data returned from query");
		}

		if (region_nodes == null)
			return regions;

		Node thisNode = null;
		for (int x = 0; x < region_nodes.getLength(); x++)
		{
			thisNode = region_nodes.item(x);
			if (thisNode == null)
				break;

			if (thisNode.getNodeName().equals("Regions"))
			{
				String code = getNodeValue(getChildByType(thisNode, "Code"));
				String region = getNodeValue(getChildByType(thisNode, "Description"));
				regions.put(region, code);
				//DVScheduler.log(code + " : " + region);
			}
		}

		return regions;
	}

	private String getNodeValue(Node node)
	{
		if (node != null && node.getFirstChild() != null)
			return node.getFirstChild().getNodeValue();
		else
			return "";
	}

	private Node getChildByType(Node node, String name)
	{
		if (!node.hasChildNodes())
			return null;

		Node found = null;
		NodeList nl = node.getChildNodes();
		for (int x = 0; x < nl.getLength(); x++)
		{
			found = nl.item(x);
			if (found.getNodeName().equals(name))
				return found;
		}

		return null;
	}

	private byte[] updateXMLTVsourceData() throws Exception
	{
		String out = "";
		String sessionID = m_urlData.getParameter("sessionID");
		if(!m_store.checkSessionId(sessionID))
		{
			out = "Security Warning: The Security Session ID you entered is not correct.";
			return out.getBytes("UTF-8");         
		}

		String fileSource = m_urlData.getParameter("fileDIR");
		if (fileSource == null)
			fileSource = "";
		/*
		else
			fileSource = new File(fileSource).getAbsolutePath();
		 */
		m_store.setProperty("guide.source.file", DVScheduler.checkBreaks(fileSource.trim()));

		String httpURL = m_urlData.getParameter("httpURL");
		if (httpURL == null)
			httpURL = "";
		m_store.setProperty("guide.source.http", DVScheduler.checkBreaks(httpURL));

		String httpUsr = m_urlData.getParameter("httpUser");
		if (httpUsr == null)
			httpUsr = "";
		m_store.setProperty("guide.source.http.usr", DVScheduler.checkBreaks(httpUsr));     

		String httpPwd = m_urlData.getParameter("httpPassword");
		if (httpPwd == null)
			httpPwd = "";
		m_store.setProperty("guide.source.http.pwd", DVScheduler.checkBreaks(httpPwd));

		String httpProxyServer = m_urlData.getParameter("httpProxyServer");
		if (httpProxyServer == null)
			httpProxyServer = "";
		m_store.setProperty("proxy.server", httpProxyServer);

		String xmlRegions = m_urlData.getParameter("xmlRegions");
		if (xmlRegions == null)
			xmlRegions = "";
		m_store.setProperty("guide.source.xml.regionsUrl", xmlRegions);

		String xmlChannels = m_urlData.getParameter("xmlChannels");
		if (xmlChannels == null)
			xmlChannels = "";
		m_store.setProperty("guide.source.xml.channelsUrl", xmlChannels);

		String xmlPrograms = m_urlData.getParameter("xmlPrograms");
		if (xmlPrograms == null)
			xmlPrograms = "";
		m_store.setProperty("guide.source.xml.programsUrl", xmlPrograms);

		String httpProxyPort = m_urlData.getParameter("httpProxyPort");
		if (httpProxyPort == null)
			httpProxyPort = "";
		m_store.setProperty("proxy.port", httpProxyPort);

		if (m_urlData.getParameter("enableProxyUsernamePassword", "0").equals("1"))
		{
			String httpProxyUser = m_urlData.getParameter("httpProxyUsername");
			if (httpProxyUser == null)
				httpProxyUser = "";
			m_store.setProperty("proxy.username", httpProxyUser);

			httpProxyUser = m_urlData.getParameter("httpProxyPassword");
			if (httpProxyUser == null)
				httpProxyUser = "";
			m_store.setProperty("proxy.password", httpProxyUser);
		}

		String type = "";
		String type2 = m_urlData.getParameter("sourceTypeFile");
		if (type2 == null)
			type2 = "";
		else if (type2.equals("1"))
			type += "File,";
		type2 = m_urlData.getParameter("sourceTypeHttp");
		if (type2 == null)
			type2 = "";
		else if (type2.equals("1"))
			type += "Http,";
		type2 = m_urlData.getParameter("sourceTypeInternal");
		if (type2 == null)
			type2 = "";
		else if (type2.equals("1"))
			type += "Internal,";
		m_store.setProperty("guide.source.type", type);

		String schOptions = "";
		String schEnabled = m_urlData.getParameter("scheduleImportEnabled");
		if (schEnabled == null)
			schEnabled = "0";
		schOptions += schEnabled.trim() + ":";

		String schHour = m_urlData.getParameter("scheduleHour");
		if (schHour == null)
			schHour = "0";
		schOptions += schHour.trim() + ":";

		String schMin = m_urlData.getParameter("scheduleMin");
		if (schMin == null)
			schMin = "0";
		schOptions += schMin.trim() + ":";

		String whenNotCapturing = m_urlData.getParameter("whenNotCapturing");
		if (whenNotCapturing == null)
			whenNotCapturing = "0";
		schOptions += whenNotCapturing.trim() + ":";

		String timeToNextSchedule = m_urlData.getParameter("timeToNextSchedule");
		if (timeToNextSchedule == null)
			timeToNextSchedule = "0";
		schOptions += timeToNextSchedule.trim() + ":";

		String wakeSystem = m_urlData.getParameter("wakeSystem");
		if (wakeSystem == null)
			wakeSystem = "0";
		schOptions += wakeSystem.trim() + ":";

		String schPeriod = m_urlData.getParameter("schedulePeriod");
		if (schPeriod == null)
			schPeriod = "24";
		schOptions += schPeriod.trim();

		int lScheduleMinuteAfterLaunch = m_urlData.getParameterInt("scheduleMinuteAfterLaunch", -1);
		if ( lScheduleMinuteAfterLaunch <= 0 )
			lScheduleMinuteAfterLaunch = -1;
		else if ( lScheduleMinuteAfterLaunch < 5 )	// Minimum 5 minutes after the program is started.
			lScheduleMinuteAfterLaunch = 5;
		m_store.setPropertyInt("guide.source.scheduleMinuteAfterLaunch", lScheduleMinuteAfterLaunch);

		String preEPGloadTask = m_urlData.getParameter("Tasks.PreEPGloadTask");
		if ( (preEPGloadTask == null) || preEPGloadTask.equals("*UNDEFINED*") )
			preEPGloadTask = "";
		m_store.setProperty("Tasks.PreEPGloadTask", preEPGloadTask);

		String preActionRun = m_urlData.getParameter("preActionRun");
		if (preActionRun == null)
			preActionRun = "";
		m_store.setProperty("guide.preActionRun", preActionRun);

		m_store.setProperty("guide.source.schedule", schOptions);

		String postScan = m_urlData.getParameter("postScanChangeTitleCase");
		if (postScan == null || postScan.isEmpty())
			postScan = "";
		m_store.setProperty("guide.postScanChangeTitleCase", postScan);

		postScan = m_urlData.getParameter("postScanAutoMarkEnabled");
		if (postScan == null || postScan.isEmpty())
			postScan = "";
		m_store.setProperty("guide.postScanAutoMark", postScan);

		postScan = m_urlData.getParameter("postScanAutoMarkRun");
		if (postScan == null)
			postScan = "";
		m_store.setProperty("guide.postScanAutoMarkRun", postScan);

		postScan = m_urlData.getParameter("postScanAutoAddEnabled");
		if (postScan == null || postScan.isEmpty())
			postScan = "";
		m_store.setProperty("guide.postScanAutoAdd", postScan);

		postScan = m_urlData.getParameter("postScanAutoAddRun");
		if (postScan == null)
			postScan = "";
		m_store.setProperty("guide.postScanAutoAddRun", postScan);

		String postEPGloadTask = m_urlData.getParameter("Tasks.PostEPGloadTask");
		if ( (postEPGloadTask == null) || postEPGloadTask.equals("*UNDEFINED*") )
			postEPGloadTask = "";
		m_store.setProperty("Tasks.PostEPGloadTask", postEPGloadTask);

		String postActionRun = m_urlData.getParameter("postActionRun");
		if (postActionRun == null)
			postActionRun = "";
		m_store.setProperty("guide.postActionRun", postActionRun);

		String postInternalScanTask = m_urlData.getParameter("Tasks.PostInternalScanTask");
		if ( (postInternalScanTask == null) || postInternalScanTask.equals("*UNDEFINED*") )
			postInternalScanTask = "";
		m_store.setProperty("Tasks.PostInternalScanTask", postInternalScanTask);

		String postInternalScanAutoMarkEnabled = m_urlData.getParameter("postInternalScanAutoMarkEnabled", "0");
		m_store.setProperty("guide.postInternalScanAutoMarkEnabled", postInternalScanAutoMarkEnabled);

		String postInternalScanAutoAddEnabled = m_urlData.getParameter("postInternalScanAutoAddEnabled", "0");
		m_store.setProperty("guide.postInternalScanAutoAddEnabled", postInternalScanAutoAddEnabled);

		String scheduleInternalEnabled = m_urlData.getParameter("scheduleInternalEnabled", "0");
		m_store.setProperty("guide.scheduleInternalEnabled", scheduleInternalEnabled);

		String scheduleInternalPeriod = m_urlData.getParameter("scheduleInternalPeriod", "24");
		m_store.setProperty("guide.scheduleInternalPeriod", scheduleInternalPeriod);

		String languagePriority = m_urlData.getParameter("languagePriority", "");
		m_store.setProperty("guide.language.priority", languagePriority);

		String maxLanguage = m_urlData.getParameter("maxLanguage", "");
		m_store.setProperty("guide.language.max", maxLanguage);

		String demodMatch = m_urlData.getParameter("chanMatch");
		m_store.setEpgScanDemodulators(demodMatch);
		m_store.saveChannels(null);

		//GuideStore guide = GuideStore.getInstance();
		//guide.setInternalEpgScheduleChannelChange();

		m_store.refreshWakeupTime();

		StringBuffer buff = new StringBuffer();
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /settings.html\n\n");

		return buff.toString().getBytes("UTF-8");
	}

	private byte[] xmlSourceSetup() throws Exception
	{
		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "config", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("checkId", m_store.isSecurity() ? "1" : "0");

		root.setAttribute("httpProxyServer", m_store.getProperty("proxy.server", ""));
		root.setAttribute("httpProxyPort", m_store.getProperty("proxy.port", ""));
		root.setAttribute("httpProxyUsername", m_store.getProperty("proxy.username", ""));
		//root.setAttribute("httpProxyPassword", m_store.getProperty("proxy.password", ""));

		String scheduleOptions = m_store.getProperty("guide.source.schedule", "0:0:0:1:10:1:24");
		String[] schOptsArray = scheduleOptions.split(":");
		root.setAttribute("scheduleImportEnabled", schOptsArray[0].trim());
		if (schOptsArray.length > 1)
			root.setAttribute("scheduleHour", schOptsArray[1].trim());
		else
			root.setAttribute("scheduleHour", "");

		if (schOptsArray.length > 2)
			root.setAttribute("scheduleMin", schOptsArray[2].trim());
		else
			root.setAttribute("scheduleMin", "");

		if (schOptsArray.length > 3)
			root.setAttribute("whenNotCapturing", schOptsArray[3].trim());
		else
			root.setAttribute("whenNotCapturing", "");

		if (schOptsArray.length > 4)
			root.setAttribute("timeToNextSchedule", schOptsArray[4].trim());
		else
			root.setAttribute("timeToNextSchedule", "");

		if (schOptsArray.length > 5)
			root.setAttribute("wakeSystem", schOptsArray[5].trim());
		else
			root.setAttribute("wakeSystem", "");

		if (schOptsArray.length > 6)
			root.setAttribute("schedulePeriod", schOptsArray[6].trim());
		else
			root.setAttribute("schedulePeriod", "24");

		long scheduleMinuteAfterLaunch = m_store.getPropertyLong("guide.source.scheduleMinuteAfterLaunch", -1);
		root.setAttribute("scheduleMinuteAfterLaunch", (scheduleMinuteAfterLaunch <= 0) ? "" : Long.toString(scheduleMinuteAfterLaunch));

		root.setAttribute("preEPGloadTask", m_store.getProperty("Tasks.PreEPGloadTask", ""));
		root.setAttribute("preActionRun", m_store.getProperty("guide.preActionRun", "All"));

		root.setAttribute("postScanChangeTitleCase", m_store.getProperty("guide.postScanChangeTitleCase", "unchanged"));
		root.setAttribute("postScanAutoMarkEnabled", m_store.getProperty("guide.postScanAutoMark", "1"));
		root.setAttribute("postScanAutoMarkRun", m_store.getProperty("guide.postScanAutoMarkRun", "All"));
		root.setAttribute("postScanAutoAddEnabled", m_store.getProperty("guide.postScanAutoAdd", "1"));
		root.setAttribute("postScanAutoAddRun", m_store.getProperty("guide.postScanAutoAddRun", "All"));
		root.setAttribute("postEPGloadTask", m_store.getProperty("Tasks.PostEPGloadTask", ""));
		root.setAttribute("postActionRun", m_store.getProperty("guide.postActionRun", "All"));

		root.setAttribute("postInternalScanTask", m_store.getProperty("Tasks.PostInternalScanTask", ""));
		root.setAttribute("postInternalScanAutoMarkEnabled", m_store.getProperty("guide.postInternalScanAutoMarkEnabled", "1"));
		root.setAttribute("postInternalScanAutoAddEnabled", m_store.getProperty("guide.postInternalScanAutoAddEnabled", "1"));
		root.setAttribute("scheduleInternalEnabled", m_store.getProperty("guide.scheduleInternalEnabled", "0"));
		root.setAttribute("scheduleInternalPeriod", m_store.getProperty("guide.scheduleInternalPeriod", "24"));
		root.setAttribute("languagePriority", m_store.getProperty("guide.language.priority", ""));
		root.setAttribute("maxLanguage", m_store.getProperty("guide.language.max", ""));

		String type = m_store.getProperty("guide.source.type", "Internal");
		if (type.contains("File"))
			root.setAttribute("fileSourceEnabled", "1");
		if (type.contains("Http"))
			root.setAttribute("httpSourceEnabled", "1");
		if (type.contains("Internal"))
			root.setAttribute("internalSourceEnabled", "1");

		Element elm = doc.createElement("fileDIR");
		Text text = doc.createTextNode(m_store.getProperty("guide.source.file", ""));
		elm.appendChild(text);
		root.appendChild(elm);

		elm = doc.createElement("httpURL");
		text = doc.createTextNode(m_store.getProperty("guide.source.http", ""));
		elm.appendChild(text);
		root.appendChild(elm);

		elm = doc.createElement("httpUser");
		text = doc.createTextNode(m_store.getProperty("guide.source.http.usr", ""));
		elm.appendChild(text);
		root.appendChild(elm);

		elm = doc.createElement("httpPassword");
		text = doc.createTextNode(m_store.getProperty("guide.source.http.pwd", ""));
		elm.appendChild(text);
		root.appendChild(elm);

		elm = doc.createElement("allTasks");
		m_store.getTaskListXml(doc, elm);
		root.appendChild(elm);

		m_store.getDemodulatorsXml(doc, root);

		//
		// Send output back 
		//
		XSL transformer = new XSL(doc, "epg-sourceSettings.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] epgAutoAction() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		StringBuffer buff = new StringBuffer();

		String reload = m_urlData.getParameter("reload");
		String rescan = m_urlData.getParameter("rescan");
		String email = m_urlData.getParameter("email");

		if (reload != null && reload.equals("1"))
		{
			guide.loadXmltv(buff, 0, 0, 1, 1, 0);
		}

		if (rescan != null && rescan.equals("1"))
		{
			m_store.removeSchedulesOfTypeEpg(buff, 0);
			guide.addEpgMatches(buff, 0, false);
			guide.saveEpg();
		}

		m_store.saveSchedules(null);

		if (email != null && email.equals("1"))
		{
			EmailSender sender = new EmailSender();
			sender.setSubject("DV Scheduler EPG Reload Results");
			sender.setBody("EPG Reload Result:\n\n" + buff.toString());

			try
			{
				Thread mailThread = new Thread(Thread.currentThread().getThreadGroup(), sender, sender.getClass().getName());
				mailThread.start();
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}

		String returnData = "";
		returnData += "HTTP/1.0 200 OK\n";
		returnData += "Content-Type: text/plain\n";
		returnData += "Pragma: no-cache\n";
		returnData += "Cache-Control: no-cache\n\n";
		returnData += "EPG Reload/Rescan Results\n";
		returnData += "==========================\n";

		returnData += buff.toString();

		return returnData.getBytes("UTF-8");
	}

	private String getChannelDay(String channel, Date time, DateFormatter2 dateFormatter2)
	{
		String text = channel + " (" + dateFormatter2.formatDateMonth(time) + ")";
		return text;
	}

	// useItemID values:
	//	0: don't use.
	//	1: use schedule item ID.
	//	2: use EPG item ID.
	private byte[] xmlEpg(int useItemID) throws Exception
	{
		Calendar start = Calendar.getInstance();
		GuideStore guideStore = GuideStore.getInstance();

		if(start.get(Calendar.HOUR_OF_DAY) < 6)
		{
			start.add(Calendar.DATE, -1);
		}

		int month = start.get(Calendar.MONTH) + 1;
		int day = start.get(Calendar.DATE);
		int year = start.get(Calendar.YEAR);
		double scrollto = -2;

		String useItemIdChannel = null;
		if (useItemID == 1)
		{
			String id = m_urlData.getParameter("id");
			if (id != null && id.length() > 0)
			{
				HashMap<String, ScheduleItem> times = m_store.getScheduleList();
				ScheduleItem item = (ScheduleItem) times.get(id);
				if (item != null)
				{
					start.setTime(item.getStart());

					if(start.get(Calendar.HOUR_OF_DAY) < 6)
					{
						start.add(Calendar.DATE, -1);
					}

					month = start.get(Calendar.MONTH) + 1;
					day = start.get(Calendar.DATE);
					year = start.get(Calendar.YEAR);
					scrollto = start.get(Calendar.HOUR_OF_DAY) + start.get(Calendar.MINUTE)/60.0;
					useItemIdChannel = item.getChannel();
				}
			}
		}
		else if (useItemID == 2)
		{
			String id = m_urlData.getParameter("id");
			String channel = m_urlData.getParameter("channel");
			if (id != null && id.length() > 0 && channel != null && channel.length() > 0)
			{
				String epgChan = guideStore.getEpgChannelIdFromMap(channel);
				GuideItem item = guideStore.getProgram(epgChan, id);
				if (item != null)
				{
					start.setTime(item.getStart());

					if(start.get(Calendar.HOUR_OF_DAY) < 6)
					{
						start.add(Calendar.DATE, -1);
					}

					month = start.get(Calendar.MONTH) + 1;
					day = start.get(Calendar.DATE);
					year = start.get(Calendar.YEAR);
					scrollto = start.get(Calendar.HOUR_OF_DAY) + start.get(Calendar.MINUTE)/60.0;
					useItemIdChannel = channel;
				}
			}
		}
		else
		{
			try
			{
				year = Integer.parseInt(m_urlData.getParameter("year"));
				month = Integer.parseInt(m_urlData.getParameter("month"));
				day = Integer.parseInt(m_urlData.getParameter("day"));
			}
			catch (Exception e)
			{
			}

			try
			{
				scrollto = Double.parseDouble(m_urlData.getParameter("scrollto"));
			}
			catch (Exception e)
			{
			}

			//			if(scrollto == -1)
			//			{
			//			scrollto = start.get(Calendar.HOUR_OF_DAY) + start.get(Calendar.MINUTE)/60.0;
			//			}
		}

		int height = -1;
		try
		{
			height = Integer.parseInt(m_urlData.getParameter("height"));
		}
		catch (Exception e)
		{
		}

		if(height == -1)
		{
			height = Integer.parseInt(m_store.getProperty("EPG.hourLength", "180"));
			if (height <= 0)
				height = 180;
		}

		int width = -1;
		try
		{
			width = Integer.parseInt(m_urlData.getParameter("width"));
		}
		catch (Exception e)
		{
		}

		if(width == -1)
		{
			width = Integer.parseInt(m_store.getProperty("EPG.channelLength", "100"));
			if (width <= 0)
				width = 100;
		}

		int quick = -1;
		try
		{
			quick = Integer.parseInt(m_urlData.getParameter("quick"));
		}
		catch (Exception e)
		{
		}

		int balloonDelay = -1;
		try
		{
			balloonDelay = Integer.parseInt(m_urlData.getParameter("balloonDelay"));
		}
		catch (Exception e)
		{
		}

		if(balloonDelay == -1)
		{
			balloonDelay = Integer.parseInt(m_store.getProperty("EPG.balloonDelay", "0"));
			if (balloonDelay < 0)
				balloonDelay = 0;
		}

		start.set(Calendar.YEAR, year);
		start.set(Calendar.MONTH, month - 1);
		start.set(Calendar.DATE, day);
		start.set(Calendar.HOUR_OF_DAY, 6);
		start.set(Calendar.MINUTE, 0);
		start.set(Calendar.SECOND, 0);
		start.add(Calendar.SECOND, -1);
		start.set(Calendar.MILLISECOND, 0);

		int daysPerPageRequested = m_urlData.getParameterInt("daysPerPage", -1);
		if (daysPerPageRequested < 1)
			m_urlData.getCookieInt("EpgDaysPerPage", -1);
		int daysPerPageRequested2 = daysPerPageRequested;
		int daysPerPageDefault = m_store.getPropertyInt("EPG.daysPerPage", 1);
		if (daysPerPageRequested2 < 1)
			daysPerPageRequested2 = daysPerPageDefault;
		if (daysPerPageRequested2 < 1)
			daysPerPageRequested2 = 1;
		int daysPerPage = daysPerPageRequested2;
		Calendar end = (Calendar) start.clone();
		end.add(Calendar.DAY_OF_MONTH, daysPerPage);
		if (end.getTimeInMillis() > guideStore.getMaxEntry().getTime())
		{
			end.setTime(guideStore.getMaxEntry());
			daysPerPage = (int) ((end.getTimeInMillis() - start.getTimeInMillis()) / 24 / 3600000 + 1);
		}
		//int hoursPerPage = daysPerPage * 24;

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("Test", "tv", null);

		Vector<String[]> channelMap = guideStore.getOldChannelMap();
		Set<?> wsChannels = m_store.getChannels().keySet();

		Element root = doc.getDocumentElement();

		Calendar nowCalendar = Calendar.getInstance();

		root.setAttribute("date", new Integer(start.get(Calendar.DATE)).toString());
		root.setAttribute("month", (String) m_store.monthNameShort.get(new Integer(start.get(Calendar.MONTH))));
		root.setAttribute("monthn0", Integer.toString(start.get(Calendar.MONTH)));
		root.setAttribute("monthn1", Integer.toString(start.get(Calendar.MONTH)+1));
		root.setAttribute("year", new Integer(start.get(Calendar.YEAR)).toString());
		root.setAttribute("day", (String) m_store.dayNameFull.get(new Integer(start.get(Calendar.DAY_OF_WEEK))));
		root.setAttribute("nowHour", new Integer(nowCalendar.get(Calendar.HOUR_OF_DAY)).toString());
		root.setAttribute("nowMinute", new Integer(nowCalendar.get(Calendar.MINUTE)).toString());
		root.setAttribute("scrollto", new Integer(new Double(scrollto).intValue()).toString());
		root.setAttribute("scrollto2", new Double(scrollto).toString());
		root.setAttribute("height", new Integer(height).toString());
		root.setAttribute("width", new Integer(width).toString());
		root.setAttribute("balloonDelay", new Integer(balloonDelay).toString());
		root.setAttribute("leftClick", m_store.getProperty("EPG.leftClick", "Show Menu"));
		root.setAttribute("rightClick", m_store.getProperty("EPG.rightClick", "Add Schedule"));
		root.setAttribute("pathTheme", m_store.getThemeDirectory(m_headers.get("User-Agent")));
		root.setAttribute("displayDescription", m_store.getProperty("EPG.displayDescription", "All"));

		if ( (!isLocal() && !m_store.isPreviewRemoteAvailable()) || 
				(isLocal() && !m_store.isPreviewAvailable()) )
			root.setAttribute("preview", "0");
		else
			root.setAttribute("preview", "1");

		CaptureDeviceList devList = CaptureDeviceList.getInstance();
		root.setAttribute("warningDevice", devList.getDeviceCount() > 0 ? "0" : "1");

		DateFormatter2 dateFormatter2 = new DateFormatter2();
		String value = "";
		String epgLoadDesc = "";
		if (guideStore.getEpgLoadTime() != Long.MAX_VALUE)
		{
			value = dateFormatter2.formatDateTime(new Date(guideStore.getEpgLoadTime()));
			root.setAttribute("epgLastLoadTime", value);
			epgLoadDesc = "EPG last loaded on:<br>" + value + "<br><br>";
		}
		epgLoadDesc += "Next EPG load:<br>";
		if (m_store.m_nextEpgLoadTime != Long.MAX_VALUE)
		{
			value = dateFormatter2.formatDateTime(new Date(m_store.m_nextEpgLoadTime));
			root.setAttribute("epgNextLoadTime", value);
		}
		if ( m_store.isEpgLoading() )
		{
			epgLoadDesc += "In progress";
			root.setAttribute("epgLoadStatus", "1");
		}
		else if (m_store.m_nextEpgLoadTime <= Calendar.getInstance().getTimeInMillis())
		{
			epgLoadDesc += "Waiting on condition";
			root.setAttribute("epgLoadStatus", "2");
		}
		else if ( (m_store.m_nextEpgLoadTime == Long.MAX_VALUE) || !m_store.m_bPrevEpgLoadEnabled )
		{
			epgLoadDesc += "No schedule";
			root.setAttribute("epgLoadStatus", "3");
		}
		else
		{
			epgLoadDesc += value;
			root.setAttribute("epgLoadStatus", "0");
		}
		root.setAttribute("epgLoadDesc", epgLoadDesc);

		// The capture types.
		Vector<String> captureTypes = m_store.getCaptureTypes();
		for (int x = 0; x < captureTypes.size(); x++)
		{
			Element capTypeEl = doc.createElement("captureType");
			capTypeEl.setAttribute("name", captureTypes.get(x));
			root.appendChild(capTypeEl);
		}

		// The pages.
		int page = m_urlData.getParameterInt("page", -1);	// page number starts at 1.

		int channelsPerPageRequested = m_urlData.getParameterInt("channelsPerPage", -1);
		if (channelsPerPageRequested < 1)
			m_urlData.getCookieInt("EpgChannelsPerPage", -1);
		int channelsPerPageRequested2 = channelsPerPageRequested;
		int channelsPerPageDefault = m_store.getPropertyInt("EPG.channelsPerPage", 50);
		if (channelsPerPageRequested2 < 1)
			channelsPerPageRequested2 = channelsPerPageDefault;
		if (channelsPerPageRequested2 < 1)
			channelsPerPageRequested2 = 1;
		int channelsPerPage = channelsPerPageRequested2;
		if (channelsPerPage > channelMap.size())
			channelsPerPage = channelMap.size();

		if (useItemIdChannel != null)
		{
			int channelIndex = guideStore.getChannelIndex(useItemIdChannel);
			if (channelIndex < 0)
			{
				page = 1;
				root.setAttribute("scrolltoChannel", "1");
			}
			else
			{
				page = channelIndex / channelsPerPage + 1;	// One-based index.
				int colNum = (channelIndex - channelsPerPage * (page - 1)) * daysPerPage + 1;
				root.setAttribute("scrolltoChannel", Integer.toString(colNum));
			}
		}
		else
		{
			if (page < 1)
			{
				String channel = m_urlData.getParameter("channel");
				if ( (channel == null) || channel.isEmpty() )
					page = 1;
				else
				{
					int channelIndex = guideStore.getChannelIndex(channel);
					if (channelIndex < 0)
						page = 1;
					else
						page = channelIndex / channelsPerPage + 1;	// One-based index.
				}
			}
			else if (channelsPerPage >= channelMap.size())
				page = 1;
			else
			{
				int maxPage = channelMap.size() / channelsPerPage + ((channelMap.size() % channelsPerPage) > 0 ? 1 : 0);
				if (page > maxPage)
					page = maxPage;
			}
			root.setAttribute("scrolltoChannel", "1");
		}
		String link = "/servlet/" + m_urlData.getServletClass() + "?action=12&";
		int pageCount = 0;
		for (int i=0; i<channelMap.size(); i += channelsPerPage)
		{
			int pageNumber = i / channelsPerPage + 1;
			Element elm = doc.createElement("page");
			elm.setAttribute("number", Integer.toString(pageNumber));
			if (page == pageNumber)
			{
				elm.setAttribute("name", "(" + Integer.toString(pageNumber) + ")");
				elm.setAttribute("selected", "1");
			}
			else
				elm.setAttribute("name", Integer.toString(pageNumber));
			int indexTo = i + channelsPerPage - 1;
			if (indexTo >= channelMap.size())
				indexTo = channelMap.size() - 1;
			elm.setAttribute("title", (channelsPerPage > 1) ? channelMap.get(i)[0] + " to " + channelMap.get(indexTo)[0] : channelMap.get(i)[0]);

			Element elmUrl = doc.createElement("url");
			Text urlText = doc.createTextNode(link + "year=" + year + "&month=" + month + "&day=" + day + "&page=" + pageNumber + "&channelsPerPage=" + channelsPerPageRequested + "&daysPerPage=" + daysPerPageRequested);
			elmUrl.appendChild(urlText);
			elm.appendChild(elmUrl);
			root.appendChild(elm);

			pageCount++;
		}
		root.setAttribute("page", Integer.toString(page));
		root.setAttribute("pageCount", Integer.toString(pageCount));
		root.setAttribute("channelsPerPage", Integer.toString(channelsPerPageRequested2));
		root.setAttribute("channelsPerPageActual", Integer.toString(channelsPerPage));
		root.setAttribute("daysPerPage", Integer.toString(daysPerPageRequested2));
		root.setAttribute("daysPerPageActual", Integer.toString(daysPerPage));

		// Day links.
		Vector<String[]> links = guideStore.getEpgLinks(start);
		for (int x = 0; x < links.size(); x++)
		{
			String[] data = (String[]) links.get(x);

			Element dayEl = doc.createElement("days");
			dayEl.setAttribute("name", data[1]);
			dayEl.setAttribute("title", data[2]);

			Element dayUrl = doc.createElement("url");
			Text dayUrlTest = doc.createTextNode(link + data[0] + "&page=" + page + "&channelsPerPage=" + channelsPerPageRequested + "&daysPerPage=" + daysPerPageRequested);
			dayUrl.appendChild(dayUrlTest);

			dayEl.appendChild(dayUrl);

			root.appendChild(dayEl);
		}

		// Days per page.
		Element daysPerPageElement = doc.createElement("daysPerPage");
		daysPerPageElement.setAttribute("name", "Days");
		daysPerPageElement.setAttribute("value", "-1");
		if (-1 == daysPerPageRequested)
			daysPerPageElement.setAttribute("selected", "1");
		root.appendChild(daysPerPageElement);
		daysPerPageElement = doc.createElement("daysPerPage");
		daysPerPageElement.setAttribute("name", "All");
		daysPerPageElement.setAttribute("value", Integer.toString(Integer.MAX_VALUE));
		if (Integer.MAX_VALUE == daysPerPageRequested)
			daysPerPageElement.setAttribute("selected", "1");
		root.appendChild(daysPerPageElement);
		for (int i=1; i<=links.size(); i++)
		{
			daysPerPageElement = doc.createElement("daysPerPage");
			daysPerPageElement.setAttribute("name", (i == daysPerPageDefault) ? Integer.toString(i) + "*" : Integer.toString(i));
			daysPerPageElement.setAttribute("value", Integer.toString(i));
			if (i == daysPerPageRequested)
				daysPerPageElement.setAttribute("selected", "1");
			root.appendChild(daysPerPageElement);
		}

		// Channels per page.
		Element channelsPerPageElement = doc.createElement("channelsPerPage");
		channelsPerPageElement.setAttribute("name", "Channels");
		channelsPerPageElement.setAttribute("value", "-1");
		if (-1 == channelsPerPageRequested)
			channelsPerPageElement.setAttribute("selected", "1");
		root.appendChild(channelsPerPageElement);
		channelsPerPageElement = doc.createElement("channelsPerPage");
		channelsPerPageElement.setAttribute("name", "All");
		channelsPerPageElement.setAttribute("value", Integer.toString(Integer.MAX_VALUE));
		if (Integer.MAX_VALUE == channelsPerPageRequested)
			channelsPerPageElement.setAttribute("selected", "1");
		root.appendChild(channelsPerPageElement);
		for (int i=1; i<=channelMap.size(); i++)
		{
			channelsPerPageElement = doc.createElement("channelsPerPage");
			channelsPerPageElement.setAttribute("name", (i == channelsPerPageDefault) ? Integer.toString(i) + "*" : Integer.toString(i));
			channelsPerPageElement.setAttribute("value", Integer.toString(i));
			if (i == channelsPerPageRequested)
				channelsPerPageElement.setAttribute("selected", "1");
			root.appendChild(channelsPerPageElement);
		}

		//
		// Channel headers.
		//
		String sChannelImageDirectory = m_store.getProperty("path.httproot", "http") + "/themes/" + m_store.getThemeDirectory(m_headers.get("User-Agent")) + "/images/channels/";
		String sChannelImageWidth = Integer.toString(m_store.getPropertyInt("channel.logoWidth", 64));
		String sChannelShow = m_store.getProperty("channel.logo", "1");
		boolean bShowChannelImage = !sChannelShow.equals("0") && !m_store.getProperty("channel.useLogo", "2").equals("0");
		boolean bChannelImage = false;
		if (--page < 0)	// Change to zero-based index.
			page = 0;
		int fromChannel = page * channelsPerPage;
		if (fromChannel >= (channelMap.size()))
			fromChannel = channelMap.size() - 1;
		root.setAttribute("startChannel", channelMap.get(fromChannel)[0]);
		int toChannel = fromChannel + channelsPerPage;	// Not including toChannels.
		if (toChannel > channelMap.size())
			toChannel = channelMap.size();

		for (int x = fromChannel; x < toChannel; x++)
		{
			String[] map = (String[]) channelMap.get(x);

			Calendar channelCalendar = (Calendar) start.clone();
			channelCalendar.set(Calendar.HOUR_OF_DAY, 6);
			channelCalendar.set(Calendar.MINUTE, 0);
			channelCalendar.set(Calendar.SECOND, 0);
			channelCalendar.set(Calendar.MILLISECOND, 0);
			for (int j = 0; j < daysPerPage; j++)
			{
				Element channel = doc.createElement("channel");
				channel.setAttribute("id", map[0]);
				channel.setAttribute("date", dateFormatter2.formatDateMonth(channelCalendar.getTime()));
				channel.setAttribute("day-week", dateFormatter2.formatDayWeek(channelCalendar.getTime()));
				channel.setAttribute("channel-day", getChannelDay(map[0], channelCalendar.getTime(), dateFormatter2));

				Element disName = doc.createElement("display-name");
				Text chaName = doc.createTextNode(map[0]);
				disName.appendChild(chaName);
				channelCalendar.add(Calendar.DAY_OF_MONTH, 1);

				channel.appendChild(disName);

				bChannelImage = Channel.getChannelLogoXml(channel, map[0], sChannelImageDirectory, sChannelImageWidth, sChannelShow, bShowChannelImage, bChannelImage);

				root.appendChild(channel);
			}
		}
		if (bChannelImage)
			root.setAttribute("channelImage", "1");

		//Calendar labelCalendar = Calendar.getInstance();
		Calendar labelCalendar;
		labelCalendar = (Calendar) start.clone();
		labelCalendar.set(Calendar.HOUR_OF_DAY, 6);
		labelCalendar.set(Calendar.MINUTE, 0);
		labelCalendar.set(Calendar.SECOND, 0);
		labelCalendar.set(Calendar.MILLISECOND, 0);
		for (int x = 0; x < 24; x++)
		{
			Element timeLabel = doc.createElement("timeLabel");
			timeLabel.setAttribute("hour", Integer.toString(labelCalendar.get(Calendar.HOUR_OF_DAY)));
			//timeLabel.setAttribute("hour2", (hoursPerPage <= 24) ? dateFormatter2.formatHour(labelCalendar.getTime()) : dateFormatter2.formatDayHour(labelCalendar.getTime()));
			timeLabel.setAttribute("hour2", dateFormatter2.formatHour(labelCalendar.getTime()));
			root.appendChild(timeLabel);
			labelCalendar.add(Calendar.HOUR, 1);
		}

		SimpleDateFormat dateFormater = new SimpleDateFormat("yyyyMMddHHmmss Z");
		StringBuffer dateBuff = new StringBuffer();

		Calendar startTime = Calendar.getInstance();
		startTime.set(Calendar.MILLISECOND, 0);

		//
		// Output the programs
		//
		for (int x = fromChannel; x < toChannel; x++)
		{
			String[] map = (String[]) channelMap.get(x);

			String channelName = map[0];

			if (!wsChannels.contains(map[0]))
			{
				channelName = "Not Mapped";
			}
			else
			{
				Calendar programStartCalendar = (Calendar) start.clone();
				Calendar programEndCalendar = (Calendar) programStartCalendar.clone();
				programEndCalendar.add(Calendar.DAY_OF_MONTH, 1);
				for (int j = 0; j < daysPerPage; j++)
				{
					String channelDay = getChannelDay(map[0], programStartCalendar.getTime(), dateFormatter2);
					GuideItem[] programs = guideStore.getProgramsInc(programStartCalendar.getTime(), programEndCalendar.getTime(), map[1]);

					for (int y = 0; y < programs.length; y++)
					{
						GuideItem item = programs[y];

						///////////////////////////////////////////////////////////////////////////////////////////////////////////
						// add start padding if needed
						//
						programStartCalendar.add(Calendar.SECOND, 1);
						startTime.setTime(item.getStart());
						long pastStart = startTime.getTime().getTime() - programStartCalendar.getTime().getTime();
						if (y == 0 && pastStart > 0)
						{
							Element program_PH = doc.createElement("programme");

							dateBuff = new StringBuffer();
							dateFormater.format(programStartCalendar.getTime(), dateBuff, new FieldPosition(0));
							program_PH.setAttribute("start", dateBuff.toString());
							program_PH.setAttribute("start2", dateFormatter2.formatTime(programStartCalendar.getTime()));

							dateBuff = new StringBuffer();
							dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
							program_PH.setAttribute("stop", dateBuff.toString());
							program_PH.setAttribute("stop2", dateFormatter2.formatTime(item.getStart()));

							program_PH.setAttribute("channel", channelName);
							program_PH.setAttribute("channel-day", channelDay);

							Element titleElement = doc.createElement("title");
							Text titleText = doc.createTextNode("EMPTY");
							titleElement.appendChild(titleText);
							program_PH.appendChild(titleElement);

							Element subTitleElement = doc.createElement("sub-title");
							Text subTitleText = doc.createTextNode("empty");
							subTitleElement.appendChild(subTitleText);
							program_PH.appendChild(subTitleElement);

							Element catElement = doc.createElement("category2");
							Text catText = doc.createTextNode("epgProgramIsEmpty");
							catElement.appendChild(catText);
							program_PH.appendChild(catElement);

							Element descElement = doc.createElement("desc");
							Text descText = doc.createTextNode("empty");
							descElement.appendChild(descText);
							program_PH.appendChild(descElement);

							Element lengthElement = doc.createElement("length");
							lengthElement.setAttribute("units", "minutes");
							Text lengthText = doc.createTextNode(new Long(pastStart / (1000 * 60) + 1).toString());
							lengthElement.appendChild(lengthText);
							program_PH.appendChild(lengthElement);

							root.appendChild(program_PH);
						}
						programStartCalendar.add(Calendar.SECOND, -1);
						///////////////////////////////////////////////////////////////////////////////////////////////////////////

						///////////////////////////////////////////////////////////////////////////////////////////////////////////
						// Add padding item if needed
						//
						if (y > 0)
						{
							long skip = item.getStart().getTime() - (programs[y - 1].getStart().getTime() + (programs[y - 1].getDuration() * 1000 * 60));
							if (skip > 0)
							{
								//DVScheduler.log("Skipping : " + skip);

								Element program_PH = doc.createElement("programme");

								dateBuff = new StringBuffer();
								dateFormater.format(programs[y - 1].getStop(), dateBuff, new FieldPosition(0));
								program_PH.setAttribute("start", dateBuff.toString());
								program_PH.setAttribute("start2", dateFormatter2.formatTime(programs[y - 1].getStop()));

								dateBuff = new StringBuffer();
								dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
								program_PH.setAttribute("stop", dateBuff.toString());
								program_PH.setAttribute("stop2", dateFormatter2.formatTime(item.getStart()));

								program_PH.setAttribute("channel", channelName);
								program_PH.setAttribute("channel-day", channelDay);

								Element titleElement = doc.createElement("title");
								Text titleText = doc.createTextNode("EMPTY");
								titleElement.appendChild(titleText);
								program_PH.appendChild(titleElement);

								Element subTitleElement = doc.createElement("sub-title");
								Text subTitleText = doc.createTextNode("empty");
								subTitleElement.appendChild(subTitleText);
								program_PH.appendChild(subTitleElement);

								Element catElement = doc.createElement("category2");
								Text catText = doc.createTextNode("epgProgramIsEmpty");
								catElement.appendChild(catText);
								program_PH.appendChild(catElement);

								Element descElement = doc.createElement("desc");
								Text descText = doc.createTextNode("empty");
								descElement.appendChild(descText);
								program_PH.appendChild(descElement);

								Element lengthElement = doc.createElement("length");
								lengthElement.setAttribute("units", "minutes");
								Text lengthText = doc.createTextNode(new Long(skip / (1000 * 60)).toString());
								lengthElement.appendChild(lengthText);
								program_PH.appendChild(lengthElement);

								root.appendChild(program_PH);
							}
						}
						///////////////////////////////////////////////////////////////////////////////////////////////////////////

						Element program = doc.createElement("programme");

						long epgStart = programStartCalendar.getTimeInMillis();
						//long epgEnd = epgStart + 24 * 3600 * 1000;
						long itemStart = item.getStart().getTime();
						//long itemEnd = itemStart + item.getDuration() * 60 * 1000;
						boolean bContinued = false;
						int adjustedDuration = 0;
						dateBuff = new StringBuffer();
						if (epgStart <= itemStart)
						{
							dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
							program.setAttribute("start", dateBuff.toString());
							program.setAttribute("start2", dateFormatter2.formatTime(item.getStart()));
						}
						else
						{
							bContinued = true;
							programStartCalendar.add(Calendar.SECOND, 1);
							adjustedDuration = (int)((item.getStop().getTime() - epgStart) / 1000 / 60);
							dateFormater.format(programStartCalendar.getTime(), dateBuff, new FieldPosition(0));
							program.setAttribute("start", dateBuff.toString());
							program.setAttribute("start2", dateFormatter2.formatTime(programStartCalendar.getTime()));
							programStartCalendar.add(Calendar.SECOND, -1);
						}
						/*
					if (itemEnd > epgEnd)
					{
						bContinued = true;
						adjustedDuration = (int)(epgEnd - itemStart) / 1000 / 60;
						dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
						program.setAttribute("start", dateBuff.toString());
						program.setAttribute("start2", dateFormatter2.formatTime(item.getStart()));
					}
						 */
						program.setAttribute("start3", dateFormatter2.formatTime(item.getStart()));

						dateBuff = new StringBuffer();
						dateFormater.format(item.getStop(), dateBuff, new FieldPosition(0));
						program.setAttribute("stop", dateBuff.toString());
						program.setAttribute("stop2", dateFormatter2.formatTime(item.getStop()));

						program.setAttribute("channel", channelName);
						program.setAttribute("channel-day", channelDay);
						program.setAttribute("id", item.toString());

						if (guideStore.isInEpgExclusion(item, channelName))
						{
							program.setAttribute("excludedAA", "1");
						}
						else
						{
							program.setAttribute("excludedAA", "0");
						}

						if (item.isAutoMark())
						{
							program.setAttribute("matchedAA", "1");
						}
						else
						{
							program.setAttribute("matchedAA", "0");
						}

						Element titleElement = doc.createElement("title");
						Text titleText = doc.createTextNode(bContinued ? "(continued) " + item.getName() : item.getName());
						titleElement.appendChild(titleText);
						program.appendChild(titleElement);

						if (!item.getSubName().isEmpty())
						{
							Element subTitleElement = doc.createElement("sub-title");
							Text subTitleText = doc.createTextNode(item.getSubName());
							subTitleElement.appendChild(subTitleText);
							program.appendChild(subTitleElement);
						}

						Vector<String> categories = item.getCategories();
						if ( (categories != null) && !categories.isEmpty() )
						{
							for (int i=0; i<categories.size(); i++)
							{
								Element catElement = doc.createElement("category");
								Text catText = doc.createTextNode(categories.get(i));
								catElement.appendChild(catText);
								program.appendChild(catElement);

								catElement = doc.createElement("category2");
								String cat2 = categories.get(i);
								for (int x2 = 0; x2 < cat2.length(); x2++)
								{
									char charAt = cat2.charAt(x2);

									if ( !((charAt >= 'a' && charAt <= 'z') || 
											(charAt >= 'A' && charAt <= 'Z') ||
											(charAt >= '0' && charAt <= '9') || 
											(charAt == '_') || 
											(charAt == '-')) )
									{
										cat2 = cat2.replace(charAt, '-');
									}
								}
								catText = doc.createTextNode(cat2);
								catElement.appendChild(catText);
								program.appendChild(catElement);
							}
						}
						else
						{
							Element catElement = doc.createElement("category2");
							Text catText = doc.createTextNode("epgProgramHasNoCategory");
							catElement.appendChild(catText);
							program.appendChild(catElement);
						}

						if (!item.getDescription().isEmpty())
						{
							Element descElement = doc.createElement("desc");
							Text descText;
							if ((quick == 1) && (item.getDescription().length() > 20))
								descText = doc.createTextNode(item.getDescription().substring(0, 20));
							else
								descText = doc.createTextNode(item.getDescription());
							descElement.appendChild(descText);
							program.appendChild(descElement);
						}

						Element lengthElement = doc.createElement("length");
						lengthElement.setAttribute("units", "minutes");
						String sAdjustedDuration = Integer.toString(adjustedDuration);
						String sDuration = Integer.toString(item.getDuration());
						//lengthElement.setAttribute("adjusted", sAdjustedDuration);
						lengthElement.setAttribute("real", sDuration);
						//lengthElement.setAttribute("pixel", Integer.toString((bContinued ? adjustedDuration : item.getDuration()) * height / 60));
						Text lengthText = doc.createTextNode(bContinued ? sAdjustedDuration : sDuration);
						lengthElement.appendChild(lengthText);
						program.appendChild(lengthElement);

						String detailsUrl = "/servlet/EpgDataRes?action=06&id=" + item.toString() + "&channel=" + Formatter.encodeURL(map[0], "UTF-8");
						Element infoUrlElement = doc.createElement("detailsUrl");
						Text infoUrlText = doc.createTextNode(detailsUrl);
						infoUrlElement.appendChild(infoUrlText);
						program.appendChild(infoUrlElement);

						root.appendChild(program);
					}
					programStartCalendar.add(Calendar.DAY_OF_MONTH, 1);
					programEndCalendar.add(Calendar.DAY_OF_MONTH, 1);
				}
			}
		}

		//
		// Add Schedules
		//
		Set<ScheduleItem> scheduleItems = new HashSet<ScheduleItem>();
		Text textNode = null;
		Element elementNaode = null;
		int totalDuration = 0;
		for (int x = fromChannel; x < toChannel; x++)
		{
			String[] map = (String[]) channelMap.get(x);

			String channelName = map[0];

			if (channelName != null)
			{
				Vector<ScheduleItem> items = new Vector<ScheduleItem>();
				Calendar scheduleStartCalendar = (Calendar) start.clone();
				Calendar scheduleEndCalendar = (Calendar) scheduleStartCalendar.clone();
				scheduleEndCalendar.add(Calendar.DAY_OF_MONTH, 1);
				for (int j = 0; j < daysPerPage; j++)
				{
					String channelDay = getChannelDay(map[0], scheduleStartCalendar.getTime(), dateFormatter2);
					items.clear();
					m_store.getSchedulesWhenInc(scheduleStartCalendar.getTime(), scheduleEndCalendar.getTime(), channelName, items);

					for (int y = 0; y < items.size(); y++)
					{
						ScheduleItem item = items.get(y);

						Element schedule = doc.createElement("schedule");

						dateBuff = new StringBuffer();
						dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
						schedule.setAttribute("start", dateBuff.toString());
						schedule.setAttribute("start2", dateFormatter2.formatTime(item.getStart()));
						schedule.setAttribute("startHour", Integer.toString(item.getStartHour()));
						schedule.setAttribute("startMinute", Integer.toString(item.getStartMinute()));

						dateBuff = new StringBuffer();
						dateFormater.format(item.getStop(), dateBuff, new FieldPosition(0));
						schedule.setAttribute("stop", dateBuff.toString());
						schedule.setAttribute("stop2", dateFormatter2.formatTime(item.getStop()));
						schedule.setAttribute("stopHour", Integer.toString(item.getStopHour()));
						schedule.setAttribute("stopMinute", Integer.toString(item.getStopMinute()));

						schedule.setAttribute("duration", new Integer(item.getDuration()).toString());

						schedule.setAttribute("channel", channelName);
						schedule.setAttribute("channel-day", channelDay);

						schedule.setAttribute("name", item.getName());

						String child_type = item.getTypeStr();
						schedule.setAttribute("type", child_type);

						schedule.setAttribute("error", Integer.toString(item.getErrorCount()));

						startTime.setTime(item.getStart());
						long pastStart = startTime.getTime().getTime() - scheduleStartCalendar.getTime().getTime();

						elementNaode = doc.createElement("id");
						textNode = doc.createTextNode(item.toString());
						elementNaode.appendChild(textNode);
						schedule.appendChild(elementNaode);

						Element topElement = doc.createElement("from_top");
						Text topText = doc.createTextNode(new Long(pastStart / (1000 * 60)).toString());
						topElement.appendChild(topText);
						schedule.appendChild(topElement);

						Element stateElement = doc.createElement("itemState");
						Text stateText = doc.createTextNode(new Integer(item.getState()).toString());
						stateElement.appendChild(stateText);
						schedule.appendChild(stateElement);

						if (!scheduleItems.add(item))
							schedule.setAttribute("duplicate", "1");
						else
						{
							totalDuration += item.getDuration();

							if (item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.ERROR)
							{
								String showLogUrl = "/servlet/ScheduleDataRes?action=07&id=" + item.toString();
								addActionUrl(showLogUrl, item, "LL", schedule, doc);               

								String delUrl = "/servlet/ScheduleDataRes?action=04&id=" + item.toString();
								addActionUrl(delUrl, item, "D", schedule, doc);

								//if(item.getType() != ScheduleItem.EPG)
								{
									String editUrl = "/servlet/ScheduleDataRes?action=01&id=" + item.toString() + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
									addActionUrl(editUrl, item, "E", schedule, doc);

									String edit2Url = "/servlet/ScheduleDataRes?action=02&id=" + item.toString() + "&day=" + startTime.get(Calendar.DAY_OF_MONTH) + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
									addActionUrl(edit2Url, item, "E2", schedule, doc);

									String changeDurationUrl = "javascript:inputForm(\"" + item.toString() + "\"," + item.getDuration() + "," + item.getStopHour() + "," + item.getStopMinute() + ",-1)";
									addActionUrl(changeDurationUrl, item, "+2", schedule, doc);

									String addTimeUrl = "/servlet/ScheduleDataRes?action=05&id=" + item.toString();
									addActionUrl(addTimeUrl, item, "+", schedule, doc);

									if (item.getType() == ScheduleItem.DAILY
											|| item.getType() == ScheduleItem.WEEKLY
											|| item.getType() == ScheduleItem.MONTHLY
											|| item.getType() == ScheduleItem.WEEKDAY)
									{
										schedule.setAttribute("skip", "1");
										String skipUrl = "/servlet/ScheduleDataRes?action=06&id=" + item.toString();
										addActionUrl(skipUrl, item, "SK", schedule, doc);
									}
								}
							}
							else if (item.getState() == ScheduleItem.RUNNING)
							{
								String showLogUrl = "/servlet/ScheduleDataRes?action=07&id=" + item.toString();
								addActionUrl(showLogUrl, item, "LL", schedule, doc);               

								String stopUrl = "/servlet/ScheduleDataRes?action=09&id=" + item.toString();
								addActionUrl(stopUrl, item, "S", schedule, doc);

								String edit2Url = "/servlet/ScheduleDataRes?action=02&id=" + item.toString() + "&day=" + startTime.get(Calendar.DAY_OF_MONTH) + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
								addActionUrl(edit2Url, item, "E2", schedule, doc);

								String changeDurationUrl = "javascript:inputForm(\"" + item.toString() + "\"," + item.getDuration() + "," + item.getStopHour() + "," + item.getStopMinute() + ",-1)";
								addActionUrl(changeDurationUrl, item, "+2", schedule, doc);

								String addTimeUrl = "/servlet/ScheduleDataRes?action=05&id=" + item.toString();
								addActionUrl(addTimeUrl, item, "+", schedule, doc);      
							}
						}

						root.appendChild(schedule);
					}
					scheduleStartCalendar.add(Calendar.DAY_OF_MONTH, 1);
					scheduleEndCalendar.add(Calendar.DAY_OF_MONTH, 1);
				}
			}

		}

		root.setAttribute("scheduledhour", new Integer(totalDuration/60).toString());
		root.setAttribute("scheduledminute", new Integer(totalDuration%60).toString());

		//
		// Do transform and return data
		//
		String compress = m_store.getProperty("EPG.compress", "Yes");
		XSL transformer = new XSL(doc, "epg-guide.xsl", m_headers, m_urlData);

		// set current page to check point for back
		// Remove the &scrollto=-1 first
		String request = m_urlData.getRequestString();
		request = request.replace("&scrollto=", "&nada=");
		transformer.addCookie("backURL", request);
		//return transformer.doTransform(compress.equals("Yes") ? 1 : 0, true);
		return transformer.transform(compress.equals("Yes") ? 1 : 0, false);
	}

	private void addActionUrl(String url, ScheduleItem item, String name, Element root, Document doc)
	{
		Text textNode = null;
		Element elementNaode = null;
		Element elementNaode02 = null;

		elementNaode = doc.createElement("actionUrl");
		elementNaode.setAttribute("name", name);
		elementNaode.setAttribute("item_id", item.toString());
		textNode = doc.createTextNode(url);
		elementNaode02 = doc.createElement("url");
		elementNaode02.appendChild(textNode);
		elementNaode.appendChild(elementNaode02);
		root.appendChild(elementNaode);
	}

	private String programAddURL(GuideItem item, Calendar startTime, String chanName, int action) throws Exception
	{
		StringBuffer buff = new StringBuffer();
		buff.append("/servlet/ScheduleDataRes?action=" + action);
		buff.append("&channel=" + Formatter.encodeURL(chanName, "UTF-8") + "&id=" + item.toString());

		return buff.toString();
	}

	protected class GuideItem2
	{
		int m_index;
		String m_channel;
		GuideItem m_guideItem;
	}

	private byte[] quicksearchEPG(String templateName, String sortCookie, String defaultSort) throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		String doSearch = m_urlData.getParameter("search");

		StringBuffer buff = new StringBuffer();

		String kwMatch = m_urlData.getParameter("kwMatch");
		if (kwMatch == null)
			kwMatch = "";

		kwMatch = kwMatch.trim();

		String kwSite = m_urlData.getParameter("kwSite");
		int nKwSite = 3;
		if (kwSite == null || kwSite.isEmpty())
			nKwSite = 3;
		else if (kwSite.equals("Title"))
			nKwSite = 1;
		else if (kwSite.equals("Description"))
			nKwSite = 2;

		String catMatch = m_urlData.getParameter("catMatch");
		Set<String> catMatchSet = new HashSet<String>();
		if (catMatch != null)
		{
			String[] catMatches = catMatch.split("\n");
			for (int x = 0; x < catMatches.length; x++)
			{
				catMatchSet.add(catMatches[x].toUpperCase());
			}
		}

		String chanMatch = m_urlData.getParameter("chanMatch");
		Set<String> chanMatchSet = new HashSet<String>();
		if (chanMatch != null)
		{
			String[] chanMatches = chanMatch.split("\n");
			for (int x = 0; x < chanMatches.length; x++)
			{
				chanMatchSet.add(chanMatches[x].toUpperCase());
			}
		}

		String flagMatch = m_urlData.getParameter("flag");
		int flag = 0xffffffff;
		if (flagMatch != null && !flagMatch.isEmpty())
		{
			flag = 0;
			String[] flagMatches = flagMatch.split("\n");
			for (int x = 0; x < flagMatches.length; x++)
			{
				if (flagMatches[x].equals("excluded"))
					flag |= 0x01;
				else if (flagMatches[x].equals("notExcluded"))
					flag |= 0x02;
				else if (flagMatches[x].equals("matched"))
					flag |= 0x04;
				else if (flagMatches[x].equals("notMatched"))
					flag |= 0x08;
			}
			if ( ((flag & 0x01) == 0) && ((flag & 0x02) == 0) )
				flag |= 0x03;
			if ( ((flag & 0x04) == 0) && ((flag & 0x08) == 0) )
				flag |= 0x0c;
		}

		String sort = m_urlData.getParameter("sort");
		if ( (sort == null) || (sort.isEmpty()) )
		{
			sort = m_urlData.getCookie(sortCookie);
			if ( (sort == null) || (sort.isEmpty()) )
				sort = ( (defaultSort != null) && !defaultSort.isEmpty() ) ? defaultSort : "channel";
		}

		HashMap<String, Vector<GuideItem>> results = new HashMap<String, Vector<GuideItem>>();
		int num = 0;

		if (doSearch != null && doSearch.equalsIgnoreCase("yes"))
		{
			num = guide.simpleEpgSearch(kwMatch, nKwSite, catMatchSet, chanMatchSet, flag, results);

			//buff.append("Your quick search for (" + HTMLEncoder.encode(kwMatch) + ") returned " + num + " results.<p>\n");
			buff.append("Found " + num + " items.<p>\n");

			if (results.size() > 0)
			{
				DateFormatter2 dateFormatter2 = new DateFormatter2();

				//buff.append("<span class='areaTitle'>Search Results:</span><br><br>\n");
				buff.append("<table class='epgSearchResults'>\n");
				buff.append("<tr>");
				buff.append("<td>Program Name</td>");
				buff.append("<td>Time</td>");
				buff.append("<td>Duration</td>");
				buff.append("<td>Action</td>");
				buff.append("</tr>");            

				Calendar startTime = Calendar.getInstance();
				if (sort.equals("autoAdd"))
				{
					Map<Long, Integer> autoAddIdToIndex = new HashMap<Long, Integer>();
					Vector<EpgMatch> autoAdds = m_store.getEpgAutoAdds();
					for (int i=0; i<autoAdds.size(); i++)
					{
						EpgMatch autoAdd = autoAdds.get(i);
						autoAddIdToIndex.put( autoAdd.getID(), i );
					}
					Vector<GuideItem2> results2 = new Vector<GuideItem2>();
					Iterator<String> iter = results.keySet().iterator();
					while (iter.hasNext())
					{
						String channel = iter.next();
						Vector<GuideItem> gis = results.get(channel);
						for (int i=0; i<gis.size(); i++)
						{
							GuideItem2 gi2 = new GuideItem2();
							gi2.m_guideItem = gis.get(i);
							gi2.m_channel = channel;
							List<Long> flaggerIds = gi2.m_guideItem.getFavoriteFlaggers();
							if ( (flaggerIds == null) || flaggerIds.isEmpty() )
								gi2.m_index = Integer.MAX_VALUE;
							else
							{
								Integer index = null;
								Iterator<Long> iter2 = flaggerIds.iterator();
								while (iter2.hasNext())
								{
									index = autoAddIdToIndex.get(iter2.next());
									if (index != null)
									{
										gi2.m_index = index;
										break;
									}
									if (index == null)
										gi2.m_index = Integer.MAX_VALUE;
								}
							}
							results2.add( gi2 );
						}
					}

					Comparator<GuideItem2> comparatorAutoAdd = new Comparator<GuideItem2>() {
						public int compare(GuideItem2 g1, GuideItem2 g2) {
							return g1.m_index - g2.m_index;
						}
					};
					Collections.sort(results2, comparatorAutoAdd);

					int previousIndex = -1;
					for (int i=0; i<results2.size(); i++)
					{
						GuideItem2 item = results2.get(i);
						String channel = item.m_channel;
						startTime.setTime(item.m_guideItem.getStart());

						int index = item.m_index;
						if ( index != previousIndex )
						{
							previousIndex = index;
							buff.append("<tr><td colspan='4'><br>\n");
							if (index == Integer.MAX_VALUE)
								buff.append("<span class='areaTitle'>Other:</span><br>\n");
							else
							{
								buff.append("<span class='areaTitle'>Auto-Add: " + 
										HTMLEncoder.encode(autoAdds.get(index).getName2()) + "</span><br>\n");
							}
							buff.append("</td></tr\n");
						}

						String startString = dateFormatter2.formatTime(startTime.getTime());
						String dateString = m_store.dayName.get(new Integer(startTime.get(Calendar.DAY_OF_WEEK))) + ", " + 
								startTime.get(Calendar.DATE) + " " + 
								m_store.monthNameShort.get(new Integer(startTime.get(Calendar.MONTH)));

						buff.append("<tr>\n");
						buff.append("<td class='epgSearchResults'" +
								"desc2=\"" + item.m_guideItem.getDescription().replaceAll("\"", "&quot;") + 
								"\" onMouseMove=\"showTip(event,this," + m_store.getProperty("EPG.balloonDelay", "0") + 
								");\" onMouseOut=\"hideTip();\">" + 
								item.m_guideItem.getName());
						if(item.m_guideItem.getSubName() != null && item.m_guideItem.getSubName().length() > 0)
							buff.append(" (" + item.m_guideItem.getSubName() + ")");
						buff.append("</td>");
						buff.append("<td class='epgSearchResults'>" + dateString + " at " + startString + "</td>");
						buff.append("<td class='epgSearchResults'>" + item.m_guideItem.getDuration() + "</td>");

						buff.append("<td class='epgSearchResults'>");

						String infoUrl = "/servlet/EpgDataRes?action=06&id=" + item.m_guideItem.toString() + "&channel=" + 
								Formatter.encodeURL(channel, "UTF-8");
						buff.append("<span class='programName' title='Show Detail' onClick=\"openDetails('" + 
								infoUrl + "')\">");
						buff.append("Details");
						buff.append("</span> | ");

						String epgUrl = "/servlet/EpgDataRes?action=26" + 
								"&id=" + item.m_guideItem.toString() + 
								"&channel=" + Formatter.encodeURL(channel, "UTF-8");
						buff.append("<a class='infoNav' title='Show in EPG' href='" + epgUrl + "'>EPG</a> | ");

						infoUrl = "/servlet/EpgDataRes?action=27&id=" + item.m_guideItem.toString() + "&channel=" + 
								Formatter.encodeURL(channel, "UTF-8");
						if (guide.isInEpgExclusion(item.m_guideItem, channel))
						{
							buff.append("<a class='infoNav' title='Include in Auto-Add Scan' href='" + infoUrl + "'>Include</a>");
						}
						else
						{
							buff.append("<a class='infoNav' title='Exclude from Auto-Add Scan' href='" + infoUrl + "'>Exclude</a>");
						}

						buff.append("</td>");

						buff.append("</tr>\n");
					}
				}
				else if (sort.equals("title"))
				{
					Map<Long, Integer> autoAddIdToIndex = new HashMap<Long, Integer>();
					Vector<EpgMatch> autoAdds = m_store.getEpgAutoAdds();
					for (int i=0; i<autoAdds.size(); i++)
					{
						autoAddIdToIndex.put( autoAdds.get(i).getID(), i );
					}
					Vector<GuideItem2> results2 = new Vector<GuideItem2>();
					Iterator<String> iter = results.keySet().iterator();
					while (iter.hasNext())
					{
						String channel = iter.next();
						Vector<GuideItem> gis = results.get(channel);
						for (int i=0; i<gis.size(); i++)
						{
							GuideItem2 gi2 = new GuideItem2();
							gi2.m_guideItem = gis.get(i);
							gi2.m_channel = channel;
							/*
							Long flaggerId = gi2.m_guideItem.getFavoriteFlagger();
							gi2.m_index = (flaggerId == null) ? Integer.MAX_VALUE : autoAddIdToIndex.get(flaggerId);
							 */
							results2.add( gi2 );
						}
					}

					Comparator<GuideItem2> comparatorTitle = new Comparator<GuideItem2>() {
						public int compare(GuideItem2 g1, GuideItem2 g2) {
							return g1.m_guideItem.getName().compareToIgnoreCase(g2.m_guideItem.getName());
						}
					};
					Collections.sort(results2, comparatorTitle);

					for (int i=0; i<results2.size(); i++)
					{
						GuideItem2 item = results2.get(i);
						String channel = item.m_channel;
						startTime.setTime(item.m_guideItem.getStart());

						String startString = dateFormatter2.formatTime(startTime.getTime());
						String dateString = m_store.dayName.get(new Integer(startTime.get(Calendar.DAY_OF_WEEK))) + ", " + 
								startTime.get(Calendar.DATE) + " " + 
								m_store.monthNameShort.get(new Integer(startTime.get(Calendar.MONTH)));

						buff.append("<tr>\n");
						buff.append("<td class='epgSearchResults'" +
								"desc2=\"" + item.m_guideItem.getDescription().replaceAll("\"", "&quot;") + 
								"\" onMouseMove=\"showTip(event,this," + m_store.getProperty("EPG.balloonDelay", "0") + 
								");\" onMouseOut=\"hideTip();\">" + 
								item.m_guideItem.getName());
						if(item.m_guideItem.getSubName() != null && item.m_guideItem.getSubName().length() > 0)
							buff.append(" (" + item.m_guideItem.getSubName() + ")");
						buff.append("</td>");
						buff.append("<td class='epgSearchResults'>" + dateString + " at " + startString + "</td>");
						buff.append("<td class='epgSearchResults'>" + item.m_guideItem.getDuration() + "</td>");

						buff.append("<td class='epgSearchResults'>");

						String infoUrl = "/servlet/EpgDataRes?action=06&id=" + item.m_guideItem.toString() + "&channel=" + 
								Formatter.encodeURL(channel, "UTF-8");
						buff.append("<span class='programName' title='Show Detail' onClick=\"openDetails('" + infoUrl + "')\">");
						buff.append("Details");
						buff.append("</span> | ");

						String epgUrl = "/servlet/EpgDataRes?action=26" + 
								"&id=" + item.m_guideItem.toString() + 
								"&channel=" + Formatter.encodeURL(channel, "UTF-8");
						buff.append("<a class='infoNav' title='Show in EPG' href='" + epgUrl + "'>EPG</a> | ");

						infoUrl = "/servlet/EpgDataRes?action=27&id=" + item.m_guideItem.toString() + "&channel=" + 
								Formatter.encodeURL(channel, "UTF-8");
						if (guide.isInEpgExclusion(item.m_guideItem, channel))
						{
							buff.append("<a class='infoNav' title='Include in Auto-Add Scan' href='" + infoUrl + "'>Include</a>");
						}
						else
						{
							buff.append("<a class='infoNav' title='Exclude from Auto-Add Scan' href='" + infoUrl + "'>Exclude</a>");
						}

						buff.append("</td>");

						buff.append("</tr>\n");
					}
				}
				else
				{
					String[] keys = (String[])results.keySet().toArray(new String[0]);
					Arrays.sort(keys);

					for (int y = 0; y < keys.length; y++)
					{
						Vector<?> result = (Vector<?>) results.get(keys[y]);//chanMap[0]);

						if (result.size() > 0)
						{
							buff.append("<tr><td colspan='4'><br>\n");
							buff.append("<span class='areaTitle'>Channel : " + HTMLEncoder.encode(keys[y]) + "</span><br>\n");
							buff.append("</td></tr\n");

							for (int x = 0; x < result.size(); x++)
							{
								GuideItem item = (GuideItem) result.get(x);
								startTime.setTime(item.getStart());

								String startString = dateFormatter2.formatTime(startTime.getTime());
								/*
							int hour = startTime.get(Calendar.HOUR);
							if(hour == 0)
								hour = 12;
							String startString = intToStr(hour) + ":" + intToStr(startTime.get(Calendar.MINUTE)) + " " + store.ampm.get(new Integer(startTime.get(Calendar.AM_PM)));
								 */
								String dateString = m_store.dayName.get(new Integer(startTime.get(Calendar.DAY_OF_WEEK))) + 
										", " + startTime.get(Calendar.DATE) + " " + 
										m_store.monthNameShort.get(new Integer(startTime.get(Calendar.MONTH)));

								buff.append("<tr>\n");
								buff.append("<td class='epgSearchResults'" +
										"desc2=\"" + item.getDescription().replaceAll("\"", "&quot;") + 
										"\" onMouseMove=\"showTip(event,this," + m_store.getProperty("EPG.balloonDelay", "0") + 
										");\" onMouseOut=\"hideTip();\">" + 
										item.getName());
								if(item.getSubName() != null && item.getSubName().length() > 0)
									buff.append(" (" + item.getSubName() + ")");
								buff.append("</td>");
								buff.append("<td class='epgSearchResults'>" + dateString + " at " + startString + "</td>");
								buff.append("<td class='epgSearchResults'>" + item.getDuration() + "</td>");

								buff.append("<td class='epgSearchResults'>");

								String infoUrl = "/servlet/EpgDataRes?action=06&id=" + item.toString() + "&channel=" + 
										Formatter.encodeURL(keys[y], "UTF-8");
								buff.append("<span class='programName' title='Show Detail' onClick=\"openDetails('" + 
										infoUrl + "')\">");
								buff.append("Details");
								buff.append("</span> | ");

								String epgUrl = "/servlet/EpgDataRes?action=26" + 
										"&id=" + item.toString() + 
										"&channel=" + Formatter.encodeURL(keys[y], "UTF-8");
								buff.append("<a class='infoNav' title='Show in EPG' href='" + epgUrl + "'>EPG</a> | ");

								infoUrl = "/servlet/EpgDataRes?action=27&id=" + item.toString() + "&channel=" + 
										Formatter.encodeURL(keys[y], "UTF-8");
								if (guide.isInEpgExclusion(item, keys[y]))
								{
									buff.append("<a class='infoNav' title='Include in Auto-Add Scan' href='" + infoUrl + 
											"'>Include</a>");
								}
								else
								{
									buff.append("<a class='infoNav' title='Exclude from Auto-Add Scan' href='" + infoUrl + 
											"'>Exclude</a>");
								}

								buff.append("</td>");

								buff.append("</tr>\n");
							}
						}
					}
				}

				buff.append("</table><p>\n");
			}
		}

		PageTemplate template = new PageTemplate(m_headers, m_urlData, templateName, true);

		template.replace("$result", buff.toString());

		if (kwMatch.length() > 0)
			template.replace("$kwMatch", HTMLEncoder.encode(kwMatch));
		else
			template.replace("$kwMatch", "");

		/*
		if (catMatch.length() > 0)
			template.replaceAll("$catMatch", HTMLEncoder.encode(catMatch));
		else
			template.replaceAll("$catMatch", ".*");
		 */

		String[] cats = guide.getCategoryStrings();
		StringBuffer bufc = new StringBuffer();
		boolean DefaultOption = (catMatch == null || catMatch.isEmpty() || catMatchSet.contains(""));
		if (cats.length > 0)
		{
			bufc.append("<option value=''" + (DefaultOption ? " Selected" : "") + ">All</option>\n");
			for (int x = 0; x < cats.length; x++)
			{
				bufc.append("<option value='" + cats[x] + "'");
				if ((!DefaultOption) && (catMatchSet.contains(cats[x].toUpperCase())))
				{
					bufc.append(" Selected");
				}
				bufc.append(">" + cats[x] + "</option>\n");
			}
		}
		else
			bufc.append("<option value=''>All</option>\n");

		template.replace("$catList", bufc.toString());

		bufc = new StringBuffer();
		bufc.append("<option value=''" + (nKwSite == 3 ? " Selected" : "") + ">Title and Description</option>\n");
		bufc.append("<option value='Title'" + (nKwSite == 1 ? " Selected" : "") + ">Title only</option>\n");
		bufc.append("<option value='Description'" + (nKwSite == 2 ? " Selected" : "") + ">Description only</option>\n");
		template.replace("$siteList", bufc.toString());

		bufc = new StringBuffer();
		HashMap<String, Channel> channels = m_store.getChannels();
		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		Arrays.sort(keys);
		DefaultOption = (chanMatch == null || chanMatch.isEmpty() || chanMatchSet.contains(""));
		if (keys.length > 0)
		{
			bufc.append("<option value=''" + (DefaultOption ? " Selected" : "") + ">All</option>\n");
			for (int x = 0; x < keys.length; x++)
			{
				bufc.append("<option value='" + keys[x] + "'");
				if ((!DefaultOption) && (chanMatchSet.contains(keys[x].toUpperCase())))
				{
					bufc.append(" Selected");
				}
				bufc.append(">" + keys[x] + "</option>\n");
			}
		}
		else
			bufc.append("<option value=''>All</option>\n");

		template.replace("$chanList", bufc.toString());

		if ( flag == 0xffffffff )
		{
			template.replace("$flagAll", "selected");
		}
		else
		{
			if ( (flag & 0x01) > 0 )
				template.replace("$flagExcluded", "selected");
			if ( (flag & 0x02) > 0 )
				template.replace("$flagNotExcluded", "selected");
			if ( (flag & 0x04) > 0 )
				template.replace("$flagMatched", "selected");
			if ( (flag & 0x08) > 0 )
				template.replace("$flagNotMatched", "selected");
		}

		if ( sort.equals("autoAdd") )
			template.replace("$sortAutoAdd", "selected");
		else if ( sort.equals("title") )
			template.replace("$sortTitle", "selected");
		else
			template.replace("$sortChannel", "selected");

		return template.getPageBytes();
	}

	private byte[] nownextEPG() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		StringBuffer buff = new StringBuffer();

		buff.append("<b>Now Showing (" + (new Date()).toString() + ")</b><p>\n");

		buff.append("<a href='/servlet/EpgDataRes?action=21");
		buff.append("'>Refresh This Screen</a><p>\n");

		buff.append("<table>\n");
		buff.append("<tr>");
		buff.append("<td>&nbsp;</td>");
		buff.append("<td>Program Name</td>");
		buff.append("<td>Time</td>");
		buff.append("</tr>");

		Vector<?> chanMap = guide.getOldChannelMap();
		Set<?> wsChannels = m_store.getChannels().keySet();

		Calendar startTime = Calendar.getInstance();
		Date Now = new Date();

		DateFormatter2 dateFormatter2 = new DateFormatter2();

		for (int y = 0; y < chanMap.size(); y++)
		{
			String[] map = (String[])chanMap.get(y);

			boolean bNext = false;

			if(wsChannels.contains(map[0]))
			{
				GuideItem[] items = guide.getProgramsForChannel(map[1]);

				buff.append("<tr><td colspan='5' bgcolor=#000000>\n");
				buff.append("<span class='areaTitle'>" + HTMLEncoder.encode(map[0]) + "</span><br>\n");
				buff.append("</td></tr\n");

				for (int x = 0; x < items.length; x++)
				{
					GuideItem item = items[x];

					startTime.setTime(item.getStart());

					if ((item.getStart().before(Now) && item.getStop().after(Now)) || bNext)
					{
						boolean bNow = !bNext;
						if (!bNext)
							bNext = true;
						else
							bNext = false;

						String startString = dateFormatter2.formatTime(startTime.getTime());
						/*
						int hour = startTime.get(Calendar.HOUR_OF_DAY);
						if(hour > 12) hour = hour - 12;
						else if(hour == 0) hour = 12;
						String startString = intToStr(hour) + ":" + intToStr(startTime.get(Calendar.MINUTE)) + " " + store.ampm.get(new Integer(startTime.get(Calendar.AM_PM)));
						 */

						buff.append("<tr>\n");
						buff.append("<td class='epgSearchResults'><b>" + (bNow ? "NOW" : "NEXT") + "</b></td>");

						buff.append("<td class='epgSearchResults'>");
						String infoUrl = "/servlet/EpgDataRes?action=06&id=" + item.toString() + "&channel=" + Formatter.encodeURL(map[0], "UTF-8");
						buff.append("<span class='programName' onClick=\"openDetails('" + infoUrl + "')\">");
						buff.append("<b>" + item.getName() + "</b>");
						buff.append("</span>");
						buff.append("</td>");

						buff.append("<td class='epgSearchResults'>" + startString + " (" + item.getDuration() + ")</td>");

						buff.append("</tr>\n");

						if (bNow)
							buff.append("<tr><td colspan=5><font size=2>" + item.getDescription() + "</font></td></tr>\n");
						if (!bNow)
							buff.append("<tr><td colspan=5>&nbsp;</td></tr>\n");

					}
				}
			}
		}

		buff.append("</table><p>\n");

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-nownext.html", true);

		template.replace("$result", buff.toString());

		return template.getPageBytes();
	}

	private byte[] toggleEpgExclusion(boolean bRefresh) throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		String channel = m_urlData.getParameter("channel");
		String id = m_urlData.getParameter("id");

		String epgChan = guide.getEpgChannelIdFromMap(channel);
		GuideItem item = guide.getProgram(epgChan, id);

		if (item == null)
		{
			return ("Invalid item.").getBytes("UTF-8");
		}

		boolean bExcluded = guide.toggleEpgExclusion(item, channel);
		guide.saveEpgExclusion();
		guide.saveEpg();

		if (bRefresh)
		{
			return getRedirUrlRedir(null).getBytes("UTF-8");
		}
		else
		{
			DVScheduler.log("Result " + bExcluded);
			return addLength("EPGSchedule|" + (bExcluded ? "1" : "0")).getBytes("UTF-8");
		}
	}

	protected String addLength(String text)
	{
		return String.format("%1$04d", text.length()+4) + text;
	}

	private byte[] showProgramDetails() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		String channel = m_urlData.getParameter("channel");
		String id = m_urlData.getParameter("id");

		String epgChan = guide.getEpgChannelIdFromMap(channel);

		GuideItem item = guide.getProgram(epgChan, id);

		StringBuffer buff = new StringBuffer(256);

		if (item == null)
		{
			buff.append("No info available for the requested Item.");
			return buff.toString().getBytes("UTF-8");
		}

		buff.append("<table align='center' border='0' cellpadding='4' cellspacing='2' width='90%'>");

		buff.append("<tr><td align='right' valign='top' colspan='2'>");

		Calendar startTime = Calendar.getInstance();
		startTime.setTime(item.getStart());
		buff.append("<a href='#' onClick=\"addItem('" + programAddURL(item, startTime, channel, 18) + "');\">");
		buff.append("<img align='absmiddle' src='/images/addsch.png' border='0' title='New Schedule' alt='New Schedule' width='24' height='24'>");
		buff.append("</a>\n");

		buff.append(" | <a href='#' onClick=\"addItem('" + programAddURL(item, startTime, channel, 12) + "');\">");
		buff.append("<img align='absmiddle' src='/images/add.png' border='0' title='Add Schedule' alt='Add Schedule' width='24' height='24'>");
		buff.append("</a>\n");

		String searchURL = "/servlet/EpgAutoAddDataRes?action=14" + 
				"&itemID=" + Formatter.encodeURL(item.toString(), "UTF-8") +
				"&chan=" + Formatter.encodeURL(channel, "UTF-8");
		buff.append(" | <a href='#' onClick=\"doSearch('" + searchURL + "');\">");
		buff.append("<img align='absmiddle' src='/images/reload.png' border='0' title='Create Auto-Add Item' alt='Create Auto-Add Item' width='24' height='24'>");
		buff.append("</a>\n");

		String toggleURL = "/servlet/EpgDataRes?action=27" + 
				"&channel=" + Formatter.encodeURL(channel, "UTF-8") + "&id=" + item.toString();
		buff.append(" | <a href='#' onClick=\"addItem('" + toggleURL + "');\">");
		buff.append("<img align='absmiddle' src='/images/toggleExclusion24.png' border='0' title='Toggle Auto-Add Exclusion' alt='Toggle Auto-Add Exclusion' width='24' height='24'>");
		buff.append("</a>\n");

		buff.append("</td></tr>\n");

		buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Name:</b></td><td width='100%'>" + item.getName());

		if (item.getSubName().length() > 0)
			buff.append(" (" + item.getSubName() + ")");

		buff.append("</td></tr>");

		SimpleDateFormat df = new SimpleDateFormat("h:mm aa");
		buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Time:</b></td><td>" + 
				df.format(item.getStart()) + "</td></tr>\n");

		df = new SimpleDateFormat("EEE MMM d");
		buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Date:</b></td><td>" + 
				df.format(item.getStart()) + "</td></tr>\n");      

		buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Duration:</b></td><td>" + item.getDuration() + " min</td></tr>\n");

		buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Channel:</b></td><td>" + channel + "</td></tr>\n");

		// add source URL link
		if (item.getURL().length() > 0)
		{
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Source:</b></td><td><font size=2><b>\n");
			buff.append("<a href='" + item.getURL() + "' target='_sourceData'>More Info</a>");
			buff.append("</b></font></td></tr>");
		} 

		// add category
		if (item.getCategoriesSize() > 0)
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Category:</b></td><td>" + item.getCategories2() + "</td></tr>\n");

		String sInfo = "";
		String[] searches = DVScheduler.getArray(m_store.getProperty("EPG.webSearch", "IMDB->http://www.imdb.com/find?q={title}\nWikipedia->http://en.wikipedia.org/wiki/Special:Search?search={title}"));
		String title = Formatter.encodeURL(item.getName(), "UTF-8");
		String subTitle = Formatter.encodeURL(item.getSubName(), "UTF-8");
		for (int i=0; i<searches.length; i++)
		{
			String[] searchEngine = searches[i].split("->");
			if (searchEngine.length != 2)
				continue;

			sInfo += "<a target='_" + searchEngine[0] + "' href='" + searchEngine[1].replace("{title}", title).replace("{subtitle}", subTitle) + "'>" + searchEngine[0] + "</a> ";
		}

		if (guide.isInEpgExclusion(item, channel))
			sInfo += "*EXCLUDED* ";
		if (item.isAutoMark())
			sInfo += "*FAVORITE* ";
		if (item.getHighDef())
			sInfo = sInfo + "HDTV ";
		if (item.getWidescreen())
			sInfo = sInfo + "WS ";
		if (item.getAC3())
			sInfo = sInfo + "AC3 ";
		if (item.getSurround())
			sInfo = sInfo + "SRS ";
		if (item.getCaptions())
			sInfo = sInfo + "CC ";
		if (item.getInteractive())
			sInfo = sInfo + "*INTERACTIVE* ";
		if (item.getPremiere())
			sInfo = sInfo + "*PREMIERE* ";
		if (item.getLive())
			sInfo = sInfo + "*LIVE* ";
		if (item.getLastChance())
			sInfo = sInfo + "*FINAL* ";
		if (item.getRepeat())
			sInfo = sInfo + "(REPEAT)";

		if (sInfo.length() > 0)
		{
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><img style='cursor:hand;cursor:pointer;' border=0 src='/images/help24.png' alt='Help' title='Help' align='absmiddle' onClick=\"showHelp('InfoField');\" width='24' height='24'><b>Info:</b></td><td><font size=2><b>\n");
			buff.append(sInfo);
			buff.append("</b></font></td></tr>");
		}

		if(item.getRatings().length() > 0)
		{
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='right' valign='top'><b>Rating:</b></td><td><font size=2><b>\n");
			buff.append(item.getRatings());
			buff.append("</b></font></td></tr>");
		}

		// add description
		if(item.getDescription() != null && item.getDescription().length() > 0)
		{
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='left' valign='top' colspan='2'>" + item.getDescription() + "<br></td></tr>\n");
		}

		List<Long> flaggers = item.getFavoriteFlaggers();
		if ( (flaggers != null) && !flaggers.isEmpty() )
		{
			buff.append("<tr><td style='border: 1px solid #FFFFFF;' align='left' valign='top' colspan='2'><b>Auto-Add Matches:</b><table>");
			Vector<EpgMatch> autoAdds = m_store.getEpgAutoAdds();
			Iterator<Long> iter = flaggers.iterator();
			while (iter.hasNext())
			{
				Long autoAddId = iter.next();
				for (int i=0; i<autoAdds.size(); i++)
				{
					EpgMatch autoAdd = autoAdds.get(i);
					if (autoAdd.getID() == autoAddId)
					{
						buff.append("<tr><td><a href='#' class='noUnder' onClick=\"addItem('/servlet/EpgAutoAddDataRes?action=03&index=" + 
								i + "');\">" + autoAdd.getName2() + "</a>");
						buff.append(" <a href='#' class='noUnder' onClick=\"addItem('/servlet/EpgAutoAddDataRes?action=32&delSel=1&sel=" + 
								i + "');\"><img align='absmiddle' src='/images/delete.png' border='0' title='Delete Auto-Add' alt='Delete Auto-Add' width='24' height='24'></a>");
						buff.append("</td></tr>");
						break;
					}
				}
			}
			buff.append("</table></td></tr>\n");
		}

		buff.append("</table>\n");

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-details.html");
		template.replace("$details", buff.toString());

		return template.getPageBytes();
	}

	private byte[] addUpdateChannelMapping() throws Exception
	{
		GuideStore guideStore = GuideStore.getInstance();

		String wsChannel = m_urlData.getParameter("wsChannel");
		String epgChannel = m_urlData.getParameter("epgChannel");
		int priority = m_urlData.getParameterInt("priority", 10);

		int index = m_urlData.getParameterInt("index", -1);
		int subIndex = m_urlData.getParameterInt("subIndex", -1);
		String update = m_urlData.getParameter("update");
		if (index >= 0 && subIndex >= 0 && update != null && update.equals("Update"))
		{
			// make sure WS channel exists
			// make sure EPG channel exists
			if (m_store.isChannelExist(wsChannel) && guideStore.isChannelExist(epgChannel))
			{
				guideStore.updateChannelMapCustom(wsChannel, epgChannel, priority, index, subIndex);
				guideStore.saveChannelMap();
			}
		}
		else
		{
			// make sure WS channel exists
			// make sure EPG channel exists
			if (m_store.isChannelExist(wsChannel) && guideStore.isChannelExist(epgChannel))
			{
				guideStore.addChannelToCustomMap(wsChannel, epgChannel, priority);
				guideStore.saveChannelMap();
			}
		}

		StringBuffer buff = new StringBuffer(256);
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/" + m_urlData.getServletClass() + "?action=04\n\n");

		return buff.toString().getBytes("UTF-8");
	}

	private byte[] showChannelMapping() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		StringBuffer buff = new StringBuffer();
		StringBuffer warnings = new StringBuffer();

		Vector<ChannelMapGroup> chanMap = guide.getChannelMapCustom();
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-mapping.html", true);

		// Have to set default to custom due to version 5.72 and earlier not having default.
		if (m_store.getProperty("guide.mapping", "custom").equals("custom"))
			template.replace("$checkedCustom", "checked");
		else
			template.replace("$checkedDefault", "checked");

		Set<String> wsChanSet = m_store.getChannels().keySet();
		String[] wsChannels = (String[])wsChanSet.toArray(new String[0]);

		for (int x = 0; x < chanMap.size(); x++)
		{
			ChannelMapGroup mapGroup = chanMap.get(x);

			buff.append("<tr>");

			if (chanMap.size() > 1)
				buff.append("<td><input type='checkbox' name='sel' value='" + x + "'></td><td>");
			else
				buff.append("<td></td><td>");
			if(!wsChanSet.contains(mapGroup.m_channelName))
			{
				warnings.append("DV Scheduler Channel (" + HTMLEncoder.encode(mapGroup.m_channelName) + ") does not exist!<br>\n");
				buff.append("<img align='absmiddle' src='/images/exclaim24.png' border='0' alt='Error' title='Error' width='24' height='24'>");
			}
			buff.append(HTMLEncoder.encode(mapGroup.m_channelName) + "</td><td><table>");

			for (int j=0; j<mapGroup.m_channelMaps.size(); j++)
			{
				ChannelMap map = mapGroup.m_channelMaps.get(j);

				buff.append("<tr><td>");
				if (guide.getProgramSizeForEpgChannel(map.m_epgChannelId) == 0)
				{
					warnings.append("There is currently no data for EPG channel (" + HTMLEncoder.encode(map.m_epgChannelId) + ")<br>\n");
					buff.append("<img align='absmiddle' src='/images/help24.png' border='0' alt='No data' title='No data' width='24' height='24'>");
				}

				buff.append(HTMLEncoder.encode(map.m_epgChannelName + " (" + map.m_epgChannelId) + ") [" + map.m_priority + "]</td>");
				if (mapGroup.m_channelMaps.size() > 1)
					buff.append("<td><a href='/servlet/EpgDataRes?action=07&id=" + x + "&subIndex=" + j + "'><img align='absmiddle' border='0' alt='Delete' title='Delete' src='/images/delete.png' width='24' height='24'></a></td>\n");
				buff.append("<td><img style='cursor:hand;cursor:pointer;' border=0 src='/images/edit.png' alt='Edit' title='Edit' onClick=\"edit('" + mapGroup.m_channelName + "','" + map.m_epgChannelId + "'," + map.m_priority + "," + x + "," + j + ");\" align='absmiddle' width='24' height='24'></td>\n");
				buff.append("</tr>");
			}

			buff.append("</table></td><td>");
			if (chanMap.size() > 1)
				buff.append("<input type='image' name='move." + x + "' align='absmiddle' border='0' alt='Move selected items above this item' title='Move selected items above this item' src='/images/moveBefore.png' width='24' height='24' onMouseOver='lineTop(this, true)' onMouseOut='lineTop(this, false)'>\n");
			buff.append("<a href='/servlet/EpgDataRes?action=07&id=" + x + "'><img align='absmiddle' border='0' alt='Delete' title='Delete' src='/images/delete.png' width='24' height='24'></a>\n");
			if (chanMap.size() > 1)
			{
				buff.append("<a href='/servlet/EpgDataRes?action=24&id=" + x + "&dir=0'><img align='absmiddle' border='0' alt='Up' title='Up' src='/images/up01.png' width='7' height='7'></a>\n");
				buff.append("<a href='/servlet/EpgDataRes?action=24&id=" + x + "&dir=1'><img align='absmiddle' border='0' alt='Down' title='Down' src='/images/down01.png' width='7' height='7'></a>\n");
			}
			buff.append("</td>\n");

			buff.append("</tr>\n");
		}
		if (chanMap.size() > 1)
		{
			buff.append("<tr><td>&nbsp;</td><td><a onClick='return confirmDeleteAll();' href='/servlet/EpgDataRes?action=32' class='noUnder'><img border=0 src='/images/deleteAll.png' align='absmiddle' width='24' height='24'> Delete All</a></td><td>&nbsp;</td><td><input type='image' name='moveEnd' align='absmiddle' border='0' alt='Move selected items to end' title='Move selected items to end' src='/images/moveBefore.png' width='24' height='24' onMouseOver='lineTop(this, true)' onMouseOut='lineTop(this, false)'> <input type='image' name='delSel' align='absmiddle' border='0' alt='Delete selected items' title='Delete selected items' src='/images/delete.png' width='24' height='24'></a></td></tr>\n");
			buff.append("<tr><td>&nbsp;</td><td colspan='2'><a href='/servlet/EpgDataRes?action=EpgSortMapping' class='noUnder'><img border='0' src='/images/sort.png' align='absmiddle' width='24' height='24'> Sort according to Channel Surf Order</a> (see <a class='under' href='/servlet/ChannelDataRes?action=14'>Channel Surf Order</a>)</td></tr>");
		}
		buff.append("<tr><td>&nbsp;</td><td colspan='2'><a href='/servlet/EpgDataRes?action=EpgAddDefaultMapping' class='noUnder'><img border='0' src='/images/Download.png' align='absmiddle' width='24' height='24'> Add Built-in EPG Scanner mapping</a></td></tr>");
		template.replace("$channelmap", buff.toString());

		Arrays.sort(wsChannels, String.CASE_INSENSITIVE_ORDER);
		buff = new StringBuffer();
		for(int x = 0; x < wsChannels.length; x++)
		{
			buff.append("<OPTION VALUE=\"" + HTMLEncoder.encode(wsChannels[x]) + "\"> " + HTMLEncoder.encode(wsChannels[x] + " (" + m_store.getChannel(wsChannels[x]).getProgramID() + ")") + "\n");
		}
		template.replace("$wsChannels", buff.toString());

		String[] epgChannels = (String[])guide.getChannelList();
		Arrays.sort(epgChannels, String.CASE_INSENSITIVE_ORDER);
		buff = new StringBuffer();
		for(int x = 0; x < epgChannels.length; x++)
		{
			GuideChannel guideChannel = guide.getGuideChannel(epgChannels[x]);
			String epgChannelName = guideChannel.getEpgChannelName();
			buff.append("<OPTION VALUE=\"" + HTMLEncoder.encode(epgChannels[x]) + "\"> " + HTMLEncoder.encode(epgChannelName + " (" + epgChannels[x]) + "), " + guideChannel.getSize() + " events\n");
		}
		template.replace("$epgChannels", buff.toString());  

		//
		// Now show any warnings
		//
		if(warnings.length() > 0)
		{
			String warningText = "<table><tr><td style='border: 1px solid #FFFFFF;'>" +
					"<img align='absmiddle' src='/images/exclaim24.png' border='0' alt='Error' title='Error' width='24' height='24'><b>Warnings and </b><img align='absmiddle' src='/images/help24.png' border='0' alt='Info' title='Info' width='24' height='24'><b>Info</b>" +
					"</td></tr><tr><td>";
			warningText += warnings.toString() + "</td></tr></table>";

			template.replace("$warning", warningText);
		}
		else
			template.replace("$warning", "");

		return template.getPageBytes();
	}

	private void reloadXMLTVdata() throws Exception
	{
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "epg-reload.html");
		m_outStream.write(template.getPageBytes(0));

		String type = m_store.getProperty("guide.source.type", "Internal");
		if (type.contains("Internal"))
			m_outStream.write("(Built-in EPG Scanner will run separately, check device status)<br><br>".getBytes("UTF-8"));

		if ( !m_store.isEpgLoading() )
		{
			GuideStore guide = GuideStore.getInstance();
			if (type.contains("Internal"))
				guide.setInternalEpgScheduleManualReload();

			m_store.m_bEpgManualLoadRunning = true;
			StringBuffer buff = new StringBuffer();
			guide.loadXmltv(buff, 1, 0);
			m_store.m_bEpgManualLoadRunning = false;
			m_outStream.write(buff.toString().getBytes("UTF-8"));
			m_outStream.write("</td></tr></table><br><br></body></html>".getBytes("UTF-8"));
		}
		else
		{
			m_outStream.write("An EPG load is already running!".getBytes("UTF-8"));
		}
	}

	private String intToStr(int num)
	{
		if (num < 10)
			return "0" + num;
		else
			return new Integer(num).toString();

	}

	private byte[] setEpgMapping() throws Exception
	{
		String mapping = m_urlData.getParameter("useDefault");

		GuideStore guide = GuideStore.getInstance();
		if (mapping.equals("0"))
		{
			m_store.setProperty("guide.mapping", "custom");
			guide.setCustomEpgChannelMapping();
		}
		else
		{
			m_store.setProperty("guide.mapping", "default");
			guide.setInternalEpgChannelMapping();
		}

		return getServletRedir("?action=04").getBytes("UTF-8");
	}

	private byte[] addDefaultEpgMapping() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();
		guide.addDefaultEpgMapping();

		return getServletRedir("?action=04").getBytes("UTF-8");
	}

	private byte[] sortEpgMapping() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();
		guide.sortChannelMapCustom();

		return getServletRedir("?action=04").getBytes("UTF-8");
	}
}
