/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class EpgMatchList
{
	private Vector<EpgMatchListItem> matchList = new Vector<EpgMatchListItem>();
	private boolean m_bUseTitleFiter = true;

	public EpgMatchList()
	{
	}   

	public EpgMatchList(Node node)
	{
		parseXML(node);
	}

	public Vector<EpgMatchListItem> getMatchList()
	{
		return matchList;
	}

	public boolean isMatch(GuideItem item, String channel, String[] titleFilters)
	{
		if(matchList.size() == 0)
			return false;

		for(int x = 0; x < matchList.size(); x++)
		{
			EpgMatchListItem matcher = (EpgMatchListItem)matchList.get(x);

			if(matcher.getType() == EpgMatchListItem.TYPE_TEXT)
			{
				if(!matcher.isTextMatch(item, channel, isUseTitleFilter() ? titleFilters : null))
					return false;
			}
			else if(matcher.getType() == EpgMatchListItem.TYPE_SPAN)
			{
				if(!matcher.isSpanMatch(item))
					return false;
			}
			else if(matcher.getType() == EpgMatchListItem.TYPE_DAYS)
			{
				if(!matcher.isDayMatch(item))
					return false;
			}
			else if(matcher.getType() == EpgMatchListItem.TYPE_OTHER)
			{
				if(!matcher.isOtherMatch(item))
					return false;
			}
			else if(matcher.getType() == EpgMatchListItem.TYPE_DURATION)
			{
				if(!matcher.isDurationMatch(item))
					return false;
			}
		}

		return true;
	}

	public void setUseTitleFilter(boolean bUseTitleFilter)
	{
		m_bUseTitleFiter = bUseTitleFilter;
	}

	public boolean isUseTitleFilter()
	{
		return m_bUseTitleFiter;
	}

	public void getXML(Document doc, String name)
	{
		Element item = doc.createElement("MatchList");

		item.setAttribute("name", name);
		item.setAttribute("useTitleFilter", m_bUseTitleFiter ? "true" : "false");

		for(int x = 0; x < matchList.size(); x++)
		{
			EpgMatchListItem matcher = (EpgMatchListItem)matchList.get(x);
			matcher.getXML(doc, item);      
		}

		Element root = doc.getDocumentElement();
		root.appendChild(item);
	}

	private void parseXML(Node node)
	{
		try
		{
			if (node.getAttributes().getNamedItem("useTitleFilter").getTextContent().equals("false"))
				m_bUseTitleFiter = false;
		}
		catch(Exception e){}

		NodeList nl = node.getChildNodes();
		Node found = null;

		for(int x = 0; x < nl.getLength(); x++)
		{
			found = nl.item(x);
			if(found.getNodeName().equals("MatchListItem"))
			{
				matchList.add(new EpgMatchListItem(found));
			}
		}      
	}

	public String getChannels()
	{
		String channels = "";

		for(int x = 0; x < matchList.size(); x++)
		{
			EpgMatchListItem matcher = (EpgMatchListItem)matchList.get(x);

			if ( (matcher.getType() == EpgMatchListItem.TYPE_TEXT) && (matcher.getField() == EpgMatchListItem.FIELD_CHANNEL) && (matcher.getExists()) )
			{
				if (!channels.isEmpty())
					channels += ", ";

				channels += matcher.getTextSearch();
			}
		}

		return channels;
	}

	public String getSpans()
	{
		String spans = "";

		for(int x = 0; x < matchList.size(); x++)
		{
			EpgMatchListItem matcher = matchList.get(x);

			if (matcher.getType() == EpgMatchListItem.TYPE_SPAN)
			{
				if (!spans.isEmpty())
					spans += ", ";

				spans += DataStore.intToStr(matcher.getSpanFromHour()) + ":" + DataStore.intToStr(matcher.getSpanFromMin()) + " to " + 
				DataStore.intToStr(matcher.getSpanToHour()) + ":" + DataStore.intToStr(matcher.getSpanToMin());
			}
		}

		return spans;
	}

	public String getDays()
	{
		String days = "";

		for(int x = 0; x < matchList.size(); x++)
		{
			EpgMatchListItem matcher = matchList.get(x);

			if (matcher.getType() == EpgMatchListItem.TYPE_DAYS)
			{
				String temp = "";
				if(matcher.getDayValue(EpgMatchListItem.DAYS_SUN))
					temp += (temp.isEmpty() ? "" : " ") + "Sun";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_MON))
					temp += (temp.isEmpty() ? "" : " ") + "Mon";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_TUE))
					temp += (temp.isEmpty() ? "" : " ") + "Tue";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_WED))
					temp += (temp.isEmpty() ? "" : " ") + "Wed";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_THUR))
					temp += (temp.isEmpty() ? "" : " ") + "Thur";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_FRI))
					temp += (temp.isEmpty() ? "" : " ") + "Fri";

				if(matcher.getDayValue(EpgMatchListItem.DAYS_SAT))
					temp += (temp.isEmpty() ? "" : " ") + "Sat";

				if (!days.isEmpty())
					days += ", ";
				days += temp;
			}
		}

		return days;
	}
}