/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


import java.util.regex.*;
import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class EpgMatchListItem
{
	// what type are we
	public static final int TYPE_TEXT = 0;
	public static final int TYPE_SPAN = 1;
	public static final int TYPE_DAYS = 2;
	public static final int TYPE_OTHER = 3;
	public static final int TYPE_DURATION = 4;

	// fileds ids
	public static final int FIELD_TITLE = 0;
	public static final int FIELD_DESCRIPTION = 1;
	public static final int FIELD_CHANNEL = 2;
	public static final int FIELD_CATEGORY = 3;
	public static final int FIELD_SUBTITLE = 4;
	public static final int FIELD_LANGUAGE = 5;
	public static final int FIELD_RATINGS = 6;

	// flag ids
	public static final int FLAG_NONE = 0;
	public static final int FLAG_CASEINSENSATIVE = 1;
	public static final int FLAG_REGEX = 2;

	// data for type TEXT
	private String searchText = "";
	private int field = 0;
	private boolean exists = true;
	private int flags = 0;

	// data for type span
	private int span_from_hour = 0;
	private int span_from_min = 0;
	private int span_to_hour = 0;
	private int span_to_min = 0;

	// data for type days
	// bit representation of days
	private boolean day_sun = true;
	private boolean day_mon = true;
	private boolean day_tue = true;
	private boolean day_wed = true;
	private boolean day_thur = true;
	private boolean day_fri = true;
	private boolean day_sat = true;

	// days ids
	public static final int DAYS_SUN = 0;
	public static final int DAYS_MON = 1;
	public static final int DAYS_TUE = 2;
	public static final int DAYS_WED = 3;
	public static final int DAYS_THUR = 4;
	public static final int DAYS_FRI = 5;
	public static final int DAYS_SAT = 6;

	// type other
	public static final int OTHER_WIDESCREEN = 0;
	public static final int OTHER_HIGHDEFINITION = 1;
	public static final int OTHER_SURROUNDSOUND = 2;
	public static final int OTHER_AC3SOUND = 3;
	public static final int OTHER_CAPTION = 4;
	public static final int OTHER_PREMIERE = 5;
	public static final int OTHER_REPEAT = 6;
	public static final int OTHER_LASTCHANCE = 7;

	private int m_other = -1;
	private boolean m_bOtherValue = true;

	// type duration
	private int m_minDuration = -1;
	private int m_maxDuration = -1;

	private int type = -1;

	public EpgMatchListItem(int matchType)
	{
		type = matchType;
	}

	public EpgMatchListItem(Node node)
	{
		parseXML(node);
	}   

	public int getType()
	{
		return type;
	}

	public void setDayData(int dayID, boolean value)
	{
		if(dayID == DAYS_SUN)
			day_sun = value;
		else if(dayID == DAYS_MON)
			day_mon = value;
		else if(dayID == DAYS_TUE)
			day_tue = value;
		else if(dayID == DAYS_WED)
			day_wed = value;
		else if(dayID == DAYS_THUR)
			day_thur = value;
		else if(dayID == DAYS_FRI)
			day_fri = value;
		else if(dayID == DAYS_SAT)
			day_sat = value;
	}

	public boolean getDayValue(int dayID)
	{
		if(dayID == DAYS_SUN)
			return day_sun;
		else if(dayID == DAYS_MON)
			return day_mon;
		else if(dayID == DAYS_TUE)
			return day_tue;
		else if(dayID == DAYS_WED)
			return day_wed;
		else if(dayID == DAYS_THUR)
			return day_thur;
		else if(dayID == DAYS_FRI)
			return day_fri;
		else if(dayID == DAYS_SAT)
			return day_sat;  
		else
			return false;
	}

	public void setSpanData(int fromHour, int fromMin, int toHour, int toMin)
	{
		span_from_hour  = fromHour;
		span_from_min = fromMin;
		span_to_hour = toHour;
		span_to_min = toMin;
	}

	public int getSpanFromHour()
	{
		return span_from_hour;
	}

	public int getSpanFromMin()
	{
		return span_from_min;
	}   

	public int getSpanToHour()
	{
		return span_to_hour;
	}   

	public int getSpanToMin()
	{
		return span_to_min;
	} 

	public void setTextSearchData(String text, int f, boolean ex, int fl)
	{
		searchText = text;
		field = f;
		exists = ex;
		flags = fl;
	}

	public String getTextSearch()
	{
		return searchText;
	}

	public int getField()
	{
		return field;
	}   

	public boolean getExists()
	{
		return exists;
	}      

	public int getFlags()
	{
		return flags;
	}    

	public boolean isTextMatch(GuideItem item, String channel, String[] titleFilters)
	{
		String searchString = searchText;

		boolean matchFound = false;
		if (field == FIELD_CATEGORY)
		{
			Vector<String> categories = item.getCategories();
			if (categories != null)
			{
				for (int i=0; i<categories.size(); i++)
				{
					String text = categories.get(i);
					if(flags == FLAG_CASEINSENSATIVE)
					{
						text = text.toUpperCase();
						searchString = searchString.toUpperCase();
					}

					if(flags == FLAG_REGEX)
					{
						// do reg ex
						matchFound = Pattern.matches(searchString, text);
					}
					else
					{
						// to text search
						if(searchString.equals("*") || searchString.equals("**"))
						{
							matchFound = !text.isEmpty();             
						}
						else if(searchString.startsWith("*") && searchString.endsWith("*"))
						{
							String search = searchString.substring(1, searchString.length() - 1);
							if(text.indexOf(search) > -1)
								matchFound = true;             
						}
						else if(searchString.startsWith("*"))
						{
							String search = searchString.substring(1, searchString.length());
							matchFound = text.endsWith(search);
						}
						else if(searchString.endsWith("*"))
						{
							String search = searchString.substring(0, searchString.length()-1);
							matchFound = text.startsWith(search);
						}
						else
						{
							matchFound = text.equals(searchString);
						}
					}
					if (matchFound)
						break;
				}
			}
		}
		else
		{
			String text = "";

			if(field == FIELD_TITLE)
				text = item.getName();
			else if(field == FIELD_DESCRIPTION)
				text = item.getDescription();
			else if(field == FIELD_CHANNEL)
				text = channel;
			else if(field == FIELD_SUBTITLE)
				text = item.getSubName();
			else if(field == FIELD_LANGUAGE)
				text = item.getLanguage();
			else if(field == FIELD_RATINGS)
				text = item.getRatings();

			if ( (titleFilters != null) && ( (field == FIELD_TITLE) || (field == FIELD_SUBTITLE) ) )
			{
				int lastIndex, textLength, titleFilterLength;
				for (int i=0; i<titleFilters.length; i++)
				{
					lastIndex = text.lastIndexOf(titleFilters[i]);
					textLength = text.length();
					titleFilterLength = titleFilters[i].length();
					if ( (lastIndex > 1) && (lastIndex < textLength) && (lastIndex >= (textLength - titleFilterLength - 3)) )
					{
						char charAt;
						for (--lastIndex; lastIndex>0; lastIndex--)
						{
							charAt = text.charAt(lastIndex);
							if ( ((charAt >= '0') && (charAt <= '9')) || ((charAt >= 'A') && (charAt <= 'Z')) || 
									((charAt >= 'a') && (charAt <= 'z')) || (charAt >= 128) )
								break;
						}
						text = text.substring(0, lastIndex+1);
						break;
					}
				}
			}

			if(flags == FLAG_CASEINSENSATIVE)
			{
				text = text.toUpperCase();
				searchString = searchString.toUpperCase();
			}

			if(flags == FLAG_REGEX)
			{
				// do reg ex
				matchFound = Pattern.matches(searchString, text);
			}
			else
			{
				// to text search
				if(searchString.equals("*") || searchString.equals("**"))
				{
					matchFound = !text.isEmpty();             
				}
				else if(searchString.startsWith("*") && searchString.endsWith("*"))
				{
					String search = searchString.substring(1, searchString.length() - 1);
					if(text.indexOf(search) > -1)
						matchFound = true;             
				}
				else if(searchString.startsWith("*"))
				{
					String search = searchString.substring(1, searchString.length());
					matchFound = text.endsWith(search);
				}
				else if(searchString.endsWith("*"))
				{
					String search = searchString.substring(0, searchString.length()-1);
					matchFound = text.startsWith(search);
				}
				else
				{
					matchFound = text.equals(searchString);
				}
			}
		}

		// now work out if we have a match or not and if that is what we wanted
		if(exists && matchFound)
			return true;
		else if(!exists && !matchFound)
			return true;
		else
			return false;
	}

	public boolean isSpanMatch(GuideItem item)
	{
		int fromMinInDay = (span_from_hour * 60) + span_from_min;
		int toMinInDay = (span_to_hour * 60) + span_to_min;

		int span = 0;
		if(fromMinInDay < toMinInDay)
		{
			span = toMinInDay - fromMinInDay;
		}
		else if(toMinInDay < fromMinInDay)
		{
			span = (24 * 60) - fromMinInDay;
			span += toMinInDay;
		}  

		//DVScheduler.log("Span = " + span);

		Calendar cal = Calendar.getInstance();
		cal.setTime(item.getStart());

		Calendar start = Calendar.getInstance();
		start.setTime(item.getStart());
		start.set(Calendar.HOUR_OF_DAY, span_from_hour);
		start.set(Calendar.MINUTE, span_from_min);  

		Calendar end = Calendar.getInstance(); 
		end.setTime(start.getTime());
		end.add(Calendar.MINUTE, span);

		if(cal.after(start) && cal.before(end))
			return true;
		else if(cal.equals(start) || cal.equals(end))
			return true;
		else
			return false;
	}

	public boolean isDayMatch(GuideItem item)
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(item.getStart());

		int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);

		if(dayOfWeek == Calendar.SUNDAY && day_sun)
			return true;
		else if(dayOfWeek == Calendar.MONDAY && day_mon)
			return true;
		else if(dayOfWeek == Calendar.TUESDAY && day_tue)
			return true;      
		else if(dayOfWeek == Calendar.WEDNESDAY && day_wed)
			return true;        
		else if(dayOfWeek == Calendar.THURSDAY && day_thur)
			return true;        
		else if(dayOfWeek == Calendar.FRIDAY && day_fri)
			return true;     
		else if(dayOfWeek == Calendar.SATURDAY && day_sat)
			return true;      
		else 
			return false;
	}

	public boolean isOtherMatch(GuideItem item)
	{
		if ( (m_other == OTHER_WIDESCREEN) && (m_bOtherValue == item.getWidescreen()) )
			return true;
		else if ( (m_other == OTHER_HIGHDEFINITION) && (m_bOtherValue == item.getHighDef()) )
			return true;
		else if ( (m_other == OTHER_SURROUNDSOUND) && (m_bOtherValue == item.getSurround()) )
			return true;
		else if ( (m_other == OTHER_AC3SOUND) && (m_bOtherValue == item.getAC3()) )
			return true;
		else if ( (m_other == OTHER_CAPTION) && (m_bOtherValue == item.getCaptions()) )
			return true;
		else if ( (m_other == OTHER_PREMIERE) && (m_bOtherValue == item.getPremiere()) )
			return true;
		else if ( (m_other == OTHER_REPEAT) && (m_bOtherValue == item.getRepeat()) )
			return true;
		else if ( (m_other == OTHER_LASTCHANCE) && (m_bOtherValue == item.getLastChance()) )
			return true;

		return false;
	}

	public void SetOther(int flag, boolean bValue)
	{
		m_other = flag;
		m_bOtherValue = bValue;
	}

	public int GetOtherFlag()
	{
		return m_other;
	}

	public static String GetOtherFlagStr(int flag)
	{
		switch (flag)
		{
		case OTHER_WIDESCREEN:
			return "Wide-screen";
		case OTHER_HIGHDEFINITION:
			return "High-definition";
		case OTHER_SURROUNDSOUND:
			return "Surround sound";
		case OTHER_AC3SOUND:
			return "AC3 sound";
		case OTHER_CAPTION:
			return "Caption";
		case OTHER_PREMIERE:
			return "Premiere";
		case OTHER_REPEAT:
			return "Repeat";
		case OTHER_LASTCHANCE:
			return "Last chance";
		default:
			return "Unknown";
		}
	}

	public String GetOtherFlagStr()
	{
		return GetOtherFlagStr(m_other);
	}

	public boolean GetOtherValue()
	{
		return m_bOtherValue;
	}

	public boolean isDurationMatch(GuideItem item)
	{
		boolean bMin = m_minDuration <= 0 ? true : (item.getDuration() >= m_minDuration ? true : false);
		boolean bMax = m_maxDuration <= 0 ? true : (item.getDuration() <= m_maxDuration ? true : false);

		return (bMin && bMax);
	}

	public void SetDuration(int minDuration, int maxDuration)
	{
		m_minDuration = minDuration;
		m_maxDuration = maxDuration;
	}

	public int GetMinDuration()
	{
		return m_minDuration;
	}

	public int GetMaxDuration()
	{
		return m_maxDuration;
	}

	public void getXML(Document doc, Element item)
	{
		Element mainElm = null;
		mainElm = doc.createElement("MatchListItem");
		mainElm.setAttribute("type", new Integer(type).toString());

		Element elm = null;
		Text text = null;

		if(type == EpgMatchListItem.TYPE_TEXT)
		{
			elm = doc.createElement("TEXT");
			elm.setAttribute("field", new Integer(field).toString());
			elm.setAttribute("exists", new Boolean(exists).toString());
			elm.setAttribute("flags", new Integer(flags).toString());
			text = doc.createTextNode(searchText);
			elm.appendChild(text);
			mainElm.appendChild(elm);
		}
		else if(type == EpgMatchListItem.TYPE_SPAN)
		{      
			elm = doc.createElement("SPAN");
			elm.setAttribute("span_from_hour", new Integer(span_from_hour).toString());
			elm.setAttribute("span_to_hour", new Integer(span_to_hour).toString());
			elm.setAttribute("span_from_min", new Integer(span_from_min).toString());
			elm.setAttribute("span_to_min", new Integer(span_to_min).toString());
			mainElm.appendChild(elm);
		}
		else if(type == EpgMatchListItem.TYPE_DAYS)
		{       
			elm = doc.createElement("DAYS");
			elm.setAttribute("sun", new Boolean(day_sun).toString());
			elm.setAttribute("mon", new Boolean(day_mon).toString());
			elm.setAttribute("tue", new Boolean(day_tue).toString());
			elm.setAttribute("wed", new Boolean(day_wed).toString());
			elm.setAttribute("thur", new Boolean(day_thur).toString());
			elm.setAttribute("fri", new Boolean(day_fri).toString());
			elm.setAttribute("sat", new Boolean(day_sat).toString());
			mainElm.appendChild(elm);      
		}
		else if(type == EpgMatchListItem.TYPE_OTHER)
		{       
			elm = doc.createElement("OTHER");
			elm.setAttribute("flag", new Integer(m_other).toString());
			elm.setAttribute("value", new Boolean(m_bOtherValue).toString());
			mainElm.appendChild(elm);      
		}

		else if(type == EpgMatchListItem.TYPE_DURATION)
		{       
			elm = doc.createElement("DURATION");
			elm.setAttribute("minDuration", new Integer(m_minDuration).toString());
			elm.setAttribute("maxDuration", new Integer(m_maxDuration).toString());
			mainElm.appendChild(elm);
		}

		item.appendChild(mainElm);
	}

	private void parseXML(Node node)
	{
		NamedNodeMap itemAttribs = node.getAttributes();
		type = Integer.parseInt(itemAttribs.getNamedItem("type").getTextContent());

		NodeList nl = node.getChildNodes();
		Node found = null;

		for(int x = 0; x < nl.getLength(); x++)
		{
			found = nl.item(x);
			if(type == EpgMatchListItem.TYPE_TEXT && found.getNodeName().equals("TEXT"))
			{
				itemAttribs = found.getAttributes();
				field = Integer.parseInt(itemAttribs.getNamedItem("field").getTextContent());
				exists = Boolean.parseBoolean(itemAttribs.getNamedItem("exists").getTextContent());
				flags = Integer.parseInt(itemAttribs.getNamedItem("flags").getTextContent());
				searchText = found.getTextContent();
			}
			else if(type == EpgMatchListItem.TYPE_SPAN && found.getNodeName().equals("SPAN"))
			{
				itemAttribs = found.getAttributes();
				span_from_hour = Integer.parseInt(itemAttribs.getNamedItem("span_from_hour").getTextContent());
				span_to_hour = Integer.parseInt(itemAttribs.getNamedItem("span_to_hour").getTextContent());
				span_from_min = Integer.parseInt(itemAttribs.getNamedItem("span_from_min").getTextContent());
				span_to_min = Integer.parseInt(itemAttribs.getNamedItem("span_to_min").getTextContent());
			}     
			else if(type == EpgMatchListItem.TYPE_DAYS && found.getNodeName().equals("DAYS"))
			{
				itemAttribs = found.getAttributes();
				day_sun = Boolean.parseBoolean(itemAttribs.getNamedItem("sun").getTextContent());;
				day_mon = Boolean.parseBoolean(itemAttribs.getNamedItem("mon").getTextContent());;
				day_tue = Boolean.parseBoolean(itemAttribs.getNamedItem("tue").getTextContent());;
				day_wed = Boolean.parseBoolean(itemAttribs.getNamedItem("wed").getTextContent());;
				day_thur = Boolean.parseBoolean(itemAttribs.getNamedItem("thur").getTextContent());;
				day_fri = Boolean.parseBoolean(itemAttribs.getNamedItem("fri").getTextContent());;
				day_sat = Boolean.parseBoolean(itemAttribs.getNamedItem("sat").getTextContent());;
			}
			else if(type == EpgMatchListItem.TYPE_OTHER && found.getNodeName().equals("OTHER"))
			{
				itemAttribs = found.getAttributes();
				m_other = Integer.parseInt(itemAttribs.getNamedItem("flag").getTextContent());
				m_bOtherValue = Boolean.parseBoolean(itemAttribs.getNamedItem("value").getTextContent());
			}     
			else if(type == EpgMatchListItem.TYPE_DURATION && found.getNodeName().equals("DURATION"))
			{
				itemAttribs = found.getAttributes();
				m_minDuration = Integer.parseInt(itemAttribs.getNamedItem("minDuration").getTextContent());
				m_maxDuration = Integer.parseInt(itemAttribs.getNamedItem("maxDuration").getTextContent());
			}     
		}      
	}    
}