/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.util.Map.Entry;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

class FileManagementDataRes extends HTTPResponse
{
	public FileManagementDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if("01".equals(action))
		{
			m_outStream.write(deleteFile());
			return;
		}
		else if("02".equals(action))
		{
			m_outStream.write(showCommandList());
			return;
		}
		else if("03".equals(action))
		{
			m_outStream.write(runCommand());
			return;
		}
		else if("04".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addAutoDelItem());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("05".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(removeAutoDelItem());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("06".equals(action))
		{
			m_outStream.write(playFile());
			return;
		}
		else if("07".equals(action))
		{
			m_outStream.write(changePlayFile());
			return;
		}
		else if("08".equals(action))
		{
			m_outStream.write(stopPlayFile());
			return;
		}
		else if("09".equals(action))
		{
			uploadFile();
			return;
		}
		else if("10".equals(action))
		{
			//uploadPlaylistOrRedirect();
			uploadPlaylist();
			return;
		}

		m_outStream.write(xmlFileList());
	}

	public void getResponse2() throws Exception
	{
		String action = m_urlData.getParameter("parameter0");
		if ("servefile".equals(action))
			uploadFile(m_urlData.getParameter("parameter1"));
		//else if ("playlistAll".equals(action))
		//	uploadPlaylist();
		else
			m_outStream.write("Error occurred".getBytes("UTF-8"));
	}

	public static File listFilesDirectories(String directory, String fileMasks, Element node, Document doc, boolean bForwardSlash, boolean bShowCapturePaths) throws Exception
	{
		DataStore store = DataStore.getInstance();
		long minFreeSpace = store.getPropertyLong("Capture.minFreeSpace", 20000) * 1024 * 1024;
		String sMinFreeSpace = Long.toString(minFreeSpace);

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = store.getServeFilePaths();

		if (directory != null && directory.equals("/"))
		{
			Element elm = doc.createElement("directories");
			Element elm2 = null;
			Element elmFiles = doc.createElement("files");
			for (int i=0; i<paths.length; i++)
			{
				String name = bForwardSlash ? paths[i].replace("\\", "/") : paths[i];
				elm2 = doc.createElement("directory");
				elm2.setAttribute("name", name);
				File file = new File(paths[i]);
				elm2.setAttribute("freeSpace", Long.toString(file.getFreeSpace()));
				elm2.setAttribute("minFreeSpace", sMinFreeSpace);
				elm.appendChild(elm2);

				elm2 = doc.createElement("file");
				elm2.setAttribute("name", name);
				elm2.setAttribute("path", name);
				elm2.setAttribute("type", "1");
				elmFiles.appendChild(elm2);
			}
			node.appendChild(elm);
			node.appendChild(elmFiles);

			return null;
		}

		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);
		int pathSelected = 0;
		String parentDirectory = paths[0];
		boolean bAtRoot = true;

		if (directory != null && directory.length() > 0)
		{
			File path = new File(directory);
			String fullPath = path.getCanonicalPath();

			// check requested file is still within the capture dir
			for(int x = paths.length-1; x >= 0; x--)
			{
				if (paths[x].isEmpty())
					paths[x] = ".";

				baseDir = new File(paths[x]);
				if (!baseDir.exists())
					baseDir.mkdirs();
				String rootFilePath = baseDir.getCanonicalPath();

				if(fullPath.equals(rootFilePath))
				{
					bAtRoot = true;
					pathSelected = x;
					parentDirectory = paths[x];
					break;
				}
				else if(fullPath.indexOf(rootFilePath) > -1)
				{
					bAtRoot = false;
					baseDir = path;
					pathSelected = x;
					parentDirectory = paths[x];
					break;
				} 
			}
		}

		File[] files = baseDir.listFiles(new FileTypeFilter(fileMasks, 2));
		if(files == null)
			files = new File[0];

		boolean dirsAtTop = "1".equals(store.getProperty("filebrowser.DirsAtTop", "1"));
		Arrays.sort(files, new CompareFiles(dirsAtTop));

		Element elm = doc.createElement("directories");
		Element elm2 = null;
		for (int i=0; i<paths.length; i++)
		{
			elm2 = doc.createElement("directory");
			elm2.setAttribute("name", bForwardSlash ? paths[i].replace("\\", "/") : paths[i]);
			File file = new File(paths[i]);
			elm2.setAttribute("freeSpace", Long.toString(file.getFreeSpace()));
			elm2.setAttribute("minFreeSpace", sMinFreeSpace);
			if (i == pathSelected)
				elm2.setAttribute("selected", "1");
			elm.appendChild(elm2);
		}
		node.appendChild(elm);

		if (baseDir.exists() && baseDir.isDirectory())
		{
			elm = doc.createElement("files");
			elm.setAttribute("freeSpace", Long.toString(baseDir.getFreeSpace()));
			elm.setAttribute("minFreeSpace", sMinFreeSpace);
			elm.setAttribute("currentDirectory", bForwardSlash ? baseDir.getAbsolutePath().replace("\\", "/") : baseDir.getAbsolutePath());
			elm.setAttribute("directory", bForwardSlash ? parentDirectory.replace("\\", "/") : parentDirectory);
			String relativeDirectory = baseDir.getAbsolutePath();
			relativeDirectory = relativeDirectory.length() <= parentDirectory.length() ? "" : relativeDirectory.substring(parentDirectory.length()+1);
			elm.setAttribute("relativeDirectory", bForwardSlash ? relativeDirectory.replace("\\", "/") : relativeDirectory);
			if (!bAtRoot || bShowCapturePaths)
			{
				elm2 = doc.createElement("file");
				elm2.setAttribute("name", "parent");
				elm2.setAttribute("path", bAtRoot ? "/" : (bForwardSlash ? baseDir.getParentFile().getAbsolutePath().replace("\\", "/") : baseDir.getParentFile().getAbsolutePath()));
				elm2.setAttribute("type", "1");
				elm.appendChild(elm2);
				node.appendChild(elm);
			}

			Map<String, KeepForDetails> autoDeleteFiles = store.getAutoDelList();
			Map<String, String> mapFilenameToId = new HashMap<String, String>();
			Iterator<Entry<String, KeepForDetails>> iter = autoDeleteFiles.entrySet().iterator();
			while (iter.hasNext())
			{
				Entry<String, KeepForDetails> entry = iter.next();
				mapFilenameToId.put(entry.getValue().getFileName(), entry.getKey());
			}

			for (int i=0; i<files.length; i++)
			{
				if(files[i].isHidden() == false)
				{
					elm2 = doc.createElement("file");
					elm2.setAttribute("name", bForwardSlash ? files[i].getName().replace("\\", "/") : files[i].getName());
					elm2.setAttribute("path", bForwardSlash ? files[i].getAbsolutePath().replace("\\", "/") : files[i].getAbsolutePath());

					if(files[i].isDirectory())
					{
						elm2.setAttribute("type", "1");
					}
					else
					{
						elm2.setAttribute("type", "0");
						elm2.setAttribute("size", Long.toString(files[i].length()));
						String autoDeleteId = mapFilenameToId.get(files[i].getCanonicalPath());
						if (autoDeleteId != null)
							elm2.setAttribute("autoDeleteId", autoDeleteId);
					}
					elm.appendChild(elm2);
					node.appendChild(elm);
				}
			}

			elm = doc.createElement("allTasks");
			store.getTaskListXml(doc, elm);
			node.appendChild(elm);
		}

		return baseDir;
	}

	private byte[] xmlFileList() throws Exception
	{
		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "filer", null);
		Element root = doc.getDocumentElement();
		String directory = m_urlData.getParameter("path");
		File baseDir = listFilesDirectories(directory, m_store.getProperty("filebrowser.masks", ".log,.mpg,.mpeg,.bin,.mpv,.mpa,.ts,.tp,.dvr-ms,.rec,.pva,.avi,.mp4,.mkv,.xml,.srt"), root, doc, false, false);
		root.setAttribute("path", baseDir.getAbsolutePath());
		root.setAttribute("freeSpace", Long.toString(baseDir.getFreeSpace()));
		long minFreeSpace = m_store.getPropertyLong("Capture.minFreeSpace", 20000) * 1024 * 1024;
		String sMinFreeSpace = Long.toString(minFreeSpace);
		root.setAttribute("minFreeSpace", sMinFreeSpace);

		//
		// Send output back 
		//
		XSL transformer = new XSL(doc, "fileList.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] deleteFile() throws Exception
	{
		File thisFile = new File(m_urlData.getParameter("file"));
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}      
		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		if(thisFile != null && thisFile.exists())
		{
			DVScheduler.log( (thisFile.delete() ? "File deleted: " : "Cannot delete: ") + thisFile.getAbsolutePath() );
		}

		return getRedirUrlRedir("?path=" + Formatter.encodeURL_UTF8E(thisFile.getParent())).getBytes("UTF-8");   
	}

	private byte[] showCommandList() throws Exception
	{
		StringBuffer out = new StringBuffer(256);
		String file = m_urlData.getParameter("file");

		HashMap<String, TaskCommand> tasks = m_store.getTaskList();
		String[] keys = (String[])tasks.keySet().toArray(new String[0]);

		Arrays.sort(keys);

		for(int x = 0; x < keys.length; x++)
		{
			TaskCommand command = (TaskCommand)tasks.get(keys[x]);

			if(command != null && command.getEnabled())
			{
				out.append("<tr><td class='itemheading'><a class='noUnder' href='/servlet/" + m_urlData.getServletClass());
				out.append("?action=03&file=" + Formatter.encodeURL(file, "UTF-8") + "&command=" + Formatter.encodeURL(keys[x], "UTF-8"));
				out.append("'><strong><center>" + HTMLEncoder.encode(keys[x]) + "</center></strong></a></td><tr>\n");
			}
		}      

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "CommandList.html");
		template.replace("$commandList", out.toString());
		template.replace("$filename", file);

		return template.getPageBytes();
	}

	private byte[] runCommand() throws Exception
	{
		String commandID = m_urlData.getParameter("command");

		File thisFile = new File(m_urlData.getParameter("file"));
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}      

		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		Map<String, TaskCommand> tasks = m_store.getTaskList();
		TaskCommand taskCommand = tasks.get(commandID);

		if(taskCommand != null)
		{
			String command = taskCommand.getCommand();
			boolean autoRem = taskCommand.getAutoRemove();

			command = command.replace("$filename", thisFile.getCanonicalPath());

			if ( taskCommand.isConditional() )
			{
				TaskItem taskItem = new TaskItem(taskCommand, command, null);
				ThreadLock.getInstance().getLock();
				try
				{
					DVScheduler.log("Queuing : " + command);
					m_store.m_waitingTaskList.add(taskItem);
				}
				finally
				{
					ThreadLock.getInstance().releaseLock();
				}
			}
			else
			{
				ThreadLock.getInstance().getLock();
				try
				{
					DVScheduler.log("Running : " + command);

					CommandWaitThread comThread = new CommandWaitThread(command, taskCommand.getDelay());
					Thread comWait = new Thread(Thread.currentThread().getThreadGroup(), comThread, comThread.getClass().getName());
					comWait.start();
					String key = new Long(new Date().getTime()).toString() + "-" + new Long(m_store.m_rand.nextLong()).toString();
					TaskItemThread taskItem = new TaskItemThread(key, comThread, command, autoRem);
					taskItem.m_allowSuspend = taskCommand.isAllowSuspend();
					Thread taskThread = new Thread(Thread.currentThread().getThreadGroup(), taskItem, taskItem.getClass().getName());
					taskThread.start();
					m_store.m_runningTaskList.put(key, taskItem);
				}
				finally
				{
					ThreadLock.getInstance().releaseLock();
				}
			}
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/TaskManagementDataRes\n\n");
		return out.toString().getBytes("UTF-8");
	}

	private byte[] addAutoDelItem() throws Exception
	{
		File thisFile = new File(m_urlData.getParameter("file"));
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}
		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		if(thisFile != null && thisFile.exists())
		{
			m_store.addAutoDeleteItem(requestedFilePath, m_store.getPropertyInt("AutoDel.KeepFor", 30) * 24 * 60);
			m_store.saveAutoDelList();
		}

		return getRedirUrlRedir("?path=" + Formatter.encodeURL_UTF8E(thisFile.getParent())).getBytes("UTF-8");
	}

	private byte[] removeAutoDelItem() throws Exception
	{
		String id = m_urlData.getParameter("id");
		m_store.removeAutoDeleteItem(id);
		m_store.saveAutoDelList();
		File thisFile = new File(m_urlData.getParameter("file"));
		return getRedirUrlRedir("?path=" + Formatter.encodeURL_UTF8E(thisFile.getParent())).getBytes("UTF-8");
	}

	private String addLength(String text)
	{
		return String.format("%1$04d", text.length()+4) + text;
	}

	private String getValue(String text)
	{
		return text.substring(text.indexOf("->") + 2);
	}

	// return value:
	// 100 file not found
	// 101 ok, also return task id
	// 102 play command file error
	// 104 error
	public byte[] playFile() throws Exception
	{
		String filePath = m_urlData.getParameter("filePath");
		if (filePath == null)
			return addLength("DVSchedulerFile|100").getBytes("UTF-8");

		File thisFile = new File(filePath);
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}
		}

		if (!bFound)
		{
			return addLength("DVSchedulerFile|100").getBytes("UTF-8");
		}

		String command = "";
		String clientGetAddress = "";
		boolean bSeekable = false;

		String prop = m_store.getProperty("Surf.remote.file", "HTTP");
		if (prop.equals("*"))
			return addLength("DVSchedulerFile|102").getBytes("UTF-8");
		String commandFile = m_store.getProperty("Capture.engine.path", "util") + File.separator + prop + ".pfr";
		File dir = new File(commandFile);

		int filenameEncoding = Formatter.TYPE_URL_UTF8;
		if (!dir.exists() || !dir.isFile())
		{
			DVScheduler.log("Play command file (" + commandFile + ") doesn't exist");
			return addLength("DVSchedulerFile|102").getBytes("UTF-8");
		}
		else if (dir.length() > 4096)
		{
			DVScheduler.log("Play command file (" + commandFile + ") > 4096 bytes - too big, check for error");
			return addLength("DVSchedulerFile|102").getBytes("UTF-8");
		}
		else
		{
			try
			{
				BufferedReader r = new BufferedReader(new FileReader(commandFile));
				String line = null;

				while ((line = r.readLine()) != null)
				{
					line = line.trim();

					if (line.startsWith("Client Get Address->"))
					{
						clientGetAddress = getValue(line).trim();
					}
					else if (line.startsWith("Is Seekable->"))
					{
						bSeekable = getValue(line).trim().equals("1");
					}
					else if (line.startsWith("Filename Encoding->"))
					{
						String enc = line.substring(19).trim();

						filenameEncoding = Formatter.getEncoding(enc, Formatter.TYPE_URL_UTF8);
					}
					else if (!line.startsWith("//") && !line.isEmpty())
						command += line + "\n";
				}

				r.close();
			}
			catch (Exception e)
			{
				DVScheduler.log("Error reading play command file (" + commandFile + ")");
				e.printStackTrace();
				return addLength("DVSchedulerFile|102").getBytes("UTF-8");
			}
		}

		ThreadLock.getInstance().getLock();
		try
		{
			int streamPort = -1;
			if ( command.contains("$streamport") || clientGetAddress.contains("$streamport") )
				streamPort = m_store.getStreamPort();

			command = replaceKeywords(command, thisFile, streamPort, filenameEncoding);
			clientGetAddress = replaceKeywords(clientGetAddress, thisFile, streamPort, filenameEncoding);

			DVScheduler.log("Running : " + command);

			CommandWaitThread comThread = new CommandWaitThread(command, 0);
			Thread comWait = new Thread(Thread.currentThread().getThreadGroup(), comThread, comThread.getClass().getName());
			comWait.start();
			String key = new Long(new Date().getTime()).toString() + "-" + new Long(m_store.m_rand.nextLong()).toString();
			TaskItemThread taskItem = new TaskItemThread(key, comThread, command, true);
			taskItem.m_streamPort = streamPort;
			Thread taskThread = new Thread(Thread.currentThread().getThreadGroup(), taskItem, taskItem.getClass().getName());
			taskThread.start();
			m_store.m_runningTaskList.put(key, taskItem);

			return addLength("DVSchedulerFile|101|" + key + "|" + 
					thisFile.getName() + "|" + 
					clientGetAddress + "|" + 
					(bSeekable ? "1" : "0")).getBytes("UTF-8");
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}
	}

	public byte[] changePlayFile() throws Exception
	{
		String taskId = m_urlData.getParameter("taskId");
		if (taskId == null)
			return addLength("DVSchedulerFile|104").getBytes("UTF-8");

		ThreadLock.getInstance().getLock();
		try
		{
			TaskItemThread task = m_store.m_runningTaskList.get(taskId);

			if (task != null)
				task.killNow();
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		return playFile();
	}

	// return value:
	// 301 ok
	// 302 task not found
	public byte[] stopPlayFile() throws Exception
	{
		String taskId = m_urlData.getParameter("taskId");
		if (taskId == null)
			return addLength("DVSchedulerFile|302").getBytes("UTF-8");

		ThreadLock.getInstance().getLock();
		try
		{
			TaskItemThread task = m_store.m_runningTaskList.get(taskId);

			if (task == null)
			{
				//DVScheduler.log("stopPlayFile: task not found: " + taskId);
				return addLength("DVSchedulerFile|302").getBytes("UTF-8");
			}

			task.killNow();
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		return addLength("DVSchedulerFile|301").getBytes("UTF-8");
	}

	private String replaceKeywords(String text, File thisFile, int streamPort, int filenameEncoding) throws Exception
	{
		text = text.replace("$streamport", Integer.toString(streamPort));

		String remoteAddress = m_headers.get("RemoteAddress");
		if ( remoteAddress.equals("127.0.0.1") || remoteAddress.equals("0:0:0:0:0:0:0:1") )
			remoteAddress = "localhost";
		text = text.replace("$clientname", remoteAddress);
		String serverPath = m_headers.get("Host");
		text = text.replace("$serverpath", serverPath);
		int sep = serverPath.lastIndexOf(":");
		String serverName = (sep > -1) ? serverPath.substring(0, sep) : serverPath;
		text = text.replace("$servername", serverName);
		text = text.replace("$serverurl", (m_store.getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) ? "https://" + serverName + ":" + m_store.getHttpsPort() : "http://" + serverName + ":" + DVScheduler.getHttpPort());

		text = text.replace("$filenameonly", thisFile.getName());
		text = text.replace("$encfilenameonly", Formatter.encode(thisFile.getName(), filenameEncoding));
		text = text.replace("$filename", thisFile.getAbsolutePath());
		text = text.replace("$encfilename", Formatter.encode(thisFile.getAbsolutePath(), filenameEncoding));
		text = text.replace("$capturepath", thisFile.getParent());
		text = text.replace("$enccapturepath", Formatter.encode(thisFile.getParent(), filenameEncoding));
		text = m_store.replaceKeywords(text);

		return text;
	}

	private void uploadFile() throws Exception
	{
		uploadFile(m_urlData.getParameter("file"));
	}

	private void uploadFile(String fileName) throws Exception
	{
		File thisFile = new File(fileName);
		String requestedFilePath = thisFile.getCanonicalPath();

		String[] paths = m_store.getServeFilePaths();
		boolean bFound = false;
		for (int i=0; i<paths.length; i++)
		{
			if (paths[i].isEmpty())
				paths[i] = ".";

			File baseDir = new File(paths[i]);
			String rootFilePath = baseDir.getCanonicalPath();

			if (requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}      
		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		String ip = m_headers.get("RemoteAddress");
		if ( ip.equals("127.0.0.1") || ip.equals("0:0:0:0:0:0:0:1") )
			ip = "localhost";
		Thread.currentThread().setName("Serve \"" + fileName + "\" to " + ip);
		FileUploader fileUploader = new FileUploader(m_outStream, m_headers);
		//fileUploader.uploadFile(thisFile, true, m_store.getProperty("Preview.disposition", "inline"));
		fileUploader.uploadFile(thisFile, true, "attachment");
	}

	protected class FileName
	{
		String m_name, m_fullPath;
		long m_length;
	}

	private void listFiles(File directory, FileTypeFilter filter, LinkedList<FileName> list)
	{
		File[] files = directory.listFiles(filter);
		for (int j=0; j<files.length; j++)
		{
			if (files[j].isFile() && !files[j].isHidden())
			{
				FileName fileName = new FileName();
				fileName.m_name = files[j].getName();
				fileName.m_fullPath = files[j].getAbsolutePath();
				fileName.m_length = files[j].length();
				list.addLast(fileName);
			}
			else if (files[j].isDirectory() && !files[j].isHidden())
				listFiles(files[j], filter, list);
		}
	}

	private String replaceKeywords(String text, ScheduleItem item)
	{
		text = m_store.replaceKeywords(text);
		text = item.replaceKeywords(text);	// Has to be before Channel::replaceKeywords(...)

		Channel ch = m_store.getChannel(item.getChannel());	// Can't get running channel.
		if (ch != null)
			text = ch.replaceKeywords(text);

		return text;
	}
	
	private void uploadPlaylist() throws Exception
	{
		LinkedList<FileName> list = new LinkedList<FileName>();
		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		FileTypeFilter filter = new FileTypeFilter(m_store.getProperty("filebrowser.surfmasks", ".mpg,.mpeg,.bin,.mpv,.mpa,.ts,.tp,.dvr-ms,.rec,.pva,.avi,.mp4,.mkv"), 2);
		for (int i=0; i<paths.length; i++)
		{
			if (paths[i].isEmpty())
				paths[i] = ".";

			listFiles(new File(paths[i]), filter, list);
		}
		Comparator<FileName> comparator = new Comparator<FileName>() {
			public int compare(FileName g1, FileName g2) {
				return g1.m_fullPath.compareToIgnoreCase(g2.m_fullPath);
			}
		};
		Collections.sort(list, comparator);

		StringBuffer text = new StringBuffer();
		text.append("#EXTM3U\n");

		boolean bLocal = isLocal();
		//String hostUrl = (m_store.getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) ? "https://" + m_headers.get("Host") : "http://" + m_headers.get("Host") + "/servefile/";
		String host = m_headers.get("Host");
		int sep = host.lastIndexOf(":");
		host = (sep > -1) ? host.substring(0, sep) : host;
		String host2 = "http://" + host + ":" + DVScheduler.getHttpPort();
		String hostUrl = host2 + "/servefile/";
		Iterator<FileName> iter = list.iterator();
		while (iter.hasNext())
		{
			FileName fileName = iter.next();
			//text.append("#EXTINF:-1," + fileName.m_name + "\n");
			text.append("#EXTINF:-1," + fileName.m_fullPath + " (" + fileName.m_length/1048576 + "MB)\n");
			if (bLocal)
				text.append(fileName.m_fullPath + "\n");
			else
				text.append(hostUrl + Formatter.encodeURL_UTF8E(fileName.m_fullPath) + "\n");
		}

		// Do all the channels.
		ScheduleItem item = new ScheduleItem(m_store.m_rand.nextLong());

		item.setPlaylist();
		item.setCapType("Playlist");

		// check name pattern exists
		String[] namePatterns = m_store.getNamePatterns();
		String namePattern = namePatterns[0];

		// build the Calendar object and set the start time
		Calendar newDate = Calendar.getInstance();

		item.setType(ScheduleItem.ONCE);
		String ip = m_headers.get("RemoteAddress");
		if ( ip.equals("127.0.0.1") || ip.equals("0:0:0:0:0:0:0:1") )
			ip = "localhost";
		item.setName("Preview from " + ip);

		// Set the initial status
		item.setState(ScheduleItem.WAITING);
		item.setStatus("Waiting");

		// Reset the abort if are editing
		item.resetAbort();

		// Set the start/stop and duration
		item.setStart(newDate);
		item.setDuration(10);

		// Set the Auto Del option
		item.setAutoDeletable(true);

		item.setFilePattern(namePattern);

		item.setKeepFor(-1);

		item.setPostTask("");

		item.setNames(m_headers);

		// Check capture type.
		String capType = item.getCaptureCommandFile();
		String capTypeFile = m_store.getProperty("Capture.engine.path", "util") + File.separator + capType;
		File dir = new File(capTypeFile);

		String createFileName = "";
		String createFileType = "";
		String createFileContent = "";
		if (!dir.exists() || !dir.isFile())
		{
			DVScheduler.log("Capture type file (" + capType + ") doesn't exist");
			throw new Exception("Capture type file (" + capType + ") doesn't exist");
		}
		else if (dir.length() > 4096)
		{
			DVScheduler.log("Capture type file (" + capType + ") > 4096 bytes - too big, check for error");
			throw new Exception("Capture type file (" + capType + ") > 4096 bytes - too big, check for error");
		}
		else
		{
			BufferedReader r = new BufferedReader(new FileReader(capTypeFile));
			String line = null;

			while ((line = r.readLine()) != null)
			{
				line = line.trim();

				if (line.startsWith("Create File Name->"))
				{
					createFileName = getValue(line).trim();
				}
				else if (line.startsWith("Create File Type->"))
				{
					createFileType = getValue(line).trim();
				}
				else if (line.startsWith("Create File Content->"))
				{
					createFileContent += getValue(line) + "\r\n";
				}
			}

			r.close();

			if ( createFileName.isEmpty() || createFileType.isEmpty() || createFileContent.isEmpty() ) 
			{
				DVScheduler.log("Error reading capture type file (" + capType + "): One or more of CreateFileName, CreateFileType, CreateFileContent missing.");
				throw new Exception("Error reading capture type file (" + capType + "): One or more of CreateFileName, CreateFileType, CreateFileContent missing.");
			}
		}
		
		Vector<String> channels = m_store.m_channelOrder.getChannels();
		for (int i=0; i<channels.size(); i++)
		{
			item.setChannel(channels.get(i));
			String createFileContent2 = replaceKeywords(createFileContent, item);
			text.append(createFileContent2);
		}

		byte[] text2 = text.toString().getBytes("UTF-8");

		StringBuffer header = new StringBuffer();
		header.append("HTTP/1.0 200 OK\nContent-Type: " + createFileType + "\n");
		header.append("Content-Disposition: " + m_store.getProperty("Preview.disposition", "inline") + "; filename=\"" + createFileName + "\"\n");
		header.append("Content-Length: " + text2.length + "\n");
		header.append("Cache-Control: no-cache\n");
		header.append("\n");
		m_outStream.write(header.toString().getBytes("UTF-8"));
		m_outStream.write(text2);
	}
	
	private void uploadPlaylistOrRedirect() throws Exception
	{
		if ( (m_store.getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) && m_store.isAllowUnsecureDownload() )
		{
			String host = m_headers.get("Host");
			int sep = host.lastIndexOf(":");
			if (sep > -1)
				host = host.substring(0, sep);
			if ( host.equals("127.0.0.1") || host.equals("0:0:0:0:0:0:0:1") )
			{
				host = "localhost";
			}
			m_outStream.write( ("HTTP/1.0 302 Moved Temporarily\nLocation: http://" + host + ":" + DVScheduler.getHttpPort() + "/playlistAll/\n\n").getBytes() );
		}
		else
			uploadPlaylist();
	}
}
