import java.io.Serializable;
import java.util.*;

public class GuideChannel implements Serializable
{
	private static final long serialVersionUID = 1L;
	private String m_epgChannelId = "";
	private String m_epgChannelName = "";
	private String m_channelSetting = "";
	private SortedMap<Long, GuideItem> m_items = new TreeMap<Long, GuideItem>();
	private long m_loadTime = 0;
	private int m_originalNetworkId = -2;
	private int m_transportStreamId = -2;
	private int m_sourceId = -2;
	private int m_programId = -2;

	public GuideChannel(String epgChannelId, String epgChannelName, String channelSetting, int originalNetworkId, 
			int transportStreamId, int sourceId, int programId)
	{
		m_epgChannelId = epgChannelId;
		m_epgChannelName = epgChannelName;
		m_channelSetting = channelSetting;
		m_originalNetworkId = originalNetworkId;
		m_transportStreamId = transportStreamId;
		m_sourceId = sourceId;
		m_programId = programId;
		m_loadTime = new Date().getTime();
	}

	public void clear()
	{
		m_items.clear();
		m_loadTime = new Date().getTime();
	}

	public String getEpgChannelId()
	{
		return m_epgChannelId;
	}

	public void setEpgChannelId(String epgChannelId)
	{
		m_epgChannelId = epgChannelId;
	}

	public String getEpgChannelName()
	{
		return m_epgChannelName;
	}

	public int getOriginalNetworkId()
	{
		return m_originalNetworkId;
	}

	public int getTransportStreamId()
	{
		return m_transportStreamId;
	}

	public int getSourceId()
	{
		return m_sourceId;
	}

	public int getProgramId()
	{
		return m_programId;
	}

	public void setOriginalNetworkId(int originalNetworkId)
	{
		m_originalNetworkId = originalNetworkId;
	}

	public void setTransportStreamId(int transportStreamId)
	{
		m_transportStreamId = transportStreamId;
	}

	public void setSourceId(int sourceId)
	{
		m_sourceId = sourceId;
	}

	public void setProgramId(int programId)
	{
		m_programId = programId;
	}

	public void setEpgChannelName(String epgChannelName)
	{
		m_epgChannelName = epgChannelName;
	}

	public void setChannelSetting(String channelSetting)
	{
		m_channelSetting = channelSetting;
	}

	public void add(GuideItem item)
	{
		m_items.put(item.getStart().getTime(), item);
	}

	public void addOldItems(GuideChannel guideChannel)
	{
		long minTime = getMinStartTime();
		Iterator<GuideItem> iter = guideChannel.m_items.values().iterator();
		while (iter.hasNext())
		{
			GuideItem item = iter.next();
			if (item.getStop().getTime() > minTime)
				break;

			add(item);
		}
	}

	public int getSize()
	{
		return m_items.size();
	}

	public boolean isEmpty()
	{
		return m_items.isEmpty();
	}

	public boolean isEmptyFutureEvents(long time)
	{
		if (isEmpty())
			return true;
		
		return (m_items.lastKey() < time);
	}
	
	public String getChannelSetting()
	{
		return m_channelSetting;
	}

	public GuideItem getItem(String startTime)
	{
		return getItem(Long.parseLong(startTime));
	}

	public GuideItem getItem(long startTime)
	{
		return m_items.get(startTime);
	}

	public GuideItem[] getItems()
	{
		return m_items.values().toArray(new GuideItem[0]);
	}

	public GuideItem[] getItems(Date start, Date end)
	{
		return getItems(start.getTime(), end.getTime());
	}

	public GuideItem[] getItems(long start, long end)
	{
		return m_items.subMap(start, end).values().toArray(new GuideItem[0]);
	}

	public GuideItem[] getItemsInc(Date start, Date end)
	{
		return getItemsInc(start.getTime(), end.getTime());
	}

	public GuideItem[] getItemsInc(long start, long end)
	{
		List<GuideItem> subList = new LinkedList<GuideItem>();
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			GuideItem item = iter.next();
			if(item.getStop().getTime() > (start + 5000)
					&& item.getStart().getTime() < end)
			{
				subList.add(item);
			}
		}

		GuideItem[] items = (GuideItem[]) subList.toArray(new GuideItem[0]);
		return items;
	}

	public GuideItem[] getItems(String name)
	{
		List<GuideItem> matchingItems = new LinkedList<GuideItem>();
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			GuideItem item = iter.next();
			if(name.equals(item.getName()))
			{
				matchingItems.add(item);
			}
		}

		GuideItem[] items = (GuideItem[]) matchingItems.toArray(new GuideItem[0]);
		return items;
	}

	public void getFirstLetters(Collection<String> col)
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			col.add(iter.next().getName().substring(0, 1).toUpperCase());
		}
	}

	public void getNamesStartingWith(String starting, Collection<String> col)
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			String name = iter.next().getName();
			if (name.toUpperCase().startsWith(starting.toUpperCase()))
				col.add(name);
		}
	}

	public void getNames(Collection<String> col)
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			col.add(iter.next().getName());
		}
	}

	public long getMinStartTime()
	{
		return m_items.firstKey();
	}

	public long getMaxStartTime()
	{
		return m_items.lastKey();
	}

	public void setLoadTime(long loadTime)
	{
		m_loadTime = loadTime;
	}

	public long getLoadTime()
	{
		return m_loadTime;
	}

	public void removeOldItems(long minTime)
	{
		m_items.headMap(minTime).clear();
	}

	public void clearAutoAddMatched()
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			iter.next().clearFavoriteFlaggers();
		}
	}

	public void setToUpperCase()
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			iter.next().setNameToUpperCase();
		}
	}

	public void setToLowerCase()
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			iter.next().setNameToLowerCase();
		}
	}

	public void setToTitleCase()
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			iter.next().setNameToTitleCase();
		}
	}

	public void getCategories(Collection<String> col)
	{
		Iterator<GuideItem> iter = m_items.values().iterator();
		while (iter.hasNext())
		{
			Vector<String> categories = iter.next().getCategories();
			if (categories != null)
				col.addAll(categories);
		}
	}

	public boolean isEpgForChannel(Channel channel)
	{
		int onid = channel.getOriginalNetworkId();
		int tsid = channel.getTransportStreamId();
		int sid = channel.getSourceId();
		int prid = channel.getProgramID();

		if ((onid >= 0) && (tsid >= 0) && (prid >= 0))
			return (onid == m_originalNetworkId) && (tsid == m_transportStreamId) && (prid == m_programId);

		if ((tsid >= 0) && (sid >= 0))
			return (tsid == m_transportStreamId) && (prid == m_programId);

		return false;

	}
}