/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;

class GuideItem implements Serializable, Comparable<GuideItem>
{
	private static final long serialVersionUID = 347376695042195476L;
	String name = "";
	String subName = "";
	String description = "";
	Date startTime = null;
	Date endTime = null;
	String language = "English";
	String ratings = "";
	String url = "";
	boolean widescreen = false;
	boolean highdef = false;
	boolean bw = false;
	boolean ac3 = false;
	boolean surround = false;
	boolean closedcaptions = false;
	boolean repeat = false;
	boolean premiere = false;
	boolean live = false;
	boolean lastchance = false;
	boolean interactive = false;
	boolean bAutoMark = false;	// Flagged as favorite.
	List<Long> m_favoriteFlaggers = null;	// Who set favorite to true.
	Vector<String> m_categories = null;

	int duration = 0;

	public GuideItem()
	{
	}

	public GuideItem clone()
	{
		GuideItem clonedItem = new GuideItem();

		clonedItem.name = name;
		clonedItem.subName = subName;
		clonedItem.description = description;
		clonedItem.startTime = startTime;
		clonedItem.setStop(endTime);
		clonedItem.description = description;
		clonedItem.language = language;
		clonedItem.ratings = ratings;
		clonedItem.url = url;
		clonedItem.widescreen = widescreen;
		clonedItem.highdef = highdef;
		clonedItem.bw = bw;
		clonedItem.ac3 = ac3;
		clonedItem.surround = surround;
		clonedItem.closedcaptions = closedcaptions;
		clonedItem.repeat = repeat;
		clonedItem.premiere = premiere;
		clonedItem.live = live;
		clonedItem.lastchance = lastchance;
		clonedItem.interactive = interactive;
		clonedItem.bAutoMark = bAutoMark;
		if ( m_favoriteFlaggers != null )
		{
			clonedItem.m_favoriteFlaggers = new LinkedList<Long>();
			clonedItem.m_favoriteFlaggers.addAll(m_favoriteFlaggers);
		}
		if ( m_categories != null )
		{
			clonedItem.m_categories = new Vector<String>();
			clonedItem.m_categories.addAll(m_categories);
		}

		return clonedItem;
	}

	public String getURL()
	{
		return url;
	}

	public void setURL(String data)
	{
		url = data;
	}

	public void setName(String name)
	{
		this.name = name;
	}
	
	public void setNameToUpperCase()
	{
		name = name.toUpperCase();
	}

	public void setNameToLowerCase()
	{
		name = name.toLowerCase();
	}

	// Note: doesn't work with some "unicode" sentences.
	public void setNameToTitleCase()
	{
		if ( (name.equals(name.toUpperCase())) || (name.equals(name.toLowerCase())) )
		{
			char prev = ' ';
			StringBuffer buf = new StringBuffer(name.toLowerCase());
			for (int i=0; i<name.length(); i++)
			{
				if ( (prev == ' ') || (prev == '.') || (prev == '-')
						|| (prev == '*') || (prev == '(') || (prev == '~') 
						|| (prev == '!') || (prev == '@') || (prev == '#') 
						|| (prev == '&') || (prev == '=') || (prev == '+') 
						|| (prev == '[') || (prev == '{') || (prev == '|') 
						|| (prev == ';') || (prev == ':') || (prev == ',') 
						|| (prev == '<') || (prev == '/') || (prev == '?') )
				{
					buf.setCharAt(i, name.substring(i, i+1).toUpperCase().charAt(0));
				}
				prev = name.charAt(i);
			}
			name = buf.toString();
		}
	}

	public void setSubName(String sub)
	{
		this.subName = sub;
	}

	public void setDescription(String desc)
	{
		this.description = desc;
	}

	public void setStart(Date start)
	{
		startTime = start;
		startTime.setTime((startTime.getTime() / 60000) * 60000);

		if(duration != 0)
		{
			Calendar cal = Calendar.getInstance();
			cal.setTime(startTime);
			cal.add(Calendar.MINUTE, duration);
			endTime = cal.getTime();
		}
	}

	public void setDuration(int duration)
	{
		this.duration = duration;

		if(startTime !=null)
		{
			Calendar cal = Calendar.getInstance();
			cal.setTime(startTime);
			cal.add(Calendar.MINUTE, duration);
			endTime = cal.getTime();
		}
	}

	public void setStop(Date stop)
	{
		endTime = stop;
		endTime.setTime((endTime.getTime() / 60000) * 60000);

		if(duration == 0 && startTime != null)
		{
			duration = (int)((endTime.getTime() - startTime.getTime()) / (1000 * 60));
		}
	}   

	public void addCategory(String cat)
	{
		if ( (cat == null) || cat.isEmpty() )
			return;

		if (m_categories == null)
			m_categories = new Vector<String>();

		m_categories.add(cat.trim());
	}

	public void setCategories(Vector<String> categories)
	{
		if (categories == null)
		{
			if ( m_categories != null )
			{
				m_categories.removeAllElements();
				m_categories = null;
			}
		}
		else
		{
			if (m_categories == null)
				m_categories = new Vector<String>();
			else
				m_categories.removeAllElements();

			m_categories.addAll(categories);
		}
	}

	public void setLanguage(String lng)  {this.language = lng;}
	public void setRatings(String rtngs)  {this.ratings = rtngs;}
	public void setWidescreen(boolean ws) { this.widescreen = ws;}
	public void setHighDef(boolean hd) { this.highdef = hd;}
	public void setRepeat(boolean rpt) { this.repeat = rpt;}
	public void setPremiere(boolean prm) { this.premiere = prm;}
	public void setLive(boolean lv) { this.live = lv;}
	public void setInteractive(boolean mhp) { this.interactive = mhp;}
	public void setBlackWhite(boolean bw) { this.bw = bw;}
	public void setSurround(boolean srnd) { this.surround = srnd;}
	public void setAC3(boolean digsnd) { this.ac3 = digsnd;}
	public void setLastChance(boolean fin) { this.lastchance = fin;}
	public void setCaptions(boolean cpt) { this.closedcaptions = cpt;}
	public void setAutoMark(boolean cpt) { this.bAutoMark = cpt;}

	public void addFavoriteFlagger(long flaggerId)
	{
		bAutoMark = true;
		if (m_favoriteFlaggers == null)
			m_favoriteFlaggers = new LinkedList<Long>();
		m_favoriteFlaggers.add(flaggerId);
	}

	public void clearFavoriteFlaggers()
	{
		bAutoMark = false;
		if (m_favoriteFlaggers != null)
		{
			m_favoriteFlaggers.clear();
			m_favoriteFlaggers = null;
		}
	}

	public Long getFavoriteFlagger()
	{
		return ( (m_favoriteFlaggers == null) || m_favoriteFlaggers.isEmpty() ) ? null : m_favoriteFlaggers.get(0);
	}

	public List<Long> getFavoriteFlaggers()
	{
		return m_favoriteFlaggers;
	}

	public String getName()
	{
		return name;
	}
	public String getSubName()
	{
		return subName;
	}
	public String getDescription()
	{
		return description;
	}
	public Date getStart()
	{
		return startTime;
	}
	public Date getStop()
	{
		return endTime;
	}
	public int getDuration()
	{
		return duration;
	}

	public String getFirstCategory()
	{
		return ((m_categories == null) || m_categories.isEmpty()) ? "" : m_categories.get(0);
	}

	public Vector<String> getCategories()
	{
		return m_categories;
	}

	public String getCategories2()
	{
		if ((m_categories == null) || m_categories.isEmpty())
			return "";

		String categories = m_categories.get(0);
		for (int i=1; i<m_categories.size(); i++)
		{
			categories += " | " + m_categories.get(i);
		}
		return categories;
	}

	public int getCategoriesSize()
	{
		return (m_categories == null) ? 0 : m_categories.size();
	}

	public String getLanguage()   {return language;}
	public String getRatings()   {return ratings;}
	public boolean getWidescreen()  { return widescreen;}
	public boolean getHighDef()  { return highdef;}
	public boolean getRepeat()  { return repeat;}
	public boolean getPremiere()  { return premiere;}
	public boolean getLive()  { return live;}
	public boolean getInteractive()  { return interactive;}
	public boolean getBlackWhite()  { return bw;}
	public boolean getSurround()  { return surround;}
	public boolean getAC3()  { return ac3;}
	public boolean getLastChance()  { return lastchance;}
	public boolean getCaptions()  { return closedcaptions;}
	public boolean isAutoMark()  { return bAutoMark;}

	public String toString()
	{
		return Long.toString(startTime.getTime());
	}

	public int compareTo(GuideItem com)
	{
		if (com.getStart().getTime() > getStart().getTime())
			return -1;
		else if (com.getStart().getTime() == getStart().getTime())
			return 0;
		else
			return 1;
	}
}
