/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.net.*;

public class HTTPurl
{
	private HashMap<String, String> m_args = new HashMap<String, String>();
	private int m_requestType = 0;
	private String m_requestString = "";
	private String m_servletClass = "";
	private String m_originalGET = "";
	HashMap<?, ?> m_headers = null;
	private HashMap<String, String> m_cookies = null;

	public HTTPurl(String requestURL, String postData, HashMap<?, ?> head) throws Exception
	{
		m_headers = head;
		getCookies();

		m_originalGET = requestURL;
		if (!requestURL.toUpperCase().startsWith("GET ") && !requestURL.toUpperCase().startsWith("POST "))
			throw new Exception("The request " + requestURL + " was not a valid type. (GET or POST)");

		int indexOfSpace = requestURL.indexOf(" ");
		m_requestString = requestURL.substring(indexOfSpace + 1, requestURL.indexOf(" ", indexOfSpace + 2));

		if (requestURL.startsWith("POST") && m_requestString.startsWith("/servlet/"))
		{
			m_requestType = 3;

			parseArgsAmpersand(postData);

			int paramStart = m_requestString.indexOf("?");

			if (paramStart > -1)
			{
				parseArgsAmpersand(m_requestString.substring(paramStart + 1, m_requestString.length()));
			}

			if (paramStart < 1)
				paramStart = m_requestString.length();

			m_servletClass = m_requestString.substring(9, paramStart);
			m_servletClass = m_servletClass.trim();

		}
		else if (m_requestString.equals("/") || m_requestString.startsWith("/?"))
		{
			m_requestType = 1;
			int paramStart = m_requestString.indexOf("?");
			if (paramStart > -1)
			{
				parseArgsAmpersand(m_requestString.substring(paramStart + 1, m_requestString.length()));
			}
		}
		else if (m_requestString.startsWith("/servlet/"))
		{
			m_requestType = 3;

			int paramStart = m_requestString.indexOf("?");

			if (paramStart > -1)
			{
				parseArgsAmpersand(m_requestString.substring(paramStart + 1, m_requestString.length()));
			}

			if (paramStart < 1)
				paramStart = m_requestString.length();

			m_servletClass = m_requestString.substring(9, paramStart);
			m_servletClass = m_servletClass.trim();
		}
		//else if ( m_requestString.startsWith("/servefile/") || m_requestString.startsWith("/playlistAll/") )
		else if ( m_requestString.startsWith("/servefile/") )
		{
			m_requestType = 4;
			m_servletClass = "FileManagementDataRes";
			parseArgsSlash(m_requestString);
		}
		else if ( m_requestString.startsWith("/stream/") || m_requestString.startsWith("/stream2/") || m_requestString.startsWith("/playlist/") )
		{
			m_requestType = 4;
			m_servletClass = "ScheduleDataRes";
			parseArgsSlash(m_requestString);
		}
		else
		{
			m_requestType = 2;
		}
	}

	public String toString()
	{
		return m_originalGET;
	}

	public String getServletClass()
	{
		return m_servletClass;
	}

	public int getRequestType()
	{
		return m_requestType;
	}

	public String getRequestString()
	{
		return m_requestString;
	}

	public String getRequestString2() throws Exception
	{
		return "&dari=" + Formatter.encodeURL(getRequestString(), "UTF-8");
	}

	public String getParameter(String name)
	{
		return (String) m_args.get(name);
	}

	public String[] getParameterList()
	{
		Set<String> params = m_args.keySet();

		String[] keys = new String[params.size()];

		Iterator<String> stepper = params.iterator();
		int index = 0;
		while (stepper.hasNext())
		{
			keys[index++] = (String) stepper.next();
		}

		return keys;
	}

	public String getParameterStartsWith(String name)
	{
		Set<String> params = m_args.keySet();

		Iterator<String> stepper = params.iterator();
		while (stepper.hasNext())
		{
			String name2 = stepper.next();
			if (name2.startsWith(name))
				return name2;
		}

		return null;
	}

	private int parseArgsAmpersand(String reqString) throws Exception
	{
		if (reqString == null)
			return 0;

		String[] paramList = reqString.split("&");
		String value2 = null;
		m_args.clear();

		for (int x = 0; x < paramList.length; x++)
		{
			int sep = paramList[x].indexOf("=");
			if (sep > 0)
			{
				String name = paramList[x].substring(0, sep);
				String value = paramList[x].substring(sep + 1, paramList[x].length());

				name = URLDecoder.decode(name, "UTF-8");
				value = URLDecoder.decode(value, "UTF-8");

				//DVScheduler.log(name + " = " + value);
				//args.put(name, value);
				// Simple workaround to allow multiple values for this map.
				value2 = m_args.get(name);
				if (value2 != null)
					m_args.put(name, (value2 + "\n" + value));
				else
					m_args.put(name, value);
			}
		}

		return 1;
	}

	private int parseArgsSlash(String reqString) throws Exception
	{
		if (reqString == null)
			return 0;

		String[] paramList = reqString.split("/");
		m_args.clear();

		for (int x = 1; x < paramList.length; x++)
		{
			m_args.put("parameter" + (x - 1), Formatter.decodeURL_UTF8E(paramList[x]));
		}

		return 1;
	}

	private void getCookies()
	{
		m_cookies = new HashMap<String, String>();
		String cookieList = (String) m_headers.get("Cookie");
		if(cookieList == null || cookieList.length() == 0)
			return;

		String[] cutCookies = cookieList.split("; ");

		for(int x = 0; x < cutCookies.length; x++)
		{
			String[] cookieBits = cutCookies[x].split("=");
			if(cookieBits.length == 2)
			{
				m_cookies.put(cookieBits[0], cookieBits[1]);
			}
		}
	}

	public String getCookie(String name)
	{
		return (String)m_cookies.get(name);
	}

	public int getCookieInt(String name, int defaultValue)
	{
		try
		{
			defaultValue = Integer.parseInt(getCookie(name));
		}
		catch (Exception e) {}

		return defaultValue;
	}

	public String getParameter(String key, String defaultValue)
	{
		String text = getParameter(key);

		return text == null ? defaultValue : text;
	}

	public int getParameterInt(String key, int defaultValue)
	{
		try
		{
			defaultValue = Integer.parseInt(getParameter(key));
		}
		catch (Exception e) {}

		return defaultValue;
	}

	public double getParameterDouble(String key, double defaultValue)
	{
		try
		{
			defaultValue = Double.parseDouble(getParameter(key));
		}
		catch (Exception e) {}

		return defaultValue;
	}
}