/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.net.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;

class KBScheduleDataRes extends HTTPResponse
{
	public KBScheduleDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if ("01".equals(action))
		{
			m_outStream.write(showCalendar());
			return;
		}
		if ("02".equals(action))
		{
			m_outStream.write(showAddForm());
			return;
		}
		else if ("03".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addUpdateItem());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}         
			return;
		}
		else if ("04".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(showItemOptions());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}         
			return;
		}
		else if ("05".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(deleteItem());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}           
			return;
		}
		else if ("06".equals(action))
		{
			m_outStream.write(stopRunningTask());
			return;
		}
		else if ("07".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addTime());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}          
			return;
		}
		else if ("08".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(skipToNext());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}          
			return;
		}
		else if ("09".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(showLog());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}          
			return;
		}
		else if ("11".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addScheduleFromGuideItem());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}         
			return;
		}
		else
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(getScheduleTable());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}           
			return;
		}

	}

	private byte[] addScheduleFromGuideItem() throws Exception
	{
		String channel = m_urlData.getParameter("channel");
		String id = m_urlData.getParameter("id");

		GuideStore guide = GuideStore.getInstance();
		String epgChan = guide.getEpgChannelIdFromMap(channel);
		GuideItem guideItem = guide.getProgram(epgChan, id);

		if(epgChan == null || epgChan.length() == 0)
			throw new Exception("WS Channel Not Found!");

		String captype = "Capture.deftype";

		ScheduleItem schItem = new ScheduleItem(guideItem, channel,
				captype, m_store.m_rand.nextLong(), false);

		String task = m_store.getProperty("Tasks.DefTask", "");
		schItem.setPostTask(task);

		// Add time buffer
		int startBuff = 0;
		int endBuff = 0;
		int epgBuff = 0;
		try
		{
			startBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.start", "0"));
			endBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.end", "0"));
			epgBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.EPG", "0"));
		}
		catch (Exception e)
		{
		}      

		String[] patterns = m_store.getNamePatterns();
		schItem.setFilePattern(patterns[0]);

		String keepFor = m_store.getProperty("AutoDel.KeepFor", "30");
		int keepInt = 30;
		try
		{
			keepInt = Integer.parseInt(keepFor);
		}
		catch (Exception e)
		{
		}
		schItem.setKeepFor(keepInt);  
		schItem.setAutoDeletable(m_store.isAutoDelByDefault());

		schItem.setCapType(captype);

		// build the Calendar object and set the start time
		Calendar cal = Calendar.getInstance();
		cal.setTime(schItem.getStart());

		cal.add(Calendar.MINUTE,
				(startBuff * -1));
		schItem.setDuration(guideItem.getDuration()
				+ guideItem.getDuration()*epgBuff/60
				+ startBuff
				+ endBuff);
		schItem.setStart(cal);

		schItem.setType(ScheduleItem.ONCE);

		// Log the add action
		schItem.log("New Schedule added/edited");

		boolean isAlreadyInList = guide.isAlreadyInList(schItem);

		if(!isAlreadyInList)
		{
			m_store.addScheduleItemAndSave(schItem);
		}      

		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/" + m_urlData.getServletClass();  

		StringBuffer out = new StringBuffer(4096);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();      
	}

	private byte[] showLog() throws Exception
	{
		String id = m_urlData.getParameter("id");
		if (id == null || id.length() == 0)
			id = "";

		ScheduleItem item = m_store.getScheduleItem(m_urlData.getParameter("id"));

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "log", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("id", item.toString());

		Element logitem = null;
		Element elm = null;
		Text text = null;

		if (item != null)
		{
			String[] lines = item.getLog().split("\n");
			for (int x = 0; x < lines.length; x++)
			{
				logitem = doc.createElement("logitem");
				elm = doc.createElement("line");
				text = doc.createTextNode(lines[x]);
				elm.appendChild(text);
				logitem.appendChild(elm);
				root.appendChild(logitem);
			}
		}

		//
		// Send output back
		//
		XSL transformer = new XSL(doc, "kb-showlog.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] skipToNext() throws Exception
	{
		String id = m_urlData.getParameter("id");
		ScheduleItem item = m_store.getScheduleItem(m_urlData.getParameter("id"));

		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons";      

		// Only allow update to waiting schedules
		if (id != null && id.length() > 0 && item != null && item.getState() == ScheduleItem.WAITING)
		{
			item.skipToNext();
			m_store.saveSchedules(item);
		}

		// Redirect back to the main page
		StringBuffer out = new StringBuffer();
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();
	}

	private byte[] addTime() throws Exception
	{
		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons";

		ScheduleItem item = m_store.getScheduleItem(m_urlData.getParameter("id"));

		if(item != null)
		{
			item.setUserModified();

			Calendar cal = Calendar.getInstance();
			cal.setTime(item.getStart());

			// Add time buffer
			int startBuff = 0;
			int endBuff = 0;

			try
			{
				startBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.start", "0"));   
				endBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.end", "0"));   
			}
			catch(Exception e){}

			// if the item is running then only add on to the duration
			if(item.getState() != ScheduleItem.RUNNING)
			{
				cal.add(Calendar.MINUTE, (startBuff * -1));  
				item.setDuration(item.getDuration() + startBuff + endBuff);
				item.setStart(cal);
			}
			else
			{
				item.setDuration(item.getDuration() + endBuff);
			}

			m_store.saveSchedules(item);
		}

		// Redirect back to the main page
		StringBuffer out = new StringBuffer();
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();
	}

	private byte[] addUpdateItem() throws Exception
	{
		String id = m_urlData.getParameter("id");

		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons";

		ScheduleItem item = null;
		if (id != null && id.length() > 0)
		{
			HashMap<?, ?> times = m_store.getScheduleList();
			item = (ScheduleItem) times.get(id);

			// if we are trying to edit a running schedule just redirect back
			// to the main page
			if (item != null && (item.getState() != ScheduleItem.FINISHED && item.getState() != ScheduleItem.WAITING && item.getState() != ScheduleItem.SKIPPED && item.getState() != ScheduleItem.ERROR))
			{
				StringBuffer out = new StringBuffer();
				out.append("HTTP/1.0 302 Moved Temporarily\n");
				out.append("Location: " + backURL + "\n\n");
				return out.toString().getBytes();
			}
		}

		if (item != null
				&& (item.getState() == ScheduleItem.FINISHED
				|| item.getState() == ScheduleItem.WAITING
				|| item.getState() == ScheduleItem.SKIPPED 
				|| item.getState() == ScheduleItem.ERROR))
		{
			item.resetCaptureDevicesUsed();
			item.resetErrorCount();
		}

		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}          

		String mes = addSchedule(item, index);

		if (mes != null)
			throw new Exception(mes);

		StringBuffer out = new StringBuffer(4096);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();
	}

	private String addSchedule(ScheduleItem item, int index) throws Exception
	{
		// Add time buffer
		int startBuff = 0;
		int endBuff = 0;
		int epgBuff = 0;
		try
		{
			startBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.start", "0"));
			endBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.end", "0"));
			epgBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.EPG", "0"));
		}
		catch (Exception e)
		{
		}
		String addBuff = m_urlData.getParameter("buffer");

		// Get all the input data
		String y = m_urlData.getParameter("year");
		String m = m_urlData.getParameter("month");
		String d = m_urlData.getParameter("day");
		String h = m_urlData.getParameter("hour");
		String mi = m_urlData.getParameter("min");
		String dur = m_urlData.getParameter("duration");
		String channel = m_urlData.getParameter("channel");
		String name = m_urlData.getParameter("name");
		String autoDel = m_urlData.getParameter("autoDel");
		String namePattern = m_urlData.getParameter("namePattern");
		String keepFor = m_urlData.getParameter("keepfor");
		String task = m_urlData.getParameter("task");    

		// check channel name exists
		HashMap<?, ?> channels = m_store.getChannels();
		if(!channels.containsKey(channel))
		{
			throw new Exception("Channel Not Found!");
		}

		// check name pattern exists
		String[] namePatterns = m_store.getNamePatterns();
		if(namePattern == null || namePattern.length() == 0)
			namePattern = namePatterns[0];

		GuideStore guide = GuideStore.getInstance();

		boolean found = false;
		for(int x = 0; x < namePatterns.length; x++)
		{
			if(namePatterns[x].equals(namePattern))
			{
				found = true;
				break;
			}
		}
		if(!found)
		{
			throw new Exception("Name Pattern Not Found!");
		}      

		if(item != null)
		{
			m_store.removeScheduleItem(item.toString());
		}

		// Calc the duration in min
		int duration = Integer.parseInt(dur);

		int type = 0;
		try
		{
			type = Integer.parseInt(m_urlData.getParameter("type"));
		}
		catch (Exception e01)
		{
		}

		String captype = m_urlData.getParameter("captype");
		if ( (captype == null) || captype.isEmpty() )
			captype = "Capture.deftype";

		// build the Calendar object and set the start time
		Calendar newDate = Calendar.getInstance();
		newDate.set(Calendar.MILLISECOND, 0);
		newDate.set(Integer.parseInt(y), Integer.parseInt(m), Integer.parseInt(d), Integer.parseInt(h), Integer.parseInt(mi), 0);

		// If needed add the Time buffer
		if("yes".equals(addBuff))
		{
			newDate.add(Calendar.MINUTE, (startBuff * -1));
			duration = duration + duration*epgBuff/60 + startBuff + endBuff;
		}

		// Crerate the new Schedule Object
		if(item == null)
			item = new ScheduleItem(m_store.m_rand.nextLong());

		item.setCapType(captype);

		item.setType(type);
		item.setCustomDays(0);
		item.setUserModified();
		item.setName(name);

		// Set the initial status
		item.setState(ScheduleItem.WAITING);
		item.setStatus("Waiting");

		// Reset the abort if are editing
		item.resetAbort();

		// Set the start/stop and duration
		item.setStart(newDate);
		item.setDuration(duration);

		// Set the channel to capture
		item.setChannel(channel);

		// Set the Auto Del option
		if("1".equalsIgnoreCase(autoDel))
			item.setAutoDeletable(true);
		else
			item.setAutoDeletable(false);

		item.setFilePattern(namePattern);

		if(keepFor != null)
		{
			int keepInt = 30;
			try
			{
				keepInt = Integer.parseInt(keepFor);
			}
			catch (Exception e)
			{
			}
			item.setKeepFor(keepInt);
		}
		else
		{
			keepFor = m_store.getProperty("AutoDel.KeepFor", "30");
			int keepInt = 30;
			try
			{
				keepInt = Integer.parseInt(keepFor);
			}
			catch (Exception e)
			{
			}
			item.setKeepFor(keepInt);
		}

		if(task != null && !task.equalsIgnoreCase("none") && task.length() > 0)
		{
			HashMap<?, ?> tasks = m_store.getTaskList();
			if(tasks.containsKey(task))
				item.setPostTask(task);
		}
		else if(task != null && task.equalsIgnoreCase("none"))
		{
			item.setPostTask("");
		}

		// Log the add action
		item.log("New Schedule added/edited");

		boolean isAlreadyInLIst = guide.isAlreadyInList(item);

		if(isAlreadyInLIst)
		{
			return null;
		}      
		m_store.addScheduleItemAndSave(item);

		return null;
	}

	private byte[] showAddForm() throws Exception
	{
		int day = -1;
		int month = -1;
		int year = -1;

		try
		{
			day = Integer.parseInt(m_urlData.getParameter("day"));
			month = Integer.parseInt(m_urlData.getParameter("month"));
			year = Integer.parseInt(m_urlData.getParameter("year"));
		}
		catch (Exception e)
		{
		}

		ScheduleItem item = null;
		String id = m_urlData.getParameter("id");

		if (id != null && id.length() > 0)
		{
			HashMap<?, ?> times = m_store.getScheduleList();
			item = (ScheduleItem) times.get(id);
		}

		// if we are trying to edit a running schedule just redirect back
		// to the main page
		if (item != null && (item.getState() != ScheduleItem.FINISHED && item.getState() != ScheduleItem.WAITING && item.getState() != ScheduleItem.SKIPPED && item.getState() != ScheduleItem.ERROR))
		{
			StringBuffer out = new StringBuffer();
			out.append("HTTP/1.0 302 Moved Temporarily\n");
			out.append("Location: /servlet/KBScheduleDataRes\n\n");
			return out.toString().getBytes();
		}

		if (day == -1 || month == -1 || year == -1)
		{
			Date start = item.getStart();
			Calendar cal = Calendar.getInstance();
			cal.setTime(start);

			day = cal.get(Calendar.DATE);
			month = cal.get(Calendar.MONTH);
			year = cal.get(Calendar.YEAR);
		}
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "item_form", null);
		Element root = doc.getDocumentElement();

		//
		// form elements
		//

		Calendar cal = Calendar.getInstance();
		if (item != null)
			cal.setTime(item.getStart());

		root.setAttribute("month", new Integer(month).toString());
		root.setAttribute("year", new Integer(year).toString());
		root.setAttribute("date", new Integer(day).toString());
		if (item != null)
			root.setAttribute("id", id);

		Element formEl = doc.createElement("startTimeMin");
		formEl.setAttribute("Name", "Minute");
		formEl.setAttribute("max", "59");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "5");
		formEl.setAttribute("value", new Integer(cal.get(Calendar.MINUTE)).toString());
		root.appendChild(formEl);

		formEl = doc.createElement("startTimeHour");
		formEl.setAttribute("Name", "Hour");
		formEl.setAttribute("max", "23");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		formEl.setAttribute("value", new Integer(cal.get(Calendar.HOUR_OF_DAY)).toString());
		root.appendChild(formEl);

		formEl = doc.createElement("duration");
		formEl.setAttribute("Name", "Duration");
		formEl.setAttribute("max", "400");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "5");
		if (item != null)
			formEl.setAttribute("value", new Integer(item.getDuration()).toString());
		else
			formEl.setAttribute("value", "5");
		root.appendChild(formEl);

		Text text = null;

		formEl = doc.createElement("name");
		if (item != null)
			text = doc.createTextNode(item.getName());
		else
			text = doc.createTextNode("");
		formEl.appendChild(text);
		root.appendChild(formEl);

		formEl = doc.createElement("channel");
		getChannelList(doc, formEl, null);
		if (item != null)
			formEl.setAttribute("value", item.getChannel());
		else
			formEl.setAttribute("value", "");
		root.appendChild(formEl);

		formEl = doc.createElement("type");
		formEl.setAttribute("Name", "Schedule Type");
		formEl.setAttribute("max", "10");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		if (item != null)
			formEl.setAttribute("value", new Integer(item.getType()).toString());
		else
			formEl.setAttribute("value", "0");
		root.appendChild(formEl);

		formEl = doc.createElement("captureType");
		/*
		formEl.setAttribute("Name", "Capture Type");
		formEl.setAttribute("max", "10");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		 */
		getCapturetypeList(doc, formEl);
		if (item != null)
			formEl.setAttribute("value", item.getCapType());
		else
			//formEl.setAttribute("value", store.getProperty("Capture.deftype", ""));
			formEl.setAttribute("value", "Capture.deftype");
		root.appendChild(formEl);

		formEl = doc.createElement("pattern");
		getPatternList(doc, formEl);
		if (item != null)
			formEl.setAttribute("value", item.getFilePattern());
		else
			formEl.setAttribute("value", "");
		root.appendChild(formEl);

		formEl = doc.createElement("autoDel");
		formEl.setAttribute("Name", "Auto Delete");
		formEl.setAttribute("max", "1");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		if (item != null)
		{
			if (item.isAutoDeletable())
				formEl.setAttribute("value", "1");
			else
				formEl.setAttribute("value", "0");
		}
		else
			formEl.setAttribute("value", m_store.isAutoDelByDefault() ? "1" : "0");
		root.appendChild(formEl);

		formEl = doc.createElement("keepfor");
		formEl.setAttribute("Name", "keep For");
		formEl.setAttribute("max", "120");
		formEl.setAttribute("min", "1");
		formEl.setAttribute("amount", "1");
		if (item != null)
			formEl.setAttribute("value", new Integer(item.getKeepFor()).toString());
		else
		{
			String keep = m_store.getProperty("AutoDel.KeepFor", "30");
			formEl.setAttribute("value", keep);
		}
		root.appendChild(formEl);

		formEl = doc.createElement("posttask");
		getTaskList(doc, formEl);
		if (item != null)
		{
			formEl.setAttribute("value", item.getPostTask());
		}
		else
		{
			String defTask = m_store.getProperty("Tasks.DefTask", "");
			formEl.setAttribute("value", defTask);
		}
		root.appendChild(formEl);

		//
		// Send output back
		//
		XSL transformer = new XSL(doc, "kb-details.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private String getTaskList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		StringBuffer buff = new StringBuffer(1024);
		HashMap<String, TaskCommand> tasks = m_store.getTaskList();

		String[] keys = (String[]) tasks.keySet().toArray(new String[0]);

		Arrays.sort(keys);

		option = doc.createElement("option");
		text = doc.createTextNode("none");
		option.appendChild(text);
		formEl.appendChild(option);

		for (int x = 0; x < keys.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(keys[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}

		return buff.toString();
	}

	private void getCapturetypeList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		Vector<String> capTypes = m_store.getCaptureTypes();

		for (int x = 0; x < capTypes.size(); x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(capTypes.get(x));
			option.appendChild(text);
			formEl.appendChild(option);
		}
	}

	private String getPatternList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		String array = "";
		String[] patterns = m_store.getNamePatterns();

		for (int x = 0; x < patterns.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(patterns[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}

		return array;

	}

	private void getChannelList(Document doc, Element formEl, String chanMatch)
	{
		Element option = null;
		Text text = null;
		boolean chanExists = false;

		HashMap<String, Channel> channels = m_store.getChannels();
		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		Arrays.sort(keys);

		for (int x = 0; x < keys.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(keys[x]);
			option.appendChild(text);
			formEl.appendChild(option);
			if (keys[x].equals(chanMatch))
				chanExists = true;
		}
		if (chanMatch != null)
		{
			if (!chanExists)
			{
				option = doc.createElement("option");
				text = doc.createTextNode(chanMatch);
				option.appendChild(text);
				formEl.appendChild(option);
			}
			if (".*".compareTo(chanMatch) != 0)
			{
				option = doc.createElement("option");
				text = doc.createTextNode(".*");
				option.appendChild(text);
				formEl.appendChild(option);
			}
		}
	}

	private byte[] showCalendar() throws Exception
	{
		String id = m_urlData.getParameter("id");
		ScheduleItem item = m_store.getScheduleItem(id);

		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e)
		{}

		// if we are trying to edit a running schedule just redirect back
		// to the main page
		if (item != null && (item.getState() != ScheduleItem.FINISHED && item.getState() != ScheduleItem.WAITING && item.getState() != ScheduleItem.SKIPPED && item.getState() != ScheduleItem.ERROR))
		{
			StringBuffer out = new StringBuffer();
			out.append("HTTP/1.0 302 Moved Temporarily\n");
			out.append("Location: /servlet/KBScheduleDataRes\n\n");
			return out.toString().getBytes();
		}

		int month = -1;
		int year = -1;
		int day = -1;
		try
		{
			month = Integer.parseInt(m_urlData.getParameter("month"));
			year = Integer.parseInt(m_urlData.getParameter("year"));
		}
		catch (Exception e)
		{
		}

		Calendar cal = Calendar.getInstance();

		if (item != null)
			cal.setTime(item.getStart());

		day = cal.get(Calendar.DATE);
		cal.set(Calendar.DATE, 1);

		if (month > -1 && year > -1)
		{
			cal.set(Calendar.MONTH, month);
			cal.set(Calendar.YEAR, year);
		}

		if (day > cal.getActualMaximum(Calendar.DATE))
			day = cal.getActualMaximum(Calendar.DATE);

		cal.set(Calendar.DATE, day);

		month = cal.get(Calendar.MONTH);
		year = cal.get(Calendar.YEAR);

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "cal", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("selectedDay", new Integer(day).toString());
		root.setAttribute("description", m_store.monthNameFull.get(new Integer(month)) + " " + year);

		cal.add(Calendar.MONTH, -1);
		root.setAttribute("prevMonth", new Integer(cal.get(Calendar.MONTH)).toString());
		root.setAttribute("prevYear", new Integer(cal.get(Calendar.YEAR)).toString());

		cal.add(Calendar.MONTH, 2);
		root.setAttribute("nextMonth", new Integer(cal.get(Calendar.MONTH)).toString());
		root.setAttribute("nextYear", new Integer(cal.get(Calendar.YEAR)).toString());

		cal.add(Calendar.MONTH, -1);

		if (item != null)
			root.setAttribute("itemID", item.toString());
		else
			root.setAttribute("itemID", "");

		if (index > -1)
			root.setAttribute("index", new Integer(index).toString());
		else
			root.setAttribute("index", "");      

		int currentMonth = cal.get(Calendar.MONTH);
		cal.set(Calendar.DATE, 1);

		int dayNameStart = cal.getFirstDayOfWeek();

		for (int x = 0; x < 7; x++)
		{
			String dayName = (String)m_store.dayName.get(new Integer(dayNameStart));
			if(dayName == null)
				dayName = "NULL";

			Element dayEl = doc.createElement("dayNames");
			Element dayUrl = doc.createElement("name");
			Text dayUrlTest = doc.createTextNode(dayName);
			dayUrl.appendChild(dayUrlTest);
			dayEl.appendChild(dayUrl);
			root.appendChild(dayEl);

			dayNameStart++;
			if(dayNameStart > cal.getActualMaximum(Calendar.DAY_OF_WEEK))
				dayNameStart = cal.getActualMinimum((Calendar.DAY_OF_WEEK));
		}      

		for (int x = cal.getFirstDayOfWeek(); x < cal.get(Calendar.DAY_OF_WEEK); x++)
		{
			Element dayEl = doc.createElement("day");
			dayEl.setAttribute("date", "0");
			dayEl.setAttribute("month", new Integer(cal.get(Calendar.MONTH)).toString());
			dayEl.setAttribute("year", new Integer(cal.get(Calendar.YEAR)).toString());
			dayEl.setAttribute("DayOfWeek", new Integer(x).toString());
			dayEl.setAttribute("week", new Integer(cal.get(Calendar.WEEK_OF_MONTH)).toString());
			Element dayUrl = doc.createElement("url");
			Text dayUrlTest = doc.createTextNode("");
			dayUrl.appendChild(dayUrlTest);
			dayEl.appendChild(dayUrl);
			root.appendChild(dayEl);
		}

		while (cal.get(Calendar.MONTH) == currentMonth)
		{
			Element dayEl = doc.createElement("day");
			dayEl.setAttribute("date", new Integer(cal.get(Calendar.DATE)).toString());
			dayEl.setAttribute("month", new Integer(cal.get(Calendar.MONTH)).toString());
			dayEl.setAttribute("year", new Integer(cal.get(Calendar.YEAR)).toString());
			dayEl.setAttribute("DayOfWeek", new Integer(cal.get(Calendar.DAY_OF_WEEK)).toString());
			dayEl.setAttribute("week", new Integer(cal.get(Calendar.WEEK_OF_MONTH)).toString());

			String action = "/servlet/" + m_urlData.getServletClass() + "?action=02&day=" + cal.get(Calendar.DATE) + "&month=" + cal.get(Calendar.MONTH) + "&year=" + cal.get(Calendar.YEAR);
			if (item != null)
				action += "&id=" + Formatter.encodeURL(item.toString(), "UTF-8");
			if(index > -1)
				action += "&index=" + index;

			Element dayUrl = doc.createElement("url");
			Text dayUrlTest = doc.createTextNode(action);
			dayUrl.appendChild(dayUrlTest);

			dayEl.appendChild(dayUrl);

			root.appendChild(dayEl);

			cal.add(Calendar.DATE, 1);
		}

		cal.add(Calendar.DATE, -1);
		for (int x = cal.get(Calendar.DAY_OF_WEEK) + 1; x <= 7; x++)
		{
			Element dayEl = doc.createElement("day");
			dayEl.setAttribute("date", "0");
			dayEl.setAttribute("month", new Integer(cal.get(Calendar.MONTH)).toString());
			dayEl.setAttribute("year", new Integer(cal.get(Calendar.YEAR)).toString());
			dayEl.setAttribute("DayOfWeek", new Integer(x).toString());
			dayEl.setAttribute("week", new Integer(cal.get(Calendar.WEEK_OF_MONTH)).toString());
			Element dayUrl = doc.createElement("url");
			Text dayUrlTest = doc.createTextNode("");
			dayUrl.appendChild(dayUrlTest);
			dayEl.appendChild(dayUrl);
			root.appendChild(dayEl);
		}

		//
		// Send output back
		//
		XSL transformer = new XSL(doc, "kb-cal.xsl", m_headers, m_urlData);
		return transformer.transform();      
	}

	private ScheduleItem[] filterItems(ScheduleItem[] itemsArray, boolean past)
	{
		Vector<ScheduleItem> filteredList = new Vector<ScheduleItem>();
		Date now = new Date();

		for (int x = 0; x < itemsArray.length; x++)
		{
			ScheduleItem item = itemsArray[x];

			if (past)
			{
				if (item.getStop().getTime() < now.getTime())
					filteredList.add(item);
			}
			else
			{
				if (item.getStop().getTime() >= now.getTime())
					filteredList.add(item);
			}

		}

		return (ScheduleItem[]) filteredList.toArray(new ScheduleItem[0]);
	}

	private byte[] getScheduleTable() throws Exception
	{
		Calendar dateFormater = Calendar.getInstance();

		int start = 0;
		int show = 10;

		try
		{
			start = Integer.parseInt(m_urlData.getParameter("start"));
			show = Integer.parseInt(m_urlData.getParameter("show"));
		}
		catch (Exception e)
		{
		}

		if (start < 0)
			start = 0;

		int filter = 0;
		try
		{
			filter = Integer.parseInt(m_urlData.getParameter("filter"));
		}
		catch (Exception e)
		{
		}

		HashMap<String, ScheduleItem> times = m_store.getScheduleList();
		ScheduleItem[] itemsArray = (ScheduleItem[]) times.values().toArray(new ScheduleItem[0]);

		// Filter items
		//itemsArray = filterItems(itemsArray, filterList);
		itemsArray = filterItems(itemsArray, (filter != 0));

		Arrays.sort(itemsArray);

		int end = show + start;
		if ((show + start) >= itemsArray.length)
			end = itemsArray.length;

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "schedules", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("start", new Integer(start).toString());
		root.setAttribute("end", new Integer(end).toString());
		root.setAttribute("show", new Integer(show).toString());
		root.setAttribute("total", new Integer(itemsArray.length).toString());
		root.setAttribute("filter", new Integer(filter).toString());

		Element sch = null;
		Element elm = null;
		Text text = null;

		DateFormatter2 dateFormatter2 = new DateFormatter2();

		for (int x = start; x < end; x++)
		{
			ScheduleItem item = itemsArray[x];
			dateFormater.setTime(item.getStart());

			sch = doc.createElement("schedule");
			sch.setAttribute("id", item.toString());

			//
			// Add the time
			//
			String timeString = dateFormatter2.formatTime(dateFormater.getTime());
			/*
			// I think there is a bug in the .get(Calendar.HOUR) of Calendar
			// it returns 0 for 12 pm so do the following to fix this
			int hour = dateFormater.get(Calendar.HOUR);
			if (hour == 0)
				hour = 12;
			String timeString = store.intToStr(hour) + ":" + store.intToStr(dateFormater.get(Calendar.MINUTE)) + " " + store.ampm.get(new Integer(dateFormater.get(Calendar.AM_PM)));
			 */

			String dateString = m_store.dayName.get(new Integer(dateFormater.get(Calendar.DAY_OF_WEEK))) + ", " + dateFormater.get(Calendar.DATE) + " " + m_store.monthNameShort.get(new Integer(dateFormater.get(Calendar.MONTH)));

			sch.setAttribute("start", timeString + " " + dateString);

			elm = doc.createElement("schName");
			if (item.getName().length() > 1)
				text = doc.createTextNode(item.getName());
			else
				text = doc.createTextNode("No Name");
			elm.appendChild(text);
			sch.appendChild(elm);

			elm = doc.createElement("schDur");
			text = doc.createTextNode(new Integer(item.getDuration()).toString());
			elm.appendChild(text);
			sch.appendChild(elm);

			elm = doc.createElement("schChannel");
			text = doc.createTextNode(item.getChannel());
			elm.appendChild(text);
			sch.appendChild(elm);

			elm = doc.createElement("schStatus");
			text = doc.createTextNode(item.getStatus());
			elm.appendChild(text);
			sch.appendChild(elm);

			String type = item.getTypeStr();
			elm = doc.createElement("schType");
			text = doc.createTextNode(type);
			elm.appendChild(text);
			sch.appendChild(elm);

			String action = "/servlet/" + m_urlData.getServletClass() + "?action=04&id=" + Formatter.encodeURL(item.toString(), "UTF-8");

			elm = doc.createElement("action");
			text = doc.createTextNode(action);
			elm.appendChild(text);
			sch.appendChild(elm);

			root.appendChild(sch);
		}

		//
		// Send output back
		//
		XSL transformer = new XSL(doc, "kb-schedules.xsl", m_headers, m_urlData);

		// set current page to check point for back
		transformer.addCookie("backURL", m_urlData.getRequestString());

		return transformer.transform();      
	}

	private void addButton(Document doc, String name, String url, boolean confirm)
	{
		Element button = null;
		Element elm = null;
		Text text = null;

		button = doc.createElement("button");
		button.setAttribute("name", name);
		elm = doc.createElement("url");

		text = doc.createTextNode(url);
		elm.appendChild(text);
		button.appendChild(elm);

		if(confirm)
		{
			elm = doc.createElement("confirm");
			text = doc.createTextNode("true");
			elm.appendChild(text);
			button.appendChild(elm);
		}

		doc.getDocumentElement().appendChild(button); 
	}

	private byte[] showItemOptions() throws Exception
	{
		String id = m_urlData.getParameter("id");

		ScheduleItem item = m_store.getScheduleItem(id);

		if (item == null)
		{
			StringBuffer out = new StringBuffer(4096);
			out.append("HTTP/1.0 302 Moved Temporarily\n");
			out.append("Location: /servlet/ApplyTransformRes?xml=root&xsl=kb-buttons\n\n");
			return out.toString().getBytes();
		}

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "buttons", null);

		String action = "javascript:jumpBack()";

		addButton(doc, "Back", action, false);

		// add stop button if running
		// add Add Time button if Running
		if(item.getState() == ScheduleItem.RUNNING)
		{
			action = "/servlet/" + m_urlData.getServletClass() + "?action=06&id=" + id;
			addButton(doc, "Stop Running Item", action, true);

			action = "/servlet/" + m_urlData.getServletClass() + "?action=07&id=" + id;
			addButton(doc, "Add Time", action, false);
		}       

		// Edit item
		if(//item.getType() != ScheduleItem.EPG && 
				(item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.ERROR))
		{
			action = "/servlet/" + m_urlData.getServletClass() + "?action=01&id=" + id;
			addButton(doc, "Edit Item", action, false);
		}

		// Delete item
		if((item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.ERROR))
		{
			action = "/servlet/" + m_urlData.getServletClass() + "?action=05&id=" + id;
			addButton(doc, "Delete Item", action, true);
		}

		// Skip to next repeating items
		if((item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.ERROR) &&
				item.getType() != ScheduleItem.EPG && 
				item.getType() != ScheduleItem.ONCE)
		{
			action = "/servlet/" + m_urlData.getServletClass() + "?action=08&id=" + id;
			addButton(doc, "Skip to Next", action, false);
		}

		// Add time buffer to start and stop times
		if((item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.ERROR) 
				//&& item.getType() == ScheduleItem.ONCE 
				)
		{
			action = "/servlet/" + m_urlData.getServletClass() + "?action=07&id=" + id;
			addButton(doc, "Add Time", action, false);
		}

		// Show item in EPG
		Calendar cal = Calendar.getInstance();
		cal.setTime(item.getStart());
		action = "/servlet/KBEpgDataRes?action=01" + "&year=" + cal.get(Calendar.YEAR) + "&month=" + (cal.get(Calendar.MONTH) + 1) + "&day=" + cal.get(Calendar.DATE) + "&start=" + cal.get(Calendar.HOUR_OF_DAY);
		addButton(doc, "Show in EPG", action, false);

		// View item log
		action = "/servlet/" + m_urlData.getServletClass() + "?action=09&id=" + Formatter.encodeURL(item.toString(), "UTF-8");
		addButton(doc, "View Item Log", action, false);

		//
		// Send output back
		//
		XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] deleteItem() throws Exception
	{
		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons";

		String id = m_urlData.getParameter("id");

		StringBuffer out = new StringBuffer(4096);
		ScheduleItem item = m_store.getScheduleItem(id);

		if(item != null && (item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.ERROR))
		{
			m_store.removeScheduleItem(id, true);
			m_store.saveSchedules(item);
		}

		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();
	}

	private byte[] stopRunningTask() throws Exception
	{
		String backURL = m_urlData.getCookie("backURL");
		try
		{
			backURL = URLDecoder.decode(backURL, "UTF-8");
		}
		catch(Exception e){}
		if(backURL == null || backURL.length() == 0)
			backURL = "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons";

		StringBuffer out = new StringBuffer(4096);
		String id = m_urlData.getParameter("id");

		ScheduleItem si = null;

		ThreadLock.getInstance().getLock();
		try
		{
			si = m_store.getScheduleItem(id);
			if(si != null && si.getState() == ScheduleItem.RUNNING)
			{
				si.abort();
				si.setStatus("Aborting");
				si.setState(ScheduleItem.ABORTED);
				si.log("Item marked for abortion");
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}      

		// wait for the item to be stopped before returning
		m_store.setToDoWake();
		int counts = 10;
		while(si.getState() == ScheduleItem.ABORTED && counts > 0)
		{
			counts--;
			Thread.sleep(1000);
		}      

		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");

		return out.toString().getBytes();
	}
}
