/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;

public class NowRunningInfo
{
	HashMap<String, CaptureTask> tasks = null;
	private String m_saveTo = "nowRunning.xml";

	public NowRunningInfo(HashMap<String, CaptureTask> data, String saveTo)
	{
		tasks = data;
		m_saveTo = saveTo;
	}

	public void writeNowRunning()
	{
		try
		{
			String xmlData = getXMLData();

			File runningCapList = new File(m_saveTo);
			FileWriter writer = new FileWriter(runningCapList);
			writer.write(xmlData);
			writer.close();
		}
		catch (Exception e)
		{
			DVScheduler.log("Error Thrown When Writing " + m_saveTo + " : " + e.toString());
		}
	}

	private String getXMLData()
	{
		ByteArrayOutputStream buff = new ByteArrayOutputStream();

		try
		{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			DOMImplementation di = db.getDOMImplementation();
			Document doc = di.createDocument("", "running_captures", null);

			Element root = doc.getDocumentElement();
			Element elm = null;
			Element elm2 = null;
			Text text = null;

			String[] keys = (String[]) tasks.keySet().toArray(new String[0]);

			for (int index = 0; index < keys.length; index++)
			{
				CaptureTask task = (CaptureTask) tasks.get(keys[index]);
				ScheduleItem item = task.getScheduleItem();

				elm = doc.createElement("capture");
				elm.setAttribute("cardID", new Integer(task.getCardID()).toString());

				elm2 = doc.createElement("name");
				text = doc.createTextNode(item.getName());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				elm2 = doc.createElement("start");
				text = doc.createTextNode(item.getStart().toString());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				elm2 = doc.createElement("stop");
				text = doc.createTextNode(item.getStop().toString());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				elm2 = doc.createElement("duration");
				text = doc.createTextNode(new Integer(item.getDuration()).toString());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				elm2 = doc.createElement("channel");
				text = doc.createTextNode(item.getChannel());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				CaptureFile cap = task.getCaptureFile();
				elm2 = doc.createElement("filename");
				text = doc.createTextNode(cap.getFullPath());
				elm2.appendChild(text);
				elm.appendChild(elm2);

				//append capture to root element
				root.appendChild(elm);
			}

			// Transform Document
			TransformerFactory factory = TransformerFactory.newInstance();
			Transformer xformer = factory.newTransformer();

			Source source = new DOMSource(doc);
			Result result = new StreamResult(buff);
			xformer.transform(source, result);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return buff.toString();
	}

}