/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.text.*;

import java.util.zip.GZIPOutputStream;

public class PageTemplate
{
	private SimpleDateFormat df = new SimpleDateFormat("EEE, d MMM yyyy H:mm:ss"); 
	private StringBuffer page = null;
	private Vector<String> cookies = new Vector<String>();
	private Map<String, String> m_headers = null;
	private HTTPurl m_urlData = null;

	public PageTemplate(Map<String, String> headers, HTTPurl urlData, String template) throws Exception
	{
		m_urlData = urlData;
		m_headers = headers;
		loadTemplate(template);
	}

	public PageTemplate(Map<String, String> headers, HTTPurl urlData, String template, boolean bBackURL) throws Exception
	{
		m_urlData = urlData;
		m_headers = headers;
		loadTemplate(template);
		if (bBackURL)
			addCookie("backURL", m_urlData.getRequestString());
	}

	public void addCookie(String n, String d) throws Exception
	{
		String name = Formatter.encodeURL(n, "UTF-8");
		String data = Formatter.encodeURL(d, "UTF-8");
		cookies.add(name + "=" + data + "; PATH=/");
	}   

	private void loadTemplate(String template) throws Exception
	{
		DataStore store = DataStore.getInstance();
		ByteArrayOutputStream ba = new ByteArrayOutputStream();      
		FileInputStream fi = new FileInputStream(store.getTemplateFile(m_headers.get("User-Agent"), template));

		int read = 0;
		byte[] bytes = new byte[1024];
		read = fi.read(bytes);
		while(read > -1)
		{
			ba.write(bytes, 0, read);
			read = fi.read(bytes);
		}

		fi.close();

		page = new StringBuffer();
		page.append(ba.toString());
	}

	public void replace(String placeHolder, String replace) throws Exception
	{
		if(page == null) new Exception("Template not loaded or failed load");

		//DVScheduler.log("replaceAll on " + placeHolder);
		int phLength = placeHolder.length();
		int replaceLength = replace.length();

		int phIndex = page.indexOf(placeHolder);
		while(phIndex != -1)
		{
			page.replace(phIndex, phIndex + phLength, replace);
			phIndex = page.indexOf(placeHolder, phIndex + replaceLength);
		}
	}

	public void replaceFile(String placeHolder, String fileName) throws Exception
	{
		if (page == null) new Exception("Template not loaded or failed load");

		//DVScheduler.log("replaceAll on " + placeHolder);

		ByteArrayOutputStream ba = new ByteArrayOutputStream();      
		FileInputStream fi = new FileInputStream(fileName);

		int read = 0;
		byte[] bytes = new byte[1024];
		read = fi.read(bytes);
		while(read > -1)
		{
			ba.write(bytes, 0, read);
			read = fi.read(bytes);
		}

		fi.close();

		String replace = ba.toString();
		int phLength = placeHolder.length();
		int replaceLength = replace.length();

		int phIndex = page.indexOf(placeHolder);
		while(phIndex != -1)
		{
			page.replace(phIndex, phIndex + phLength, replace);
			phIndex = page.indexOf(placeHolder, phIndex + replaceLength);
		}
	}

	public void replaceRedirUrl(String back) throws Exception
	{
		replace(back, HTTPResponse.getRedirUrl(m_headers, m_urlData, null));
	}

	public byte[] getPageBytes() throws Exception
	{
		return getPageBytes(-1);
	}

	public byte[] getPageBytes(int nCompress) throws Exception
	{
		if(page == null)
			new Exception("Template not loaded or filed load");

		/*
		StringBuffer returnData = new StringBuffer(buildResponce());
		returnData.append(page);

		return returnData.toString().getBytes("UTF-8");
		 */

		DataStore store = DataStore.getInstance();
		if ( (nCompress < 0) || (nCompress > 1) )
		{
			String compress = store.getProperty("HTTP.compress", "Yes");
			if (compress.equals("Yes"))
				nCompress = 1;
			else
				nCompress = 0;
		}
		if (nCompress == 1)
		{
			if (m_headers != null)
			{
				String acceptEncoding = (String)m_headers.get("Accept-Encoding");
				if ( (acceptEncoding == null) || !acceptEncoding.toLowerCase().contains("gzip") || HTTPResponse.isLocal(m_headers) )
					nCompress = 0;
			}
			else
				nCompress = 0;
		}

		ByteArrayOutputStream out2 = new ByteArrayOutputStream();
		out2.write(buildResponce(nCompress).getBytes());
		if (nCompress == 1)
		{
			GZIPOutputStream gzipOut = new GZIPOutputStream(out2);
			gzipOut.write(page.toString().getBytes("UTF-8"));
			gzipOut.finish();
			gzipOut.flush();
			return out2.toByteArray();
		}
		else
		{
			out2.write(page.toString().getBytes("UTF-8"));
			return out2.toByteArray();
		}
	}

	private String buildResponce(int nCompress)
	{
		StringBuffer header = new StringBuffer();

		header.append("HTTP/1.0 200 OK\n");
		if (nCompress == 1)
			header.append("Content-Encoding: gzip\n");
		header.append("Content-Type: text/html; charset=utf-8\n");
		header.append("Pragma: no-cache\n");
		header.append("Cache-Control: no-cache\n");

		String timeStamp = df.format(new Date());
		timeStamp += " GMT";

		header.append("Last-Modified: " + timeStamp + "\n");
		header.append("Date: " + timeStamp + "\n");
		header.append("Expires: " + timeStamp + "\n");

		for(int x = 0; x < cookies.size(); x++)
		{
			String cook = (String)cookies.get(x);
			header.append("Set-Cookie: " + cook + "\n");
		}      

		header.append("\n");

		return header.toString();
	}
}