/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.net.*;
import java.util.HashMap;

class RequestRedir implements Runnable
{
	private Socket m_server = null;
	private int m_httpsPort = -1;

	// Redirect to HTTPS
	RequestRedir(Socket server, int httpsPort)
	{
		this.m_server=server;
		try {
			m_server.setSendBufferSize(65536);
		} catch (SocketException e) {
			e.printStackTrace();
		}
		m_httpsPort = httpsPort;
	}

	public void run ()
	{
		String request = null;
		try
		{
			// Get input from the client
			InputStream inStream = m_server.getInputStream();
			BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
			OutputStream out = m_server.getOutputStream();

			request = in.readLine();
			int indexOfSpace = request.indexOf(" ");
			String request2 = request.substring(indexOfSpace + 1, request.indexOf(" ", indexOfSpace + 2));

			String request3;
			HashMap<String, String> headers = new HashMap<String, String>();
			do
			{
				request3 = in.readLine();
				int index = request3.indexOf(": ");
				if(index > 0)
				{
					headers.put(request3.substring(0, index), request3.substring(index + 2, request3.length()));
				}
			}
			while( (request3 != null) && !(request3.equals("")) );

			headers.put("RemoteAddress", m_server.getInetAddress().getHostAddress());
			headers.put("HostAddress", m_server.getLocalAddress().getHostAddress());
			headers.put("HostName", m_server.getLocalAddress().getHostName());

			DataStore store = DataStore.getInstance();
			boolean bUnsecureDownload = store.isAllowUnsecureDownload();

			if ( bUnsecureDownload && (request2.startsWith("/servefile/") || 
					request2.startsWith("/stream/") || request2.startsWith("/stream2/") || 
					//request2.startsWith("/playlistAll/") || request2.startsWith("/playlist/")) )
					request2.startsWith("/playlist/")) )
			{
				HTTPRequest httpRequest = new HTTPRequest(request, headers, "", out);
				httpRequest.sendResponseData(bUnsecureDownload);
			}
			else
			{
				//String host = m_server.getLocalAddress().getHostAddress();
				//if ( m_server.getLocalAddress().getHostName().equalsIgnoreCase("localhost") || 
				//		host.equals("127.0.0.1") || host.equals("0:0:0:0:0:0:0:1") )

				String host = headers.get("Host");
				int sep = host.lastIndexOf(":");
				if (sep > -1)
					host = host.substring(0, sep);
				if ( host.equals("127.0.0.1") || host.equals("0:0:0:0:0:0:0:1") )
				{
					host = "localhost";
				}

				out.write( ("HTTP/1.0 302 Moved Temporarily\nLocation: https://" + host + ":" + m_httpsPort + request2 + "\n\n").getBytes() );
			}
		}
		catch (Exception e)
		{
			DVScheduler.log("Worker RequestRedir Exception: " + request);
			e.printStackTrace();
		}
		finally
		{
			try
			{
				m_server.close();
			}
			catch(Exception e){}
		}

		//DVScheduler.log("Request Thread Exiting");
	}
}