/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.File;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class SystemStatusData
{

	public SystemStatusData()
	{
	}

	public byte[] getStatusXML(Map<String, String> headers, HTTPurl urlData) throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "sys-info", null); 

		Element root = doc.getDocumentElement();
		Element item = null;   

		DataStore store = DataStore.getInstance();
		GuideStore guide = GuideStore.getInstance();

		int balloonDelay = urlData.getParameterInt("balloonDelay", -1);

		if(balloonDelay == -1)
		{
			balloonDelay = Integer.parseInt(store.getProperty("EPG.balloonDelay", "0"));
			if (balloonDelay < 0)
				balloonDelay = 0;
		}
		root.setAttribute("balloonDelay", new Integer(balloonDelay).toString());

		root.setAttribute("leftClick", store.getProperty("EPG.leftClick", "Show Menu"));
		root.setAttribute("rightClick", store.getProperty("EPG.rightClick", "Add Schedule"));
		root.setAttribute("pathTheme", store.getThemeDirectory(headers.get("User-Agent")));
		root.setAttribute("local", HTTPResponse.isLocal(headers) ? "1" : "0");
		if ( (!HTTPResponse.isLocal(headers) && !store.isPreviewRemoteAvailable()) || 
				(HTTPResponse.isLocal(headers) && !store.isPreviewAvailable()) )
			root.setAttribute("preview", "0");
		else
			root.setAttribute("preview", "1");

		CaptureDeviceList devList = CaptureDeviceList.getInstance();
		root.setAttribute("warningDevice", devList.getDeviceCount() > 0 ? "0" : "1");
		root.setAttribute("numRecordingError", Integer.toString(store.m_nRecordingErrorCount - store.m_nLastRecordingErrorCount));

		DateFormatter2 dateFormatter2 = new DateFormatter2();
		String value = "";
		String epgLoadDesc = "";
		if (guide.getEpgLoadTime() != Long.MAX_VALUE)
		{
			value = dateFormatter2.formatDateTime(new Date(guide.getEpgLoadTime()));
			root.setAttribute("epgLastLoadTime", value);
			epgLoadDesc = "EPG last loaded on:<br>" + value + "<br><br>";
		}
		epgLoadDesc += "Next EPG load:<br>";
		if (store.m_nextEpgLoadTime != Long.MAX_VALUE)
		{
			value = dateFormatter2.formatDateTime(new Date(store.m_nextEpgLoadTime));
			root.setAttribute("epgNextLoadTime", value);
		}
		if ( store.isEpgLoading() )
		{
			epgLoadDesc += "In progress";
			root.setAttribute("epgLoadStatus", "1");
		}
		else if (store.m_nextEpgLoadTime <= Calendar.getInstance().getTimeInMillis())
		{
			epgLoadDesc += "Waiting on condition";
			root.setAttribute("epgLoadStatus", "2");
		}
		else if ( (store.m_nextEpgLoadTime == Long.MAX_VALUE) || !store.m_bPrevEpgLoadEnabled )
		{
			epgLoadDesc += "No schedule";
			root.setAttribute("epgLoadStatus", "3");
		}
		else
		{
			epgLoadDesc += value;
			root.setAttribute("epgLoadStatus", "0");
		}
		root.setAttribute("epgLoadDesc", epgLoadDesc);

		// The capture types.
		Vector<String> capTypes = store.getCaptureTypes();
		for (int x = 0; x < capTypes.size(); x++)
		{
			Element capTypeEl = doc.createElement("captureType");
			capTypeEl.setAttribute("name", capTypes.get(x));
			root.appendChild(capTypeEl);
		}

		// Tasks.
		root.setAttribute("taskWaiting", Integer.toString(store.m_waitingTaskList.size()));
		root.setAttribute("taskRunning", Integer.toString(store.m_runningTaskList.size()));
		root.setAttribute("taskFinished", Integer.toString(store.m_finishedTaskList.size()));

		// time
		Calendar start = Calendar.getInstance();
		start.setTime(new Date());      
		item = doc.createElement("time");
		int hour = start.get(Calendar.HOUR_OF_DAY);
		if(hour > 12) hour = hour - 12;
		else if(hour == 0) hour = 12;      
		item.setAttribute("hour_12" , intToXchar(hour, 2));      
		item.setAttribute("hour_24" , intToXchar(start.get(Calendar.HOUR_OF_DAY), 2));
		item.setAttribute("minute" , intToXchar(start.get(Calendar.MINUTE), 2));
		if(start.get(Calendar.AM_PM) == Calendar.AM)
			item.setAttribute("am_pm" , "am");
		else
			item.setAttribute("am_pm" , "pm");
		item.setAttribute("time2", dateFormatter2.formatTime(start.getTime()));
		root.appendChild(item);        

		// add now running/and next
		ScheduleItem[] itemsArray = (ScheduleItem[])store.getScheduleList().values().toArray(new ScheduleItem[0]);
		Arrays.sort(itemsArray);

		Vector<ScheduleItem> nextItems = new Vector<ScheduleItem>();
		ScheduleItem firstNext = null;

		// add now running data
		item = doc.createElement("now_running");
		for(int x = 0; x < itemsArray.length; x++)
		{
			if(nextItems.size() == 0 && 
					itemsArray[x].getState() == ScheduleItem.WAITING &&
					itemsArray[x].getStop().getTime() > new Date().getTime())
			{
				firstNext = itemsArray[x];
				nextItems.add(itemsArray[x]);
			}
			else if(firstNext != null &&
					itemsArray[x].getState() == ScheduleItem.WAITING &&
					itemsArray[x].getStart().getTime() < firstNext.getStop().getTime())
			{
				nextItems.add(itemsArray[x]);
			}

			if(itemsArray[x].getState() == ScheduleItem.RUNNING)
			{
				addNowRunningItem(itemsArray[x], item, doc);
			}
		}
		root.appendChild(item); 

		// add next item
		item = doc.createElement("next");
		for(int x = 0; x < nextItems.size(); x++)
		{
			ScheduleItem next = nextItems.get(x);
			addNextItem(next, item, doc);
		}
		root.appendChild(item); 

		// add now and next epg items
		item = doc.createElement("now_and_next");
		addNOWandNEXT(store, item, doc, headers.get("User-Agent"));
		root.appendChild(item);

		// Add all the channels
		item = doc.createElement("channels");
		addChannels(store, item, doc, headers.get("User-Agent"));
		root.appendChild(item);

		item = doc.createElement("devices");
		item.setAttribute("total", Integer.toString(devList.getDeviceCount()));
		item.setAttribute("total_running", Integer.toString(devList.getRunningDeviceCount()));
		addDevices(store, item, doc);
		root.appendChild(item);

		item = doc.createElement("directories");
		addCapturePathStats(store, item, doc);
		root.appendChild(item);

		// Downloads.
		item = doc.createElement("downloads");
		int total = addDownloads(store, item, doc);
		item.setAttribute("total", Integer.toString(total));
		root.appendChild(item);

		// Do transform and return data
		XSL transformer = new XSL(doc, "status.xsl", headers, urlData);
		transformer.addCookie("backURL", "/");
		return transformer.transform(-1, true);
	}

	public static void addNOWandNEXT(DataStore store, Element item, Document doc, String userAgent)
	{
		GuideStore guide = GuideStore.getInstance();

		Vector<String[]> chanMap = guide.getOldChannelMap();
		Set<String> wsChannels = store.getChannels().keySet();
		Date now = new Date();

		String sChannelImageDirectory = store.getProperty("path.httproot", "http") + "/themes/" + store.getThemeDirectory(userAgent) + "/images/channels/";		String sChannelImageWidth = Integer.toString(store.getPropertyInt("channel.logoWidth", 64));
		String sChannelShow = store.getProperty("channel.logo", "1");
		boolean bShowChannelImage = !sChannelShow.equals("0") && !store.getProperty("channel.useLogo", "2").equals("0");
		boolean bChannelImage = false;
		for (int y = 0; y < chanMap.size(); y++)
		{
			String[] map = (String[])chanMap.get(y);

			if(wsChannels.contains(map[0]))
			{
				Element channel = doc.createElement("channel");
				channel.setAttribute("epg_channel", map[1]);
				channel.setAttribute("ws_channel", map[0]);

				bChannelImage = Channel.getChannelLogoXml(channel, map[0], sChannelImageDirectory, sChannelImageWidth, sChannelShow, bShowChannelImage, bChannelImage);

				GuideItem[] items = guide.getProgramsForChannel(map[1]);         

				for (int x = 0; x < items.length; x++)
				{
					GuideItem gitem = items[x];

					if(gitem.getStart().before(now) && gitem.getStop().after(now))
					{
						Element elmNow = doc.createElement("now");
						addGuideItem(items[x], guide.isInEpgExclusion(items[x], map[0]), elmNow, doc);
						channel.appendChild(elmNow);

						if(x+1 < items.length)
						{
							Element elmNext = doc.createElement("next");
							addGuideItem(items[x+1], guide.isInEpgExclusion(items[x+1], map[0]), elmNext, doc);
							channel.appendChild(elmNext);
						}
						break;
					}
					else if ( gitem.getStart().after(now) )
					{
						Element elmNext = doc.createElement("next");
						addGuideItem(items[x], guide.isInEpgExclusion(items[x], map[0]), elmNext, doc);
						channel.appendChild(elmNext);
						break;
					}
				}

				item.appendChild(channel);
			}
		}
		if (bChannelImage)
			item.setAttribute("channelImage", "1");
	}

	private void addChannels(DataStore store, Element item, Document doc, String userAgent)
	{
		HashMap<String, Channel> channels = store.getChannels();
		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		List<String> chList = new ArrayList<String>();

		for (int x = 0; x < keys.length; x++)
		{
			Channel ch = channels.get(keys[x]);
			//if (ch.isGroup())
			//	continue;
			chList.add(ch.getSetting() + "\n" + keys[x]);
		}
		String[] chArray = chList.toArray(new String[0]);
		Arrays.sort(chArray);
		String prevSetting = "";
		String setting = "";
		Integer group = 0;
		Integer subGroup = 0;
		int subCount = 0;

		String sChannelImageDirectory = store.getProperty("path.httproot", "http") + "/themes/" + store.getThemeDirectory(userAgent) + "/images/channels/";		String sChannelImageWidth = Integer.toString(store.getPropertyInt("channel.logoWidth", 64));
		String sChannelShow = store.getProperty("channel.logo", "1");
		boolean bShowChannelImage = !sChannelShow.equals("0") && store.getProperty("channel.useLogo", "2").equals("2");
		boolean bChannelImage = false;
		for (int x = 0; x < chArray.length; x++)
		{
			String[] chs = chArray[x].split("\n");
			Element channel = doc.createElement("channel");
			channel.setAttribute("ws_channel", chs[1]);
			setting = chs[0];
			channel.setAttribute("setting", setting);
			if ( (x != 0) && !setting.equals(prevSetting) )
			{
				group++;
				subGroup = 0;
				subCount = 0;
			}
			if (subCount++ % 5 == 0)
				subGroup++;
			prevSetting = setting;
			channel.setAttribute("group", group.toString());
			channel.setAttribute("subgroup", subGroup.toString());

			bChannelImage = Channel.getChannelLogoXml(channel, chs[1], sChannelImageDirectory, sChannelImageWidth, sChannelShow, bShowChannelImage, bChannelImage);

			item.appendChild(channel);
		}
		if (bChannelImage)
			item.setAttribute("channelImage", "1");
	}

	private static void addGuideItem(GuideItem gItem, boolean bExcludedAA, Element parent, Document doc)
	{
		Element name = null;
		Text text = null; 

		name = doc.createElement("name");
		name.setAttribute("sub_name", gItem.getSubName());
		text = doc.createTextNode(gItem.getName());
		name.appendChild(text);
		parent.appendChild(name);

		if (bExcludedAA)
		{
			name.setAttribute("excludedAA", "1");
		}
		else
		{
			name.setAttribute("excludedAA", "0");
		}

		if (gItem.isAutoMark())
		{
			name.setAttribute("matchedAA", "1");
		}
		else
		{
			name.setAttribute("matchedAA", "0");
		}

		name = doc.createElement("desc");
		text = doc.createTextNode(gItem.getDescription());
		name.appendChild(text);
		parent.appendChild(name);

		Calendar start = Calendar.getInstance();
		start.setTime(gItem.getStart());
		name = doc.createElement("start");

		int hour = start.get(Calendar.HOUR_OF_DAY);
		if(hour > 12) hour = hour - 12;
		else if(hour == 0) hour = 12;

		name.setAttribute("hour_12" , intToXchar(hour, 2));      
		name.setAttribute("hour_24" , intToXchar(start.get(Calendar.HOUR_OF_DAY), 2));
		name.setAttribute("minute" , intToXchar(start.get(Calendar.MINUTE), 2));
		if(start.get(Calendar.AM_PM) == Calendar.AM)
			name.setAttribute("am_pm" , "am");
		else
			name.setAttribute("am_pm" , "pm");
		DateFormatter2 dateFormatter2 = new DateFormatter2();
		name.setAttribute("time2", dateFormatter2.formatTime(start.getTime()));
		parent.appendChild(name);    

		// add duration
		name = doc.createElement("duration");
		text = doc.createTextNode(new Integer(gItem.getDuration()).toString());
		name.appendChild(text);
		parent.appendChild(name);

		// add item id
		name = doc.createElement("id");
		text = doc.createTextNode(gItem.toString());
		name.appendChild(text);
		parent.appendChild(name);      
	}

	private void addNextItem(ScheduleItem schItem, Element item, Document doc)
	{
		Element nextItem = doc.createElement("item");

		Element name = null;
		Text text = null;   

		// add name
		name = doc.createElement("name");
		text = doc.createTextNode(schItem.getName());
		name.appendChild(text);
		nextItem.appendChild(name);

		// add start time
		Calendar start = Calendar.getInstance();
		start.setTime(schItem.getStart());
		name = doc.createElement("start");

		int hour = start.get(Calendar.HOUR_OF_DAY);
		if(hour > 12) hour = hour - 12;
		else if(hour == 0) hour = 12;

		name.setAttribute("hour_12" , intToXchar(hour, 2));      
		name.setAttribute("hour_24" , intToXchar(start.get(Calendar.HOUR_OF_DAY), 2));
		name.setAttribute("minute" , intToXchar(start.get(Calendar.MINUTE), 2));
		if(start.get(Calendar.AM_PM) == Calendar.AM)
			name.setAttribute("am_pm" , "am");
		else
			name.setAttribute("am_pm" , "pm");
		DateFormatter2 dateFormatter2 = new DateFormatter2();
		name.setAttribute("time2", dateFormatter2.formatTime(start.getTime()));
		nextItem.appendChild(name);


		// add duration
		name = doc.createElement("id");
		text = doc.createTextNode(schItem.toString());
		name.appendChild(text);
		nextItem.appendChild(name);

		// add duration
		name = doc.createElement("duration");
		text = doc.createTextNode(new Integer(schItem.getDuration()).toString());
		name.appendChild(text);
		nextItem.appendChild(name);

		// status
		name = doc.createElement("status");
		text = doc.createTextNode(schItem.getStatus());
		name.appendChild(text);
		nextItem.appendChild(name);  

		// time till run
		Date now = new Date();
		long timeLeft = schItem.getStart().getTime() - now.getTime();

		long days = timeLeft / (1000 * 60 * 60 *24);
		long hours = (timeLeft - (days * 1000 * 60 * 60 *24)) / (1000 * 60 * 60);
		long min = (timeLeft - (days * 1000 * 60 * 60 *24) - (hours * 1000 * 60 * 60)) / (1000 * 60);
		long seconds = (timeLeft - (days * 1000 * 60 * 60 *24) - (hours * 1000 * 60 * 60) - (min * 1000 * 60)) / 1000;

		name = doc.createElement("time_to_action");
		name.setAttribute("days" , new Long(days).toString());
		name.setAttribute("hours" , new Long(hours).toString());
		name.setAttribute("minutes" , new Long(min).toString());  
		name.setAttribute("seconds" , new Long(seconds).toString());
		nextItem.appendChild(name);  

		addScheduleInfo(schItem, nextItem, doc);

		item.appendChild(nextItem); 
	}

	private void addNowRunningItem(ScheduleItem schItem, Element item, Document doc)
	{
		Element runnintItem = null;

		Element name = null;
		Text text = null;

		runnintItem = doc.createElement("item");

		// add name
		name = doc.createElement("name");
		text = doc.createTextNode(schItem.getName());
		name.appendChild(text);
		runnintItem.appendChild(name);

		// add start time
		Calendar start = Calendar.getInstance();
		start.setTime(schItem.getStart());
		name = doc.createElement("start");
		int hour = start.get(Calendar.HOUR_OF_DAY);
		if(hour > 12) hour = hour - 12;
		else if(hour == 0) hour = 12;

		name.setAttribute("hour_12" , intToXchar(hour, 2));      
		name.setAttribute("hour_24" , intToXchar(start.get(Calendar.HOUR_OF_DAY), 2));
		name.setAttribute("minute" , intToXchar(start.get(Calendar.MINUTE), 2));
		if(start.get(Calendar.AM_PM) == Calendar.AM)
			name.setAttribute("am_pm" , "am");
		else
			name.setAttribute("am_pm" , "pm");
		DateFormatter2 dateFormatter2 = new DateFormatter2();
		name.setAttribute("time2", dateFormatter2.formatTime(start.getTime()));
		runnintItem.appendChild(name);

		// id
		name = doc.createElement("id");
		text = doc.createTextNode(schItem.toString());
		name.appendChild(text);
		runnintItem.appendChild(name);

		// add duration
		name = doc.createElement("duration");
		text = doc.createTextNode(new Integer(schItem.getDuration()).toString());
		name.appendChild(text);
		runnintItem.appendChild(name);

		// status
		name = doc.createElement("status");
		text = doc.createTextNode(schItem.getStatus());
		name.appendChild(text);
		runnintItem.appendChild(name);

		addScheduleInfo(schItem, runnintItem, doc);

		item.appendChild(runnintItem); 
	}

	private static String intToXchar(int val, int len)
	{
		String finalString = "";
		String rawInt = new Integer(val).toString();

		int toAdd = len - rawInt.length();

		for(int x = 0; x < toAdd; x ++)
		{
			finalString += "0";
		}

		finalString += rawInt;
		return finalString;
	}

	private void addActionUrl(String url, ScheduleItem item, String name, Element root, Document doc)
	{
		Text textNode = null;
		Element elementNaode = null;
		Element elementNaode02 = null;

		elementNaode = doc.createElement("actionUrl");
		elementNaode.setAttribute("name", name);
		elementNaode.setAttribute("item_id", item.toString());
		textNode = doc.createTextNode(url);
		elementNaode02 = doc.createElement("url");
		elementNaode02.appendChild(textNode);
		elementNaode.appendChild(elementNaode02);
		root.appendChild(elementNaode);
	}

	private void addScheduleInfo(ScheduleItem item, Element element, Document doc)
	{
		DateFormatter2 dateFormatter2 = new DateFormatter2();
		SimpleDateFormat dateFormater = new SimpleDateFormat("yyyyMMddHHmmss Z");
		StringBuffer dateBuff = new StringBuffer();
		Calendar startTime = Calendar.getInstance();
		startTime.set(Calendar.MILLISECOND, 0);

		Element schedule = doc.createElement("schedule");

		dateBuff = new StringBuffer();
		dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
		schedule.setAttribute("start", dateBuff.toString());
		schedule.setAttribute("start2", dateFormatter2.formatTime(item.getStart()));
		schedule.setAttribute("startHour", Integer.toString(item.getStartHour()));
		schedule.setAttribute("startMinute", Integer.toString(item.getStartMinute()));

		dateBuff = new StringBuffer();
		dateFormater.format(item.getStop(), dateBuff, new FieldPosition(0));
		schedule.setAttribute("stop", dateBuff.toString());
		schedule.setAttribute("stop2", dateFormatter2.formatTime(item.getStop()));
		schedule.setAttribute("stopHour", Integer.toString(item.getStopHour()));
		schedule.setAttribute("stopMinute", Integer.toString(item.getStopMinute()));

		schedule.setAttribute("duration", new Integer(item.getDuration()).toString());

		schedule.setAttribute("channel", item.getChannel());

		schedule.setAttribute("name", item.getName());

		String child_type = item.getTypeStr();
		schedule.setAttribute("type", child_type);

		schedule.setAttribute("error", Integer.toString(item.getErrorCount()));

		startTime.setTime(item.getStart());
		/*
		long pastStart = startTime.getTime().getTime() - start.getTime().getTime();

		elementNaode = doc.createElement("id");
		textNode = doc.createTextNode(item.toString());
		elementNaode.appendChild(textNode);
		schedule.appendChild(elementNaode);

		Element topElement = doc.createElement("from_top");
		Text topText = doc.createTextNode(new Long(pastStart / (1000 * 60)).toString());
		topElement.appendChild(topText);
		schedule.appendChild(topElement);
		 */

		Element stateElement = doc.createElement("itemState");
		Text stateText = doc.createTextNode(new Integer(item.getState()).toString());
		stateElement.appendChild(stateText);
		schedule.appendChild(stateElement);

		if (item.getState() == ScheduleItem.WAITING || item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.SKIPPED || item.getState() == ScheduleItem.ERROR)
		{

			String showLogUrl = "/servlet/ScheduleDataRes?action=07&id=" + item.toString();
			addActionUrl(showLogUrl, item, "LL", schedule, doc);               

			String delUrl = "/servlet/ScheduleDataRes?action=04&id=" + item.toString();
			addActionUrl(delUrl, item, "D", schedule, doc);

			String epgUrl = "/servlet/EpgDataRes?action=25&id=" + item.toString();
			addActionUrl(epgUrl, item, "EPG", schedule, doc);

			//if(item.getType() != ScheduleItem.EPG)
			{
				String editUrl = "/servlet/ScheduleDataRes?action=01&id=" + item.toString() + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
				addActionUrl(editUrl, item, "E", schedule, doc);

				String edit2Url = "/servlet/ScheduleDataRes?action=02&id=" + item.toString() + "&day=" + startTime.get(Calendar.DAY_OF_MONTH) + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
				addActionUrl(edit2Url, item, "E2", schedule, doc);

				String changeDurationUrl = "javascript:inputForm(\"" + item.toString() + "\"," + item.getDuration() + "," + item.getStopHour() + "," + item.getStopMinute() + ",-1)";
				addActionUrl(changeDurationUrl, item, "+2", schedule, doc);

				String addTimeUrl = "/servlet/ScheduleDataRes?action=05&id=" + item.toString();
				addActionUrl(addTimeUrl, item, "+", schedule, doc);

				if (item.getType() == ScheduleItem.DAILY
						|| item.getType() == ScheduleItem.WEEKLY
						|| item.getType() == ScheduleItem.MONTHLY
						|| item.getType() == ScheduleItem.WEEKDAY)
				{
					schedule.setAttribute("skip", "1");
					String skipUrl = "/servlet/ScheduleDataRes?action=06&id=" + item.toString();
					addActionUrl(skipUrl, item, "SK", schedule, doc);
				}
			}
		}
		else if (item.getState() == ScheduleItem.RUNNING)
		{
			String showLogUrl = "/servlet/ScheduleDataRes?action=07&id=" + item.toString();
			addActionUrl(showLogUrl, item, "LL", schedule, doc);

			String epgUrl = "/servlet/EpgDataRes?action=25&id=" + item.toString();
			addActionUrl(epgUrl, item, "EPG", schedule, doc);

			String stopUrl = "/servlet/ScheduleDataRes?action=09&id=" + item.toString();
			addActionUrl(stopUrl, item, "S", schedule, doc);

			String edit2Url = "/servlet/ScheduleDataRes?action=02&id=" + item.toString() + "&day=" + startTime.get(Calendar.DAY_OF_MONTH) + "&month=" + startTime.get(Calendar.MONTH) + "&year=" + startTime.get(Calendar.YEAR);
			addActionUrl(edit2Url, item, "E2", schedule, doc);

			String changeDurationUrl = "javascript:inputForm(\"" + item.toString() + "\"," + item.getDuration() + "," + item.getStopHour() + "," + item.getStopMinute() + ",-1)";
			addActionUrl(changeDurationUrl, item, "+2", schedule, doc);

			String addTimeUrl = "/servlet/ScheduleDataRes?action=05&id=" + item.toString();
			addActionUrl(addTimeUrl, item, "+", schedule, doc);      
		}

		element.appendChild(schedule);
	}

	private void addDevices(DataStore store, Element item, Document doc)
	{
		CaptureDeviceList devList = CaptureDeviceList.getInstance();
		int status;
		int total = devList.getDeviceCount();
		for (int i=0; i<total; i++)
		{
			CaptureDevice captureDevice = devList.getDevice(i);
			status = captureDevice.getStatus();
			Element item2 = doc.createElement("device");
			item2.setAttribute("index", Integer.toString(i));
			item2.setAttribute("name", captureDevice.getName());
			item2.setAttribute("number", Integer.toString(captureDevice.getNumber()));
			item2.setAttribute("unique_name", captureDevice.getUniqueName());
			item2.setAttribute("status", Integer.toString(status));
			if (status > 0)
			{
				item2.setAttribute("setting", captureDevice.getSettingName());
				item2.setAttribute("total_capture", status == 2 ? Integer.toString(captureDevice.getCaptureProcess().getTotalCapture()) : "0");
			}
			item.appendChild(item2);
		}
	}

	private void addCapturePathStats(DataStore store, Element item, Document doc)
	{
		String[] paths = DVScheduler.getArray(store.getProperty("Capture.path", "capture"));
		if (paths[0].isEmpty())
			paths[0] = ".";

		long minFreeSpace = store.getPropertyLong("Capture.minFreeSpace", 20000) * 1024 * 1024;
		String sMinFreeSpace = Long.toString(minFreeSpace);

		Element elm2 = null;
		int count = 0;
		for (int i=0; i<paths.length; i++)
		{
			elm2 = doc.createElement("directory");
			elm2.setAttribute("name", paths[i]);
			File file = new File(paths[i]);
			long freeSpace = file.getFreeSpace();
			if (freeSpace < minFreeSpace)
				count++;
			elm2.setAttribute("freeSpace", Long.toString(freeSpace));
			elm2.setAttribute("minFreeSpace", sMinFreeSpace);
			item.appendChild(elm2);
		}
		item.setAttribute("total", Integer.toString(paths.length));
		item.setAttribute("totalBelowMin", Integer.toString(count));
	}

	private int addDownloads(DataStore store, Element item, Document doc)
	{
		ThreadGroup top = Thread.currentThread().getThreadGroup();
		while (true)
		{
			if(top.getParent() != null)
				top = top.getParent();
			else
				break;
		}
		Thread[] theThreads = new Thread[top.activeCount()];
		top.enumerate(theThreads);
		int count = 0;
		for (int i = 0; i < theThreads.length; i++)
		{
			Thread thread = theThreads[i];
			String text = thread.getName();
			if (!text.startsWith("Serve"))
				continue;

			count++;

			String[] texts = text.split("\n");
			Element item2 = doc.createElement("download");
			item2.setAttribute("name", texts[0]);
			if (texts.length >= 4)
			{
				item2.setAttribute("completed", texts[1]);
				item2.setAttribute("speed", texts[2]);
				item2.setAttribute("timeRemaining", texts[3]);
			}
			item.appendChild(item2);
		}

		return count;
	}
}
