/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class TaskCommand
{
	private String m_name = "";
	private String m_command = "";
	private boolean enabled = true;
	private boolean autoRemove = false;
	private int delay = 0;

	private boolean m_bWhenNotCapturing = false;
	private int m_maxConcurrentTask = 0;
	private int m_minIdleTime = 0;	// In minutes
	private boolean m_allowSuspend = false;

	public TaskCommand(String nameID)
	{   
		m_name = nameID;
	}

	public TaskCommand(Node item)
	{
		NamedNodeMap itemAttribs = item.getAttributes();
		m_name = itemAttribs.getNamedItem("name").getTextContent();

		NodeList itemNodes = item.getChildNodes();
		Node itemNode = null;  
		for(int y = 0; y < itemNodes.getLength(); y++)
		{
			itemNode = itemNodes.item(y);

			if(itemNode.getNodeName().equals("enabled"))
			{
				if("true".equals(itemNode.getTextContent()))
					enabled = true;
				else
					enabled = false;
			}
			else if(itemNode.getNodeName().equals("command"))
			{
				m_command = itemNode.getTextContent();
			}
			else if(itemNode.getNodeName().equals("autoRemove"))
			{
				if("true".equals(itemNode.getTextContent()))
					autoRemove = true;
				else
					autoRemove = false;
			}
			else if(itemNode.getNodeName().equals("delay"))
			{
				delay = new Integer(itemNode.getTextContent()).intValue();
			}
			else if(itemNode.getNodeName().equals("whenIdle"))
			{
				if("true".equals(itemNode.getTextContent()))
					setWhenNotCapturing(true);
				else
					setWhenNotCapturing(false);
			}
			else if(itemNode.getNodeName().equals("maxConcurrentTask"))
			{
				setMaxConcurrentTask(Integer.parseInt(itemNode.getTextContent()));
			}
			else if(itemNode.getNodeName().equals("minIdleTime"))
			{
				setMinIdleTime(Integer.parseInt(itemNode.getTextContent()));
			}
			else if(itemNode.getNodeName().equals("allowSuspend"))
			{
				m_allowSuspend = "true".equals(itemNode.getTextContent());
			}
		}
	}

	public void setCommand(String com)
	{
		m_command = com;
	}

	public void setEnabled(boolean en)
	{
		enabled = en;
	}

	public void setAutoRemove(boolean autoRem)
	{
		autoRemove = autoRem;
	}

	public boolean getAutoRemove()
	{
		return autoRemove;
	}

	public void setAllowSuspend(boolean allowSuspend)
	{
		m_allowSuspend = allowSuspend;
	}

	public boolean isAllowSuspend()
	{
		return m_allowSuspend;
	}

	public boolean getEnabled()
	{
		return enabled;
	}

	public String getCommand()
	{
		return m_command;
	}

	public void setName(String name)
	{
		m_name = name;
	}

	public String getName()
	{
		return m_name;
	}

	public int getDelay()
	{
		return delay;
	}

	public void setDelay(int pause)
	{
		delay = pause;
	}

	public void setWhenNotCapturing(boolean bWhenNotCapturing)
	{
		m_bWhenNotCapturing = bWhenNotCapturing;
	}

	public boolean isWhenNotCapturing()
	{
		return m_bWhenNotCapturing;
	}

	public void setMaxConcurrentTask(int maxConcurrentTask)
	{
		m_maxConcurrentTask = maxConcurrentTask;
	}

	public int getMaxConcurrentTask()
	{
		return m_maxConcurrentTask;
	}

	public void setMinIdleTime(int minIdleTime)
	{
		m_minIdleTime = minIdleTime;
	}

	public int getMinIdleTime()
	{
		return m_minIdleTime;
	}

	public boolean isConditional()
	{
		return ( isWhenNotCapturing() || (getMaxConcurrentTask() > 0) || (getMinIdleTime() > 0) );
	}

	public void getXML(Document doc, Element root)
	{
		Element elm = null;
		Text text = null;

		Element item = doc.createElement("task");
		item.setAttribute("name", m_name);

		elm = doc.createElement("enabled");
		text = doc.createTextNode(new Boolean(enabled).toString());
		elm.appendChild(text);
		item.appendChild(elm);  

		elm = doc.createElement("command");
		text = doc.createTextNode(m_command);
		elm.appendChild(text);
		item.appendChild(elm);      

		elm = doc.createElement("autoRemove");
		text = doc.createTextNode(new Boolean(autoRemove).toString());
		elm.appendChild(text);
		item.appendChild(elm);  

		elm = doc.createElement("allowSuspend");
		text = doc.createTextNode(new Boolean(m_allowSuspend).toString());
		elm.appendChild(text);
		item.appendChild(elm);  

		elm = doc.createElement("delay");
		text = doc.createTextNode(new Integer(delay).toString());
		elm.appendChild(text);
		item.appendChild(elm);        

		elm = doc.createElement("whenIdle");
		text = doc.createTextNode(new Boolean(isWhenNotCapturing()).toString());
		elm.appendChild(text);
		item.appendChild(elm);

		elm = doc.createElement("maxConcurrentTask");
		text = doc.createTextNode(new Integer(getMaxConcurrentTask()).toString());
		elm.appendChild(text);
		item.appendChild(elm);

		elm = doc.createElement("minIdleTime");
		text = doc.createTextNode(new Integer(getMinIdleTime()).toString());
		elm.appendChild(text);
		item.appendChild(elm);

		root.appendChild(item);
	}

}